#!/usr/bin/python3
# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# This file is a is a tool to help translate EKD, update POT and PO 
# files, and manage translations (eg from French to English, but 
# also vice versa).
# Copyright (C) 2004-2012  Lama Angelo <ekd.forum@gmail.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os, sys, glob, time, shutil, string, stat, platform, locale, subprocess, datetime


class MurExpo_outils_traduction(object):
    
    def __init__(self):
        
        # -----------------------------------------------
        # Tous les fichiers .py sont à la racine donc 
        # pas de fichiers à récupérer avec __init__.py
        # -----------------------------------------------
        self.liste_fichiers = os.listdir(os.getcwd())
        # Liste des fichiers .py dans l'arborescence de MurExpo
        self.fichiers_a_exclure = ['barre_de_progression_export_image.py', 
                                   'capter_commandes_murexpo.py', 
                                   'chemin_data_ressources.py', 
                                   'creation_iss_exe_compil_inno_setup.py', 
                                   'fonte.py', 'murexpo_fichiers_traduction.py', 
                                   'libdeco.py', 'ligne_horizontale_ligne_verticale.py', 
                                   'push_button_icone_droite.py', 'push_button_icone_gauche.py', 
                                   'pyinstaller_creation_executable_murexpo.py', 
                                   'tempo_chemin_plateforme.py', 'tempo_ecriture.py', 
                                   'tempo_lecture.py']
        # -----------------------------------------------

        # Mise en place de la date (et heure, min, sec)
        date_time = datetime.datetime.now()
        # Le résultat sera affiché comme ceci (exemple) :
        # ... _ 22_04_2021_17h_44mn_53sec
        self.date_heure = date_time.strftime('%d_%m_%Y_%Hh_%Mmn_%Ssec')
		
        # Version de la locale (fr pour langue française)
        self.langue = locale.getdefaultlocale()[0][:2]
		
        # Conditions pour la langue (et textes dans les variables)

        if self.langue == 'fr': # français
		
            entete_1 = "OUTILS TRADUCTION DE MUREXPO"
            entete_2 = "* 1) Créer un fichier .po (locale/murexpo.po) à partir de tous les textes contenus dans MurExpo.\n# --------------\n* 2) Permet de mettre à jour les fichiers .po (lors de traductions ultérieures). Les fichiers mis\nà jour seront enregistrés dans votre répertoire utilisateur et seront de la forme\nmurexpo_symbole_langue_self.date_heure_maj.pot (par exemple: murexpo_de_maj_26_04_2021_02h_03mn_35sec.po).\n"
            annonce_crea_po_1er = "* Tapez 1 pour créer un fichier .po (locale/murexpo.po) à partir de tous les textes contenus dans MurExpo"
            annonce_maj_po = "* Tapez 2 pour mettre à jour vos fichiers .po (à faire quand des fichiers .po ont été traduits)"
            annonce_quitter = "* Tapez 0 sur le clavier pour QUITTER sans rien faire"
            annonce_choix = "Quel est votre choix ; 1, 2 ou 0 (?): "
            self.annonce_crea_fich_1 = 'Création des fichiers .po pour: '
            self.maj_txt_murexpo = 'La mise à jour a été faite sur le(s) fichier(s): '
            self.annonce_crea_fich_2 = "Création du fichier: "
            self.annonce_crea_future_maj = "Création pour future mise à jour: "
            self.erreur_msgmerge_no_install = "ERREUR: Il y a un problème !, msgmerge n'est peut-être pas\ninstallé ou la syntaxe de msgmerge a changé."
            
        else: # pour les autres langues ...

            entete_1 = "TRANSLATION TOOLS FOR MUREXPO"
            entete_2 = "* 1) Create a (locale/murexpo.po) file from all the texts in MUREXPO.\n# --------------\n* 2) Turns update .po files (translations in later). Update files will be saved in your home\ndirectory and will be of the form: murexpo_symbole_language_date_time_maj.pot\n(eg murexpo_de_maj_26_04_2021_02h_03mn_35sec.po).\n"
            annonce_crea_po_1er = "* Type 1 to create a .po file (locale/muexpo.po) from all the texts in MurExpo"
            annonce_maj_po = "* Type 2 to update your files .po files (to do when .po files have been translated)"
            annonce_quitter = "* Enter 0 on the keyboard to exit without doing anything"
            annonce_choix = "What is your choice, 1, 2 or 0 (?): "
            self.annonce_crea_fich_1 = 'Creating .po files for: '
            self.maj_fichiers_racine = "Updated files:\nlocale/murexpo.po\nlocale/murexpoDoc.po\n... and that in the tree of MurExpo."
            self.maj_txt_murexpo = 'The update was made on the file(s): '
            self.annonce_crea_fich_2 = "File creation: "
            self.annonce_crea_future_maj = "Created for future update: "
            self.erreur_msgmerge_no_install = "ERROR: There is a problem !, msgmerge is perhaps not install or msgmerge syntax has changed."

        # ---------------------------------------------------------------------------------
        # MENU PRINCIPAL DU SCRIPT 
        # ---------------------------------------------------------------------------------
        print('')
        print("================================================================================================")
        print(entete_1)
        print("================================================================================================")
        print(entete_2)
        print("================================================================================================")
        print(annonce_crea_po_1er)
        print("------------------------------------------------------------------------------------------------")
        print(annonce_maj_po)
        print("------------------------------------------------------------------------------------------------")
        print(annonce_quitter)
        print("------------------------------------------------------------------------------------------------")
        choix = input(annonce_choix)
        # choix.valeur défini dans la classe EkdInput dans: 
        # moteur_modules_common.EkdCompatiblePython2Et3.py
        if choix == '1': self.creation_fichier_po_lignes_msgstr_vides()
        elif choix == '2': self.mise_a_jour_fichiers_po()
        elif choix == '0': sys.exit()
        print('')
        
        
    def recup_fichiers_py_a_traduire(self) :
        """ """
        
        # Récupération de la liste des fichiers .py à traiter (traductions)
        liste_fichiers_string = [' ' + fich for fich in self.liste_fichiers if os.path.splitext(fich)[1] == '.py' and fich not in self.fichiers_a_exclure]
        fichiers_liste = [fich for fich in self.liste_fichiers if os.path.splitext(fich)[1] == '.py' and fich not in self.fichiers_a_exclure]
        # Les fichiers sont transformés en : ' fichier_1.py fichier_2.py ...'
        fichiers_concat = "".join(liste_fichiers_string)
        # -----------------------------------------------
        
        return fichiers_concat, fichiers_liste


    # ----------------------------------------------------------------
    # 1) Création des fichiers PO à partir des textes de MurExpo 
    # (les champs msgstr seront ici vide). En fait il s'agit
    # de la création des fichiers locale/murexpo.po et murexpoDoc.po
    def creation_fichier_po_lignes_msgstr_vides(self) :
        """
        # =================================================================================
        # Création préalable du fichier locale/murexpo.po (lors de la toute première 
        # traduction) par le menu : * Tapez 1 pour créer un fichier .po 
        # (locale/murexpo.po) à partir de tous les textes contenus dans MurExpo
        # =================================================================================
        """
        
        # Récupération des fichiers par la fonction
        # recup_fichiers_py_a_traduire (return) ...
        # Le return récupère la variable à l'indice 0
        fichiers = self.recup_fichiers_py_a_traduire()[0]

        # commande de création des fichier-sources de traduction *.pot (un pour l'interface de base et l'autre pour l'aide)
        ### (--no-wrap : ne pas couper les lignes de messages qui sont plus longues que la largeur des pages de sortie., -o, --output=FICHIER)
        os.system("xgettext --no-wrap -L Python -o locale/murexpo.po " + fichiers)
        #subprocess.Popen("xgettext --no-wrap -k --keyword='tr' -o locale/murexpoDoc.po " + totalFichiersSource)
        
        print('')
        print(self.annonce_crea_fich_1)
        print(fichiers)  # test
        print('')
        
    # ----------------------------------------------------------------------------------------    
    # 2) Fonction pour la mise à jour des fichiers .po ... en rajoutant les nouveaux textes 
    # implémentés dans MurExpo (quand des nouvelles fonctionnalités sont ajoutées) en prenant 
    # comme base locale/murexpo.po ... et en exportant les textes rajoutés sous la forme d'un 
    # fichier :
    # 1) maj_traduction_etape_1/murexpo_symbole_langue_maj.po et
    # maj_traduction_etape_1/murexpo_symbole_langue_maj.pot ... par exemple
    # maj_traduction_etape_1/murexpo_de_maj.po et maj_traduction_etape_1/murexpo_de_maj.pot
    # 2) maj_traduction_final/murexpo_symbole_langue_maj_date_heure.po et
    # maj_traduction_final/murexpo_symbole_langue_maj_date_heure.pot ... par exemple :
    # maj_traduction_final/murexpo_de_maj_25_04_2021_16h_06mn_16sec.po et
    # maj_traduction_final/murexpo_de_maj_25_04_2021_16h_06mn_16sec.po
    def mise_a_jour_fichiers_po(self) :
        """
        # =================================================================================
        # Fonction pour mise à jour des fichiers *.po dans le répertoire 
        # ---------------------------------------------------------------
        # maj_traduction_final (lui-même contenu ds le rep. du projet)
        # ------------------------------------------------------------
        # ... et le final dans le rep. maj_traduction_final
        # --------------------------------------------------
        #
        # Voilà comment procéder:
        # -----------------------
        #
        # Créer les fichiers (lors de la toute première traduction) :
        # -----------------------------------------------------------
        #
        # 1) Si le fichier locale/murexpo.po n'existe pas, il faut le créer
        # par la commande : python3 murexpo_outils_traductions.py (--> en 
        # en fait parle script que vous êtes en train d'utiliser) --> choisir :
        # * Tapez 1 pour créer un fichier .po (locale/murexpo.po) à partir de tous 
        # les textes contenus dans MurExpo ... dans le menu). Cette action va
        # créer le fichier murexpo.po contenant tous les textes à traduire.
        #
        # 2) Faire la traduction du fichier murexpo.po et ekdDoc.po dans PoEdit et
        # les sauvegarder dans le répertoire locale/es/LC_MESSAGES (dans le cas de
        # la traduction espagnole) sous le nom murexpo.po
        #
        # Mettre à jour les fichiers (lors de traductions ultérieures)
        # ------------------------------------------------------------
        # par la suite dans le développement de l'application quand
        # ----------------------------------------------------------
        # de nouvelles fonctionnalités (et de nouveaux textes) auront
        # ------------------------------------------------------------
        # été rajouté(es) :
        # -----------------
        #
        # 1) Ouvrir une console dans le répertoire où se trouve ce script 
        # murexpo_outils_traductions.py (dans la console : 
        # cd /chemin/vers/le/répertoire/de MurExpo)
        # et mettre à jour les fichiers .po par la commande :
        # python3 murexpo_outils_traductions.py (--> en fait par le script 
        # que l'on est en train d'utiliser --> choisir : * Tapez 2 pour mettre 
        # à jour vos fichiers .po (à faire quand des fichiers .po ont été traduits).
        # ==> Les fichiers .po de mise à jour seront crées dans le rep. 
        # maj_traduction_final (lui-même contenu ds le rep. du projet)
        # et porteront les noms:
        # murexpo_symbole_langue_maj_jour_mois_année_heure_minutes_secondes.po
        # (par exemple murexpo_de_maj_22_04_2021_01h_32mn_47sec.po)
        #
        # 2) Mettez en téléchargement quelque part ces fichiers pour les traducteurs du 
        # projet ... de votre côté passez-les à la moulinette dans PoEdit pour obtenir les 
        # fichiers murexpo.po et murexpo.mo (en les ayant renommés) puis copiez-les 
        # (toujours dans le cas d'une traduction en espagnol) dans locale/es/LC_MESSAGES 
        # avec un système en espagnol ... et vérifier que tout s'affiche correctement ... 
        # (gardez tout de même une copie de sauvegarde de vos fichiers murexpo.po quelque 
        # part [fruit d'une traduction préalable ... histoire de ne pas perdre le travail 
        # déjà effectué], avant de faire une quelconque mise à jour des différents 
        # fichiers .po !!!).
        # ====================================================================================
        """
        
        print('')
        
        # Chemin pour la 1ère étape de la mise à jour :
        rep_1_maj_langue = os.path.join(os.getcwd(), 'maj_traduction_etape_1')
        # Chemin pour récupérer la mise à jour finale
        # des fichiers po de traduction
        rep_final_maj_langue = os.path.join(os.getcwd(), 'maj_traduction_final')
        
        #
        if not os.path.exists(os.path.join(os.getcwd(), 'maj_traduction_etape_1')) :
            print('')
            print("Le répertoire {} n'existe pas ... il est donc créée.".format(rep_1_maj_langue))
            print('')
            os.mkdir('maj_traduction_etape_1')
            
        #
        if not os.path.exists(os.path.join(os.getcwd(), 'maj_traduction_final')) :
            print("Le répertoire {} n'existe pas ... il est donc créée.".format(rep_final_maj_langue))
            print('')
            os.mkdir('maj_traduction_final')
               
        # ---------------------------------------------------------
        # Le répertoire de l'étape 1 de la mise à jour est sondé et
        # si il contient déjà des fichiers .po, ils sont supprimés
        # ---------------------------------------------------------
        fich_rep_1_maj_langue = os.listdir(rep_1_maj_langue)
        if len(fich_rep_1_maj_langue) > 0 :
            for f in fich_rep_1_maj_langue :
                chem_complet = os.path.join(rep_1_maj_langue, f)
                os.remove(chem_complet)

        # Récup. de tous les répertoires des langues présentes
        # dans le rep. locale. Affiche ['de', 'en', 'es', 'it']
        listeRepLocale = [os.path.basename(parcLoc) for parcLoc in glob.glob(os.path.join('locale', '*')) if os.path.basename(parcLoc) not in 'murexpo.po']
        listeRepLocale.sort()

        # Liste sélectionnant les fichiers .po des 
        # répertoires de chaque langue gérée. Affiche 
        # ['locale/de/LC_MESSAGES/murexpo.po', 
        # 'locale/en/LC_MESSAGES/murexpo.po', 
        # 'locale/es/LC_MESSAGES/murexpo.po', 
        # 'locale/it/LC_MESSAGES/murexpo.po']
        listeFichPo = [parcPo for parcRepLoc in listeRepLocale for parcPo in glob.glob(os.path.join('locale', parcRepLoc, 'LC_MESSAGES', '*.*')) if parcPo.endswith('po') or parcPo.endswith('pot')]
        listeFichPo.sort()

        # Liste de listes comportant le chemin associé au nom de fichier 
        # (sans extension) affiche par exemple: 
        # ['locale/de/LC_MESSAGES/murexpo', 'locale/en/LC_MESSAGES/murexpo', 
        # 'locale/es/LC_MESSAGES/murexpo', 'locale/it/LC_MESSAGES/murexpo']
        lChemNFich = ["".join(parSplit.split('.')[0]) for parSplit in listeFichPo]
        lChemNFich.sort()

        # Mise en relation entre le chemin et le titre de chaque fichier.po et le symbole langue, 
        # ça donne par exemple la liste:
        # [('locale/de/LC_MESSAGES/murexpo', 'de'), ..., ('locale/it/LC_MESSAGES/murexpo', 'it')]
        unionPoEtLangue = list(zip(lChemNFich, listeRepLocale))

        # ------------------------------------------------
        # Mise en relation du symbole langue dans le titre et copie 
        # des fichiers dans le répertoire maj_traduction_etape_1 
        # (ça donne par exemple: murexpo_en.po, ..., murexpo_es.po)
        for parc_copie_maj_1 in unionPoEtLangue :
            if 'murexpo' in parc_copie_maj_1[0] :
                shutil.copy(os.path.join(os.getcwd(), parc_copie_maj_1[0]+'.po'), os.path.join(rep_1_maj_langue, os.path.basename(parc_copie_maj_1[0])+'_'+parc_copie_maj_1[1]+'.po'))
        # ------------------------------------------------

        # Récupération des fichiers par la fonction
        # recup_fichiers_py_a_traduire (return) ...
        # Le return récupère la variable à l'indice 1
        liste_fichiers = self.recup_fichiers_py_a_traduire()[1]
        
        # Récupération de  la liste des fichiers .py à traiter (traductions)
        pre_fichiers = [fich for fich in liste_fichiers if os.path.splitext(fich)[1] == '.py' and fich not in self.fichiers_a_exclure]
        
        # --------------------------------------------------------
        # Les parties nouvellement ajoutées (et non encore 
        # traduites) viennent s'ajouter aux fichiers .po se
        # trouvant dans locale/langue/LC_MESSAGES de chaque langue
        # ... par exemple dans locale/de/LC_MESSAGES/murexpo.po
        # --------------------------------------------------------
        # Boucle principale (parcours des fichiers 
        # .py dont les textes sont à traduire)
        for parc_fich in pre_fichiers :
            # Parcours des fichiers contenus 
            # dans le répertoire locale ... 
            # langue ... LC_MESSAGES et 
            # récup des fichiers .po
            for trad in listeFichPo :
                # Lancement de la commande ...
                os.system('xgettext --keyword=__ --keyword=_e --default-domain=murexpo --language=python ' + parc_fich + ' --join-existing --output='+trad)
        # --------------------------------------------------------

        # Scanne le contenu du répertoire : maj_traduction_etape_1'
        contenu_rep_maj = glob.glob(os.path.join(rep_1_maj_langue, '*.*'))
        
        # ---------------------------------------------------------------------
        # Mise à jour des fichiers avec msgmerge (permet d'enlever les anciens 
        # textes inutilisés et permet en même temps d'alléger les fichiers PO)
        try :
            for fichPO in contenu_rep_maj :
                # f --> /home/angelo/projet_MurExpo/maj_traduction_etape_1/murexpo_de ...
                # e --> .po 
                f, e = os.path.splitext(fichPO)
                if 'murexpo_' in f :
                    # La syntaxe ... > ... peut aussi être utilisée à la place de ... -o ...
                    # Fichier PO
                    os.system('msgmerge '+fichPO+' '+os.path.join(os.getcwd(), 'locale', 'murexpo.po')+' -o '+os.path.join(os.path.dirname(contenu_rep_maj[0]), os.path.basename(f)+'_maj.po'))
                    # Fichier POT
                    os.system('msgmerge '+fichPO+' '+os.path.join(os.getcwd(), 'locale', 'murexpo.po')+' -o '+os.path.join(os.path.dirname(contenu_rep_maj[0]), os.path.basename(f)+'_maj.pot'))   
        except :
            print("ERREUR: Il y a un problème !, msgmerge n'est peut-être pas\ninstallé ou la syntaxe de msgmerge a changé.")
            print(self.erreur_msgmerge_no_install)
        # ---------------------------------------------------------------------

        # ---------------------------------------------------------------------
        # Renommage des fichiers .po ... et .pot (par exemple: 
        # murexpo_de_maj_21_04_2021_18h_59mn_27sec.po et
        # murexpo_de_maj_21_04_2021_18h_59mn_27sec.pot,
        # et copie des fichiers dans le rép. maj_traduction_final
        # ---------------------------------------------------------------------
        collectePO = glob.glob(rep_1_maj_langue+os.sep+'*.po')
        for collRenom in collectePO :
            if 'maj.po' in collRenom :
                fich, ext = os.path.splitext(collRenom)
                # Renommage du fichier PO
                shutil.copy(collRenom, os.path.join(rep_final_maj_langue, os.path.basename(fich)+'_'+self.date_heure+'.po'))
                # Renommage du fichier POT
                shutil.copy(collRenom, os.path.join(rep_final_maj_langue, os.path.basename(fich)+'_'+self.date_heure+'.pot'))
                print('')
                print(self.annonce_crea_fich_2 + " {0}".format(os.path.join(rep_final_maj_langue, os.path.basename(fich)+'_'+self.date_heure+'.po')))
                print(self.annonce_crea_fich_2 + " {0}".format(os.path.join(rep_final_maj_langue, os.path.basename(fich)+'_'+self.date_heure+'.pot')))
        # ---------------------------------------------------------------------
         
        
# Appel de la classe
main = MurExpo_outils_traduction()
