#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2020
#-------------------------------------------------------------------------------

# Merci à eyllanesc pour son aide est pour cette classe ... import
# --> pour les icônes à droite dans le bouton
# --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
from push_button_icone_droite import PushButtonIconeDroite
# --> pour les icônes à gauche dans le bouton
from push_button_icone_gauche import PushButtonIconeGauche

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture
# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Traitement des oeuvres avec Pillow (par exemple
# l'oeuvre est encadrée ou entoilée)
from traitement_image_oeuvre import Traitement_Image_Oeuvre

# Appel du module (fonction avec return) de redimensionnement
from redimensionnement_oeuvre import redimensionnement_image_oeuvre

# Appel du module fonte (fonction 
# fonte avec arguments) contenu 
# dans le script fonte.py ... pour
# attribuer une fonte spécifique
# à un widget (le type de police)
from fonte import fonte

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

# Appel de la classe pour mettre en place une barre de progression (factice 
# --> fonctionnant avec un QTimer) utilisée pour l'exportation de l'image
# de l'exposition
from barre_de_progression_export_image import BarreDeProgressionExportImage

# Appel de la classe (QThread) pour le traitement de la
# 2ème partie (transformation en gif animé avec imageio)
from gif_anime_thread import Fenetre_barre_Gif_Anime

# Appel de la classe pour exporter l'exposition sous 
# la forme d'un fichier html (et aussi sous la forme 
# d'un fichier PDF) ... aussi par un QThread
from murexpo_export_html import Murexpo_Export_Html

# Capture des toutes les oeuvres chargées sur le mur d'exposition.
from capture_oeuvres_chargees_murexpo import CapterToutesOeuvresChargees
# Capture des toutes les oeuvres supprimées sur le mur d'exposition.
from capture_oeuvres_supprimees_murexpo import CapterToutesOeuvresSupprimees

import os, sys, glob, platform, imageio

# Import de Pillow
try : import Image
except ImportError : from PIL import Image

# Import du timer du module threading
from threading import Timer

# Imports PyQt5 -----------------------------------------------------------------------
from PyQt5.QtWidgets import QComboBox, QWidget, QStackedWidget, QVBoxLayout, QHBoxLayout, \
                            QGroupBox, QPushButton, QGridLayout, QLabel, \
                            QGraphicsScene, QGraphicsView, QGraphicsPixmapItem, \
                            QGraphicsItem, QApplication, QTextEdit, QSlider, QFileDialog, \
                            QGraphicsDropShadowEffect, QSpinBox, QCheckBox, QPushButton, QProgressBar # QProgressBar
from PyQt5.QtCore import QSize, Qt, QTimer, QRectF, QFileInfo, QDir, pyqtSignal, QLocale, QRect, QPointF # (QPointF indispensable !)
from PyQt5.QtGui import QPixmap, QIcon, QColor, QTransform, QPainter, QImage, QBrush, QFont, QPen, QPainterPath#, QMovie
# --------------------------------------------------------------------------------------

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------

# ------------------------------------------------ A garder pour l'instant -------------
"""
# --------------------------------------------------
# Liste GLOBALE de positionnement des 
# items (les images des oeuvres)
liste_position_PixmapItem = []
# --------------------------------------------------
"""
# ------------------------------------------------ A garder pour l'instant -------------


class Fabrique(QWidget):
    ''' Classe pour disposer et faire les réglages 
    du mur d'exposition et des oeuvres qui vont y 
    être accrochées '''
    
    # =========================================================================
    # Création du signal en vue de lier le bouton 'Ouvrir image' (page
    # 2 du QStackedWidget) à l'onglet 'Fichier ouvrir'. 1ère partie. 
    # Un grand merci à eyllanesc !!!
    # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
    # qtabwidget-defined-in-the-main-class-in-another-class#63065558
    customSignal_onglet_ouvrir_oeuvre = pyqtSignal(int)
    # =========================================================================
    # Création du signal en vue de lier le bouton 'Visuel' (page
    # 2 du QStackedWidget) à l'onglet 'Visionnage'. 1ère partie. 
    # Un grand merci à eyllanesc !!!
    customSignal_onglet_visuel_exposition = pyqtSignal(int)
    # =========================================================================
    # Création du signal en vue de lier le bouton 'Récupérer police de 
    # caractère' (page 3 du QStackedWidget) à l'onglet 'Choix police'. 1ère 
    # partie. Un grand merci à eyllanesc !!!
    customSignal_onglet_choix_police = pyqtSignal(int)
    # =========================================================================
    # Création du signal en vue de lier le bouton 'Modifier les couleurs' 
    # (page 3 du QStackedWidget) à l'onglet 'Choix couleur'. 1ère partie. 
    # Un grand merci à eyllanesc !!!
    customSignal_onglet_choix_couleur = pyqtSignal(int)
    # =========================================================================
    
    def __init__(self, parent=None):
        super(Fabrique, self).__init__(parent)

        # -------------------------------------------------
        # Appel de la classe définie au QGraphicsView
        # (c'est à dire la vue dans laquelle le mur et
        # les oeuvres vont être ajoutées)
        self.graphe_vue = MonGraphiqueVueMur()
        # -------------------------------------------------
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------

        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        print(_("Chemin du répertoire temporaire : "), self.chemin_tempo_system_exploit)
        
        # -------------------------------------------------------------------------------------------
        # Non encore utilisé (mais sera certainement utile plus tard)
        # -------------------------------------------------------------------------------------------
        # Drapeau écrit dans le rep. tempo. (donnée écrite à 0, c'est à dire l'index originel)
        # -------------------------------------------------------------------------------------------
        # Ecriture de l'index de la phase (dans l'onglet Conception et exportation) 
        # --> à la page 4 de l'affichage des phases [index 3 du stacked --> self.stack]
        # --------------------------------------
        # Pour l'instant on est à la page 4 (index 3) mais cela peut changer si une
        # page/phase est rajoutée pour la procédure de conception de l'exposition
        # --------------------------------------
        ecr_tempo_index_phase_page_exposition = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_index_page_phase_murexpo_", "data_index_page_phase_murexpo", ".txt", '3')
        # -------------------------------------------------------------------------------------------
        
        # -------------------------------------------------------------------------------------------
        # Non encore utilisé (mais sera certainement utile plus tard)
        # -------------------------------------------------------------------------------------------
        # Drapeau écrit dans le rep. tempo. (donnée écrite à 0, c'est à dire l'index originel)
        # -------------------------------------------------------------------------------------------
        # Ecriture de l'index de la phase d'exportation (dans l'onglet Conception et exportation) 
        # --> à la page 4 de l'affichage des phases [index 3 du stacked --> self.stack] ... et 
        # ... et à l'index 0 du stacked --> self.stack_sauvegarde ... de même à l'index 0 dans 
        # la combo self.liste_sauvegarde (ici pour l'instant le choix est : Exporter (sous la 
        # forme d'une image) l'exposition)
        # --------------------------------------
        # Pour l'instant, tant que rien n'a été sélectionné dans la combo self.liste_sauvegarde, on
        # est toujours à l'index 0, c'est à dire avec le choix : Exporter (sous la forme d'une 
        # image) l'exposition ...
        # --------------------------------------
        ecr_tempo_index_exportation = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_index_exportation_conversion_murexpo_", "data_index_exportation_conversion_murexpo", ".txt", '0')
        # -------------------------------------------------------------------------------------------
        
        # -------------------------------------------------------------------------------------------
        # Fichier tempo qui fait office de drapeau ... (très important). Utilisé pour l'enregistrement 
        # de l'exposition sous la forme d'une image (lors de cette étape ce fichier tempo est ré-écrit 
        # et prend la valeur '1'). Utilisé ensuite pour afficher la barre de de progression pour 
        # l'exportation sous la forme d'une image.
        ecr_tempo_drapeau_progression_enregistr_img = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_fin_exportation_exposition_image_murexpo_", "data_fin_exportation_exposition_image_murexpo", ".txt", '0')
        # -------------------------------------------------------------------------------------------
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Info) Chemin du répertoire temporaire : {}\n").format(self.chemin_tempo_system_exploit))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        # Liste des icônes pour les rubriques
        liste_icones_rubriques = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))]
        # Liste des rubriques
        self.liste_rubriques = [_("Sélection du mur (phase 1)"), _("Chargement, disposition et réglage des oeuvres (phase 2)"), _("Chargement de la police de caractère, disposition et réglages des panneaux/cartels (phase 3)"), _("Ajout de personnages, cadres et accessoires (phase 4)"), _("Exportation/Conversion (phase 5)")]
        self.liste_menu = QComboBox()
        self.liste_menu.setMinimumHeight(36) # 36
        # La liste du menu se fait par
        # l'intermédiaire d'un QComBoBox
        for icones_rub, rubriques in zip(liste_icones_rubriques, self.liste_rubriques) :
            self.liste_menu.addItem(icones_rub, rubriques)
        self.liste_menu.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")

        # --------------------------------------------------------------------------
        # QTextEdit de visualisation de l'image 
        # (de l'oeuvre) sélectionnéee
        self.widgetEditAffOeuvre = QTextEdit()
        self.widgetEditAffOeuvre.setMinimumHeight(100)
        self.widgetEditAffOeuvre.setMaximumWidth(200)
        # Le QTextEdit est placé en lecture seule
        self.widgetEditAffOeuvre.setReadOnly(True)
        #
        self.widgetEditAffOeuvre.setStyleSheet("QTextEdit { background-color: rgb(100, 100, 100) }")
        # --------------------------------------------------------------------------

        # ---------------------------------------------------------------------
        # Les widgets pour le QStackedWidget
        self.menu_stack_1 = QWidget()
        self.menu_stack_2 = QWidget()
        self.menu_stack_3 = QWidget()
        self.menu_stack_4 = QWidget()
        self.menu_stack_5 = QWidget()
        
        # Fonctions des menus 
        # du QStackedWidget
        self.reg_stack_1()
        self.reg_stack_2()
        self.reg_stack_3()
        self.reg_stack_4()
        self.reg_stack_5()
        # Ajout des fonctions dans
        # le QStackedWidget
        self.stack = QStackedWidget(self)
        self.stack.addWidget(self.menu_stack_1)
        self.stack.addWidget(self.menu_stack_2)
        self.stack.addWidget(self.menu_stack_3)
        self.stack.addWidget(self.menu_stack_4)
        self.stack.addWidget(self.menu_stack_5)
        # ---------------------------------------------------------------------

        # ********************************
        # Groupes 
        groupe_1 = QGroupBox()
        groupe_2 = QGroupBox()
        groupe_3 = QGroupBox()
        # --------------------------------
        # Affichage du QGraphicsView
        hbox_1 = QHBoxLayout()
        hbox_1.addWidget(self.graphe_vue)
        #
        groupe_1.setLayout(hbox_1)
        # --------------------------------
        # Affichage du combo de navigation
        # (pages) et du bouton quitter
        hbox_2 = QHBoxLayout()
        hbox_2.addWidget(self.liste_menu)
        #
        groupe_2.setLayout(hbox_2)
        # --------------------------------
        # Affichage du QStackedWidget
        vbox_2 = QVBoxLayout()
        vbox_2.addWidget(self.stack)
        groupe_3.setLayout(vbox_2)
        # --------------------------------
        # La grille
        grid = QGridLayout()
        #
        grid.addWidget(groupe_1, 0, 0)
        grid.addWidget(groupe_2, 1, 0)
        grid.addWidget(groupe_3, 2, 0)
        
        #
        self.setLayout(grid)
        # ********************************

        # =====================================================
        # Signal pour afficher les différentes phases/pages
        # choix dur mur et son affichage, choix des oeuvres, 
        # les réglages et leur affichage et l'exportation
        # (par exemple sous la forme d'image(s))
        self.liste_menu.currentIndexChanged.connect(self.affiche_phases)
        # =====================================================

        #------------------------------------------------------------
        # Création d'un QTimer pour l'affichage de 
        # l'image sélectionnée dans self.widgetEditAffOeuvre
        #------------------------------------------------------------
        # Création du timer
        self.timer = QTimer(self)
        # Appel de la fonction visuel_oeuvre_selectionnee
        # à partir du timer (pour mise à jour de
        # l'interface (après coup labels, labels, 
        # boutons et lignes d'informations)
        self.timer.timeout.connect(self.visuel_oeuvre_selectionnee)
        # MAJ ttes les deux secondes
        self.timer.start(2000)
        #------------------------------------------------------------


    def quitter(self) :
        """ Fermer tout le logiciel """
        
        sys.exit(0)
        
        
    def visuel_oeuvre_selectionnee(self) :
        """ Fonction pour afficher l'image sélectionnée dans 
        le QTreeView simplement en sélectionnant l'image """
        
        try :
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_chemin_image = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_image_oeuvre_murexpo_', 'data_chemin_image_oeuvre_murexpo')
            # ... avec appel de la méthode affichage
            lecture_chemin_image = "".join(pre_lecture_chemin_image.affichage())
        except : 
            lecture_chemin_image = chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png'))

        # Quand l'utilisateur sélectionne une image, 
        # l'image en question est contenue dans la 
        # variable lecture_chemin_image (venant du 
        # fichier data_chemin_image_oeuvre_murexpo_.txt 
        # contenu dans le répertoire temporaire)
        try : 
            # Ouverture de l'image sélectionnée 
            # par l'utilisateur avec Pillow
            img = Image.open(lecture_chemin_image)
        # Si le fichier n'existe pas (il est recrée 
        # --> voir dans widget_arborescence.py) ou 
        # s'il est vide, on continue ...
        except : pass

        # -----------------------------------------------
        # Si l'utilisateur ne selectionne aucune image
        # dans le QTreeView (cas typique au moment du
        # démarrage du logiciel) ... c'est à dire que 
        # le fichier temporaire :
        # data_chemin_image_oeuvre_murexpo_.txt ne
        # contient aucune donnée ...
        # -----------------------------------------------
        if lecture_chemin_image == '' :
            largeur = 160
            hauteur = 120
            # Un fond gris est affiché dans le QTextEdit
            self.widgetEditAffOeuvre.setHtml('<center><img src="{}" width={} height="{}" /></center>'.format(chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png')), largeur, hauteur))
            # Le QTextEdit est redimensionné en hauteur (pour
            # ne pas avoir de décalage en hauteur au moment
            # de l'affichage)
            self.widgetEditAffOeuvre.setFixedHeight(int(hauteur))
            # Affichage en lecture seule
            self.widgetEditAffOeuvre.setReadOnly(True)
            
        # -----------------------------------------------
        # Dès que l'utilisateur sélectionne une
        # image dans le QTreeView ...
        # -----------------------------------------------
        else :
            try :
                # -----------------------------------------------
                # Définition préalable des tailles
                # et calcul de la hauteur
                # -----------------------------------------------
                largeur = 180
                # Vérification de la résolution
                # (en pixels) de l'image chargée
                w, h = img.size        
                # Calcul du ratio de l'image
                ratio = float(w)/float(h)
                # Calcul de la future largeur
                # (ce coup-ci on multiplie)
                hauteur = int(float(largeur/ratio))
            except :
                self.widgetEditAffOeuvre.setHtml('<center><img src="{}" width={} height="{}" /></center>'.format(chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png')), largeur, hauteur))
                self.widgetEditAffOeuvre.setFixedHeight(80)  
                
            # -----------------------------------------------
            # Dans le cas où la largeur de l'image est plus
            # grande que la hauteur (type image paysage)
            # -----------------------------------------------
            try :
                if largeur > int(hauteur) :
                    largeur = 150 # taille définie
                    # Vérification de la résolution
                    # (en pixels) de l'image chargée
                    w, h = img.size        
                    # Calcul du ratio de l'image
                    ratio = float(w)/float(h)
                    # Calcul de la future largeur
                    # (ce coup-ci on multiplie)
                    hauteur = int(float(largeur/ratio))
            except :
                self.widgetEditAffOeuvre.setHtml('<center><img src="{}" width={} height="{}" /></center>'.format(chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png')), largeur, hauteur))
                self.widgetEditAffOeuvre.setFixedHeight(80)
                
            # -----------------------------------------------
            # Dans le cas où la hauteur de l'image est plus
            # grande que la largeur (type image portrait)
            # -----------------------------------------------
            try :
                if int(hauteur) > largeur :
                    # Hauteur définie
                    hauteur = 120
                    # Vérification de la résolution
                    # (en pixels) de l'image chargée
                    w, h = img.size        
                    # Calcul du ratio de l'image
                    ratio = float(w)/float(h)
                    # Calcul de la future largeur
                    # (ce coup-ci on multiplie)
                    largeur = int(float(hauteur*ratio))
            except :
                self.widgetEditAffOeuvre.setHtml('<center><img src="{}" width={} height="{}" /></center>'.format(chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png')), largeur, hauteur))
                self.widgetEditAffOeuvre.setFixedHeight(80) 
                
            # -----------------------------------------------
            # Dans le cas où la largeur de 
            # l'image est égale à la hauteur
            # -----------------------------------------------
            try :
                if int(hauteur) == largeur :
                    # On redéfinit la 
                    # largeur et la hauteur 
                    largeur = 120
                    hauteur = 120
            except :
                self.widgetEditAffOeuvre.setHtml('<center><img src="{}" width={} height="{}" /></center>'.format(chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png')), largeur, hauteur))
                self.widgetEditAffOeuvre.setFixedHeight(80) 
                
            # -----------------------------------------------
            
            try :
                # Mise en place des données en html dans le QTextEdit
                self.widgetEditAffOeuvre.setHtml('<center><img src="{}" width={} height="{}" /></center>'.format(lecture_chemin_image, largeur, hauteur))
                # Le QTextEdit est redimensionné en hauteur (pour
                # ne pas avoir de décalage en hauteur au moment
                # de l'affichage)
                self.widgetEditAffOeuvre.setFixedHeight(hauteur)
                # Affichage en lecture seule
                self.widgetEditAffOeuvre.setReadOnly(True)
            except :
                self.widgetEditAffOeuvre.setHtml('<center><img src="{}" width={} height="{}" /></center>'.format(chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png')), largeur, hauteur))


    def reg_stack_1(self) :
        """ Fonction de la page 1 du QStackedWidget """

        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe 1 --> elle est ici : affichage_apsc/push_button_icone_droite.py ... 
        # en fait le texte dans le bouton est centré et l'icône positionné à droite.
        # J'ai un peu modifié la classe et adaptée. Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : chemin_data_ressources.py)
        icone_derniere_page = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_3_fleches_droite_murexpo_256x256.png')))
        icone_suivant = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_droite_murexpo_256x256.png')))
        #
        # Icône pour le bouton principal d'affichage (du mur)
        icone_affichage = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png')))
        #
        bout_dernier_phase_1 = PushButtonIconeDroite(_("Dernière page"), icon = icone_derniere_page, iconSize = QSize(16, 16))
        bout_dernier_phase_1.setFixedWidth(180)
        #
        bout_suivant_phase_1 = PushButtonIconeDroite(_("Page suivante"), icon = icone_suivant, iconSize = QSize(16, 16))
        bout_suivant_phase_1.setFixedWidth(180)
        #
        # Bouton pour 'lancer' la prévisualisation (l'affichage principal dans le QGraphicsView)
        self.bout_visio = PushButtonIconeGauche(_("Affichage du mur"), icon = icone_affichage, iconSize = QSize(16, 16))
        self.bout_visio.setMaximumWidth(230)
        self.bout_visio.setMinimumWidth(230)
        # ToolTip
        self.bout_visio.setToolTip(_("Pour afficher/voir l'image du mur d'exposition"))

        # ----------------------------------------------------------------------------
        # Combobox du choix des images du mur d'exposition
        self.combo_choix_mur = QComboBox()
        self.combo_choix_mur.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 2px")
        # Liste des images du mur
        liste_images_mur = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mur_001.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mur_002.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mur_003.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mur_004.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mur_005.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mur_006.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mur_007.png'))), \
                            QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mur_008.png')))]
        # Taille des images dans la combo
        self.combo_choix_mur.setIconSize(QSize(200, 60)) # 200, 60
        # Liste des sélections (au niveau texte)
        liste_txt_mur = [_("Mur d'exposition 1"), _("Mur d'exposition 2"), _("Mur d'exposition 3"), _("Mur d'exposition 4"), _("Mur d'exposition 5"), _("Mur d'exposition 6"), _("Mur d'exposition 7"), _("Mur d'exposition 8")]
        # Attribution ...
        for img_mur, txt_mur in zip(liste_images_mur, liste_txt_mur) :
            self.combo_choix_mur.addItem(img_mur, txt_mur)
        # ----------------------------------------------------------------------------
        
        groupe_1 = QGroupBox()
        groupe_2 = QGroupBox()
        
        grid_1 = QGridLayout()
        grid_1.setSpacing(8)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        grid_2 = QGridLayout()
        grid_2.setSpacing(8)
        grid_2.setContentsMargins(4, 4, 4, 4)
        
        grid_1.addWidget(self.combo_choix_mur, 0, 0)
        grid_1.addWidget(self.bout_visio, 0, 1)
        #
        grid_2.addWidget(bout_dernier_phase_1, 0, 0)
        grid_2.addWidget(bout_suivant_phase_1, 0, 1)

        #
        groupe_1.setLayout(grid_1)
        groupe_2.setLayout(grid_2)
             
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        layout.addWidget(groupe_2, 1, 0)
        
        #
        self.menu_stack_1.setLayout(layout)
        
        # =====================================================
        # Signal du combobox du choix du mur
        self.combo_choix_mur.currentIndexChanged.connect(self.changements_combo_choix_mur)
        # --------
        # Signal pour afficher le mur dans le QGraphicsView
        # --------
        # Définition de l'argument (appel de la variable 
        # retournée dans la fonction afficher_mur dans 
        # la classe MonGraphiqueVueMur)
        argument = self.graphe_vue.recup_choix_mur
        # Le signal lui-même
        self.bout_visio.clicked.connect(lambda: self.graphe_vue.afficher_mur(argument))
        # =====================================================
        # Signal pour passer à la dernière page
        bout_dernier_phase_1.clicked.connect(lambda: self.liste_menu.setCurrentIndex(4))
        # Signal pour passer à la page de la phase 2
        bout_suivant_phase_1.clicked.connect(lambda: self.liste_menu.setCurrentIndex(1))
        # =====================================================
        
        ##################################################################### ESSAI
        # ---------- Récup du libellé du bouton sélectionné par l'utilisateur
        #self.bout_visio.released.connect(self.libelle_bouton_aff_mur)
        ##################################################################### ESSAI

    
    def reg_stack_2(self) :
        """ Fonction de la page 2 du QStackedWidget """
        
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe 1 --> elle est ici : affichage_apsc/push_button_icone_droite.py ... 
        # en fait le texte dans le bouton est centré et l'icône positionné à droite.
        # J'ai un peu modifié la classe et adaptée. Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : config_apsc/chemin_data_ressources.py)
        icone_ouvrir_fichier_oeuvre = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_png_dossier_murexpo_256x256.png')))
        icone_transfert_img_vue = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_plus_murexpo_256x256.png')))
        icone_supprimer_img_vue = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_moins_murexpo_256x256.png')))
        icone_appliquer_cadre_ou_toile = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
        icone_visualiser = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png')))
        icone_relever_item = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_relever_256x256_murexpo.png')))
        icone_rabaisser_item = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_abaisser_256x256_murexpo.png')))
        icone_tourner_droite_item = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_tourner_a_droite_256x256_murexpo.png')))
        icone_tourner_gauche_item = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_tourner_a_gauche_256x256_murexpo.png')))
        icone_retour = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_gauche_murexpo_256x256.png')))
        icone_suivant = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_droite_murexpo_256x256.png')))
        
        #
        bout_ouvrir_fichier_oeuvre = PushButtonIconeGauche(_("Ouvrir image"), icon = icone_ouvrir_fichier_oeuvre, iconSize = QSize(16, 16))
        bout_ouvrir_fichier_oeuvre.setFixedWidth(150)
        bout_ouvrir_fichier_oeuvre.setToolTip(_("Charge une image (oeuvre) pour le mur d'exposition"))
        #
        bout_affiche_oeuvre = PushButtonIconeGauche("", icon = icone_transfert_img_vue, iconSize = QSize(16, 16))
        bout_affiche_oeuvre.setFixedWidth(30)
        bout_affiche_oeuvre.setToolTip(_("Ajoute l'image chargée au mur d'exposition"))
        #
        bout_supprimer_oeuvre = PushButtonIconeGauche("", icon = icone_supprimer_img_vue, iconSize = QSize(16, 16))
        bout_supprimer_oeuvre.setFixedWidth(30)
        bout_supprimer_oeuvre.setToolTip(_("Supprime l'image chargée du mur d'exposition"))
        #
        self.bout_appliquer = PushButtonIconeGauche(_("Appliquer"), icon = icone_appliquer_cadre_ou_toile, iconSize = QSize(16, 16))
        self.bout_appliquer.setFixedWidth(170)
        self.bout_appliquer.setToolTip(_("Pose d'un cadre, mise sous verre ou entoilage de la sélection"))
        #
        self.bout_visualiser = PushButtonIconeGauche(_("Visuel"), icon = icone_visualiser, iconSize = QSize(16, 16))
        self.bout_visualiser.setFixedWidth(100)
        self.bout_visualiser.setToolTip(_("Visualiser la pose des oeuvres pendant la confection de l'exposition"))
        #
        bout_relever_item = PushButtonIconeGauche(_("Relever"), icon = icone_relever_item, iconSize = QSize(16, 16))
        bout_relever_item.setToolTip(_("Releve la sélection au dessus des autres"))
        #
        bout_rabaisser_item = PushButtonIconeGauche(_("Abaisser"), icon = icone_rabaisser_item, iconSize = QSize(16, 16))
        bout_rabaisser_item.setToolTip(_("Abaisse la sélection en dessous des autres"))
        #
        bout_tourner_droite_item = PushButtonIconeGauche("", icon = icone_tourner_droite_item, iconSize = QSize(16, 16))
        bout_tourner_droite_item.setFixedWidth(30)
        bout_tourner_droite_item.setToolTip(_("Tourne la sélection vers la droite"))
        #
        bout_tourner_gauche_item = PushButtonIconeGauche("", icon = icone_tourner_gauche_item, iconSize = QSize(16, 16))
        bout_tourner_gauche_item.setFixedWidth(30)
        bout_tourner_gauche_item.setToolTip(_("Tourne la sélection vers la gauche"))
        #
        bout_retour_phase_2 = PushButtonIconeGauche(_("Retour page précédente"), icon = icone_retour, iconSize = QSize(16, 16))
        bout_retour_phase_2.setFixedWidth(250)
        #
        bout_suivant_phase_2 = PushButtonIconeDroite(_("Page suivante"), icon = icone_suivant, iconSize = QSize(16, 16))
        bout_suivant_phase_2.setFixedWidth(180)
        # ----------------------------------------------------------------------------
        # Les labels
        self.label_largeur_baget_cadre = QLabel(_("Largeur baguette cadre :"))
        self.label_epaisscadre_ou_toile = QLabel(_("Epaisseur cadre (ou toile) :"))
        self.label_largeur_bord_sous_verre = QLabel(_("Largeur :"))
        self.label_hauteur_bord_sous_verre = QLabel(_("Hauteur :"))
        self.label_rayon_estompage_ombre = QLabel(_("Rayon estompage ombre :"))
        self.label_decal_horiz_ombre = QLabel(_("Décalage horizontal ombre :"))
        self.label_decal_vertical_ombre = QLabel(_("Décalage vertical ombre :"))
        self.label_intensite_reflet = QLabel(_("Intensité du reflet :"))
        # Les labels ci-dessous sont dégrisé par défaut
        self.label_rayon_estompage_ombre.setEnabled(True)
        self.label_decal_horiz_ombre.setEnabled(True)
        self.label_decal_vertical_ombre.setEnabled(True)
        # ----------------------------------------------------------------------------
        # QSlider pour le redimensionnement des images
        slider_redimensionnement_item = QSlider()
        slider_redimensionnement_item.setOrientation(Qt.Horizontal)
        slider_redimensionnement_item.setMinimum(100)
        slider_redimensionnement_item.setMaximum(2200)
        slider_redimensionnement_item.setValue(600)
        slider_redimensionnement_item.setSingleStep(1)
        # Le tooltip (renseignements sur le widget)
        slider_redimensionnement_item.setToolTip(_("Permet de redimensionner l'image sélectionnée dans la vue."))
        # ----------------------------------------------------------------------------
        # Combo pour le choix du média
        self.liste_medias = [_("Photo brute"), _("Cadre noir classique"), _("Sous verre, bordure"), _("Toile peinte")]
        self.combo_medias = QComboBox()
        # La liste du menu se fait par
        # l'intermédiaire d'un QComBoBox
        for medias in self.liste_medias :
            self.combo_medias.addItem(medias)
        self.combo_medias.setFixedWidth(172)
        self.combo_medias.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # ----------------------------------------------------------------------------
        # Spinbox pour la largeur de la baguette du cadre
        # Choix : Cadre noir classique
        self.spinbox_larg_bag_cadre = QSpinBox()
        self.spinbox_larg_bag_cadre.setValue(24)
        self.spinbox_larg_bag_cadre.setRange(5, 250)
        # ----------------------------------------------------------------------------
        # Spinbox pour l'épaisseur du cadre ou de la toile
        # Choix : Cadre noir classique (ou Toile peinte)
        self.spinbox_ep_cadre_ou_toile = QSpinBox()
        self.spinbox_ep_cadre_ou_toile.setValue(8)
        self.spinbox_ep_cadre_ou_toile.setRange(3, 80)
        # ----------------------------------------------------------------------------
        # Spinbox pour le rayon d'estompage de l'ombre
        # --> Tous les choix
        self.spinbox_rayon_estompe_ombre = QSpinBox()
        self.spinbox_rayon_estompe_ombre.setValue(20)
        self.spinbox_rayon_estompe_ombre.setRange(2, 80)
        # Le spinbox est dégrisé par défaut
        self.spinbox_rayon_estompe_ombre.setEnabled(True)
        # ----------------------------------------------------------------------------
        # Spinbox pour le décalage horizontal de l'ombre
        # --> Tous les choix
        self.spinbox_decal_h_ombre = QSpinBox()
        self.spinbox_decal_h_ombre.setValue(3)
        self.spinbox_decal_h_ombre.setRange(1, 100)
        # Le spinbox est dégrisé par défaut
        self.spinbox_decal_h_ombre.setEnabled(True)
        # ----------------------------------------------------------------------------
        # Spinbox pour le décalage vertical de l'ombre
        # --> Tous les choix
        self.spinbox_decal_v_ombre = QSpinBox()
        self.spinbox_decal_v_ombre.setValue(3)
        self.spinbox_decal_v_ombre.setRange(1, 100)
        # Le spinbox est dégrisé par défaut
        self.spinbox_decal_v_ombre.setEnabled(True)
        # ----------------------------------------------------------------------------
        # Spinbox pour la largeur de la bordure
        # Choix : Sous verre, bordure
        self.spinbox_larg_bord_sous_verre = QSpinBox()
        self.spinbox_larg_bord_sous_verre.setValue(20)
        self.spinbox_larg_bord_sous_verre.setRange(10, 250)
        self.spinbox_larg_bord_sous_verre.setToolTip(_("Permet de régler la largeur de la bordure du sous verre."))
        # ----------------------------------------------------------------------------
        # Spinbox pour la hauteur de la bordure
        # Choix : Sous verre, bordure
        self.spinbox_haut_bord_sous_verre = QSpinBox()
        self.spinbox_haut_bord_sous_verre.setValue(20)
        self.spinbox_haut_bord_sous_verre.setRange(10, 250)
        self.spinbox_haut_bord_sous_verre.setToolTip(_("Permet de régler la hauteur de la bordure du sous verre."))
        # ----------------------------------------------------------------------------
        # Combo pour le choix de la couleur de la
        # bordure --> Sous verre, bordure
        self.liste_couleur_bordure = [_("Bordure noire"), _("Bordure grise"), _("Bordure blanche")]
        self.combo_couleur_bordure = QComboBox()
        # La liste du menu se fait par
        # l'intermédiaire d'un QComBoBox
        for c_bordure in self.liste_couleur_bordure :
            self.combo_couleur_bordure.addItem(c_bordure)
        self.combo_couleur_bordure.setFixedWidth(286)
        self.combo_couleur_bordure.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # ----------------------------------------------------------------------------
        # Cases à cocher gérant les reflets         
        self.case_reflet_1 = QCheckBox(_("Reflet sur l'oeuvre"))
        self.case_reflet_2 = QCheckBox(_("Reflet sur le sol")) # A enlever (pas utile --> voir + tard)
        self.case_reflet_2.hide() # Caché pour l'instant
        # La case Reflet sur l'oeuvre
        # est cochée par défaut
        self.case_reflet_1.setChecked(True)
        # ----------------------------------------------------------------------------
        # Spinbox pour l'intensité du reflet
        # Choix : Sous verre, cadre noir
        self.spinbox_intensite_reflet = QSpinBox()
        self.spinbox_intensite_reflet.setValue(44)
        self.spinbox_intensite_reflet.setRange(10, 256)
        # ----------------------------------------------------------------------------
        # Cases à cocher gérant l'ombre portée         
        self.case_ombre_portee = QCheckBox(_("Apposer une ombre portée"))
        # La case Ombre portée est cochée par défaut
        self.case_ombre_portee.setChecked(True)
        
        # Ecriture des différents réglages (les spinbox, 
        # les checkbox, ...) dans le répertoire temporaire
        ecr_tempo_larg_bag_cadre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_baguette_cadre_oeuvre_murexpo_", "data_largeur_baguette_cadre_oeuvre_murexpo", ".txt", '30')
        ecr_tempo_ep_cadre_ou_toile = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_epaisseur_cadre_ou_toile_oeuvre_murexpo_", "data_epaisseur_cadre_ou_toile_oeuvre_murexpo", ".txt", '10')
        ecr_tempo_rayon_estompe_ombre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_rayon_estompe_ombre_oeuvre_murexpo_", "data_rayon_estompe_ombre_oeuvre_murexpo", ".txt", '20')
        ecr_tempo_decal_h_ombre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_decalage_horizontal_ombre_oeuvre_murexpo_", "data_decalage_horizontal_ombre_oeuvre_murexpo", ".txt", '3')
        ecr_tempo_decal_v_ombre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_decalage_vertical_ombre_oeuvre_murexpo_", "data_decalage_vertical_ombre_oeuvre_murexpo", ".txt", '3')
        ecr_tempo_largeur_bordure_sous_verre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_bordure_oeuvre_sous_verre_murexpo_", "data_largeur_bordure_oeuvre_sous_verre_murexpo", ".txt", '20')
        ecr_tempo_hauteur_bordure_sous_verre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_hauteur_bordure_oeuvre_sous_verre_murexpo_", "data_hauteur_bordure_oeuvre_sous_verre_murexpo", ".txt", '20')
        ecr_tempo_coul_bord_sous_verre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_couleur_bordure_oeuvre_murexpo_", "data_largeur_couleur_bordure_oeuvre_murexpo", ".txt", '0, 0, 0')
        #
        ecr_tempo_case_reflet_oeuvre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_reflet_sur_oeuvre_oeuvre_murexpo_", "data_reflet_sur_oeuvre_oeuvre_murexpo", ".txt", _("Reflet sur l'oeuvre"))
        ecr_tempo_intensite_reflet = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_intensite_reflet_oeuvre_murexpo_", "data_intensite_reflet_oeuvre_murexpo", ".txt", '44')
        ecr_tempo_case_ombre_portee_oeuvre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombre_portee_sur_oeuvre_murexpo_", "data_ombre_portee_sur_oeuvre_murexpo", ".txt", _("Ombre portée oeuvre"))
        
        # ----------------------------------------------------------------
        groupe_1 = QGroupBox()
        groupe_2 = QGroupBox()
        groupe_3 = QGroupBox()
        groupe_4 = QGroupBox()
        groupe_5 = QGroupBox()

        # ----------------------------------------------------------------
        grid_1 = QGridLayout()
        #grid_1.setSpacing(4)
        grid_1.setSpacing(2)
        grid_1.setContentsMargins(2, 2, 2, 2)
        
        # Boutons Ouvrir, Ajouter et Supprimer 
        # (par un QHBox)
        hbox_001 = QHBoxLayout()
        hbox_001.addWidget(bout_ouvrir_fichier_oeuvre)
        hbox_001.addWidget(bout_affiche_oeuvre)
        hbox_001.addWidget(bout_supprimer_oeuvre)

        # QHBox pour accueillir les widgets de
        # pose d'un cadre ou un entoilage de l'oeuvre
        vbox_gr_1 = QVBoxLayout()

        # Par la méthode addLayout de la grille 1
        vbox_gr_1.addLayout(hbox_001)
        # Widget de l'affichage de l'oeuvre
        # chargée (alignement central)
        # -------------------------------------
        # Par la méthode addWidget de la grille
        vbox_gr_1.addWidget(self.widgetEditAffOeuvre, alignment=Qt.AlignCenter)
        
        #
        groupe_1.setLayout(vbox_gr_1)
        # ----------------------------------------------------------------
        
        grid_2 = QGridLayout()
        #grid_2.setSpacing(4)
        grid_2.setSpacing(2)
        grid_2.setContentsMargins(2, 2, 2, 2)
        
        # Réglage 2 (par un QHBox) du cadre ou toile
        hbox_002 = QHBoxLayout()
        hbox_002.addWidget(QLabel(_("Choix du média :")))
        hbox_002.addWidget(self.combo_medias)
        
        # Réglage 3 (par un QHBox) du cadre ou toile
        hbox_003 = QHBoxLayout()
        hbox_003.addWidget(self.label_largeur_baget_cadre)
        hbox_003.addWidget(self.spinbox_larg_bag_cadre)
        
        # Réglage 4 (par un QHBox) du cadre ou toile
        hbox_004 = QHBoxLayout()
        hbox_004.addWidget(self.label_epaisscadre_ou_toile)
        hbox_004.addWidget(self.spinbox_ep_cadre_ou_toile)
        
        # Réglage 5 (par un QHBoxLayout) du sous verre
        hbox_005 = QHBoxLayout()
        hbox_005.addWidget(self.combo_couleur_bordure)
        
        # Réglage 6 (par un QHBox) du sous verre
        hbox_006 = QHBoxLayout()
        hbox_006.addWidget(self.label_largeur_bord_sous_verre)
        hbox_006.addWidget(self.spinbox_larg_bord_sous_verre)
        hbox_006.addWidget(self.label_hauteur_bord_sous_verre)
        hbox_006.addWidget(self.spinbox_haut_bord_sous_verre)
        
        # Réglage des boutons pour appliquer le changement
        # pose d'un cadre noir, sous verre ou entoilage et
        # et pour visualier (dans l'onglet de visualisation)
        # la disposition des éléments (image, cadre, 
        # sous-verre) ... et vérifier l'épaisseur du cadre
        # en grandeur réelle (par un QHBoxLayout)
        hbox_appliquer_visualiser = QHBoxLayout()
        hbox_appliquer_visualiser.addWidget(self.bout_appliquer)
        hbox_appliquer_visualiser.addWidget(self.bout_visualiser)

        # QHBox pour accueillir les widgets de
        # pose d'un cadre ou un entoilage de l'oeuvre
        vbox_gr_2 = QVBoxLayout()

        # Par la méthode addLayout de la grille 1
        vbox_gr_2.addLayout(hbox_002)
        vbox_gr_2.addLayout(hbox_003)
        vbox_gr_2.addLayout(hbox_004)
        vbox_gr_2.addLayout(hbox_005)
        vbox_gr_2.addLayout(hbox_006)
        vbox_gr_2.addLayout(hbox_appliquer_visualiser)

        #
        groupe_2.setLayout(vbox_gr_2)
        # ----------------------------------------------------------------

        grid_3 = QGridLayout()
        #grid_3.setSpacing(4)
        grid_3.setSpacing(2)
        #grid_3.setContentsMargins(4, 4, 4, 4)
        grid_3.setContentsMargins(2, 2, 2, 2)
        
        # Réglage 7 (par un QHBox) pour redimensionner
        hbox_007 = QHBoxLayout()
        hbox_007.addWidget(QLabel(_("Redimensionner la sélection :")))
        hbox_007.addWidget(slider_redimensionnement_item)
        
        # Réglage 8 (par un QHBox) du
        # rayon d'estompage de l'ombre
        hbox_008 = QHBoxLayout()
        hbox_008.addWidget(self.label_rayon_estompage_ombre)
        hbox_008.addWidget(self.spinbox_rayon_estompe_ombre)
        
        # Réglage 9 (par un QHBox) du 
        # décalage horizontal de l'ombre
        hbox_009 = QHBoxLayout()
        hbox_009.addWidget(self.label_decal_horiz_ombre)
        hbox_009.addWidget(self.spinbox_decal_h_ombre)
        
        # Réglage 10 (par un QHBox) du 
        # décalage vertical de l'ombre
        hbox_010 = QHBoxLayout()
        hbox_010.addWidget(self.label_decal_vertical_ombre)
        hbox_010.addWidget(self.spinbox_decal_v_ombre)

        # QVBoxLayout pour accueillir les widgets de
        # pose d'un cadre ou un entoilage de l'oeuvre
        vbox_gr_3 = QVBoxLayout()

        # Par la méthode addLayout de la grille 3
        vbox_gr_3.addLayout(hbox_007)
        vbox_gr_3.addLayout(hbox_008)
        vbox_gr_3.addLayout(hbox_009)
        vbox_gr_3.addLayout(hbox_010)
        
        #
        groupe_3.setLayout(vbox_gr_3)
        # ----------------------------------------------------------------
        
        grid_4 = QGridLayout()
        #grid_4.setSpacing(6)
        grid_4.setSpacing(2)
        #grid_4.setContentsMargins(4, 4, 4, 4)
        grid_4.setContentsMargins(2, 2, 2, 2)

        # QHBox pour accueillir les widgets
        # faire passer la sélection (l'item)
        # sur l'autre ou la rabaisser
        hbox_gr_4_001 = QHBoxLayout()
        hbox_gr_4_001.addWidget(bout_relever_item)
        hbox_gr_4_001.addWidget(bout_rabaisser_item)

        # QHBox pour accueillir les widgets
        # de sélection du type de reflet (sur 
        # l'oeuvre ou sur le sol)
        hbox_gr_4_002 = QHBoxLayout()
        hbox_gr_4_002.addWidget(self.case_reflet_1)
        hbox_gr_4_002.addWidget(self.case_reflet_2) # A enlever (pas utile --> voir + tard)
        hbox_gr_4_002.addWidget(bout_tourner_gauche_item)
        hbox_gr_4_002.addWidget(bout_tourner_droite_item)
        # Centrer le HBox concerné
        hbox_gr_4_002.setAlignment(Qt.AlignCenter)
        
        # QHBoxLayout pour accueillir les widgets
        # concernant l'intensité du reflet
        hbox_gr_4_003 = QHBoxLayout()
        hbox_gr_4_003.addWidget(self.label_intensite_reflet)
        hbox_gr_4_003.addWidget(self.spinbox_intensite_reflet)
        
        # HBox pour accueillir le widget
        # concernant l'ombre portée
        hbox_gr_4_ombre_portee = QHBoxLayout()
        hbox_gr_4_ombre_portee.addWidget(self.case_ombre_portee)
        # Centrer le HBox concerné
        hbox_gr_4_ombre_portee.setAlignment(Qt.AlignCenter)
        
        # QVBoxLayout pour accueillir les widgets 
        # pour Relever, Abaisser, Tourner à gauche,
        # Tourner à droire ... la sélection
        vbox_gr_4 = QVBoxLayout()

        # Par la méthode addLayout de la grille 4
        vbox_gr_4.addLayout(hbox_gr_4_001)
        vbox_gr_4.addLayout(hbox_gr_4_002)
        vbox_gr_4.addLayout(hbox_gr_4_003)
        vbox_gr_4.addLayout(hbox_gr_4_ombre_portee)
        # Permet de tasser les widgets sur la hauteur
        vbox_gr_4.setAlignment(Qt.AlignVCenter)
        
        #
        groupe_4.setLayout(vbox_gr_4)
        # ----------------------------------------------------------------
        
        grid_5 = QGridLayout()
        #grid_5.setSpacing(6)
        grid_5.setSpacing(2)
        grid_5.setContentsMargins(4, 4, 4, 4)

        # Ajout dans la grille ...
        grid_5.addWidget(bout_retour_phase_2, 0, 0)
        grid_5.addWidget(bout_suivant_phase_2, 0, 1)
        # ----------------------------------------------------------------
        
        # Les groupes
        groupe_1.setLayout(grid_1)
        groupe_2.setLayout(grid_2)
        groupe_3.setLayout(grid_3)
        groupe_4.setLayout(grid_4)
        groupe_5.setLayout(grid_5)
             
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        layout.addWidget(groupe_2, 0, 1)
        layout.addWidget(groupe_3, 0, 2)
        layout.addWidget(groupe_4, 0, 3) 
        layout.addWidget(groupe_5, 1, 0, 1, 4)
        
        #
        self.menu_stack_2.setLayout(layout)
        
        # ----------------------------------------------------
        # L'index 0 est défini par défaut
        # dans la combo du choix du média
        self.combo_medias.setCurrentIndex(0)
        # Si l'utilisateur choisit l'index 0 dans la combo,
        # c'est à dire le choix "Photo brute"), la fonction
        # (qui notamment cache les widgets inutiles pour ce
        # choix) est lancée.
        if self.combo_medias.currentIndex() == 0 :
            self.changements_combo_choix_media()
        # Ecriture du choix du média dans le fichier tempo concerné
        ecr_tempo_choix_combo_medias = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_media_oeuvre_murexpo_", "data_choix_media_oeuvre_murexpo", ".txt", self.combo_medias.currentText())
        # ----------------------------------------------------
        
        # L'index 0 est défini par défaut
        # dans la combo du choix de la
        # couleur de la bordure (pour le
        # sous-verre)
        self.combo_couleur_bordure.setCurrentIndex(0)
        # Si l'utilisateur choisit l'index 0 dans la combo,
        # c'est à dire le choix "Bordure noire"), la fonction
        # (qui notamment cache les widgets inutiles pour ce
        # choix) est lancée.
        """
        if self.combo_couleur_bordure.currentIndex() == 0 :
            self.changements_couleur_bordure_sous_verre()
        """
        # Ecriture du choix du média dans le fichier tempo concerné
        couleur_noire = '0, 0, 0'
        ecr_tempo_choix_combo_coul_bord = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_couleur_bordure_oeuvre_murexpo_", "data_choix_couleur_bordure_oeuvre_murexpo", ".txt", couleur_noire)
        # ----------------------------------------------------

        # =====================================================
        # Signaux
        # =====================================================
        # Signal pour permettre au bouton 'Ouvrir image'
        # d'accéder à l'onglet 'Fichier ouvrir'
        bout_ouvrir_fichier_oeuvre.clicked.connect(self.direction_onglet_ouvrir)
        # =====================================================
        # Signal pour ajouter l'oeuvre sélectionnée dans
        # le QTreeView (l'image) dans le QGraphicsView
        bout_affiche_oeuvre.clicked.connect(self.graphe_vue.afficher_image_oeuvre)
        # Signal pour supprimer l'oeuvre sélectionnée dans
        # le QTreeView (l'image) dans le QGraphicsView
        bout_supprimer_oeuvre.clicked.connect(self.graphe_vue.supprimer_image_oeuvre)
        # =====================================================
        # Signal du combobox du choix du média
        self.combo_medias.currentIndexChanged.connect(self.changements_combo_choix_media)
        # Signal du réglage de la largeur de la baguette du cadre
        self.spinbox_larg_bag_cadre.valueChanged.connect(self.changement_spin_bag_cadre)
        # Signal du réglage de l'épaisseur du cadre ou de la toile
        self.spinbox_ep_cadre_ou_toile.valueChanged.connect(self.changement_spin_epaiss_cadre)
        # Signal du réglage de la couleur de la bordure (choix du média --> Sous verre, bordure)
        self.combo_couleur_bordure.currentIndexChanged.connect(self.changements_couleur_bordure_sous_verre)
        # Signal largeur du bord (choix du média --> Sous verre, bordure)
        self.spinbox_larg_bord_sous_verre.valueChanged.connect(self.changements_spin_larg_bordure_sous_verre)
        # Signal hauteur du bord (choix du média --> Sous verre, bordure)
        self.spinbox_haut_bord_sous_verre.valueChanged.connect(self.changements_spin_haut_bordure_sous_verre)
        # Signal du rayon d'estompage de l'ombre
        self.spinbox_rayon_estompe_ombre.valueChanged.connect(self.changement_spin_rayon_estomp_ombre)
        # Signal du décalage horizontal de l'ombre
        self.spinbox_decal_h_ombre.valueChanged.connect(self.changement_spin_decal_h_ombre)
        # Signal du décalage vertical de l'ombre
        self.spinbox_decal_v_ombre.valueChanged.connect(self.changement_spin_decal_v_ombre)
        # Signal pour réaliser le traitement de l'oeuvre, c'est à dire
        # la pose d'un cadre classique, la pose d'un sous verre, l'entoilage, ...
        self.bout_appliquer.clicked.connect(self.graphe_vue.traitement_de_l_oeuvre)
        # Signal pour visualiser l'exposition en cours de traitement (1ère étape).
        # L'image de l'exposition est visualisée dans l'onglet Visionnage
        self.bout_visualiser.clicked.connect(self.graphe_vue.image_de_l_exposition)
        # Signal pour visualiser l'exposition en cours de traitement (2ème étape).
        # Utilisation du pyqtSignal : customSignal_onglet_visuel_exposition, quand
        # on clique sur le bouton Visuel, on ouvre l'onglet Visionnage
        self.bout_visualiser.clicked.connect(self.direction_onglet_visionner)
        # Signal pour relever la sélection (faire passer l'image 
        # sélectionnée au dessus des autres images)
        bout_relever_item.clicked.connect(self.graphe_vue.relever_item)
        # Signal pour rabaisser la sélection (faire passer l'image 
        # sélectionnée en dessous des autres images)
        bout_rabaisser_item.clicked.connect(self.graphe_vue.rabaisser_item)
        # Signal pour tourner vers la droite la sélection
        bout_tourner_droite_item.clicked.connect(self.graphe_vue.tourner_vers_la_droite)
        # Signal pour tourner vers la gauche la sélection
        bout_tourner_gauche_item.clicked.connect(self.graphe_vue.tourner_vers_la_gauche)
        # Signal (le slider) pour redimensionner la sélection
        slider_redimensionnement_item.valueChanged.connect(self.graphe_vue.redimensionner_selection)
        # Les cases à cocher QCheckBox (utilisation de lambda
        # car il y a un argument dans la fonction appelée)
        # 1 Fonction pour chaque QCheckBox
        # -----------------------------------------
        # Signaux pour les cases pour le reflet sur l'oeuvre et reflet sur le sol
        self.case_reflet_1.toggled.connect(lambda : self.case_reflet_oeuvre(self.case_reflet_1))
        self.case_reflet_2.toggled.connect(lambda : self.case_reflet_sol(self.case_reflet_2)) # A enlever (pas utile --> voir + tard)
        # Signaux pour les cases pour l'ombre portée (ombre portée à l'oeuvre)
        self.case_ombre_portee.toggled.connect(lambda : self.fonction_ombre_portee_oeuvre(self.case_ombre_portee))
        # -----------------------------------------
        # Signal pour le changement de valeur de l'intensité du reflet
        self.spinbox_intensite_reflet.valueChanged.connect(self.changement_spin_intensite_reflet)
        # =====================================================
        # Signal (avec le bouton) pour revenir à la page de la phase 1
        bout_retour_phase_2.clicked.connect(lambda: self.liste_menu.setCurrentIndex(0))
        # Signal pour passer à la page de la phase 3
        bout_suivant_phase_2.clicked.connect(lambda: self.liste_menu.setCurrentIndex(2))
        # =====================================================
        
        ##################################################################### ESSAI
        # ---------- Récup du libellé du bouton sélectionné par l'utilisateur
        #bout_affiche_oeuvre.released.connect(self.libelle_bouton_aff_oeuvre)
        ##################################################################### ESSAI
        
        
    def reg_stack_3(self) :
        """ Fonction de la page 3 du QStackedWidget """

        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe 1 --> elle est ici : affichage_apsc/push_button_icone_droite.py ... 
        # en fait le texte dans le bouton est centré et l'icône positionné à droite.
        # J'ai un peu modifié la classe et adaptée. Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : config_apsc/chemin_data_ressources.py)
        icone_plus = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_plus_murexpo_256x256.png')))
        icone_supprimer = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_moins_murexpo_256x256.png')))
        icone_charger_police = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_choix_police_murexpo_256x256.png')))
        icone_modifier_couleurs = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_choix_couleur_murexpo_256x256.png')))
        icone_relever_item = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_relever_256x256_murexpo.png')))
        icone_rabaisser_item = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_abaisser_256x256_murexpo.png')))
        icone_visualiser = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png')))
        icone_retour = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_gauche_murexpo_256x256.png')))
        icone_suivant = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_droite_murexpo_256x256.png')))
        
        #
        bout_charger_police = PushButtonIconeGauche(_("Choix de la police de caractère"), icon = icone_charger_police, iconSize = QSize(16, 16))
        bout_charger_police.setFixedWidth(410)
        bout_charger_police.setToolTip(_("Permet de sélectionner une police de caractère"))
        #
        bout_modifier_couleurs = PushButtonIconeGauche(_("Modifier les couleurs"), icon = icone_modifier_couleurs, iconSize = QSize(16, 16))
        bout_modifier_couleurs.setFixedWidth(410)
        bout_modifier_couleurs.setToolTip(_("Permet de sélectionner la couleur du panneau et de la police de caractère"))
        #
        bout_appliquer_panneau = PushButtonIconeGauche(_("Panneau"), icon = icone_plus, iconSize = QSize(16, 16))
        bout_appliquer_panneau.setToolTip(_("Permet d'ajouter un panneau (qui peut être associé à un texte)"))
        #
        bout_appliquer_texte = PushButtonIconeGauche(_("Texte"), icon = icone_plus, iconSize = QSize(16, 16))
        bout_appliquer_texte.setToolTip(_("Permet d'ajouter du texte (le texte en question doit d'abord être ajouté dans la zone de texte à gauche)"))
        #
        bout_supprimer = PushButtonIconeGauche(_("Supprimer"), icon = icone_supprimer, iconSize = QSize(16, 16))
        bout_supprimer.setToolTip(_("Supprime la sélection du mur d'exposition"))
        #
        bout_relever_item = PushButtonIconeGauche(_("Relever"), icon = icone_relever_item, iconSize = QSize(16, 16))
        bout_relever_item.setToolTip(_("Releve la sélection au dessus des autres"))
        #
        bout_rabaisser_item = PushButtonIconeGauche(_("Abaisser"), icon = icone_rabaisser_item, iconSize = QSize(16, 16))
        bout_rabaisser_item.setToolTip(_("Abaisse la sélection en dessous des autres"))
        #
        self.bout_visualiser = PushButtonIconeGauche(_("Visuel"), icon = icone_visualiser, iconSize = QSize(16, 16))
        self.bout_visualiser.setToolTip(_("Visualiser le panneau/cartel ou le texte, pendant la confection de l'exposition"))
        #
        bout_retour_phase_3 = PushButtonIconeGauche(_("Retour page précédente"), icon = icone_retour, iconSize = QSize(16, 16))
        bout_retour_phase_3.setFixedWidth(250)
        #
        bout_suivant_phase_3 = PushButtonIconeDroite(_("Page suivante"), icon = icone_suivant, iconSize = QSize(16, 16))
        bout_suivant_phase_3.setFixedWidth(180)
        # ----------------------------------------------------------------------------
        # La liste des icônes pour la combobox
        # -------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        liste_icones = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_format_carre_coins_arrondis.png'))), 
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_format_paysage_coins_arrondis.png'))),
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_format_portrait_coins_arrondis.png'))), 
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_panoramique_coins_arrondis.png'))),
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_format_carre_coins_carres.png'))), 
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_format_paysage_coins_carres.png'))),
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_format_portrait_coins_carres.png'))), 
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_panoramique_coins_carres.png'))),
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_coins_arrondis_redimensionnement_libre.png'))),
                        QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'image_panneau_coins_carres_redimensionnement_libre.png')))]
        # Liste des items qui seront contenus dans la combo
        self.liste_choix_panneau = [_("Panneau format carré, coins arrondis"), _("Panneau format paysage, coins arrondis"), \
                                    _("Panneau format portrait, coins arrondis"), _("Panneau panoramique, coins arrondis"), \
                                    _("Panneau format carré, coins carrés"), _("Panneau format paysage, coins carrés"), \
                                    _("Panneau format portrait, coins carrés"), _("Panneau panoramique, coins carrés"), \
                                    _("Panneau coins arrondis avec redimensionnement libre"), \
                                    _("Panneau coins carrés avec redimensionnement libre")]
        self.combo_choix_panneau = QComboBox()
        self.combo_choix_panneau.setMinimumHeight(36) # 36
        self.combo_choix_panneau.setFixedWidth(410)
        # La liste du menu se fait par
        # l'intermédiaire d'un QComBoBox
        for icones, panneaux in zip(liste_icones, self.liste_choix_panneau) :
            self.combo_choix_panneau.addItem(icones, panneaux)
        self.combo_choix_panneau.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # ----------------------------------------------------------------------------
        # Spinbox pour le contour (avec QPen)
        self.spinbox_contour_qpen = QSpinBox()
        self.spinbox_contour_qpen.setValue(5)
        self.spinbox_contour_qpen.setRange(2, 20)
        # ----------------------------------------------------------------------------
        # Label pour l'arrondi
        self.label_arrondi_rounded_rect = QLabel(_("Arrondi :"))
        # Le label est dégrisé par défaut (--> 
        # Important : grisé/dégrisé  au lieu de 
        # montré/caché ... autrement bug !)
        self.label_arrondi_rounded_rect.setEnabled(True)
        # ----------------------------------------------------------------------------
        # Spinbox pour l'arrondi (coins arrondis)
        # --> méthode addRoundedRect
        self.spinbox_arrondi_rounded_rect = QSpinBox()
        self.spinbox_arrondi_rounded_rect.setValue(15)
        self.spinbox_arrondi_rounded_rect.setRange(2, 100)
        # Le sipinbox est dégrisé par défaut 
        # (--> Important : grisé/dégrisé  au 
        # lieu de montré/caché ... autrement bug !)
        self.spinbox_arrondi_rounded_rect.setEnabled(True)
        # ----------------------------------------------------------------------------
        # Zone d'écriture (QTextEdit) du 
        # contenu pour les panneau/cartels
        self.texte_edit_panneau = QTextEdit()
        self.texte_edit_panneau.setMinimumHeight(150)
        self.texte_edit_panneau.setMaximumHeight(150)
        self.texte_edit_panneau.setMinimumWidth(200)
        self.texte_edit_panneau.setToolTip(_("Ecrivez votre texte ici ..."))
        # ----------------------------------------------------------------------------
        # QLabel du redimensionnement de la sélection
        self.label_redim_select_panneau_texte = QLabel(_("Redimensionner la sélection :"))
        # Le label est montré par défaut
        self.label_redim_select_panneau_texte.show()
        # ----------------------------------------------------------------------------
        # QSlider pour le redimensionnement des panneaux/cartels
        self.slider_redimensionnement_selection = QSlider()
        self.slider_redimensionnement_selection.setMinimumWidth(200)
        self.slider_redimensionnement_selection.setOrientation(Qt.Horizontal)
        self.slider_redimensionnement_selection.setMinimum(100)
        self.slider_redimensionnement_selection.setMaximum(2200)
        self.slider_redimensionnement_selection.setValue(500)
        self.slider_redimensionnement_selection.setSingleStep(1)
        # Le tooltip (renseignements sur le widget)
        self.slider_redimensionnement_selection.setToolTip(_("Permet de redimensionner la sélection (le panneau/cartel ou le texte)"))
        # Le slider est montré par défaut
        #self.slider_redimensionnement_selection.show()
        # ----------------------------------------------------------------------------
        # QLabel pour le redimensionnement en 
        # largeur (voir juste en dessous)
        self.label_redim_largeur_selection = QLabel(_("Redim. largeur :"))
        # Le label est caché par défaut
        self.label_redim_largeur_selection.hide()
        # ----------------------------------------------------------------------------
        # Spinbox pour le redimensionnement en 
        # largeur (redimensionnement libre) de 
        # la sélection
        self.spinbox_redim_largeur_selection = QSpinBox()
        self.spinbox_redim_largeur_selection.setRange(50, 2500)
        self.spinbox_redim_largeur_selection.setValue(400)
        # Le spinbox est caché par défaut
        self.spinbox_redim_largeur_selection.hide()
        # ----------------------------------------------------------------------------
        # QLabel pour le redimensionnement en 
        # hauteur (voir juste en dessous)
        self.label_redim_hauteur_selection = QLabel(_("Redim. hauteur :"))
        # Le label est caché par défaut
        self.label_redim_hauteur_selection.hide()
        # ----------------------------------------------------------------------------
        # Spinbox pour le redimensionnement en 
        # hauteur (redimensionnement libre) de 
        # la sélection
        self.spinbox_redim_hauteur_selection = QSpinBox()
        self.spinbox_redim_hauteur_selection.setRange(50, 2500)
        self.spinbox_redim_hauteur_selection.setValue(400)
        # Le spinbox est caché par défaut
        self.spinbox_redim_hauteur_selection.hide()
        # ----------------------------------------------------------------------------
        # Case à cocher pour gérer l'ombrage         
        self.case_ombrage_panneau_ou_texte = QCheckBox(_("Apposer une ombre portée à la sélection"))
        # La case Ombre portée
        # est décochée par défaut
        self.case_ombrage_panneau_ou_texte.setChecked(False)
        
        # Ecriture des différents réglages (les spinbox 
        # et le checkbox) dans le répertoire temporaire
        ecr_tempo_contour_panneau = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_contour_panneau_cartel_murexpo_", "data_largeur_contour_panneau_cartel_murexpo", ".txt", '5')
        ecr_tempo_arrondi_panneau = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_arrondi_panneau_cartel_murexpo_", "data_arrondi_panneau_cartel_murexpo", ".txt", '15')
        # 0 --> Non, 1 --> Oui (ici la case n'est pas cochée par défaut, donc 0 pour Non)
        ecr_tempo_ombrage_panneau_ou_texte = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_panneau_ou_texte_murexpo_", "data_ombrage_panneau_ou_texte_murexpo", ".txt", '0')
        
        groupe_1 = QGroupBox()
        groupe_2 = QGroupBox()
        
        # ------------------------------------
        grid_1 = QGridLayout()
        grid_1.setSpacing(4)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        # HBox pour le combobox
        hbox_combo_type_panneau = QHBoxLayout()
        hbox_combo_type_panneau.addWidget(self.combo_choix_panneau)
        
        # HBox pour le label du contour + le spinbox 
        # et le label de l'arrondi + le spinbox
        hbox_spin_contarrondi = QHBoxLayout()
        hbox_spin_contarrondi.addWidget(QLabel(_("Contour :")))
        hbox_spin_contarrondi.addWidget(self.spinbox_contour_qpen)
        hbox_spin_contarrondi.addWidget(self.label_arrondi_rounded_rect)
        hbox_spin_contarrondi.addWidget(self.spinbox_arrondi_rounded_rect)
        # Centrer le HBox concerné
        hbox_spin_contarrondi.setAlignment(Qt.AlignCenter)
        
        # HBox pour le bouton pour charger la police 
        hbox_bout_charg_police = QHBoxLayout()
        hbox_bout_charg_police.addWidget(bout_charger_police)
        
        # HBox pour le bouton pour modifier les couleurs 
        hbox_bout_modif_couleurs = QHBoxLayout()
        hbox_bout_modif_couleurs.addWidget(bout_modifier_couleurs)
        
        # Le VBox pour mettre ces éléments en place
        vbox_combo_lab_spin_contarrondi = QVBoxLayout()
        vbox_combo_lab_spin_contarrondi.addLayout(hbox_combo_type_panneau)
        vbox_combo_lab_spin_contarrondi.addLayout(hbox_spin_contarrondi)
        vbox_combo_lab_spin_contarrondi.addLayout(hbox_bout_charg_police)
        vbox_combo_lab_spin_contarrondi.addLayout(hbox_bout_modif_couleurs)
        
        # Permet de tasser les widgets sur la hauteur
        vbox_combo_lab_spin_contarrondi.setAlignment(Qt.AlignVCenter)

        # Zone de texte pour écrire le texte 
        # présent dans le panneau/cartel
        vbox_zone_texte_panneau = QVBoxLayout()
        vbox_zone_texte_panneau.addWidget(self.texte_edit_panneau)
        
        # *****************************************************************
        # HBox des boutons Panneau,
        # Texte et bouton Supprimer
        hbox_bout_appliquer = QHBoxLayout()
        hbox_bout_appliquer.addWidget(bout_appliquer_panneau)
        hbox_bout_appliquer.addWidget(bout_appliquer_texte)
        hbox_bout_appliquer.addWidget(bout_supprimer)
        
        # HBox des boutons Relever, Abaisser et Supprimer
        hbox_bout_relev_abaiss_visio = QHBoxLayout()
        hbox_bout_relev_abaiss_visio.addWidget(bout_relever_item)
        hbox_bout_relev_abaiss_visio.addWidget(bout_rabaisser_item)
        hbox_bout_relev_abaiss_visio.addWidget(self.bout_visualiser)
        
        # HBox pour le label Redimensionner la sélection
        # et le slider de redimensionnement correspondant
        hbox_slide_redim_selection = QHBoxLayout()
        hbox_slide_redim_selection.addWidget(self.label_redim_select_panneau_texte)
        hbox_slide_redim_selection.addWidget(self.slider_redimensionnement_selection)
        
        # HBox pour le label et le spinbox pour Redimensionner en
        # largeur et en hauteur la sélection (redimensionnement libre)
        hbox_spin_redim_libre_select_larg_haut = QHBoxLayout()
        hbox_spin_redim_libre_select_larg_haut.addWidget(self.label_redim_largeur_selection)
        hbox_spin_redim_libre_select_larg_haut.addWidget(self.spinbox_redim_largeur_selection)
        hbox_spin_redim_libre_select_larg_haut.addWidget(self.label_redim_hauteur_selection)
        hbox_spin_redim_libre_select_larg_haut.addWidget(self.spinbox_redim_hauteur_selection)
        # Centrer le HBox concerné
        hbox_spin_redim_libre_select_larg_haut.setAlignment(Qt.AlignCenter)
        
        # HBox pour la case à cocher ombre portée
        hbox_case_ombre_portee = QHBoxLayout()
        hbox_case_ombre_portee.addWidget(self.case_ombrage_panneau_ou_texte)
        # Centrer le HBox concerné
        hbox_case_ombre_portee.setAlignment(Qt.AlignCenter)

        # Le VBox pour mettre ces éléments en place
        vbox_ajouter_supp_redim_selection = QVBoxLayout()
        vbox_ajouter_supp_redim_selection.addLayout(hbox_bout_appliquer)
        vbox_ajouter_supp_redim_selection.addLayout(hbox_bout_relev_abaiss_visio)
        vbox_ajouter_supp_redim_selection.addLayout(hbox_slide_redim_selection)
        vbox_ajouter_supp_redim_selection.addLayout(hbox_spin_redim_libre_select_larg_haut)
        vbox_ajouter_supp_redim_selection.addLayout(hbox_case_ombre_portee)
        
        # Permet de tasser les widgets sur la hauteur
        vbox_ajouter_supp_redim_selection.setAlignment(Qt.AlignVCenter)
        # *****************************************************************
        
        # QHBoxLayout pour ...
        hbox_police_et_texte = QHBoxLayout()
        
        # Par la méthode addLayout (mise en place des vbox horizontalement)
        hbox_police_et_texte.addLayout(vbox_combo_lab_spin_contarrondi)
        hbox_police_et_texte.addLayout(vbox_zone_texte_panneau)
        hbox_police_et_texte.addLayout(vbox_ajouter_supp_redim_selection)
        
        #
        groupe_1.setLayout(hbox_police_et_texte)

        # ------------------------------------
        grid_2 = QGridLayout()
        grid_2.setSpacing(8)
        grid_2.setContentsMargins(4, 4, 4, 4)
        
        #
        grid_2.addWidget(bout_retour_phase_3, 0, 0)
        grid_2.addWidget(bout_suivant_phase_3, 0, 1)

        #
        groupe_1.setLayout(grid_1)
        groupe_2.setLayout(grid_2)
             
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        layout.addWidget(groupe_2, 1, 0)
        
        #
        self.menu_stack_3.setLayout(layout)
        
        # ----------------------------------------------------
        # L'index 0 est défini par défaut dans 
        # la combo du choix du type de panneau
        self.combo_choix_panneau.setCurrentIndex(0)
        # Ecriture du choix du type de panneau/cartel dans le fichier tempo concerné
        ecr_tempo_choix_type_de_panneau_cartel = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_type_de_panneau_cartel_murexpo_", "data_choix_type_de_panneau_cartel_murexpo", ".txt", self.combo_choix_panneau.currentText())
        # ----------------------------------------------------
        
        # -----------------------------------------------------
        # Ecriture de la valeur par défaut du redimensionnement 
        # (libre) du spinbox de la largeur et la hauteur du 
        # panneau (si l'utilisateur sélectionne les 2 dernières
        # entrées du combobox)
        ecr_tempo_redim_libre_largeur_panneau = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_redim_libre_panneau_cartel_murexpo_", "data_largeur_redim_libre_panneau_cartel_murexpo", ".txt", "400")
        ecr_tempo_redim_libre_hauteur_panneau = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_hauteur_redim_libre_panneau_cartel_murexpo_", "data_hauteur_redim_libre_panneau_cartel_murexpo", ".txt", "400")
        # -----------------------------------------------------
        
        # =====================================================
        # Signaux
        # =====================================================
        # Signal du combobox du choix du panneau/cartel
        self.combo_choix_panneau.currentIndexChanged.connect(self.fonction_changements_combo_choix_panneau)
        # Signal du réglage de la largeur du contour (panneau/cartel)
        self.spinbox_contour_qpen.valueChanged.connect(self.fonction_changement_spin_contour_panneau_cartel)
        # Signal du réglage de la valeur de l'arrondi du panneau/cartel
        self.spinbox_arrondi_rounded_rect.valueChanged.connect(self.fonction_changement_spin_arrondi_panneau_cartel)
        # Signal pour permettre au bouton 'Choix de la police
        # de caractère' d'accéder à l'onglet 'Choix police'
        bout_charger_police.clicked.connect(self.direction_onglet_choix_police)
        # Signal pour permettre au bouton 'Modifier les
        # couleurs' d'accéder à l'onglet 'Choix couleur'
        bout_modifier_couleurs.clicked.connect(self.direction_onglet_choix_couleur)
        # Signal pour signifier que le le texte du QTextEdit a changé
        self.texte_edit_panneau.textChanged.connect(self.fonction_mise_en_forme_texte_ds_qtextedit)
        # Signal du bouton pour appliquer les changements 
        # au niveau du panneau/cartel
        bout_appliquer_panneau.clicked.connect(self.graphe_vue.fonction_panneau)
        # Signal du bouton pour appliquer les changements 
        # au niveau du texte
        bout_appliquer_texte.clicked.connect(self.graphe_vue.fonction_texte)
        # Signal pour supprimer la sélection
        bout_supprimer.clicked.connect(self.graphe_vue.supprimer_image_oeuvre)
        # Signal pour relever la sélection
        bout_relever_item.clicked.connect(self.graphe_vue.relever_item)
        # Signal pour rabaisser la sélection
        bout_rabaisser_item.clicked.connect(self.graphe_vue.rabaisser_item)
        # Signal pour visualiser l'exposition en cours de traitement (1ère étape).
        # L'image de l'exposition est visualisée dans l'onglet Visionnage
        self.bout_visualiser.clicked.connect(self.graphe_vue.image_de_l_exposition)
        # Signal pour visualiser l'exposition en cours de traitement (2ème étape).
        # Utilisation du pyqtSignal : customSignal_onglet_visuel_exposition, quand
        # on clique sur le bouton Visuel, on ouvre l'onglet Visionnage
        self.bout_visualiser.clicked.connect(self.direction_onglet_visionner)
        # Signal (le slider) pour redimensionner la sélection
        self.slider_redimensionnement_selection.valueChanged.connect(self.graphe_vue.redimensionner_selection)
        # Signal pour redimensionner en largeur (redimensionnement libre)
        # si l'utilisateur a sélectionné une des 2 dernières entrées 
        # dans la combo
        self.spinbox_redim_largeur_selection.valueChanged.connect(self.fonction_changement_spin_redim_libre_largeur_panneau_cartel)
        # Signal pour redimensionner en hauteur (redimensionnement libre)
        # si l'utilisateur a sélectionné une des 2 dernières entrées 
        # dans la combo
        self.spinbox_redim_hauteur_selection.valueChanged.connect(self.fonction_changement_spin_redim_libre_hauteur_panneau_cartel)
        # ---------------------------------------------------------------------------
        # Signaux pour la case par rapport à l'ombrage du panneau/cartel ou du texte
        self.case_ombrage_panneau_ou_texte.toggled.connect(lambda : self.fonction_case_ombrage_panneau_ou_texte(self.case_ombrage_panneau_ou_texte))
        # ---------------------------------------------------------------------------
        # =====================================================
        # Signal (avec le bouton) pour revenir à la page de la phase 2
        bout_retour_phase_3.clicked.connect(lambda: self.liste_menu.setCurrentIndex(1))
        # Signal pour passer à la page de la phase 4
        bout_suivant_phase_3.clicked.connect(lambda: self.liste_menu.setCurrentIndex(3))
        # =====================================================

        
    def reg_stack_4(self) :
        """ Fonction de la page 4 du QStackedWidget """
        
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe 1 --> elle est ici : affichage_apsc/push_button_icone_droite.py ... 
        # en fait le texte dans le bouton est centré et l'icône positionné à droite.
        # J'ai un peu modifié la classe et adaptée. Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : config_apsc/chemin_data_ressources.py)
        icone_plus = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_plus_murexpo_256x256.png')))
        icone_supprimer = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_moins_murexpo_256x256.png')))
        icone_relever_item = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_relever_256x256_murexpo.png')))
        icone_rabaisser_item = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_abaisser_256x256_murexpo.png')))
        icone_visualiser = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png')))
        icone_retour = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_gauche_murexpo_256x256.png')))
        icone_suivant = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_droite_murexpo_256x256.png')))
        
        #
        bout_ajouter = PushButtonIconeGauche(_("Ajouter"), icon = icone_plus, iconSize = QSize(16, 16))
        bout_ajouter.setToolTip(_("Permet d'ajouter l'élément sélectionné à la vue"))
        #
        bout_supprimer = PushButtonIconeGauche(_("Supprimer"), icon = icone_supprimer, iconSize = QSize(16, 16))
        bout_supprimer.setToolTip(_("Supprime la sélection du mur d'exposition"))
        #
        bout_relever_item = PushButtonIconeGauche(_("Relever"), icon = icone_relever_item, iconSize = QSize(16, 16))
        bout_relever_item.setToolTip(_("Releve la sélection au dessus des autres"))
        #
        bout_rabaisser_item = PushButtonIconeGauche(_("Abaisser"), icon = icone_rabaisser_item, iconSize = QSize(16, 16))
        bout_rabaisser_item.setToolTip(_("Abaisse la sélection en dessous des autres"))
        #
        self.bout_visualiser_phase_4 = PushButtonIconeGauche(_("Visuel"), icon = icone_visualiser, iconSize = QSize(16, 16))
        self.bout_visualiser_phase_4.setToolTip(_("Visualiser le résultat pendant la confection de l'exposition"))
        
        #
        bout_retour_phase_4 = PushButtonIconeGauche(_("Retour page précédente"), icon = icone_retour, iconSize = QSize(16, 16))
        bout_retour_phase_4.setFixedWidth(250)
        #
        bout_suivant_phase_4 = PushButtonIconeDroite(_("Page suivante"), icon = icone_suivant, iconSize = QSize(16, 16))
        bout_suivant_phase_4.setFixedWidth(180)

        # Case à cocher pour gérer l'affichage de 
        # la combo consacrée aux personnages entiers        
        self.case_personnages_entiers = QCheckBox(_("Personnages entiers"))
        # La case des personnages entiers est cochée par défaut
        self.case_personnages_entiers.setChecked(True)
        
        # Case à cocher pour gérer l'affichage de 
        # la combo consacrée aux personnages coupés        
        self.case_personnages_coupes = QCheckBox(_("Personnages coupés"))
        # La case des personnages coupes est décochée par défaut
        self.case_personnages_coupes.setChecked(False)
        
        # Case à cocher pour gérer l'affichage 
        # de la combo consacrée aux cadres        
        self.case_cadres = QCheckBox(_("Cadres"))
        # La case des cadres est décochée par défaut
        self.case_cadres.setChecked(False)
        
        # Case à cocher pour gérer l'affichage de 
        # la combo consacrée au mobilier entiers        
        self.case_mobilier = QCheckBox(_("Mobilier"))
        # La case du mobilier est décochée par défaut
        self.case_mobilier.setChecked(False)
        
        # Case à cocher pour gérer l'affichage la
        # combo consacrée aux accessoires divers      
        self.case_accessoires = QCheckBox(_("Accessoires divers"))
        # La case des accessoires divers est décochée par défaut
        self.case_accessoires.setChecked(False)
        
        # Case à cocher pour gérer l'affichage 
        # de la combo consacrée à la nature       
        self.case_nature = QCheckBox(_("Nature"))
        # La case de la nature est décochée par défaut
        self.case_nature.setChecked(False)
        
        # ----------------------------------------------------------------------------
        # La liste des icônes pour la combobox des personnages entiers
        # -------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        liste_icones_persos_entiers = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_001.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_002.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_003.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_004.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_005.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_006.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_007.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_008.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_009.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_010.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_011.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_012.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_013.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_014.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_015.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_016.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_017.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_018.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_019.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_020.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_021.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_022.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_023.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_024.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_025.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_026.png'))),
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_027.png'))), 
                                       QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_028.png')))]
        # Liste des items qui seront contenus dans la combo
        self.liste_choix_persos_entiers = [_("Personnage entier 1"), _("Personnage entier 2"), _("Personnage entier 3"), _("Personnage entier 4"), \
                                           _("Personnage entier 5"), _("Personnage entier 6"), _("Personnage entier 7"), _("Personnage entier 8"), \
                                           _("Personnage entier 9"), _("Personnage entier 10"), _("Personnage entier 11"), _("Personnage entier 12"), \
                                           _("Personnage entier 13"), _("Personnage entier 14"), _("Personnage entier 15"), _("Personnage entier 16"), \
                                           _("Personnage entier 17"), _("Personnage entier 18"), _("Personnage entier 19"), _("Personnage entier 20"), \
                                           _("Personnage entier 21"), _("Personnage entier 22"), _("Personnage entier 23"), _("Personnage entier 24"), \
                                           _("Personnage entier 25"), _("Personnage entier 26"), _("Personnage entier 27"), _("Personnage entier 28")]
        self.combo_choix_persos_entiers = QComboBox()
        self.combo_choix_persos_entiers.setMinimumHeight(50) # 50
        # Taille des images dans la combo
        self.combo_choix_persos_entiers.setIconSize(QSize(80, 40)) # 100, 60
        # La liste du menu se fait par l'intermédiaire d'un QComBoBox
        for icones_pe, persos_entiers in zip(liste_icones_persos_entiers, self.liste_choix_persos_entiers) :
            self.combo_choix_persos_entiers.addItem(icones_pe, persos_entiers)
        self.combo_choix_persos_entiers.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # La combo est montrée par défaut
        self.combo_choix_persos_entiers.show()
        # ----------------------------------------------------------------------------
        # La liste des icônes pour la combobox des personnages coupés
        # -------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        liste_icones_persos_coupes = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_001.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_002.png'))),
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_003.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_004.png'))),
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_005.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_006.png'))),
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_007.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_008.png'))),
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_009.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_010.png'))),
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_011.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_012.png'))),
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_013.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_014.png'))),
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_015.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_016.png'))),
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_017.png'))), 
                                      QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_018.png')))]
        # Liste des items qui seront contenus dans la combo
        self.liste_choix_persos_coupes = [_("Personnage coupé 1"), _("Personnage coupé 2"), _("Personnage coupé 3"), _("Personnage coupé 4"), \
                                           _("Personnage coupé 5"), _("Personnage coupé 6"), _("Personnage coupé 7"), _("Personnage coupé 8"), \
                                           _("Personnage coupé 9"), _("Personnage coupé 10"), _("Personnage coupé 11"), _("Personnage coupé 12"), \
                                           _("Personnage coupé 13"), _("Personnage coupé 14"), _("Personnage coupé 15"), _("Personnage coupé 16"), \
                                           _("Personnage coupé 17"), _("Personnage coupé 18")]
        self.combo_choix_persos_coupes = QComboBox()
        self.combo_choix_persos_coupes.setMinimumHeight(50) # 50
        # Taille des images dans la combo
        self.combo_choix_persos_coupes.setIconSize(QSize(80, 40)) # 100, 60
        # La liste du menu se fait par l'intermédiaire d'un QComBoBox
        for icones_pc, persos_coupes in zip(liste_icones_persos_coupes, self.liste_choix_persos_coupes) :
            self.combo_choix_persos_coupes.addItem(icones_pc, persos_coupes)
        self.combo_choix_persos_coupes.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # La combo est cachée par défaut
        self.combo_choix_persos_coupes.hide()
        # ----------------------------------------------------------------------------
        # La liste des icônes pour la combobox des cadres
        # -------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        liste_icones_cadres = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_001.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_002.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_003.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_004.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_005.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_006.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_007.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_008.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_009.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_010.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_011.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_012.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_013.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_014.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_015.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_016.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_017.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_018.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_019.png')))]
        # Liste des items qui seront contenus dans la combo
        self.liste_choix_cadres = [_("Cadre 1"), _("Cadre 2"), _("Cadre 3"), _("Cadre 4"), _("Cadre 5"), _("Cadre 6"), _("Cadre 7"), _("Cadre 8"), \
                                   _("Cadre 9"), _("Cadre 10"), _("Cadre 11"), _("Cadre 12"), _("Cadre 13"), _("Cadre 14"), _("Cadre 15"), _("Cadre 16"), \
                                   _("Cadre 17"), _("Cadre 18"), _("Cadre 19")]
        self.combo_choix_cadres = QComboBox()
        self.combo_choix_cadres.setMinimumHeight(50) # 50
        # Taille des images dans la combo
        self.combo_choix_cadres.setIconSize(QSize(80, 40)) # 100, 60
        # La liste du menu se fait par l'intermédiaire d'un QComBoBox
        for icones_cad, cadres in zip(liste_icones_cadres, self.liste_choix_cadres) :
            self.combo_choix_cadres.addItem(icones_cad, cadres)
        self.combo_choix_cadres.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # La combo est cachée par défaut
        self.combo_choix_cadres.hide()
        # ----------------------------------------------------------------------------
        # La liste des icônes pour la combobox du mobilier
        # --------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        liste_icones_mobilier = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_001.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_002.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_003.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_004.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_005.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_006.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_007.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_008.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_009.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_010.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_011.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_012.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_013.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_014.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_015.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_016.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_017.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_018.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_019.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_020.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_021.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_022.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_023.png'))), 
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_024.png'))),
                                 QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_025.png')))]
        # Liste des items qui seront contenus dans la combo
        self.liste_choix_mobilier = [_("Mobilier 1"), _("Mobilier 2"), _("Mobilier 3"), _("Mobilier 4"), _("Mobilier 5"), _("Mobilier 6"), _("Mobilier 7"), _("Mobilier 8"), \
                                     _("Mobilier 9"), _("Mobilier 10"), _("Mobilier 11"), _("Mobilier 12"), _("Mobilier 13"), _("Mobilier 14"), _("Mobilier 15"), _("Mobilier 16"), \
                                     _("Mobilier 17"), _("Mobilier 18"), _("Mobilier 19"), _("Mobilier 20"), _("Mobilier 21"), _("Mobilier 22"), _("Mobilier 23"), _("Mobilier 24"), \
                                     _("Mobilier 25")]
        self.combo_choix_mobilier = QComboBox()
        self.combo_choix_mobilier.setMinimumHeight(50) # 50
        # Taille des images dans la combo
        self.combo_choix_mobilier.setIconSize(QSize(80, 40)) # 100, 60
        # La liste du menu se fait par l'intermédiaire d'un QComBoBox
        for icones_mob, mobilier in zip(liste_icones_mobilier, self.liste_choix_mobilier) :
            self.combo_choix_mobilier.addItem(icones_mob, mobilier)
        self.combo_choix_mobilier.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # La combo est cachée par défaut
        self.combo_choix_mobilier.hide()
        # ----------------------------------------------------------------------------
        # La liste des icônes pour la combobox des accessoires
        # ------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        liste_icones_accessoires = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_001.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_002.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_003.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_004.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_005.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_006.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_007.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_008.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_009.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_010.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_011.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_012.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_013.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_014.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_015.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_016.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_017.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_018.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_019.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_020.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_021.png'))), 
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_022.png'))),
                                    QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_023.png')))]
        # Liste des items qui seront contenus dans la combo
        self.liste_choix_accessoires = [_("Accessoire 1"), _("Accessoire 2"), _("Accessoire 3"), _("Accessoire 4"), _("Accessoire 5"), _("Accessoire 6"), _("Accessoire 7"), \
                                        _("Accessoire 8"), _("Accessoire 9"), _("Accessoire 10"), _("Accessoire 11"), _("Accessoire 12"), _("Accessoire 13"), _("Accessoire 14"), \
                                        _("Accessoire 15"), _("Accessoire 16"), _("Accessoire 17"), _("Accessoire 18"), _("Accessoire 19"), _("Accessoire 20"), _("Accessoire 21"), \
                                        _("Accessoire 22"), _("Accessoire 23")]
        self.combo_choix_accessoires = QComboBox()
        self.combo_choix_accessoires.setMinimumHeight(50) # 50
        # Taille des images dans la combo
        self.combo_choix_accessoires.setIconSize(QSize(80, 40)) # 100, 60
        # La liste du menu se fait par l'intermédiaire d'un QComBoBox
        for icones_acc, accessoires in zip(liste_icones_accessoires, self.liste_choix_accessoires) :
            self.combo_choix_accessoires.addItem(icones_acc, accessoires)
        self.combo_choix_accessoires.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # La combo est cachée par défaut
        self.combo_choix_accessoires.hide()
        # ----------------------------------------------------------------------------
        # La liste des icônes pour la combobox de nature
        # ------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        liste_icones_nature = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_001.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_002.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_003.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_004.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_005.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_006.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_007.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_008.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_009.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_010.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_011.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_012.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_013.png'))), 
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_014.png'))),
                               QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'nature_015.png')))]
        # Liste des items qui seront contenus dans la combo
        self.liste_choix_nature = [_("Nature 1"), _("Nature 2"), _("Nature 3"), _("Nature 4"), _("Nature 5"), _("Nature 6"), _("Nature 7"), _("Nature 8"), \
                                   _("Nature 9"), _("Nature 10"), _("Nature 11"), _("Nature 12"), _("Nature 13"), _("Nature 14"), _("Nature 15")]
        self.combo_choix_nature = QComboBox()
        self.combo_choix_nature.setMinimumHeight(50) # 30
        # Taille des images dans la combo
        self.combo_choix_nature.setIconSize(QSize(80, 40)) # 100, 60
        # La liste du menu se fait par l'intermédiaire d'un QComBoBox
        for icones_nat, nature in zip(liste_icones_nature, self.liste_choix_nature) :
            self.combo_choix_nature.addItem(icones_nat, nature)
        self.combo_choix_nature.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # La combo est cachée par défaut
        self.combo_choix_nature.hide()
        
        # ----------------------------------------------------
        # L'index 0 est défini par défaut dans la combo du 
        # choix des personnages entiers (c'est la combo
        # affichée par défaut)
        #self.combo_choix_persos_entiers.setCurrentIndex(0)
        # Si l'utilisateur choisit l'index 0 dans la combo, l'item à
        # l'index 0 (c'est à dire l'image : personnage_entier_001.png)
        # l'image en question est sauvegardée dans le répertoire 
        # temporaire ... avec le nom : 
        # image_ajout_perso_cadre_ou_accessoire.png
        if self.combo_choix_persos_entiers.currentIndex() == 0 :
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
        # ----------------------------------------------------

        # ----------------------------------------------------------------------------
        # Case à cocher pour gérer l'ombrage         
        self.case_ombrage_image_alpha_map = QCheckBox(_("Apposer une ombre portée à la sélection"))
        # La case Ombre portée
        # est décochée par défaut
        self.case_ombrage_image_alpha_map.setChecked(False)
        # La case à cocher est cachée par défaut
        self.case_ombrage_image_alpha_map.hide()
        # ----------------------------------------------
        # Ecriture des différents réglages (les spinbox 
        # et le checkbox) dans le répertoire temporaire
        # ----------------------------------------------
        # 0 --> Non, 1 --> Oui (ici la case n'est pas cochée par défaut, donc 0 pour Non)
        ecr_tempo_ombrage_image_alpha_map = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_image_alpha_map_murexpo_", "data_ombrage_image_alpha_map_murexpo", ".txt", '0')
        # ----------------------------------------------------------------------------

        # ----------------------------------------------------------------------------
        # Création du label d'affichage ...
        self.label_visio_alpha_map = QLabel()
        # Mise en place du fond quadrillé (en 160 x 160) pour montrer les alpha maps
        self.label_visio_alpha_map.setStyleSheet("background-image : url({});background-repeat: no-repeat;background-position: center;background-origin: content;".format(chemin_ressources(os.path.join('ressources_murexpo', 'murexpo_quadrillage_canal_alpha_160x160.png'))))
        #self.label.setStyleSheet("""border-image : url({});""".format(chemin_ressources(os.path.join('ressources_murexpo', 'murexpo_quadrillage_canal_alpha_160x160.png'))))
        
        # -----------------------------------------
        # Appel de la fonction pour insérer (avec 
        # le QPixmap défini dans la fonction en 
        # question) l'image de l'alpha map choisi 
        # dans la combo (visible) dans le label. 
        # Visionnement de la sélection --> 
        # ----------------------------------------
        # Fonction appelée : 
        # maj_selection_alpha_map
        # ----------------------------------------
        # Le label a été crée : (ici) dans la 
        # fonction reg_stack_4 et il est mis
        # à jour dans la fonction appelée
        # -----------------------------------------
        self.maj_selection_alpha_map()

        # ----------------------------------------------------------------------------
        # Le label du redimensionnement de la sélection
        label_redim_selection = QLabel(_("Redimensionner la sélection"))
        # ----------------------------------------------------------------------------
        # QSlider pour le redimensionnement des panneaux/cartels
        slider_redim_selection = QSlider()
        slider_redim_selection.setOrientation(Qt.Horizontal)
        slider_redim_selection.setMinimum(60)
        slider_redim_selection.setMaximum(2200)
        slider_redim_selection.setValue(400)
        slider_redim_selection.setSingleStep(1)
        # Le tooltip (renseignements sur le widget)
        slider_redim_selection.setToolTip(_("Permet de redimensionner la sélection"))
        # ----------------------------------------------------------------------------

        groupe_1 = QGroupBox()
        groupe_2 = QGroupBox()
        
        # ------------------------------------
        grid_1 = QGridLayout()
        grid_1.setSpacing(4)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        # VBox Colonne des checkbox des choix 1
        vbox_cases_des_choix_1 = QVBoxLayout()
        vbox_cases_des_choix_1.addWidget(self.case_personnages_entiers)
        vbox_cases_des_choix_1.addWidget(self.case_personnages_coupes)
        vbox_cases_des_choix_1.addWidget(self.case_cadres)
        # Alignement
        vbox_cases_des_choix_1.setAlignment(Qt.AlignHCenter)
        
        # VBox Colonne des checkbox des choix 2
        vbox_cases_des_choix_2 = QVBoxLayout()
        vbox_cases_des_choix_2.addWidget(self.case_mobilier)
        vbox_cases_des_choix_2.addWidget(self.case_accessoires)
        vbox_cases_des_choix_2.addWidget(self.case_nature)
        # Alignement
        vbox_cases_des_choix_2.setAlignment(Qt.AlignHCenter)
        
        # -------------------------------------------------
        # VBox Colonne des combos (affichées selon la case cochée)
        vbox_combos_des_choix = QVBoxLayout()
        vbox_combos_des_choix.addWidget(self.combo_choix_persos_entiers)
        vbox_combos_des_choix.addWidget(self.combo_choix_persos_coupes)
        vbox_combos_des_choix.addWidget(self.combo_choix_cadres)
        vbox_combos_des_choix.addWidget(self.combo_choix_mobilier)
        vbox_combos_des_choix.addWidget(self.combo_choix_accessoires)
        vbox_combos_des_choix.addWidget(self.combo_choix_nature)
        #
        vbox_combos_des_choix.addWidget(self.case_ombrage_image_alpha_map)
        # Alignement
        vbox_combos_des_choix.setAlignment(Qt.AlignHCenter)
        vbox_combos_des_choix.setAlignment(Qt.AlignVCenter)

        # -------------------------------------------------
        # VBox Mise en place du label de visionnement
        # de la sélection des alpha maps (ds la combo)
        vbox_visuel_label_alpha_map = QVBoxLayout()
        vbox_visuel_label_alpha_map.addWidget(self.label_visio_alpha_map)
        # Alignement
        vbox_visuel_label_alpha_map.setAlignment(Qt.AlignHCenter)
        
        # -------------------------------------------------
        # HBox pour les boutons relever et rabaissser item
        hbox_ligne_relev_abaiss = QHBoxLayout()
        hbox_ligne_relev_abaiss.addWidget(bout_relever_item)
        hbox_ligne_relev_abaiss.addWidget(bout_rabaisser_item)
        
        # HBox pour les labels de redimensionnement
        hbox_ligne_redim = QHBoxLayout()
        hbox_ligne_redim.addWidget(label_redim_selection)
        hbox_ligne_redim.addWidget(slider_redim_selection)
        
        # HBox pour les boutons ajouter et supprimer
        hbox_ligne_ajout_supp = QHBoxLayout()
        hbox_ligne_ajout_supp.addWidget(bout_ajouter)
        hbox_ligne_ajout_supp.addWidget(bout_supprimer)
        
        # QBox pour le bouton visualiser
        hbox_bout_visualiser = QHBoxLayout()
        hbox_bout_visualiser.addWidget(self.bout_visualiser_phase_4)
        
        # Layout Mise en place des boutons pour 
        # relever, rabaisser, du label et slider 
        # de redimensionnement, des boutons ajouter
        # et supprimer et du bouton visualiser
        vbox_widgets_affichage = QVBoxLayout()
        vbox_widgets_affichage.addLayout(hbox_ligne_relev_abaiss)
        vbox_widgets_affichage.addLayout(hbox_ligne_redim)
        vbox_widgets_affichage.addLayout(hbox_ligne_ajout_supp)
        vbox_widgets_affichage.addLayout(hbox_bout_visualiser)
        
        # *****************************************************************
        
        # HBoxLayout pour l'association de tous les widgets
        hbox_ajout_personn_cadres_access = QHBoxLayout()
        # Par la méthode addLayout (mise en place des vbox horizontalement)
        hbox_ajout_personn_cadres_access.addLayout(vbox_cases_des_choix_1)
        hbox_ajout_personn_cadres_access.addLayout(vbox_cases_des_choix_2)
        hbox_ajout_personn_cadres_access.addLayout(vbox_combos_des_choix)
        hbox_ajout_personn_cadres_access.addLayout(vbox_visuel_label_alpha_map)
        hbox_ajout_personn_cadres_access.addLayout(vbox_widgets_affichage)

        # Mise en place finale
        groupe_1.setLayout(hbox_ajout_personn_cadres_access)
        
        # ------------------------------------
        grid_2 = QGridLayout()
        grid_2.setSpacing(8)
        grid_2.setContentsMargins(4, 4, 4, 4)
        
        #
        grid_2.addWidget(bout_retour_phase_4, 0, 0)
        grid_2.addWidget(bout_suivant_phase_4, 0, 1)

        #
        groupe_1.setLayout(grid_1)
        groupe_2.setLayout(grid_2)
             
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        layout.addWidget(groupe_2, 1, 0)
        
        #
        self.menu_stack_4.setLayout(layout)
        
        # =====================================================
        # Signaux
        # =====================================================
        # ---------------------------------------------------------------------------
        # Signaux pour la case des personnages entiers
        self.case_personnages_entiers.toggled.connect(lambda : self.fonction_case_choix_personnages_entiers(self.case_personnages_entiers))
        # Signaux pour la case des personnages coupés
        self.case_personnages_coupes.toggled.connect(lambda : self.fonction_case_choix_personnages_coupes(self.case_personnages_coupes))
        # Signaux pour la case des cadres
        self.case_cadres.toggled.connect(lambda : self.fonction_case_choix_cadres(self.case_cadres))
        # Signaux pour la case du mobilier
        self.case_mobilier.toggled.connect(lambda : self.fonction_case_choix_mobilier(self.case_mobilier))
        # Signaux pour la case des acessoires
        self.case_accessoires.toggled.connect(lambda : self.fonction_case_choix_accessoires(self.case_accessoires))
        # Signaux pour la case des éléments naturels
        self.case_nature.toggled.connect(lambda : self.fonction_case_choix_nature(self.case_nature))
        # ---------------------------------------------------------------------------
        # Signal du combobox du choix des personnages entiers
        self.combo_choix_persos_entiers.currentIndexChanged.connect(self.fonction_changements_combo_persos_entiers)
        # Signal du combobox du choix des personnages coupés
        self.combo_choix_persos_coupes.currentIndexChanged.connect(self.fonction_changements_combo_persos_coupes)
        # Signal du combobox du choix des cadres
        self.combo_choix_cadres.currentIndexChanged.connect(self.fonction_changements_combo_cadres)
        # Signal du combobox du choix du mobilier
        self.combo_choix_mobilier.currentIndexChanged.connect(self.fonction_changements_combo_mobilier)
        # Signal du combobox du choix des accessoires
        self.combo_choix_accessoires.currentIndexChanged.connect(self.fonction_changements_combo_accessoires)
        # Signal du combobox du choix des différents éléments naturels
        self.combo_choix_nature.currentIndexChanged.connect(self.fonction_changements_combo_nature)
        # ---------------------------------------------------------------------------
        # Signaux pour la case par rapport à l'ombrage de l'alpha map (ombre portée 
        # sur l'image sélectionnée dans la combo et ajoutée dans la scène)
        self.case_ombrage_image_alpha_map.toggled.connect(lambda : self.fonction_case_ombrage_image_alpha_map(self.case_ombrage_image_alpha_map))
        # ---------------------------------------------------------------------------
        # Signal pour le bouton qui sert à ajouter l'image alpha map 
        # sélectionnée par l'intermédiaire du combo, dans la vue
        bout_ajouter.clicked.connect(self.graphe_vue.fonction_ajout_image_alpha_map)
        # ---------------------------------------------------------------------------
        # Signal (le slider) pour redimensionner la sélection
        slider_redim_selection.valueChanged.connect(self.graphe_vue.redimensionner_selection)
        # ---------------------------------------------------------------------------
        # Signal pour supprimer la sélection
        bout_supprimer.clicked.connect(self.graphe_vue.supprimer_image_oeuvre)
        # Signal pour relever la sélection
        bout_relever_item.clicked.connect(self.graphe_vue.relever_item)
        # Signal pour rabaisser la sélection
        bout_rabaisser_item.clicked.connect(self.graphe_vue.rabaisser_item)
        # Signal pour visualiser l'exposition en cours de traitement (1ère étape).
        # L'image de l'exposition est visualisée dans l'onglet Visionnage
        self.bout_visualiser_phase_4.clicked.connect(self.graphe_vue.image_de_l_exposition)
        # Signal pour visualiser l'exposition en cours de traitement (2ème étape).
        # Utilisation du pyqtSignal : customSignal_onglet_visuel_exposition, quand
        # on clique sur le bouton Visuel, on ouvre l'onglet Visionnage
        self.bout_visualiser_phase_4.clicked.connect(self.direction_onglet_visionner)
        # =====================================================
        # Signal (avec le bouton) pour revenir à la page de la phase 3
        bout_retour_phase_4.clicked.connect(lambda: self.liste_menu.setCurrentIndex(2))
        # Signal pour passer à la page de la phase 4
        bout_suivant_phase_4.clicked.connect(lambda: self.liste_menu.setCurrentIndex(4))
        # =====================================================
        
 
    def reg_stack_5(self) :
        """ Fonction de la page 5 du QStackedWidget """
        
        # --------------------------------------------------------------------
        # Le QStackedWidget spécifique pour l'affichage des pages concernant
        # la sauveagrde de l'exposition (sous la forme d'une image, d'un
        # gif animé, d'un fichier html ou d'une vidéo)
        # --------------------------------------------------------------------
        # Liste des icônes pour la sauvegarde de l'exposition
        liste_icones_sauvegarde = [QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_exportation_murexpo_256x256.png'))), \
                                   QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_exportation_murexpo_256x256.png'))), \
                                   QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_exportation_murexpo_256x256.png'))), \
                                   QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_exportation_murexpo_256x256.png')))]
        self.liste_types_de_sauvegarde = [_("Exporter l'exposition sous la forme d'une image"), _("Exporter l'exposition sous la forme d'un gif animé"), _("Exporter l'exposition sous la forme d'un fichier html ou pdf"), _("Exporter l'exposition sous la forme d'une vidéo")]
        self.liste_sauvegarde = QComboBox()
        self.liste_sauvegarde.setMinimumHeight(36)
        # Taille des images dans la combo
        self.liste_sauvegarde.setIconSize(QSize(36, 36)) #
        # Remplissage du QComBoBox par l'intermédiaire de la liste
        for icones_sauv, types_de_sauvegarde in zip(liste_icones_sauvegarde, self.liste_types_de_sauvegarde) :
            self.liste_sauvegarde.addItem(icones_sauv, types_de_sauvegarde)
        self.liste_sauvegarde.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        
        # Les widgets pour le QStackedWidget
        # de dauvegarde / Exportation
        self.menu_stack_sauv_type_image = QWidget()
        self.menu_stack_sauv_type_gif_anime = QWidget()
        #self.menu_stack_sauv_type_fichier_html = QWidget() # COMMENTE (on passe par la classe juste en dessous) ##### ESSAI
        self.menu_stack_sauv_type_video = QWidget()
        
        # Fonctions des menus du QStackedWidget
        self.fonction_stack_sauv_type_image()
        self.fonction_stack_sauv_type_gif_anime()
        #self.fonction_stack_sauv_type_fichier_html() # COMMENTE (on passe par la classe juste en dessous) ##### ESSAI
        self.fonction_stack_sauv_type_video()
        # Ajout des widgets dans le QStackedWidget
        self.stack_sauvegarde = QStackedWidget(self)
        self.stack_sauvegarde.addWidget(self.menu_stack_sauv_type_image)
        self.stack_sauvegarde.addWidget(self.menu_stack_sauv_type_gif_anime)
        #self.stack_sauvegarde.addWidget(self.menu_stack_sauv_type_fichier_html) # COMMENTE (on passe par la classe juste en dessous) ##### ESSAI

        # ------------------------------------------------------------
        # Au lieu de passer par une fonction (comme dans les autres 
        # menus) on passe ici par une classe (Murexpo_Export_Html)
        # qui embarque directement le QWidget contenant les champs
        # de texte et les boutons pour le traitement du html.
        # ------------------------------------------------------------
        # Appel de la la classe Murexpo_Export_Html
        self.export_html = Murexpo_Export_Html()
        # Mise en place du menu ...
        self.stack_sauvegarde.addWidget(self.export_html)
        # ------------------------------------------------------------

        self.stack_sauvegarde.addWidget(self.menu_stack_sauv_type_video)
        # --------------------------------------------------------------------
        
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe 1 --> elle est ici : affichage_apsc/push_button_icone_droite.py ... 
        # en fait le texte dans le bouton est centré et l'icône positionné à droite.
        # J'ai un peu modifié la classe et adaptée. Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : config_apsc/chemin_data_ressources.py)
        # ---------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : config_apsc/chemin_data_ressources.py)
        # ------------------------------------------------------------------------------------------
        # Icônes pour la navigation dans les pages
        icone_retour = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_gauche_murexpo_256x256.png')))
        icone_retour_1ere_page = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_3_fleches_gauche_murexpo_256x256.png')))

        #
        bout_retour_phase_5 = PushButtonIconeGauche(_("Retour page précédente"), icon = icone_retour, iconSize = QSize(16, 16))
        bout_retour_phase_5.setFixedWidth(250)
        #
        bout_1ere_page_phase_5 = PushButtonIconeGauche(_("Retour à la première page"), icon = icone_retour_1ere_page, iconSize = QSize(16, 16))
        bout_1ere_page_phase_5.setFixedWidth(280)
        # ----------------------------------------------------------------------------

        groupe_1 = QGroupBox()
        groupe_2 = QGroupBox()
        groupe_3 = QGroupBox()

        grid_1 = QGridLayout()
        grid_1.setSpacing(8)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        grid_1.addWidget(self.liste_sauvegarde, 0, 0)

        grid_2 = QGridLayout()
        grid_2.setSpacing(8)
        grid_2.setContentsMargins(4, 4, 4, 4)

        grid_2.addWidget(self.stack_sauvegarde, 0, 0)
        
        grid_3 = QGridLayout()
        grid_3.setSpacing(8)
        grid_3.setContentsMargins(4, 4, 4, 4)
        
        grid_3.addWidget(bout_retour_phase_5, 0, 0)
        grid_3.addWidget(bout_1ere_page_phase_5, 0, 1)
        
        #
        groupe_1.setLayout(grid_1)
        groupe_2.setLayout(grid_2)
        groupe_3.setLayout(grid_3)
        
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        layout.addWidget(groupe_2, 1, 0)
        layout.addWidget(groupe_3, 2, 0)
        
        #
        self.menu_stack_5.setLayout(layout)

        # =====================================================
        # Signal pour afficher les différentes pages pour
        # le choix de l'exportation/sauvegarde (sous la forme
        # d'image, sous la forme d'un gif animé, sous la forme 
        # d'un fichier html et sous la forme d'une vidéo)
        # --> QStackedWidget spécifique à la sauveagrde (!)
        self.liste_sauvegarde.currentIndexChanged.connect(self.affiche_phases_sauvegarde_exportation)
        # =====================================================
        # Signal (avec le bouton) pour revenir à la page de la phase 3
        bout_retour_phase_5.clicked.connect(lambda: self.liste_menu.setCurrentIndex(3))
        # Signal pour revenir à la toute 1ère page
        bout_1ere_page_phase_5.clicked.connect(lambda: self.liste_menu.setCurrentIndex(0))
        # =====================================================
        
        ##################################################################### ESSAI
        # ---------- Récup du libellé du bouton sélectionné par l'utilisateur
        #bout_affiche_oeuvre.released.connect(self.libelle_bouton_aff_oeuvre)
        ##################################################################### ESSAI
        
        
    def direction_onglet_ouvrir(self) :
        """ Fonction utilisable quand l'utilisateur clique sur 
        le bouton Ouvrir image à la page 2 du QStackedWidget.
        En fait cela permet à l'utilisateur d'accéder à l'onglet 
        Chargement des images """
        
        # =========================================================================
        # Utilisation du signal pour afficher automatiquement l'onglet 
        # Chargement des images. 
        # 2ème partie ==> le signal est émis pour l'id 1 (du QTabWidget défini 
        # [les onglets] défini dans le script main (murexpo_gui.py)). A voir aussi
        # Une ligne de code par rapport à cela se trouve dans murexpo_gui.py. 
        # Un grand merci à eyllanesc !!!
        # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
        # qtabwidget-defined-in-the-main-class-in-another-class#63065558
        self.customSignal_onglet_ouvrir_oeuvre.emit(1)
        # =========================================================================
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Direction onglet 'Chargement des images' à l'indice 1\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
      
    
    def direction_onglet_visionner(self) :
        """ Fonction utilisable quand l'utilisateur 
        clique sur le bouton Visuel à la page 2 du 
        QStackedWidget. En fait cela permet à 
        l'utilisateur d'accéder à l'onglet Visionnage """
        
        import time 
        time.sleep(0.5)
        
        # =========================================================================
        # Utilisation du signal pour afficher automatiquement l'onglet 
        # Visionnage. 
        # 2ème partie ==> le signal est émis pour l'id 4 (du QTabWidget défini 
        # [les onglets] défini dans le script main (murexpo_gui.py)). A voir aussi
        # Une ligne de code par rapport à cela se trouve dans murexpo_gui.py. 
        # Un grand merci à eyllanesc !!!
        # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
        # qtabwidget-defined-in-the-main-class-in-another-class#63065558
        self.customSignal_onglet_visuel_exposition.emit(5)
        # =========================================================================
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Direction onglet 'Visionnage' à l'indice 5\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        
    def direction_onglet_choix_police(self) :
        """ Fonction utilisable quand l'utilisateur clique sur 
        le bouton Choix de la police de caractère à la page 3
        du QStackedWidget. En fait cela permet à l'utilisateur 
        d'accéder à l'onglet Choix police """
        
        # =========================================================================
        # Utilisation du signal pour afficher automatiquement l'onglet 
        # Choix police. 
        # 2ème partie ==> le signal est émis pour l'id 2 (du QTabWidget défini 
        # [les onglets] défini dans le script main (murexpo_gui.py)). A voir aussi
        # Une ligne de code par rapport à cela se trouve dans murexpo_gui.py. 
        # Un grand merci à eyllanesc !!!
        # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
        # qtabwidget-defined-in-the-main-class-in-another-class#63065558
        self.customSignal_onglet_choix_police.emit(2)
        # =========================================================================
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Direction onglet 'Choix police' à l'indice 2\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        
    def direction_onglet_choix_couleur(self) :
        """ Fonction utilisable quand l'utilisateur clique sur 
        le bouton Modifier les couleurs à la page 3 du 
        QStackedWidget. En fait cela permet à l'utilisateur 
        d'accéder à l'onglet Choix Couleur """
        
        # =========================================================================
        # Utilisation du signal pour afficher automatiquement l'onglet 
        # Choix couleur. 
        # 2ème partie ==> le signal est émis pour l'id 3 (du QTabWidget défini 
        # [les onglets] défini dans le script main (murexpo_gui.py)). A voir aussi
        # Une ligne de code par rapport à cela se trouve dans murexpo_gui.py. 
        # Un grand merci à eyllanesc !!!
        # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
        # qtabwidget-defined-in-the-main-class-in-another-class#63065558
        self.customSignal_onglet_choix_couleur.emit(3)
        # =========================================================================
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Direction onglet 'Choix couleur' à l'indice 3\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        
    def affiche_phases(self, i):
        """ Appel des QStackedWidget et
        mise en place pour naviguer entre
        les différentes pages/phases """
        
        self.stack.setCurrentIndex(i)
        
        # -------------------------------------------------------------------------------------------
        # Non encore utilisé (mais sera certainement utile plus tard)
        # -------------------------------------------------------------------------------------------
        # Drapeau écrit dans le rep. tempo. (donnée écrite à n, c'est à dire l'index de la phase
        # de conception choisie dans le stacked)
        # -------------------------------------------------------------------------------------------
        # Ecriture de l'index de la phase de conception (dans l'onglet Conception et exportation) 
        # --> à la page 4 de l'affichage des phases [index 3 du stacked --> self.stack] ... et 
        # ... et à l'index 0 du stacked --> self.stack ...
        ecr_tempo_index_phase_page_exposition = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_index_page_phase_murexpo_", "data_index_page_phase_murexpo", ".txt", str(i))
        # -------------------------------------------------------------------------------------------
       
        
    def changements_combo_choix_mur(self) :
        """ Fonction du choix des différents 
        murs d'exposition (pour le combo) """
        
        if self.combo_choix_mur.currentText() == _("Mur d'exposition 1") : 
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_001.png'))
            # La sélection (le choix) dur mur est écrit dans le répertoire temporaire
            ecr_tempo_selection_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_001'))
        elif self.combo_choix_mur.currentText() == _("Mur d'exposition 2") : 
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_002.png'))
            # ...
            ecr_tempo_selection_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_002'))
        elif self.combo_choix_mur.currentText() == _("Mur d'exposition 3") : 
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_003.png'))
            ecr_tempo_selection_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_003'))
        elif self.combo_choix_mur.currentText() == _("Mur d'exposition 4") : 
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_004.png'))
            ecr_tempo_selection_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_004'))
        elif self.combo_choix_mur.currentText() == _("Mur d'exposition 5") : 
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_005.png'))
            ecr_tempo_selection_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_005'))
        elif self.combo_choix_mur.currentText() == _("Mur d'exposition 6") : 
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_006.png'))
            ecr_tempo_selection_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_006'))
        elif self.combo_choix_mur.currentText() == _("Mur d'exposition 7") : 
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_007.png'))
            ecr_tempo_selection_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_007'))
        elif self.combo_choix_mur.currentText() == _("Mur d'exposition 8") : 
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_008.png'))
            ecr_tempo_selection_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_008'))
            
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement de mur d'exposition ; choix du mur {}\n").format(self.combo_choix_mur.currentText()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
  

    def changements_combo_choix_media(self) :
        """ Fonction du choix des différents medias (c'est à 
        dire l'image brure sans cadre ni entoilage, l'image 
        encadrée et l'image entoilée) et ce par l'intermédiaire 
        d'un combobox. Certains widgets de réglage (selon les
        cas) sont grisés ... suivant le traitement à effectuer. """
        
        # Choix : Photo brute
        if self.combo_medias.currentText() == _('Photo brute') :
            """
            self.label_largeur_baget_cadre.setEnabled(False)
            self.spinbox_larg_bag_cadre.setEnabled(False)
            self.label_epaisscadre_ou_toile.setEnabled(False)
            self.spinbox_ep_cadre_ou_toile.setEnabled(False)
            """
            self.label_largeur_baget_cadre.hide()
            self.spinbox_larg_bag_cadre.hide()
            self.label_epaisscadre_ou_toile.hide()
            self.spinbox_ep_cadre_ou_toile.hide()
            self.combo_couleur_bordure.hide() # Caché
            self.label_largeur_bord_sous_verre.hide() # Caché
            self.spinbox_larg_bord_sous_verre.hide() # Caché
            self.label_hauteur_bord_sous_verre.hide() # Caché
            self.spinbox_haut_bord_sous_verre.hide() # Caché
            self.bout_appliquer.setEnabled(False)
            
        # Choix : Cadre noir classique
        if self.combo_medias.currentText() == _('Cadre noir classique') :
            self.label_largeur_baget_cadre.show()
            self.spinbox_larg_bag_cadre.show()
            self.label_epaisscadre_ou_toile.show()
            self.spinbox_ep_cadre_ou_toile.show()
            self.combo_couleur_bordure.hide() # Caché
            self.label_largeur_bord_sous_verre.hide() # Caché
            self.spinbox_larg_bord_sous_verre.hide() # Caché
            self.label_hauteur_bord_sous_verre.hide() # Caché
            self.spinbox_haut_bord_sous_verre.hide() # Caché
            self.bout_appliquer.setEnabled(True)
           
        # Choix : Sous verre, bordure
        if self.combo_medias.currentText() == _('Sous verre, bordure') :
            #
            self.label_largeur_baget_cadre.hide() # Caché
            #
            self.spinbox_larg_bag_cadre.hide() # Caché
            #
            self.label_epaisscadre_ou_toile.hide() # Caché
            #
            self.spinbox_ep_cadre_ou_toile.hide() # Caché
            #
            self.combo_couleur_bordure.show() # Montré
            #
            self.label_largeur_bord_sous_verre.show() # Montré
            #
            self.spinbox_larg_bord_sous_verre.show() # Montré
            #
            self.label_hauteur_bord_sous_verre.show() # Montré
            #
            self.spinbox_haut_bord_sous_verre.show() # Montré
            #
            self.bout_appliquer.setEnabled(True)
            
        # Choix : Toile peinte
        if self.combo_medias.currentText() == _('Toile peinte') :
            # 
            self.label_largeur_baget_cadre.hide() # Caché
            #
            self.spinbox_larg_bag_cadre.hide() # Caché
            # ----------------------------------------------
            # Le spinbox de l'épaisseur du cadre ou toile
            # est caché pour l'instant car le traitement
            # toile/entoilage n'est pas encore fait (!)
            # --> de même pour le label ... il est caché
            # ----------------------------------------------
            #self.label_epaisscadre_ou_toile.show() # Montré
            #self.spinbox_ep_cadre_ou_toile.show() # Montré
            #
            self.label_epaisscadre_ou_toile.hide() # Caché
            #
            self.spinbox_ep_cadre_ou_toile.hide() # Caché
            # ----------------------------------------------
            #
            self.combo_couleur_bordure.hide() # Caché
            #
            self.label_largeur_bord_sous_verre.hide() # Caché
            #
            self.spinbox_larg_bord_sous_verre.hide() # Caché
            #
            self.label_hauteur_bord_sous_verre.hide() # Caché
            #
            self.spinbox_haut_bord_sous_verre.hide() # Caché
            # ----------------------------------------------
            # Le bouton appliquer est grisé pour l'instant 
            # car le traitement toile/entoilage n'est pas 
            # encore fait (!)
            # ----------------------------------------------
            #self.bout_appliquer.setEnabled(True)
            #
            self.bout_appliquer.setEnabled(False)
            
        # Ecriture du choix du média dans le fichier tempo concerné
        ecr_tempo_choix_combo_medias = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_media_oeuvre_murexpo_", "data_choix_media_oeuvre_murexpo", ".txt", self.combo_medias.currentText())

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement de type de media ; choix du type {}\n").format(self.combo_medias.currentText()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def changements_couleur_bordure_sous_verre(self) :
        """ Fonction pour ecrire le changement de couleur 
        (au choix de l'utilisateur) de la largeur de la 
        bordure du sous verre quand choix media : Sous 
        verre avec bordure ... et écriture de la valeur 
        dans le fichier concerné dans le répertoire 
        temporaire """
        
        # Choix : Couleur noire
        if self.combo_couleur_bordure.currentText() == _('Bordure noire') :
            # Ecriture du choix du média dans le fichier tempo concerné
            couleur_noire = '0, 0, 0'
            ecr_tempo_coul_bord_sous_verre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_couleur_bordure_oeuvre_murexpo_", "data_largeur_couleur_bordure_oeuvre_murexpo", ".txt", couleur_noire)
        # Choix : Couleur grise
        elif self.combo_couleur_bordure.currentText() == _('Bordure grise') :
            couleur_grise = '127, 127, 127'
            ecr_tempo_coul_bord_sous_verre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_couleur_bordure_oeuvre_murexpo_", "data_largeur_couleur_bordure_oeuvre_murexpo", ".txt", couleur_grise)
        # Choix : Couleur blanche
        elif self.combo_couleur_bordure.currentText() == _('Bordure blanche') :
            couleur_blanche = '255, 255, 255'
            ecr_tempo_coul_bord_sous_verre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_couleur_bordure_oeuvre_murexpo_", "data_largeur_couleur_bordure_oeuvre_murexpo", ".txt", couleur_blanche)

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement de la couleur de la bordure du sous verre ; choix d'une {}\n").format(self.combo_couleur_bordure.currentText()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def changements_spin_larg_bordure_sous_verre(self) :
        """ Fonction pour ecrire le changement de largeur 
        de la bordure du sous verre quand choix media : 
        Sous verre, bordure ... et écriture de la 
        valeur dans le fichier concerné dans le répertoire 
        temporaire """
        
        ecr_tempo_larg_bord_sous_verre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_bordure_oeuvre_sous_verre_murexpo_", "data_largeur_bordure_oeuvre_sous_verre_murexpo", ".txt", str(self.spinbox_larg_bord_sous_verre.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement de la largeur de la bordure du sous verre : {}\n").format(self.spinbox_larg_bord_sous_verre.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def changements_spin_haut_bordure_sous_verre(self) :
        """ Fonction pour ecrire le changement de hauteur 
        de la bordure du sous verre quand choix media : 
        Sous verre, bordure ... et écriture de la 
        valeur dans le fichier concerné dans le répertoire 
        temporaire """
        
        ecr_tempo_haut_bord_sous_verre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_hauteur_bordure_oeuvre_sous_verre_murexpo_", "data_hauteur_bordure_oeuvre_sous_verre_murexpo", ".txt", str(self.spinbox_haut_bord_sous_verre.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement de la hauteur de la bordure du sous verre : {}\n").format(self.spinbox_haut_bord_sous_verre.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def changement_spin_bag_cadre(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox de la largeur de la baguette du cadre 
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """
        
        ecr_tempo_larg_bag_cadre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_baguette_cadre_oeuvre_murexpo_", "data_largeur_baguette_cadre_oeuvre_murexpo", ".txt", str(self.spinbox_larg_bag_cadre.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement de la largeur de la baguette du cadre (pour un 'Cadre noir classique) : {}\n").format(self.spinbox_larg_bag_cadre.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        
    def changement_spin_epaiss_cadre(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox de l'épaisseur du cadre ou de la toile
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """
        
        ecr_tempo_ep_cadre_ou_toile = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_epaisseur_cadre_ou_toile_oeuvre_murexpo_", "data_epaisseur_cadre_ou_toile_oeuvre_murexpo", ".txt", str(self.spinbox_ep_cadre_ou_toile.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement de l'épaisseur du cadre (pour un 'Cadre noir classique) : {}\n").format(self.spinbox_ep_cadre_ou_toile.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def changement_spin_rayon_estomp_ombre(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox du rayon d'estompage de l'ombre
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """
        
        ecr_tempo_rayon_estompe_ombre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_rayon_estompe_ombre_oeuvre_murexpo_", "data_rayon_estompe_ombre_oeuvre_murexpo", ".txt", str(self.spinbox_rayon_estompe_ombre.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement du rayon de l'estompage de l'ombre : {}\n").format(self.spinbox_rayon_estompe_ombre.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------

        
    def changement_spin_decal_h_ombre(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox du décalage horizontal de l'ombre
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """

        ecr_tempo_decal_h_ombre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_decalage_horizontal_ombre_oeuvre_murexpo_", "data_decalage_horizontal_ombre_oeuvre_murexpo", ".txt", str(self.spinbox_decal_h_ombre.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement du décalage horizontal de l'ombre : {}\n").format(self.spinbox_decal_h_ombre.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def changement_spin_decal_v_ombre(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox du décalage vertical de l'ombre
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """

        ecr_tempo_decal_v_ombre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_decalage_vertical_ombre_oeuvre_murexpo_", "data_decalage_vertical_ombre_oeuvre_murexpo", ".txt", str(self.spinbox_decal_v_ombre.value()))
    
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement du décalage vertical de l'ombre : {}\n").format(self.spinbox_decal_v_ombre.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def case_reflet_oeuvre(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) pour le reflet 
        sur l'oeuvre a été cochée """
        
        if c.isChecked() == True :
            reflet_oeuvre = str(c.text())
            # Ecriture de "Reflet sur l'oeuvre" dans le fichier tempo. 
            # ATENTION !!! : "Reflet &sur l'oeuvre" risque d'être écrit ... donc 
            # il faudra tester "Reflet sur l'oeuvre" ou "Reflet &sur l'oeuvre"
            ecr_tempo_case_reflet_oeuvre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_reflet_sur_oeuvre_oeuvre_murexpo_", "data_reflet_sur_oeuvre_oeuvre_murexpo", ".txt", reflet_oeuvre)

            # Le label d'info du réglage de
            # l'intensité reflet est montré
            self.label_intensite_reflet.show()
            # Le spinbox du réglage de
            # l'intensité reflet est montré
            self.spinbox_intensite_reflet.show()
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Choix d'un reflet sur l'oeuvre\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # Si la case est décochée "" est écrit
            ecr_tempo_case_reflet_oeuvre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_reflet_sur_oeuvre_oeuvre_murexpo_", "data_reflet_sur_oeuvre_oeuvre_murexpo", ".txt", "")
            pass
        
            # Le label d'info du réglage de
            # l'intensité reflet est caché
            self.label_intensite_reflet.hide()
            # Le spinbox du réglage de
            # l'intensité reflet est caché
            self.spinbox_intensite_reflet.hide()
        
       
    def fonction_ombre_portee_oeuvre(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) pour l'ombre
        portée sur l'oeuvre a été cochée """
        
        if c.isChecked() == True :
            # Ecriture de "Ombre portée oeuvre" dans le fichier tempo. 
            ecr_tempo_case_ombre_portee_oeuvre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombre_portee_sur_oeuvre_murexpo_", "data_ombre_portee_sur_oeuvre_murexpo", ".txt", _("Ombre portée oeuvre"))

            # Les labels et les spinbox sont dégrisés
            self.label_rayon_estompage_ombre.setEnabled(True)
            self.spinbox_rayon_estompe_ombre.setEnabled(True)
            self.label_decal_horiz_ombre.setEnabled(True)
            self.spinbox_decal_h_ombre.setEnabled(True)
            self.label_decal_vertical_ombre.setEnabled(True)
            self.spinbox_decal_v_ombre.setEnabled(True)

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Choix d'une ombre portée pour l'oeuvre\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # Si la case est décochée "" est écrit
            ecr_tempo_case_ombre_portee_oeuvre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombre_portee_sur_oeuvre_murexpo_", "data_ombre_portee_sur_oeuvre_murexpo", ".txt", "")
            pass
        
            # Les labels et les spinbox sont grisés
            self.label_rayon_estompage_ombre.setEnabled(False)
            self.spinbox_rayon_estompe_ombre.setEnabled(False)
            self.label_decal_horiz_ombre.setEnabled(False)
            self.spinbox_decal_h_ombre.setEnabled(False)
            self.label_decal_vertical_ombre.setEnabled(False)
            self.spinbox_decal_v_ombre.setEnabled(False)
        
        
    #========================================= A enlever --> voir + haut (pas utile --> voir + tard)  
    def case_reflet_sol(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) pour le reflet 
        sur le sol a été cochée """
        
        if c.isChecked() == True :
            reflet_sol = str(c.text())
            # Ecriture de "Reflet sur le sol" dans le fichier tempo. 
            # ATENTION !!! : "Reflet sur &le sol" risque d'être écrit ... donc 
            # il faudra tester "Reflet sur le sol" ou "Reflet sur &le sol"
            ecr_tempo_case_reflet_sol = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_reflet_sur_sol_oeuvre_murexpo_", "data_reflet_sur_sol_oeuvre_murexpo", ".txt", reflet_sol)

        if c.isChecked() == False :
            # Si la case est décochée "" est écrit
            ecr_tempo_case_reflet_sol = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_reflet_sur_sol_oeuvre_murexpo_", "data_reflet_sur_sol_oeuvre_murexpo", ".txt", "")
            pass
    #========================================= A enlever --> voir + haut (pas utile --> voir + tard)
    

    def changement_spin_intensite_reflet(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox de l'intensité du reflet (de l'oeuvre)
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """

        ecr_tempo_intensite_reflet = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_intensite_reflet_oeuvre_murexpo_", "data_intensite_reflet_oeuvre_murexpo", ".txt", str(self.spinbox_intensite_reflet.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Changement de l'intesité du reflet sur l'oeuvre : {}\n").format(self.spinbox_intensite_reflet.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def fonction_mise_en_forme_texte_ds_qtextedit(self) :
        """ Fonction connectée au QTextEdit de l'écriture 
        et la modification pour le panneau/cartel """
        
        try :
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            # ------------------------------------------------------
            # --> Lecture du type de la police
            pre_lecture_type_police = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_police_de_caractere_choisie_murexpo_", "data_police_de_caractere_choisie_murexpo")
            # ... avec appel de la méthode affichage
            lecture_type_police = "".join(pre_lecture_type_police.affichage())
            # Récupération des données, par exemple
            # ['aakar', 'medium', '48']
            type_police = lecture_type_police.split(',')
    
            # Appel de la fonction (module fonte) contenue dans le script fonte.py. 
            # Fonction avec arguments. Attribution de la police (police, style et taille)
            # au QTextEdit self.texte_edit_panneau
            fonte(self.texte_edit_panneau, type_police[0], type_police[1], int(type_police[2]))
        except : pass

        # Récupération du texte écrit par l'utilisateur dans le QTextEdit
        texte_utilisateur_panneau_cartel = self.texte_edit_panneau.toPlainText()
        # Récupération du texte écrit dans le QTextWidget et écriture dans le fichier temporaire
        # data_contenu_texte_utilisateur_panneau_cartel
        ecr_contenu = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_contenu_texte_utilisateur_panneau_cartel_murexpo_", "data_contenu_texte_utilisateur_panneau_cartel_murexpo", ".txt", texte_utilisateur_panneau_cartel)

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Texte entré par l'utilisateur : {}\n").format(texte_utilisateur_panneau_cartel))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def fonction_changements_combo_choix_panneau(self) :
        """ Fonction du choix du type panneau/cartel, les 
        widgets sont montrés ou cachés suivant le traitement 
        à effectuer. Ecriture dans le répertoire temporaire 
        (dans le fichier concerné) du choix du type de panneau """
        
        # Choix déterminé dans la structure conditionnelle juste en dessous ...
        if self.combo_choix_panneau.currentText() in [_("Panneau format carré, coins arrondis"), \
                                                    _("Panneau format paysage, coins arrondis"), \
                                                    _("Panneau format portrait, coins arrondis"), \
                                                    _("Panneau panoramique, coins arrondis")] :
            # Les widgets sont dégrisés ...
            self.label_arrondi_rounded_rect.setEnabled(True)
            self.spinbox_arrondi_rounded_rect.setEnabled(True)
            # Les widgets du redimensionnement (le label
            # et le slider) --> largeur et hauteur 
            # groupés (!!) sont montrés
            self.label_redim_select_panneau_texte.show()
            self.slider_redimensionnement_selection.show()
            # Les widgets relatifs au redimensionnement
            # individualisé les labels et les spinbox 
            # (largeur et hauteur) (!!!) sont cachés
            self.label_redim_largeur_selection.hide()
            self.spinbox_redim_largeur_selection.hide()
            self.label_redim_hauteur_selection.hide()
            self.spinbox_redim_hauteur_selection.hide()
            # Ecriture du choix du type de panneau/cartel dans le fichier tempo concerné
            ecr_tempo_choix_type_de_panneau_cartel = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_type_de_panneau_cartel_murexpo_", "data_choix_type_de_panneau_cartel_murexpo", ".txt", self.combo_choix_panneau.currentText())
        
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Choix du type de panneau/cartel : {}\n").format(self.combo_choix_panneau.currentText()))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
        
        # Choix déterminé dans la structure conditionnelle juste en dessous ...
        elif self.combo_choix_panneau.currentText() in [_("Panneau format carré, coins carrés"), \
                                                    _("Panneau format paysage, coins carrés"), \
                                                    _("Panneau format portrait, coins carrés"), \
                                                    _("Panneau panoramique, coins carrés")] :
            # Les widgets sont grisés ...
            self.label_arrondi_rounded_rect.setEnabled(False)
            self.spinbox_arrondi_rounded_rect.setEnabled(False)
            # Les widgets du redimensionnement (le label
            # et le slider) --> largeur et hauteur 
            # groupés (!!) sont montrés
            self.label_redim_select_panneau_texte.show()
            self.slider_redimensionnement_selection.show()
            # Les widgets relatifs au redimensionnement
            # individualisé les labels et les spinbox 
            # (largeur et hauteur) (!!!) sont cachés
            self.label_redim_largeur_selection.hide()
            self.spinbox_redim_largeur_selection.hide()
            self.label_redim_hauteur_selection.hide()
            self.spinbox_redim_hauteur_selection.hide()
            # Ecriture du choix du type de panneau/cartel dans le fichier tempo concerné
            ecr_tempo_choix_type_de_panneau_cartel = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_type_de_panneau_cartel_murexpo_", "data_choix_type_de_panneau_cartel_murexpo", ".txt", self.combo_choix_panneau.currentText())
    
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Choix du type de panneau/cartel : {}\n").format(self.combo_choix_panneau.currentText()))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
        # Choix déterminé dans la structure conditionnelle juste en dessous ...
        elif self.combo_choix_panneau.currentText() in [_("Panneau coins arrondis avec redimensionnement libre")] :
            # Les widgets sont dégrisés ...
            self.label_arrondi_rounded_rect.setEnabled(True)
            self.spinbox_arrondi_rounded_rect.setEnabled(True)
            # Les widgets du redimensionnement (le label
            # et le slider) --> largeur et hauteur 
            # groupés (!!) sont cachés
            self.label_redim_select_panneau_texte.hide()
            self.slider_redimensionnement_selection.hide()
            # Les widgets relatifs au redimensionnement
            # individualisé les labels et les spinbox 
            # (largeur et hauteur) (!!!) sont montrés
            self.label_redim_largeur_selection.show()
            self.spinbox_redim_largeur_selection.show()
            self.label_redim_hauteur_selection.show()
            self.spinbox_redim_hauteur_selection.show()
            # Ecriture du choix du type de panneau/cartel dans le fichier tempo concerné
            ecr_tempo_choix_type_de_panneau_cartel = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_type_de_panneau_cartel_murexpo_", "data_choix_type_de_panneau_cartel_murexpo", ".txt", self.combo_choix_panneau.currentText())            
    
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Choix du type de panneau/cartel : {}\n").format(self.combo_choix_panneau.currentText()))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
    
        # Choix déterminé dans la structure conditionnelle juste en dessous ...
        elif self.combo_choix_panneau.currentText() in [_("Panneau coins carrés avec redimensionnement libre")] :
            # Les widgets sont grisés ...
            self.label_arrondi_rounded_rect.setEnabled(False)
            self.spinbox_arrondi_rounded_rect.setEnabled(False)
            # Les widgets du redimensionnement (le label
            # et le slider) --> largeur et hauteur 
            # groupés (!!) sont cachés
            self.label_redim_select_panneau_texte.hide()
            self.slider_redimensionnement_selection.hide()
            # Les widgets relatifs au redimensionnement
            # individualisé les labels et les spinbox 
            # (largeur et hauteur) (!!!) sont montrés
            self.label_redim_largeur_selection.show()
            self.spinbox_redim_largeur_selection.show()
            self.label_redim_hauteur_selection.show()
            self.spinbox_redim_hauteur_selection.show()
            # Ecriture du choix du type de panneau/cartel dans le fichier tempo concerné
            ecr_tempo_choix_type_de_panneau_cartel = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_type_de_panneau_cartel_murexpo_", "data_choix_type_de_panneau_cartel_murexpo", ".txt", self.combo_choix_panneau.currentText())            
    
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Choix du type de panneau/cartel : {}\n").format(self.combo_choix_panneau.currentText()))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
    
    def fonction_changement_spin_contour_panneau_cartel(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox de la largeur du contour (panneau/
        cartel) ... et écriture de la valeur dans le 
        fichier concerné dans le répertoire temporaire """
        
        ecr_tempo_contour_panneau = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_contour_panneau_cartel_murexpo_", "data_largeur_contour_panneau_cartel_murexpo", ".txt", str(self.spinbox_contour_qpen.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Largeur du contour sur le panneau/cartel : {}\n").format(self.spinbox_contour_qpen.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def fonction_changement_spin_arrondi_panneau_cartel(self) :
        """ Fonction pour écrire le changement de valeur du
        spinbox de la valeur de l'arrondi du panneau/cartel
        ... et écriture de la valeur dans le fichier concerné 
        dans le répertoire temporaire """

        ecr_tempo_arrondi_panneau = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_arrondi_panneau_cartel_murexpo_", "data_arrondi_panneau_cartel_murexpo", ".txt", str(self.spinbox_arrondi_rounded_rect.value()))
    
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Valeur de l'arrondi (en degrés) sur les coins du panneau/cartel : {}\n").format(self.spinbox_arrondi_rounded_rect.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        
    def fonction_changement_spin_redim_libre_largeur_panneau_cartel(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox de la largeur de la sélection --> 
        redimensionnement libre (panneau/cartel) ... 
        et écriture de la valeur dans le fichier concerné 
        dans le répertoire temporaire """
        
        ecr_tempo_redim_libre_largeur_panneau = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_redim_libre_panneau_cartel_murexpo_", "data_largeur_redim_libre_panneau_cartel_murexpo", ".txt", str(self.spinbox_redim_largeur_selection.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Largeur du redimensionnement (libre) du panneau/cartel : {}\n").format(str(self.spinbox_redim_largeur_selection.value())))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        
    def fonction_changement_spin_redim_libre_hauteur_panneau_cartel(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox de la hauteur de la sélection --> 
        redimensionnement libre (panneau/cartel) ... 
        et écriture de la valeur dans le fichier concerné 
        dans le répertoire temporaire """
        
        ecr_tempo_redim_libre_hauteur_panneau = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_hauteur_redim_libre_panneau_cartel_murexpo_", "data_hauteur_redim_libre_panneau_cartel_murexpo", ".txt", str(self.spinbox_redim_hauteur_selection.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Hauteur du redimensionnement (libre) du panneau/cartel : {}\n").format(str(self.spinbox_redim_hauteur_selection.value())))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
    
    
    def fonction_case_ombrage_panneau_ou_texte(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) pour l'ombrage 
        du panneau/cartel ou du texte a été cochée """
        
        if c.isChecked() == True :
            # Ecriture de la valeur "1" dans le fichier tempo. 
            # ... si la case à cocher "Apposer une ombre portée 
            # à la sélection" est cochée (valeur 1 pour oui)
            ecr_tempo_ombrage_panneau_ou_texte = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_panneau_ou_texte_murexpo_", "data_ombrage_panneau_ou_texte_murexpo", ".txt", '1')

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Ombrage sélectionné sur le panneau/cartel\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # Si la case est décochée "0" est écrit ... si 
            # la case à cocher "Apposer une ombre portée à 
            #la sélection" est cochée (valeur 0 pour non)
            ecr_tempo_ombrage_panneau_ou_texte = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_panneau_ou_texte_murexpo_", "data_ombrage_panneau_ou_texte_murexpo", ".txt", '0')
    
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Ombrage désélectionné sur le panneau/cartel\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------


    # --- Fonctions spécifiques concernant les widgets de la phase/page 4 (les alpaha maps à superposer ds l'expo) ---
    
    def fonction_selection_automatique_index_ds_combo_phase_4(self, combo, item_index_0, image_sauvegardee) :
        """ Fonction spéciale pour récupérer les index 0 des combos
        concernant les personnages coupés, les cadres, le mobolier,
        les accessoires divers et la nature (la combo pour les
        personnages entiers n'est pas concernée, car l'image est
        à la base automatiquement sauvegardée dans lerep. tempo).
        On vérifie ensuite si l'image à sauvegarder existe déjà
        dans les rep. tempo, si oui elle est supprimée et 
        elle est dans la foulée enregistrée.
        # -------------------------------------------------------
        *** Par ce biais dès que l'utilisateur coche une des
        cases : Personnages coupés, Cadres, Mobilier, Accessoires
        divers ou Nature ... l'image à l'index 0 dans la combo
        est automatiquement enregistrée dans le repertoire tempo. """

        # Récupération de l'index de l'item automatiquement
        # sélectionné dans la combo des personnages coupés
        # (ce sera l'index 0)
        index = combo.currentIndex()
        # Si c'est ok ...
        if index == 0 :
            # Scan des fichiers du répertoire temporaire ...
            for supression in os.listdir(self.chemin_tempo_system_exploit) :
                # Si l'image ... est bien dedans ...
                if supression == image_sauvegardee :
                    # Elle est préalablement éliminée ...
                    os.remove(os.path.join(self.chemin_tempo_system_exploit, supression))
                    # ... et l'image à l'index 0 est sauveagrdée dans le rep. tempo.
                    Image.open(chemin_ressources(os.path.join('ressources_murexpo', item_index_0))).save(os.path.join(self.chemin_tempo_system_exploit, image_sauvegardee), "PNG")
    
    
    def maj_selection_alpha_map(self) :
        """ Fonction servant à afficher dans un label ... 
        et en passant par un pixmap l'image sélectionnée 
        dans la combo (affichée) des alpha map. Cette 
        fonction est appelée dans la fonction reg_stack_4 """

        # L'image à visionner est chargée dans le pixmap
        self.pixmap = QPixmap(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'))
        # Définition de la taille du pixmap (et en gardant le ratio)
        self.taille_pixmap = self.pixmap.scaled(160, 160, Qt.KeepAspectRatio)
        # L'image est affectée au label
        self.label_visio_alpha_map.setPixmap(self.taille_pixmap)
        # Mise à jour des éléments (!!!)
        # --------- IMPORTANT ! --------
        QApplication.processEvents()
    
    
    def fonction_case_choix_personnages_entiers(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) ... a été cochée """
        
        # Sauvegarde image ... si l'utilisateur sélectionne l'index 0
        # de la combo des personnages entiers, l'image à l'index 0 est 
        # sauvegardée dans le répertoire temporaire (Important !)
        if self.combo_choix_persos_entiers.currentIndex() == 0 : Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
        
        if c.isChecked() == True :
            # La combo est montrée (défini par défaut)
            self.combo_choix_persos_entiers.show()
            # Les autres combos sont cachées
            self.combo_choix_persos_coupes.hide()
            self.combo_choix_cadres.hide()
            self.combo_choix_mobilier.hide()
            self.combo_choix_accessoires.hide()
            self.combo_choix_nature.hide()
            
            # Les cases à cocher sont sélectionnées 
            # ou déselectionnées selon les cas
            self.case_personnages_entiers.setChecked(True)
            self.case_personnages_coupes.setChecked(False)
            self.case_cadres.setChecked(False)
            self.case_mobilier.setChecked(False)
            self.case_accessoires.setChecked(False)
            self.case_nature.setChecked(False)
            
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
            
            # La case à cocher pour ajouter une ombre 
            # portée reste cachée (cette case pour 
            # l'ombre portée ne se montrée que pour 
            # les cadres)
            self.case_ombrage_image_alpha_map.hide()

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Personnages entiers' sélectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Personnages entiers' déselectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
    
    def fonction_case_choix_personnages_coupes(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) ... a été cochée """

        if c.isChecked() == True :
            # La combo est montrée (défini par défaut)
            self.combo_choix_persos_coupes.show()
            # Les autres combos sont cachées
            self.combo_choix_persos_entiers.hide()
            self.combo_choix_cadres.hide()
            self.combo_choix_mobilier.hide()
            self.combo_choix_accessoires.hide()
            self.combo_choix_nature.hide()
            
            # Les cases à cocher sont sélectionnées 
            # ou déselectionnées selon les cas
            self.case_personnages_coupes.setChecked(True)
            self.case_personnages_entiers.setChecked(False)
            self.case_cadres.setChecked(False)
            self.case_mobilier.setChecked(False)
            self.case_accessoires.setChecked(False)
            self.case_nature.setChecked(False)

            # ------------------------------------------------------------------------------------------------------
            # Appel de la fonction pour sélectionner automatiquement ... l'image à l'index 0 dans
            # la combo ... et ce dès que l'utilisateur coche la case concernée (et l'enregistrer 
            # ds le rep.  tempo) ... sous le nom : 'image_ajout_perso_cadre_ou_accessoire.png'
            self.fonction_selection_automatique_index_ds_combo_phase_4(self.combo_choix_persos_coupes, \
                                                                       'personnage_coupe_001.png', \
                                                                       'image_ajout_perso_cadre_ou_accessoire.png')
            # ------------------------------------------------------------------------------------------------------
            
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
            
            # La case à cocher pour ajouter une ombre 
            # portée reste cachée (cette case pour 
            # l'ombre portée ne se montrée que pour 
            # les cadres)
            self.case_ombrage_image_alpha_map.hide()

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Personnages coupés' sélectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Personnages coupés' déselectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
            
    def fonction_case_choix_cadres(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) ... a été cochée """
        
        # Sauvegarde image ... si l'utilisateur sélectionne l'index 0 de 
        # la combo des personnages coupés, l'image à l'index 0 est 
        # sauvegardée dans le répertoire temporaire
        #if self.combo_choix_persos_entiers.currentIndex() == 0 : Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
        
        if c.isChecked() == True :
            # La combo est montrée (défini par défaut)
            self.combo_choix_cadres.show()
            # Les autres combos sont cachées
            self.combo_choix_persos_entiers.hide()
            self.combo_choix_persos_coupes.hide()
            self.combo_choix_mobilier.hide()
            self.combo_choix_accessoires.hide()
            self.combo_choix_nature.hide()
            
            # Les cases à cocher sont sélectionnées 
            # ou déselectionnées selon les cas
            self.case_cadres.setChecked(True)
            self.case_personnages_entiers.setChecked(False)
            self.case_personnages_coupes.setChecked(False)
            self.case_mobilier.setChecked(False)
            self.case_accessoires.setChecked(False)
            self.case_nature.setChecked(False)
            
            # ------------------------------------------------------------------------------------------------------
            # Appel de la fonction pour sélectionner automatiquement ... l'image à l'index 0 dans
            # la combo ... et ce dès que l'utilisateur coche la case concernée (et l'enregistrer 
            # ds le rep.  tempo) ... sous le nom : 'image_ajout_perso_cadre_ou_accessoire.png'
            self.fonction_selection_automatique_index_ds_combo_phase_4(self.combo_choix_cadres, 'cadre_001.png', \
                                                                       'image_ajout_perso_cadre_ou_accessoire.png')
            # ------------------------------------------------------------------------------------------------------
            
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()

            # La case à cocher pour ajouter une ombre 
            # portée est ici montrée (cette case pour 
            # l'ombre portée ne se montre qu'ici pour 
            # les cadres)
            self.case_ombrage_image_alpha_map.show()

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Cadres' sélectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Cadres' déselectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
        
    def fonction_case_choix_mobilier(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) ... a été cochée """
        
        if c.isChecked() == True :
            # La combo est montrée (défini par défaut)
            self.combo_choix_mobilier.show()
            # Les autres combos sont cachées
            self.combo_choix_persos_entiers.hide()
            self.combo_choix_persos_coupes.hide()
            self.combo_choix_cadres.hide()
            self.combo_choix_accessoires.hide()
            self.combo_choix_nature.hide()
            
            # Les cases à cocher sont sélectionnées 
            # ou déselectionnées selon les cas
            self.case_mobilier.setChecked(True)
            self.case_personnages_entiers.setChecked(False)
            self.case_personnages_coupes.setChecked(False)
            self.case_cadres.setChecked(False)
            self.case_accessoires.setChecked(False)
            self.case_nature.setChecked(False)
            
            # ------------------------------------------------------------------------------------------------------
            # Appel de la fonction pour sélectionner automatiquement ... l'image à l'index 0 dans
            # la combo ... et ce dès que l'utilisateur coche la case concernée (et l'enregistrer 
            # ds le rep.  tempo) ... sous le nom : 'image_ajout_perso_cadre_ou_accessoire.png'
            self.fonction_selection_automatique_index_ds_combo_phase_4(self.combo_choix_mobilier, 'mobilier_001.png', \
                                                                       'image_ajout_perso_cadre_ou_accessoire.png')
            # ------------------------------------------------------------------------------------------------------
            
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
            
            # La case à cocher pour ajouter une ombre 
            # portée reste cachée (cette case pour 
            # l'ombre portée ne se montrée que pour 
            # les cadres)
            self.case_ombrage_image_alpha_map.hide()

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Mobilier' sélectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Mobilier' déselectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
            
    def fonction_case_choix_accessoires(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) ... a été cochée """
        
        if c.isChecked() == True :
            # La combo est montrée (défini par défaut)
            self.combo_choix_accessoires.show()
            # Les autres combos sont cachées
            self.combo_choix_persos_entiers.hide()
            self.combo_choix_persos_coupes.hide()
            self.combo_choix_cadres.hide()
            self.combo_choix_mobilier.hide()
            self.combo_choix_nature.hide()
            
            # Les cases à cocher sont sélectionnées 
            # ou déselectionnées selon les cas
            self.case_accessoires.setChecked(True)
            self.case_personnages_entiers.setChecked(False)
            self.case_personnages_coupes.setChecked(False)
            self.case_cadres.setChecked(False)
            self.case_mobilier.setChecked(False)
            self.case_nature.setChecked(False)
            
            # ------------------------------------------------------------------------------------------------------
            # Appel de la fonction pour sélectionner automatiquement ... l'image à l'index 0 dans
            # la combo ... et ce dès que l'utilisateur coche la case concernée (et l'enregistrer 
            # ds le rep.  tempo) ... sous le nom : 'image_ajout_perso_cadre_ou_accessoire.png'
            self.fonction_selection_automatique_index_ds_combo_phase_4(self.combo_choix_accessoires, \
                                                                       'accessoire_001.png', \
                                                                       'image_ajout_perso_cadre_ou_accessoire.png')
            # ------------------------------------------------------------------------------------------------------
            
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()

            # La case à cocher pour ajouter une ombre 
            # portée reste cachée (cette case pour 
            # l'ombre portée ne se montrée que pour 
            # les cadres)
            self.case_ombrage_image_alpha_map.hide()

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Accessoires' sélectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Accessoires' déselectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
            
    def fonction_case_choix_nature(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) ... a été cochée """
        
        if c.isChecked() == True :
            # La combo est montrée (défini par défaut)
            self.combo_choix_nature.show()
            # Les autres combos sont cachées
            self.combo_choix_persos_entiers.hide()
            self.combo_choix_persos_coupes.hide()
            self.combo_choix_cadres.hide()
            self.combo_choix_mobilier.hide()
            self.combo_choix_accessoires.hide()
            
            # Les cases à cocher sont sélectionnées 
            # ou déselectionnées selon les cas
            self.case_nature.setChecked(True)
            self.case_personnages_entiers.setChecked(False)
            self.case_personnages_coupes.setChecked(False)
            self.case_cadres.setChecked(False)
            self.case_mobilier.setChecked(False)
            self.case_accessoires.setChecked(False)
            
            # ------------------------------------------------------------------------------------------------------
            # Appel de la fonction pour sélectionner automatiquement ... l'image à l'index 0 dans
            # la combo ... et ce dès que l'utilisateur coche la case concernée (et l'enregistrer 
            # ds le rep.  tempo) ... sous le nom : 'image_ajout_perso_cadre_ou_accessoire.png'
            self.fonction_selection_automatique_index_ds_combo_phase_4(self.combo_choix_nature, 'nature_001.png', \
                                                                       'image_ajout_perso_cadre_ou_accessoire.png')
            # ------------------------------------------------------------------------------------------------------
            
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
            
            # La case à cocher pour ajouter une ombre 
            # portée reste cachée (cette case pour 
            # l'ombre portée ne se montrée que pour 
            # les cadres)
            self.case_ombrage_image_alpha_map.hide()

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Nature' sélectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Case 'Nature' déselectionnée\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
    
    def fonction_changements_combo_persos_entiers(self) :
        """ Fonction qui enregistre les images sélectionnées
        dans le rep. tempo. les images des items sélectionnés
        dans la combo des personnages entiers """

        if self.combo_choix_persos_entiers.currentText() == _("Personnage entier 1") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 2") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_002.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 3") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_003.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 4") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_004.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 5") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_005.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 6") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_006.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 7") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_007.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 8") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_008.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 9") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_009.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 10") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_010.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 11") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_011.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 12") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_012.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 13") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_013.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 14") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_014.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 15") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_015.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 16") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_016.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 17") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_017.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 18") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_018.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 19") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_019.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 20") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_020.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 21") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_021.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 22") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_022.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 23") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_023.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 24") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_024.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 25") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_025.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 26") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_026.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 27") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_027.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_entiers.currentText() == _("Personnage entier 28") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_entier_028.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
    
    
    def fonction_changements_combo_persos_coupes(self) :
        """ Fonction qui enregistre les images sélectionnées
        dans le rep. tempo. les images des items sélectionnés
        dans la combo des personnages coupés """
        
        if self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 1") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 2") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_002.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 3") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_003.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 4") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_004.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 5") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_005.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 6") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_006.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 7") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_007.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 8") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_008.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 9") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_009.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 10") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_010.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 11") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_011.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 12") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_012.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 13") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_013.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 14") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_014.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 15") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_015.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 16") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_016.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 17") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_017.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_persos_coupes.currentText() == _("Personnage coupé 18") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'personnage_coupe_018.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
            
        
    def fonction_changements_combo_cadres(self) :
        """ Fonction qui enregistre les images sélectionnées
        dans le rep. tempo. les images des items sélectionnés
        dans la combo des différents cadres """
        
        if self.combo_choix_cadres.currentText() == _("Cadre 1") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 2") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_002.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 3") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_003.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 4") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_004.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 5") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_005.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 6") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_006.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 7") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_007.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 8") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_008.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 9") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_009.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 10") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_010.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 11") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_011.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 12") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_012.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 13") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_013.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 14") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_014.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 15") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_015.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 16") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_016.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 17") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_017.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 18") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_018.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_cadres.currentText() == _("Cadre 19") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'cadre_019.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
            
        
    def fonction_changements_combo_mobilier(self) :
        """ Fonction qui enregistre les images sélectionnées
        dans le rep. tempo. les images des items sélectionnés
        dans la combo du mobilier """
        
        if self.combo_choix_mobilier.currentText() == _("Mobilier 1") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 2") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_002.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 3") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_003.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 4") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_004.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 5") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_005.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 6") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_006.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 7") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_007.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 8") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_008.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 9") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_009.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 10") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_010.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 11") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_011.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 12") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_012.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 13") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_013.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 14") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_014.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 15") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_015.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 16") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_016.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 17") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_017.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 18") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_018.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 19") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_019.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 20") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_020.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 21") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_021.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 22") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_022.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 23") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_023.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 24") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_024.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_mobilier.currentText() == _("Mobilier 25") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mobilier_025.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
            
        
    def fonction_changements_combo_accessoires(self) :
        """ Fonction qui enregistre les images sélectionnées
        dans le rep. tempo. les images des items sélectionnés
        dans la combo des différents accessoires """
        
        if self.combo_choix_accessoires.currentText() == _("Accessoire 1") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 2") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_002.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 3") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_003.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 4") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_004.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 5") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_005.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 6") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_006.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 7") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_007.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 8") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_008.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 9") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_009.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 10") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_010.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 11") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_011.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 12") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_012.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 13") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_013.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 14") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_014.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 15") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_015.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 16") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_016.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 17") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_017.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 18" ): 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_018.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 19") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_019.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 20") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_020.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 21") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_021.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 22") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_022.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_accessoires.currentText() == _("Accessoire 23") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'accessoire_023.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
            
        
    def fonction_changements_combo_nature(self) :
        """ Fonction qui enregistre les images sélectionnées
        dans le rep. tempo. les images des items sélectionnés
        dans la combo des éléments naturels """
        
        if self.combo_choix_nature.currentText() == _("Nature 1") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_001.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            # -----------------------------------------
            # Appel de la fonction pour insérer (avec 
            # le QPixmap défini dans la fonction en 
            # question) l'image de l'alpha map choisi 
            # dans la combo (visible) dans le label. 
            # Visionnement de la sélection --> 
            # ----------------------------------------
            # Fonction appelée : 
            # maj_selection_alpha_map
            # ----------------------------------------
            # Le label a été crée : dans la 
            # fonction reg_stack_4 et il est mis
            # à jour dans la fonction appelée
            # -----------------------------------------
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 2") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_002.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 3") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_003.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 4") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_004.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 5") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_005.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 6") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_006.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 7") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_007.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 8") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_008.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 9") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_009.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 10") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_010.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 11") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_011.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 12") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_012.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 13") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_013.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 14") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_014.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
        elif self.combo_choix_nature.currentText() == _("Nature 15") : 
            Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'nature_015.png'))).save(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'), "PNG")
            self.maj_selection_alpha_map()
            
            
    def fonction_case_ombrage_image_alpha_map(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) pour l'ombrage 
        de l'objet image alpha map sélectionné a été cochée """
        
        if c.isChecked() == True :
            # Ecriture de la valeur "1" dans le fichier tempo. 
            # ... si la case à cocher "Apposer une ombre portée 
            # à la sélection" est cochée (valeur 1 pour oui)
            ecr_tempo_ombrage_image_alpha_map = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_image_alpha_map_murexpo_", "data_ombrage_image_alpha_map_murexpo", ".txt", '1')

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Ombrage sélectionné sur l'image alpha map\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        if c.isChecked() == False :
            # Si la case est décochée "0" est écrit ... si 
            # la case à cocher "Apposer une ombre portée à 
            #la sélection" est cochée (valeur 0 pour non)
            ecr_tempo_ombrage_image_alpha_map = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_image_alpha_map_murexpo_", "data_ombrage_image_alpha_map_murexpo", ".txt", '0')
    
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Ombrage désélectionné sur l'image alpha map\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
    # --- Fonctions spécifiques concernant les widgets de la phase/page 4 (les alpaha maps à superposer ds l'expo) ---


    # --- Fonctions spécifiques du QStackedWidget pour la sauvegarde / exportation ---
       
    def afficher_barre_progression_et_bout_visuel_image(self) :
        """ Mise en place (après coup) d'une barre de 
        progression (factice) pour montrer que le traitement 
        (l'exportation sous la forme d'une image) a eu lieu.
        De même affichage du bouton pour visualiser le résultat """
        
        # --------------------------------------------------------------------------------
        # Lecture du fichier tempo. data_fin_exportation_exposition_image_murexpo
        # ré-écrit (et positionné à '1') dans la fonction sauvegarder_image_entiere_expo
        # dans la classe MonGraphiqueVueMur (voir plus bas)
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_fin_exportation = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_fin_exportation_exposition_image_murexpo_', 'data_fin_exportation_exposition_image_murexpo')
        # ... avec appel de la méthode affichage
        lecture_fin_exportation = "".join(pre_lecture_fin_exportation.affichage())

        # --------------------------------------------------------------------------------
        # Appel de la classe BarreDeProgressionExportImage avec les attributs correspondants
        barre_de_progression_image = BarreDeProgressionExportImage(self, minimum=0, maximum=100)
        
        # --------------------------------------------------------------------------------
        # Si le fichier tempo. data_fin_exportation_exposition_image_murexpo contient
        # bien la valeur '1', c'est à dire que l'exportation/enregistrement a été
        # faite (fonction sauvegarder_image_entiere_expo, ici plus bas dans la classe
        # MonGraphiqueVueMur) --> image de l'exposition enregistrée, la barre de 
        # progression (qui est en fait une barre factice qui fonctionne avec un 
        # QTimer --> pour faire croire à un vrai traitement ==> voir dans le fichier
        # barre_de_progression_export_image ... la classe BarreDeProgressionExportImage)
        if lecture_fin_exportation == '1' :
            # Ajout (après coup) du widget de la barre et du bouton pour
            # visualiser l'image de l'exposition (après exportation)
            self.grid_1_sauv_type_image.addWidget(barre_de_progression_image, 0, 1)
            self.grid_1_sauv_type_image.addWidget(self.bout_visuel_image_expo, 0, 2)
        # --------------------------------------------------------------------------------


    def fonction_stack_sauv_type_image(self) :
        """ Fonction du QStackedWidget pour la
        sauvegarde / exportation sous la forme 
        d'une image. Utilise la fonction
        afficher_barre_progression_et_bout_visuel_image
        se trouvant juste au dessus """
        
        #
        icone_enregistrer_image_exposition = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_enregistrer_image_murexpo_256x256.png')))
        icone_visualiser_image_exposition = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png')))
        #
        bout_enregistrer_image_expo = PushButtonIconeGauche(_("Enregistrer image"), icon = icone_enregistrer_image_exposition, iconSize = QSize(16, 16))
        bout_enregistrer_image_expo.setFixedWidth(180)
        #
        self.bout_visuel_image_expo = PushButtonIconeGauche(_("Visuel de l'exposition"), icon = icone_visualiser_image_exposition, iconSize = QSize(16, 16))
        self.bout_visuel_image_expo.setFixedWidth(250)
        
        groupe_1 = QGroupBox()
        
        # La grille est nommée avec self car elle est utilisée
        # dans la fonction afficher_barre_progression_et_bout_visuel_image
        self.grid_1_sauv_type_image = QGridLayout()
        self.grid_1_sauv_type_image.setSpacing(8)
        self.grid_1_sauv_type_image.setContentsMargins(4, 4, 4, 4)
        #
        self.grid_1_sauv_type_image.addWidget(bout_enregistrer_image_expo, 0, 0)

        groupe_1.setLayout(self.grid_1_sauv_type_image)

        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        
        self.menu_stack_sauv_type_image.setLayout(layout)

        # =====================================================
        # Signaux
        # =====================================================
        # Signal pour enregistrer l'image de l'exposition
        bout_enregistrer_image_expo.clicked.connect(self.graphe_vue.sauvegarder_image_entiere_expo)
        # Utilisation de la fonction afficher_barre_progression_et_bout_visuel_image
        # notamment pour afficher la barre de progression ...
        bout_enregistrer_image_expo.clicked.connect(self.afficher_barre_progression_et_bout_visuel_image)
        # Signal pour visualiser l'exposition à la fin du traitement. Utilisation 
        # du pyqtSignal : customSignal_onglet_visuel_exposition, quand on clique 
        # sur le bouton Enregistrer image, on ouvre l'onglet Visionnage
        self.bout_visuel_image_expo.clicked.connect(self.direction_onglet_visionner)
        # =====================================================
        
    
    def fonction_stack_sauv_type_gif_anime(self) :
        """ Fonction du QStackedWidget pour 
        la sauvegarde / exportation sous la 
        forme d'un gif animé """
        
        # -------------------------------------------------------------------------
        # Combobox du décalage (passage) d'une image à l'autre 
        # (en pixels). La découpe (crop) se fera par exemple
        # tous les 5 pixels (si l'utilisateur choisit 5)
        self.liste_choix_decalage_gif_anim = ['25', '20', '10', '5']
        self.combo_choix_decalage_gif_anim = QComboBox()
        self.combo_choix_decalage_gif_anim.setMinimumHeight(36) # 36
        self.combo_choix_decalage_gif_anim.setMinimumWidth(230)
        for decal_img in self.liste_choix_decalage_gif_anim : 
            self.combo_choix_decalage_gif_anim.addItem(decal_img)
        # Par défaut
        self.combo_choix_decalage_gif_anim.setCurrentText('10')
        self.combo_choix_decalage_gif_anim.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # -------------------------------------------------------------------------
        # QSpinbox de la fréquence (img/sec) (choix utilisateur)
        self.spinbox_fps_gif_anim = QSpinBox()
        self.spinbox_fps_gif_anim.setRange(2, 120)
        self.spinbox_fps_gif_anim.setValue(15)
        # Le tooltip
        self.spinbox_fps_gif_anim.setToolTip(_("Permet de définir la fréquence (le nombre d'images/seconde)"))
        # -------------------------------------------------------------------------
        #
        icone_enregistrer_image_exposition = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_enregistrer_image_murexpo_256x256.png')))
        icone_visualiser_image_exposition = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png')))
        # ga (gif animé)
        self.bout_visuel_image_expo_ga = PushButtonIconeGauche(_("Visuel image exposition"), icon = icone_visualiser_image_exposition, iconSize = QSize(16, 16))
        self.bout_visuel_image_expo_ga.setFixedWidth(220)
        # Le bouton visuel est caché
        self.bout_visuel_image_expo_ga.hide()
        #
        bout_enregistrer_gif_anime_expo = PushButtonIconeGauche(_("Enregistrer le gif animé"), icon = icone_enregistrer_image_exposition, iconSize = QSize(16, 16))
        bout_enregistrer_gif_anime_expo.setFixedWidth(250)

        # Ecriture de la valeur par défaut du choix du décalage (en pixels) passage d'une image à l'autre, pour le futur gif animé
        ecr_tempo_choix_decalage_ga = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_decalage_gif_anime_murexpo_", "data_choix_decalage_gif_anime_murexpo", ".txt", "10")
        # Ecriture de la valeur par défaut du nombre d'images/seconde dans le futur gif animé
        ecr_tempo_choix_fps_sec_ga = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_fps_gif_anime_murexpo_", "data_choix_fps_gif_anime_murexpo", ".txt", "15")
        
        # --------------------------------------------------------------------
        # Le label pour visualiser le gif animé
        self.label_vision_gif_anime = QLabel()
        # Résolution du label (la zone de visualisation
        # du gif animé) : 120 x 120 pix
        self.label_vision_gif_anime.setGeometry(QRect(0, 0, 63, 90))
        self.label_vision_gif_anime.setMaximumSize(QSize(63, 90))
        self.label_vision_gif_anime.setFixedSize(63, 90)
        
        # Couleur (grise) du label par défaut
        self.label_vision_gif_anime.setStyleSheet("""background-color: rgb(100, 100, 100);""")
        # --------------------------------------------------------------------
        
        groupe_1 = QGroupBox()
        
        # *********************************************************************
        # HBox pour le décalage (passage d'une img à 
        # l'autre) du gif animé (label + combo) et de 
        # la durée aux choix (label + slider)
        hbox_decalage_fps_anim = QHBoxLayout()
        hbox_decalage_fps_anim.addWidget(QLabel(_("Décalage d'une image à l'autre (en pixels) :")))
        hbox_decalage_fps_anim.addWidget(self.combo_choix_decalage_gif_anim)
        hbox_decalage_fps_anim.addWidget(QLabel(_("Fréquence (images/seconde) :")))
        hbox_decalage_fps_anim.addWidget(self.spinbox_fps_gif_anim)
        hbox_decalage_fps_anim.addWidget(self.bout_visuel_image_expo_ga)
        hbox_decalage_fps_anim.addWidget(bout_enregistrer_gif_anime_expo)
        # Centrer le HBox concerné
        hbox_decalage_fps_anim.setAlignment(Qt.AlignCenter)

        # Le VBox pour mettre ces éléments en place
        vbox_reglages_enregistrement_ga = QVBoxLayout()
        vbox_reglages_enregistrement_ga.addLayout(hbox_decalage_fps_anim)

        # Permet de tasser les widgets sur la hauteur
        vbox_reglages_enregistrement_ga.setAlignment(Qt.AlignVCenter)
        
        # QHBoxLayout pour ...
        hbox_group_reglages_et_enr_gif_anim = QHBoxLayout()
        # Par la méthode addLayout (mise en place des vbox horizontalement)
        hbox_group_reglages_et_enr_gif_anim.addLayout(vbox_reglages_enregistrement_ga)
        
        #
        groupe_1.setLayout(hbox_group_reglages_et_enr_gif_anim)
        
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        
        # =====================================================
        # Signaux
        # =====================================================
        # Signal du combobox du choix du décalage pour le gif animé
        self.combo_choix_decalage_gif_anim.currentIndexChanged.connect(self.fonction_changements_choix_decalage_gif_anim)
        # Signal du spinbox du choix de la fréquence du gif animé
        self.spinbox_fps_gif_anim.valueChanged.connect(self.fonction_changement_choix_fps_gif_anim)
        # Bouton pour lancer le traitement du gif animé
        bout_enregistrer_gif_anime_expo.clicked.connect(self.graphe_vue.sauvegarder_gif_anime)
        # =====================================================
        
        self.menu_stack_sauv_type_gif_anime.setLayout(layout)
        
       
    def fonction_changements_choix_decalage_gif_anim(self) :
        """ Fonction pour écrire la valeur du décalage """ 
            
        if self.combo_choix_decalage_gif_anim.currentText() == "25" :
            ecr_tempo_choix_decalage_ga = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_decalage_gif_anime_murexpo_", "data_choix_decalage_gif_anime_murexpo", ".txt", "25")
        elif self.combo_choix_decalage_gif_anim.currentText() == "20" :
            ecr_tempo_choix_decalage_ga = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_decalage_gif_anime_murexpo_", "data_choix_decalage_gif_anime_murexpo", ".txt", "20")
        elif self.combo_choix_decalage_gif_anim.currentText() == "10" :
            ecr_tempo_choix_decalage_ga = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_decalage_gif_anime_murexpo_", "data_choix_decalage_gif_anime_murexpo", ".txt", "10")
        elif self.combo_choix_decalage_gif_anim.currentText() == "5" :
            ecr_tempo_choix_decalage_ga = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_decalage_gif_anime_murexpo_", "data_choix_decalage_gif_anime_murexpo", ".txt", "5")
            
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Choix du décalage (en pixels) des images dans le gif animé : {}\n").format(str(self.combo_choix_decalage_gif_anim.currentText())))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
            
            
    def fonction_changement_choix_fps_gif_anim(self) :
        """ Fonction pour écrire la valeur de la fréquence (img/sec) """

        ecr_tempo_choix_fps_sec_ga = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_fps_gif_anime_murexpo_", "data_choix_fps_gif_anime_murexpo", ".txt", str(self.spinbox_fps_gif_anim.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Choix de la fréquence (images/seconde) du gif animé : {}\n").format(str(self.spinbox_fps_gif_anim.value())))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
    
    '''
    # -------------------------------------------------------------------
    # Commenté car ... :
    # Au lieu de passer par une fonction (comme dans les autres 
    # menus) on passe ici par une classe (Murexpo_Export_Html)
    # qui embarque directement le QWidget contenant les champs
    # de texte et les boutons pour le traitement du htm/pdfl.
        
    def fonction_stack_sauv_type_fichier_html(self) :
        """ Fonction du QStackedWidget pour 
        la sauvegarde / exportation sous la 
        forme d'un fichier html ou pdf """
        
        label_info = QLabel(_("Cette fonctionnalité n'est pas encore disponible."))
        label_info.setAlignment(Qt.AlignCenter)
        
        groupe_1 = QGroupBox()

        grid_1 = QGridLayout()
        grid_1.setSpacing(8)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        grid_1.addWidget(label_info, 0, 0)

        groupe_1.setLayout(grid_1)
        
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        
        self.menu_stack_sauv_type_fichier_html.setLayout(layout)
        
        #
        self.export_html = Murexpo_Export_Html()
    # -------------------------------------------------------------------
    '''
        
        
    def fonction_stack_sauv_type_video(self) :
        """ Fonction du QStackedWidget pour 
        la sauvegarde / exportation sous la 
        forme d'une vidéo """
        
        label_info = QLabel(_("Cette fonctionnalité n'est pas encore disponible."))
        label_info.setAlignment(Qt.AlignCenter)
        
        groupe_1 = QGroupBox()

        grid_1 = QGridLayout()
        grid_1.setSpacing(8)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        grid_1.addWidget(label_info, 0, 0)

        groupe_1.setLayout(grid_1)
        
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        
        self.menu_stack_sauv_type_video.setLayout(layout)
        
        
    def affiche_phases_sauvegarde_exportation(self, i):
        """ Appel du QStackedWidget et
        mise en place pour naviguer entre
        les différentes pages concernant
        la sauvegarde / exportation (image,
        gif animé, fichier html ou vidéo) """
        
        self.stack_sauvegarde.setCurrentIndex(i)
        
        # -------------------------------------------------------------------------------------------
        # Non encore utilisé (mais sera certainement utile plus tard)
        # -------------------------------------------------------------------------------------------
        # Drapeau écrit dans le rep. tempo. (donnée écrite à n, c'est à dire l'index de la phase
        # d'exportation choisie dans le stacked)
        # -------------------------------------------------------------------------------------------
        # Ecriture de l'index de la phase d'exportation (dans l'onglet Conception et exportation) 
        # --> à la page 4 de l'affichage des phases [index 3 du stacked --> self.stack] ... et 
        # ... et à l'index 0 du stacked --> self.stack_sauvegarde ... de même à l'index 0 dans 
        # la combo self.liste_sauvegarde (ici pour l'instant le choix est : Exporter (sous la 
        # forme d'une image) l'exposition)
        # --------------------------------------
        # Pour l'instant, tant que rien n'a été sélectionné dans la combo self.liste_sauvegarde, on
        # est toujours à l'index 0, c'est à dire avec le choix : Exporter (sous la forme d'une 
        # image) l'exposition ...
        # --------------------------------------
        ecr_tempo_index_exportation = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_index_exportation_conversion_murexpo_", "data_index_exportation_conversion_murexpo", ".txt", str(i))
        # -------------------------------------------------------------------------------------------
        
    # --- Fonctions spécifiques du QStackedWidget pour la sauvegarde / exportation ---


    # --------------------------------- sender boutons (à garder on sait jamais) -----
    """
    def libelle_bouton_aff_mur(self) :
        '''
        Sélectionner le textedu bouton pressé en vue de
        ...
        '''
        
        # La récupération semble fonctionner !!!
        
        # Récupérer l'action du bouton
        message_bouton_aff_mur = self.sender()

        # Le texte du bouton (par ex pour le bouton ...) est
        # de la forme : ...
        # Cela récupère le texte de l'action du bouton
        texte_bouton_aff_mur = message_bouton_aff_mur.text()
        
        return texte_bouton_aff_mur
    
    
    def libelle_bouton_aff_oeuvre(self) :
        '''
        Sélectionner le textedu bouton pressé en vue de
        ...
        '''
        
        # La récupération semble fonctionner !!!
        
        # Récupérer l'action du bouton
        message_bouton_aff_oeuvre = self.sender()

        # Le texte du bouton (par ex pour le bouton ...) est
        # de la forme : ...
        # Cela récupère le texte de l'action du bouton
        texte_bouton_aff_oeuvre = message_bouton_aff_oeuvre.text()
        
        return texte_bouton_aff_oeuvre
    """
    # --------------------------------- sender boutons (à garder on sait jamais) -----


class MaScene(QGraphicsScene) :
    """ Classe de la scène (QGraphicsScene) """
    
    def __init__(self, parent=None) :
        super(MaScene, self).__init__(parent)
        
        
class MonGraphiquePixmapItemMur(QGraphicsPixmapItem) :
    """ Classe pour l'affichage de
    l'image du mur d'exposition """
    
    def __init__(self, parent=None):
        super(MonGraphiquePixmapItemMur, self).__init__(parent)

        
class MonGraphiquePixmapItemOeuvre(QGraphicsPixmapItem) :
    """ Classe pour l'affichage des oeuvres (les images). 
    Cette classe MonGraphiquePixmapItemOeuvre est 
    appelée dans la classe MonGraphiqueVueMur """
    
    def __init__(self, parent=None):
        super(MonGraphiquePixmapItemOeuvre, self).__init__(parent)

        # Fonctionne avec un import de QGraphicsItem
        # -------------------------------------------
        # L'item peut être bougé
        self.setFlag(QGraphicsItem.ItemIsMovable, True)
        # L'item peut être sélectionné
        self.setFlag(QGraphicsItem.ItemIsSelectable, True)
        # On peut donner un focus à l'item
        self.setFlag(QGraphicsItem.ItemIsFocusable, True)
        
        # Envoie un signal quand un 
        # QGraphicsPixmapItem est 
        # sélectionné
        self.setAcceptHoverEvents(True)
       
    
    # ------------------------------------------------ A garder pour l'instant ---------------    
    '''
    def hoverEnterEvent(self, event):
        """ Vu ici : https://stackoverflow.com/questions/37090727/hover-event-for-a-qgraphicsitem-pyqt4 """

        # La liste globale de positionnement des items (les images des oeuvres
        # se remplit au fur et à mesure que l'utilisateur les déplace à la souris)
        liste_position_PixmapItem.append((event.pos().x(), event.pos().y()))
        
        print("liste_position_PixmapItem ... Dernière position :", liste_position_PixmapItem[-1])
    '''
    # ------------------------------------------------ A garder pour l'instant ---------------  
    

class MonGraphiqueVueMur(QGraphicsView) :
    """ Classe du QGraphicsView pour affichage du mur et des oeuvres """
    
    # Couleur du background
    backgroundcolor = QColor(100, 100, 100)
    
    def __init__(self, parent=None) :
        super(MonGraphiqueVueMur, self).__init__(parent)
        
        self.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform | QPainter.HighQualityAntialiasing) #
        
        # ------------------------------------------------ A garder pour l'instant ---------------
        """
        # On peut sélectionner une zone dans la vue
        #self.setDragMode(QGraphicsView.RubberBandDrag)
        #
        self.setRenderHint(QPainter.Antialiasing) # A voir + tard si utile
        self.setRenderHint(QPainter.TextAntialiasing) # A voir + tard si utile
        """
        # ------------------------------------------------ A garder pour l'instant ---------------
        
        # Application du fond gris
        self.setBackgroundBrush(self.backgroundcolor)
    
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # Appel de la classe MaScene
        self.scene  = MaScene()

        # Ajout de la scène
        self.setScene(self.scene)
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        # Méthode permettant de récupérer l'objet
        # sélectionné dans la scène quand l'utilisateur 
        # sélectionne un item image
        self.scene.selectionChanged.connect(self.selection_changee)
        self.selection_courante = []

        # (Pré) Liste pour collecter (dans un 1er temps) 
        # les images des oeuvres insérées dans la scène
        self.liste_pre_collecte_images_oeuvres = []
        
        # Liste de collecte de l'item du pixmap
        # (le widget contenant l'image chargée)
        # dans la scène
        self.liste_collecte_item_pixmap = []
        
        # Amorce de la liste qui se remplit
        # au fur et à mesure que des oeuvres
        # sont supprimées (Important)
        self.item_oeuvre_supprime = []
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # ---------------------------------------------------------------
        # Appel de la classe pour capturer toutes les images
        # chargées pendant la disposition des oeuvres sur
        # le mur d'exposition
        self.capture_oeuvres_chargees = CapterToutesOeuvresChargees()
        # ********************************************************
        # --> Voir la suite dans la fonction pre_selection_oeuvre
        # ********************************************************
        # ---------------------------------------------------------------

        # ---------------------------------------------------------------
        # Appel de la classe pour capturer toutes les images
        # supprimées pendant la disposition des oeuvres sur
        # le mur d'exposition
        self.capture_oeuvres_supprimees = CapterToutesOeuvresSupprimees()
        # ********************************************************
        # --> Voir la suite dans la fonction supprimer_image_oeuvre
        # ********************************************************
        # ---------------------------------------------------------------
        
        #####
        
        # --------------------------------------------
        # Drapeaux (important !)
        # --------------------------------------------
        # Drapeau pour l'encadrement classique de 
        # l'oeuvre (traitement image) positionné à 0
        self.drapeau_cadre_noir_classique = 0
        # Drapeau pour Sous verre, bordure 
        # (traitement image) positionné à 0
        self.drapeau_sous_verre_avec_bordure = 0
        
        # ---------------------------------------------------------------------
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        #print(chem_rep_tempo)
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        # ---------------------------------------------------------------------
  
        # Listage le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
        # ... ou du répertoire courant ...
        liste_contenu_rep_courant = os.listdir(os.getcwd())
        # ------------------------------------------------------------
        # Le fichier data_choix_du_mur_murexpo_.txt
        # est éliminé à chaque démarrage de MurExpo
        # ------------------------------------------------------------
        try :
            for contenu in liste_contenu_tmp :
                # Si le fichier tempo concerné est bien dedans ...
                if 'data_choix_du_mur_murexpo' in contenu :
                    os.remove(self.chemin_tempo_system_exploit+os.sep+contenu)      
        except :
            for contenu in liste_contenu_rep_courant :
                if 'data_choix_du_mur_murexpo' in contenu :
                    os.remove(os.getcwd()+os.sep+contenu)
                    
        # ========================================================================================
        # AFFICHAGE (ET RESOLUTION)
        # ========================================================================================
        
        # ------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture 
        # des données dans le répertoire (dit) temporaire
        # ------------------------------------------------------
        # --> Lecture de la résolution de l'écran
        pre_lecture_resolution = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_desktop_murexpo_", "data_resolution_desktop_murexpo")
        # ... avec appel de la méthode affichage
        lecture_resolution = "".join(pre_lecture_resolution.affichage())
        # Récupération des données ... pour une résolution
        # de 1920 x 1080, le résultat est : ['1920', '1080']
        self.resolution = lecture_resolution.split('x')

        # Attribution ...
        self.resolution_x, self.resolution_y = int(self.resolution[0]), int(self.resolution[1])
        
        # ------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture 
        # des données dans le répertoire (dit) temporaire
        # ------------------------------------------------------
        # --> Lecture de l'espace disponible en hauteur (sans la barre des tâches)
        # --> il s'agit de l'espace restant quand on enlève la taille de la barre
        # des tâches (contenant par ex. le menu démarrage sous windows ...).
        pre_lecture_espace_dispo_hauteur = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_espace_dispo_haut_desktop_murexpo_", "data_espace_dispo_haut_desktop_murexpo")
        # ... avec appel de la méthode affichage
        lecture_espace_dispo_haut_desktop = "".join(pre_lecture_espace_dispo_hauteur.affichage())
        # Récupération des données ... pour l'espace disponible
        self.espace_dispo_desktop = lecture_espace_dispo_haut_desktop.split('x')
        # Attribution ...
        self.espace_dispo_desk_x, self.espace_dispo_desk_y = int(self.espace_dispo_desktop[0]), int(self.espace_dispo_desktop[1])
        
        # ---- Pour tests -------------------------------- essais ----
        #self.resolution_x, self.resolution_y = 1440, 1080 # Pour essais
        #self.espace_dispo_desk_x = 1440 # Pour essais
        #self.espace_dispo_desk_y = 1080 # Pour essais
        # ---- Pour tests -------------------------------- essais ----

        # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # Appel de la fonction pour l'affichage de l'interface graphique (la fenêtre 
        # principale de MurExpo) pour qu'elle s'adapte au maximum à la bonne résolution
        self.affichage_placement_resolution(self.resolution_x, self.resolution_y, self.espace_dispo_desk_x, self.espace_dispo_desk_y)
        # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        # ========================================================================================
        
        # Appel de la fonction pour afficher le logo 
        # de MurExpo au démarrage du logiciel
        self.affichage_du_logo()
        
        
    def affichage_placement_resolution(self, res_x, res_y, espac_dispo_x, espac_dispo_y) :
        """ Fonction pour la mise en place de la zone d'affichage 
        de travail (la résolution de la zone d'affichage ; largeur, 
        hauteur) de MurExpo. """

        # Résolutions (en largeur) prises en compte
        resolutions_supportees = [1920, 1829, 1807, 1792, 1776, 1768, 1750, 1722, 1707, 1700, 1683, 1680, 1654, 1644, 
                                  1600, 1564, 1544, 1536, 1517, 1504, 1500, 1477, 1463, 1444, 1440, 1415, 1400, 1397, 
                                  1368, 1366, 1360, 1344, 1334, 1301, 1280, 1224, 1216, 1210, 1200]
        
        # -----------------------------------------------------------------------------------------
        # Dimensionnement du QGraphicsView (à voir ici :
        # https://forum.qt.io/topic/48564/force-qgraphicsview-size-to-match-qgraphicsscene-size/5)
        # -----------------------------------------------------------------------------------------
        # Mise en place du dimensionnement statique ... cas précis suivant la résolution 
        # de l'écran ...
        # -------------------------------------------------
        # "Résolution programmée" et "Résolution au choix"
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1920 x 1200 ou inférieure à 1200 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        if res_x == 1920 and espac_dispo_y <= 1200 :
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1920 x 1080 ou inférieure à 1080 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        if res_x == 1920 and espac_dispo_y <= 1080 : # 16/9ème
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1792 x 1120 ou inférieure à 1120 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        if res_x == 1792 and espac_dispo_y <= 1120 : # 5/3 (5 tiers)
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -----------------------------------
        # Pour une résolution de 1722 x 1070 ou inférieure à 1070 (barres des tâches en moins)
        # -----------------------------------
        elif res_x == 1722 and self.espace_dispo_desk_y <= 1070 : # 16/10ème
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -----------------------------------
        # Pour une résolution de 1707 x 960 ou inférieure à 960 (barres des tâches en moins)
        # -----------------------------------
        elif res_x == 1707 and self.espace_dispo_desk_y <= 960 :
            # Sous GNU/Linux, MacOS, Windows --> 420
            ajustement_y = 420
        # -----------------------------------
        # Pour une résolution de 1680 x 1050 ou inférieure à 1050 (barres des tâches en moins)
        # -----------------------------------
        elif res_x == 1680 and self.espace_dispo_desk_y <= 1050 : # 16/10ème
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1600 x 1200 ou inférieure à 1200 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1600 and espac_dispo_y <= 1200 : # 16/9ème
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1600 x 1000 ou inférieure à 1000 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1600 and espac_dispo_y <= 1000 :
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1600 x 900 ou inférieure à 900 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1600 and espac_dispo_y <= 900 : # 16/9ème
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1536 x 960 ou inférieure à 960 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1536 and espac_dispo_y <= 960 :
            # Sous GNU/Linux, MacOS, Windows --> 420
            ajustement_y = 420
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1536 x 864 ou inférieure à 864 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1536 and espac_dispo_y <= 864 :
            # Sous GNU/Linux, MacOS, Windows --> 420
            ajustement_y = 420
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1517 x 853 ou inférieure à 853 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1517 and espac_dispo_y <= 853 :
            # Sous GNU/Linux, MacOS, Windows --> 420
            ajustement_y = 420
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1504 x 1003 ou inférieure à 1003 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1504 and espac_dispo_y <= 1003 :
            # Sous GNU/Linux, MacOS, Windows --> 420
            ajustement_y = 420
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1440 x 1080 ou inférieure à 1080 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1440 and espac_dispo_y <= 1080 :
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1440 x 900 ou inférieure à 900 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1440 and espac_dispo_y <= 900 :
            # Sous GNU/Linux, MacOS, Windows --> 420
            ajustement_y = 420
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1400 x 1050 ou inférieure à 1050 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1400 and espac_dispo_y <= 1050 : # 4/3 (quatre tiers)
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1368 x 912 ou inférieure à 912 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1368 and espac_dispo_y <= 912 :
            # Sous GNU/Linux et MacOS --> 450
            # Sous Windows --> 420
            # Si on est sous windows ...
            if platform.system() == 'Windows' :
                ajustement_y = 400
            # Si on est sous GNU/Linux ou MacOS ...
            elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
                ajustement_y = 420
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1366 x 768 ou inférieure à 768 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1366 and espac_dispo_y <= 768 : # 
            # Sous GNU/Linux et MacOS --> 450
            # Sous Windows --> 420
            # Si on est sous windows ...
            if platform.system() == 'Windows' :
                ajustement_y = 420
            # Si on est sous GNU/Linux ou MacOS ...
            elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
                ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1360 x 768 ou inférieure à 768 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1360 and espac_dispo_y <= 768 :
            # Sous GNU/Linux et MacOS --> 450
            # Sous Windows --> 420
            # Si on est sous windows ...
            if platform.system() == 'Windows' :
                ajustement_y = 420
            # Si on est sous GNU/Linux ou MacOS ...
            elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
                ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1344 x 840 ou inférieure à 840 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1344 and espac_dispo_y <= 840 :
            # Sous GNU/Linux et MacOS --> 420
            # Sous Windows --> 400
            # Si on est sous windows ...
            if platform.system() == 'Windows' :
                ajustement_y = 400
            # Si on est sous GNU/Linux ou MacOS ...
            elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
                ajustement_y = 420
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1334 x 800 ou inférieure à 800 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1334 and espac_dispo_y <= 800 :
            # Sous GNU/Linux et MacOS --> 420
            # Sous Windows --> 400
            # Si on est sous windows ...
            if platform.system() == 'Windows' :
                ajustement_y = 400
            # Si on est sous GNU/Linux ou MacOS ...
            elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
                ajustement_y = 420
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1280 x 1024 ou inférieure à 1024 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1280 and espac_dispo_y <= 1024 :
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1280 x 960 ou inférieure à 960 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1280 and espac_dispo_y <= 960 : # 16/10ème
            # Sous GNU/Linux, MacOS, Windows --> 350
            ajustement_y = 350
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1280 x 800 ou inférieure à 800 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1280 and espac_dispo_y <= 800 : # 16/10ème
            # Sous GNU/Linux, MacOS, Windows --> 450
            ajustement_y = 450
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1280 x 768 ou inférieure à 768 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1280 and espac_dispo_y <= 768 :
            # Sous GNU/Linux et MacOS --> 430
            # Sous Windows --> 420
            # Si on est sous windows ...
            if platform.system() == 'Windows' :
                ajustement_y = 420
            # Si on est sous GNU/Linux ou MacOS ...
            elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
                ajustement_y = 430     
        # -------------------------------------------------------------------------------------
        # Pour une résolution de 1280 x 720 ou inférieure à 720 (barres des tâches en moins)
        # -------------------------------------------------------------------------------------
        elif res_x == 1280 and espac_dispo_y <= 720 : # 16/9ème
            # Sous GNU/Linux et MacOS --> 386
            # Sous Windows --> 420
            # Si on est sous windows ...
            if platform.system() == 'Windows' :
                ajustement_y = 420
            # Si on est sous GNU/Linux ou MacOS ...
            elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
                ajustement_y = 386

        # -----------------------------------
        # "Résolution programmée" uniquement
        # -----------------------------------
        # Pour une résolution ne faisant pas
        # partie de cette liste (!) ... on
        # utilise la résolution 1280 x 720
        # -----------------------------------
        if res_x not in resolutions_supportees :
            res_x, res_y = 1280, 720
            # Sous GNU/Linux et MacOS --> 386
            # Sous Windows --> 420
            # Si on est sous windows ...
            if platform.system() == 'Windows' :
                ajustement_y = 420
            # Si on est sous GNU/Linux ou MacOS ...
            elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
                ajustement_y = 386
            self.setFixedSize(res_x - 50, espac_dispo_y - ajustement_y) 
            # Placement et dimensionnement
            self.setSceneRect(0, 0, res_x - 50, espac_dispo_y - ajustement_y)
            # Le dimensionnement s'adapte à la vue
            self.fitInView(0, 0, res_x - 50, espac_dispo_y - ajustement_y, Qt.KeepAspectRatio)
        # --------------------------------------------
        # "Résolution programmée" et "Résolution 
        # au choix"
        # --------------------------------------------
        # Pour une résolution conforme aux 
        # résolutions prises en compte
        # --------------------------------------------
        if res_x in resolutions_supportees :
            try :
                # --------------------------------------------------
                # Résolutions affectées lors du choix (par défaut)
                # "Résolution programmée" dans la configuration
                # de la résolution ...
                # --------------------------------------------------
                self.setFixedSize(espac_dispo_x - 50, espac_dispo_y - ajustement_y)
            except :
                # --------------------------------------------------
                # Résolutions affectées lors du choix
                # "Résolution au choix" dans la configuration de
                # la résolution ... (450 dimension à soustraire)
                # --------------------------------------------------
                self.setFixedSize(res_x - 50, res_y - 450)
            # Placement et dimensionnement
            try :
                # --------------------------------------------------
                # ... "Résolution programmée" ...
                # --------------------------------------------------
                self.setSceneRect(0, 0, espac_dispo_x - 50, espac_dispo_y - ajustement_y)
            except :
                # --------------------------------------------------
                # ... "Résolution au choix" ...
                # --------------------------------------------------
                self.setSceneRect(0, 0, res_x - 50, res_y - 450)
            # Le dimensionnement s'adapte à la vue
            try :
                # --------------------------------------------------
                # ... "Résolution programmée" ...
                # --------------------------------------------------
                self.fitInView(0, 0, espac_dispo_x - 50, espac_dispo_y - ajustement_y, Qt.KeepAspectRatio)
            except :
                # --------------------------------------------------
                # ... "Résolution au choix" ...
                # --------------------------------------------------
                self.fitInView(0, 0, res_x - 50, res_y - 450, Qt.KeepAspectRatio)
        
        # --------------------------------------------
        # "Résolution programmée" uniquement
        # --------------------------------------------
        # Pour une résolution supérieure à (x) 1920
        # --------------------------------------------
        if res_x > 1920 :
            res_x, res_y = 1920, 1080
            # Dimensions fixes (1920 - 50 pour x et 1080 - 430 pour y)
            self.setFixedSize(res_x - 50, res_y - 430) 
            # Placement et dimensionnement
            self.setSceneRect(0, 0, res_x - 50, res_y - 430)
            # Le dimensionnement s'adapte à la vue
            self.fitInView(0, 0, res_x - 50, res_y - 430, Qt.KeepAspectRatio)
    
    
    # ---------------------------------------------------------------------------------
    # Ces fonctions (événements à la souris, en l'occurrence pointeur gauche de la
    # souris + touche Shift [touche Maj avec un clavier de PC ... et Shift avec un 
    # clavier Mac]) permettent de faire défiler le panneau (la vue) de gauche à 
    # droite ou de haut en bas (en fait un pan)
    # ---------------------------------------------------------------------------------
    # Voir ce post sur stackedoverflow (merci à watersnake !) :
    # https://stackoverflow.com/questions/51566884/pyqt5-in-a-qgraphicsview-rebind-
    # scrollhanddrag-to-work-with-middle-mouse-butto
    # ---------------------------------------------------------------------------------
    
    def toggleDragMode(self) :
        if self.dragMode() == QGraphicsView.ScrollHandDrag :
            self.setDragMode(QGraphicsView.NoDrag)
        else :
            self.setDragMode(QGraphicsView.ScrollHandDrag)

    
    def mousePressEvent(self, event):
        # Si l'utilisateur appuye en même temps sur la touche Shift
        # (Maj avec un clavier de PC ... ou Shft clavier Mac) + 
        # pointeur souris gauche, on peut translater le panneau
        if (event.buttons() == Qt.LeftButton and event.modifiers() == Qt.ShiftModifier) :
            self.toggleDragMode()
        super().mousePressEvent(event)
    
    
    def mouseReleaseEvent(self, event):
        if self.dragMode() == QGraphicsView.ScrollHandDrag:
            self.setDragMode(QGraphicsView.NoDrag)
        super().mouseReleaseEvent(event)

    # ---------------------------------------------------------------------------------
    
    def wheelEvent(self, event):
        """
        # --------------------------------
        Merci à rengel pour cette solution
        pour zoomer et dézoomer la vue (et 
        donc l'image qu'elle contient) ...
        Solution à voir ici :
        https://stackoverflow.com/questions/19113532/
        qgraphicsview-zooming-in-and-out-under-mouse-position-using-mouse-wheel
        # --------------------------------
        Zoom in or out of the view.
        --> Zoom avant : roulette milieu de la souris vers le haut
        --> Zoom arrière : roulette milieu de la souris vers la bas
        """

        zoomInFactor = 1.1
        zoomOutFactor = 1 / zoomInFactor
        
        # Set Anchors
        self.setTransformationAnchor(QGraphicsView.NoAnchor)
        self.setResizeAnchor(QGraphicsView.NoAnchor)

        # Save the scene pos
        oldPos = self.mapToScene(event.pos())
    
        # Zoom
        if event.angleDelta().y() > 0:
            zoomFactor = zoomInFactor
        else:
            zoomFactor = zoomOutFactor
        self.scale(zoomFactor, zoomFactor)
    
        # Get the new position
        newPos = self.mapToScene(event.pos())
    
        # Move scene to old position
        delta = newPos - oldPos
        self.translate(delta.x(), delta.y())

        
    def recup_choix_mur(self) :
        """ Fonction du choix automatique du mur d'exposition """
        
        # Exception pour s'assurer que le 1er mur dans la liste
        # (combo du choix du mur dans la classe Fabrique)
        # s'affiche dans la vue si l'utilisateur ne sélectionne
        # aucun item dans la combo
        try :
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_choix_mur = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_choix_du_mur_murexpo_', 'data_choix_du_mur_murexpo')
            # ... avec appel de la méthode affichage
            self.lecture_choix_mur = "".join(pre_lecture_choix_mur.affichage())
        except :
            self.lecture_choix_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_001.jpg'))
        
        return self.lecture_choix_mur
    
    
    def recup_image_mur(self) :
        """ Récupération de l'image du mur 
        (image ouverte avec Pillow) """
        
        # Appel de la fonction recup_choix_mur
        choix_mur = self.recup_choix_mur()
        
        # Conditions pour sélectionner l'image du mur
        if choix_mur == _('mur_001') :
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_001.png'))
        if choix_mur == _('mur_002') :
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_002.png'))
        if choix_mur == _('mur_003') :
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_003.png'))
        if choix_mur == _('mur_004') :
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_004.png'))
        if choix_mur == _('mur_005') :
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_005.png'))
        if choix_mur == _('mur_006') :
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_006.png'))
        if choix_mur == _('mur_007') :
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_007.png'))
        if choix_mur == _('mur_008') :
            self.image_du_mur = chemin_ressources(os.path.join('ressources_murexpo', 'mur_008.png'))

        # Exception pour récupérer l'image du mur (avec
        # Pillow). Si l'utilisateur ne sélectionne aucun 
        # mur dans le combo (défini dans la classe 
        # Fabrique) c'est le 1er mur de la liste qui
        # est automatiquement choisi.
        try :
            # Ouverture de l'image avec Pillow
            self.img_mur = Image.open(self.image_du_mur)
        except :
            self.img_mur = Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'mur_001.png')))
            # Ecriture dans le répertoire temporaire du choix du mur ...
            ecr_choix_mur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_du_mur_murexpo_", "data_choix_du_mur_murexpo", ".txt", _('mur_001'))
            
        return self.img_mur
    
    
    def verif_resolution_img_pour_affichage(self) :
        """ Cette fonction vérifie la résolution 
        de l'image du mur choisie et recalcule 
        une nouvelle résolution afin que cette 
        image s'affiche correctement dans le 
        QGraphicsView (fonction avec return) """
        
        # Vérification de la résolution (en 
        # pixels) de l'image (du mur) chargée
        w, h = self.img_mur.size        
        
        # Calcul du ratio de l'image
        ratio = float(w)/float(h)

        # Calcul de future hauteur
        h = int(float(largeur_graphic_vue)/ratio)
        # Nouvelle dimension : 
        # tuple : (largeur, hauteur)
        nouv_dim_img = (largeur_graphic_vue, h)
        
        return nouv_dim_img
    
    
    def afficher_mur(self, images) :
        """ Fonction d'affichage du mur d'exposition 
        (avec le bouton Affichage du mur) dans la vue 
        La fonction possède un argument --> images """
        
        # Arrêt du QTimer
        # (défini ds l'init)
        #self.timer.stop()

        # Maj scène et vue QGraphicsView
        self.scene.clear()
        self.items().clear()

        # ------------------------------------
        # Détection de la taille de l'image 
        # (pour harmoniser la vue)
        # ------------------------------------
        # Appel de la fonction recup_image_mur
        # (avec un return) pour récupérer 
        # l'image du mur d'exposition
        im = self.recup_image_mur()
        largeur, hauteur = im.size

        # Conversion de l'image en JPEG
        image_mur = im.convert("RGB").save(os.path.join(self.chemin_tempo_system_exploit, 'image_mur.jpg'))

        # Récupération de l'image traitée dans le rép. tempo
        images = os.path.join(self.chemin_tempo_system_exploit, 'image_mur.jpg')
        
        # Le pixmap récupère l'image ...
        pixmap = QPixmap(images)

        # Insère l'image dans le QGraphicsPixmapItem (pour
        # le mur d'exposition) .--> Appel de la classe 
        # MonGraphiquePixmapItemMur avec comme argument 
        # le pixmap
        item = MonGraphiquePixmapItemMur(pixmap)
        
        # ------------------------------------------------ A garder pour l'instant ---------------
        """
        print("item.boundingRect().width()", item.boundingRect().width(), "item.boundingRect().height()", item.boundingRect().height()) ###############
        # Résultat : item.boundingRect().width() 1920.0 item.boundingRect().height() 1080.0 ... ou ...
        # Résultat : item.boundingRect().width() 2716.0 item.boundingRect().height() 1450.0 ... en fait
        # il s'agit de la résolution de l'image du mur chargée. En fait la scène doit avoir (normalement)
        # les dimensions de l'image (déployée à 100%)
        print("largeur image", largeur, "hauteur image", hauteur)
        # Résultat : largeur image 1920 hauteur image 1080 ... ou ...
        # Résultat : largeur image 2716 hauteur image 1450 ... pareil !
        # --------------------------------------------------
        # Get the size of your graphicsview
        # -----------------------------------
        # Permet de récupérer la taille (w, h) 
        # de la vue après appui sur le bouton
        # "Affichage du mur"
        vue_rectangle = self.viewport().rect()
        print("vue_rectangle.size()", str(vue_rectangle.size()))
        # Résultat : vue_rectangle.size() PyQt5.QtCore.QSize(1212, 428)
        # --> C'est quoi cette dimension ???
        """
        # ------------------------------------------------ A garder pour l'instant ---------------

        # Amélioration de la qualité
        item.setTransformationMode(Qt.SmoothTransformation)

        # Insère QGraphicsPixmapItem
        # dans la scène
        self.scene.addItem(item)

        # Remise en service du QTimer (défini ds l'init)
        #self.timer.start(self.temps_de_latence_vision)

        # ---------------------------------------------------------------------
        self.setScene(self.scene)
        # 
        self.setSceneRect(QRectF(self.viewport().rect()))

        x, y = 0, 0
        self.setSceneRect(x, y, pixmap.width(), pixmap.height())
        self.centerOn(0, 0)
        
        # Alignement
        self.setAlignment(Qt.Alignment(Qt.AlignHCenter|Qt.AlignVCenter))
        
        # Adapter la vue (le QGraphicsView) pour qu'il soit totalement
        # visible --> L'image déployée s'adapte à la taille de la vue
        self.fitInView(self.sceneRect(), Qt.KeepAspectRatio)
         
        # Affichage de la vue
        self.show()
        # ---------------------------------------------------------------------
        
        
    # ****************************** Gestion des oeuvres ******************************
    
    def afficher_image_oeuvre(self) :
        """ Fonction d'affichage de l'oeuvre (préalablement 
        sélectionnée dans le QTreeView) dans la vue ... et 
        ce par l'intermédiaire du bouton Ajouter la sélection
        (l'oeuvre) à la vue """
        
        try :
            
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_chemin_image = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_image_oeuvre_murexpo_', 'data_chemin_image_oeuvre_murexpo')
            # ... avec appel de la méthode affichage
            lecture_chemin_image = "".join(pre_lecture_chemin_image.affichage())
            
            # --------------------------------------------------------------
            # On vérifie si l'utilisateur a modifié la taille de l'image
            # (dans l'onglet Configuration > Traitement intermédiaire)
            # --------------------------------------------------------------
            # On liste le contenu du répertoire temporaire ...
            liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
            # Comprehension-liste pour récupérer (ou non) le fichier
            # data_taille_image_traitement_intermediaire du rep. tempo
            # --> si le fichier a été crée
            liste_presence_fichier_taille_img = [contenu for contenu in liste_contenu_tmp if 'data_taille_image_traitement_intermediaire' in contenu]
            # Si le fichier existe (la liste contiendra 1 élément) ...
            if len(liste_presence_fichier_taille_img) > 0 :
                # Utilisation de la classe Tempo_lecture pour la lecture
                # des données dans le répertoire (dit) temporaire
                rep_tempo_lecture_taille = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_taille_image_traitement_intermediaire_', 'data_taille_image_traitement_intermediaire')
                # ... avec appel de la méthode affichage
                rep_tempo_aff_taille = rep_tempo_lecture_taille.affichage()[0]
                # On récupère la valeur de la taille de l'image ...
                taille_image = int(rep_tempo_aff_taille)
                # ==============================================================
                # Adaptation de la taille de l'image pour le traitement
                # interne de l'image : image_oeuvre.png
                # ==============================================================
                # Appel du module de redimensionnement ... pour afficher les 
                # images d'oeuvres plus rapidement (fonction avec return)
                image = redimensionnement_image_oeuvre(lecture_chemin_image, taille_image, os.path.join(self.chemin_tempo_system_exploit, 'image_oeuvre.png'))
                # ==============================================================
            # Autrement si le fichier n'existe pas ...
            elif len(liste_presence_fichier_taille_img) == 0 :
                # ... une valeur par défaut est utilisée
                taille_image = 950
                # ==============================================================
                # Adaptation de la taille de l'image pour le traitement
                # interne de l'image : image_oeuvre.png
                # ==============================================================
                # Appel du module de redimensionnement ... pour afficher les 
                # images d'oeuvres plus rapidement (fonction avec return)
                image = redimensionnement_image_oeuvre(lecture_chemin_image, taille_image, os.path.join(self.chemin_tempo_system_exploit, 'image_oeuvre.png'))
                # ==============================================================
            
            # Appel de la fonction widget_image_oeuvre -->
            # fonction qui sert à insérer chaque item (chaque
            # image chargée par l'utilisateur) dans la scène.
            # Gestion du pixmap et son redimensionnement, appel 
            # de la classe dérivée MonGraphiquePixmapItemOeuvre
            # (avec comme argument le pixmap), gestion de l'ombrage
            # ... et affichage de la scène
            self.widget_image_oeuvre(image)
    
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Taille initiale en pixels de l'oeuvre chargée et adaptée à la vue : {}x{}\n").format(int(self.item_pixmap.boundingRect().width()), int(self.item_pixmap.boundingRect().height())))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            # Complément d'information :
            # --------------------------
            # Affiche la taille width de l'oeuvre (float) :
            # --> self.item_pixmap.boundingRect().width()
            # Affiche la taille height de l'oeuvre (float) :
            # self.item_pixmap.boundingRect().height()
            # ---------------------------------------------------------------
            
        except : return


    def widget_image_oeuvre(self, image) :
        """ Fonction servant à insérer chaque item (chaque
        image chargée par l'utilisateur) dans la scène.
        Gestion du pixmap et son redimensionnement, appel 
        de la classe dérivée MonGraphiquePixmapItemOeuvre
        (avec comme argument le pixmap), gestion de l'ombrage
        (appel de la fonction) ... et affichage de la scène """
        
        # ---------------------------------------------------------------------
        # Lecture du fichier tempo par rapport à l'ombre portée por l'oeuvre 
        # (si à la page/phase 2 la case 'Apposer une ombre portée') a été 
        # cochée (ou non) ...
        # ---------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture 
        # des données dans le répertoire (dit) temporaire
        pre_lecture_ombre_portee_oeuvre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_ombre_portee_sur_oeuvre_murexpo_', 'data_ombre_portee_sur_oeuvre_murexpo')
        # ... avec appel de la méthode affichage
        lecture_ombre_portee_oeuvre = "".join(pre_lecture_ombre_portee_oeuvre.affichage())
        
        # Le pixmap récupère l'image ...
        pixmap = QPixmap(image)
        
        # Redimensionnement du pixmap ... pour affichage immédiat dans la vue (largeur = 400 pixels)
        pixmap = pixmap.scaled(400, 400, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        
        # Insère l'image dans le QGraphicsPixmapItem (pour
        # le mur d'exposition) .--> Appel de la classe 
        # MonGraphiquePixmapItemOeuvre avec comme argument 
        # le pixmap
        self.item_pixmap = MonGraphiquePixmapItemOeuvre(pixmap)
        
        # Amélioration de la qualité
        self.item_pixmap.setTransformationMode(Qt.SmoothTransformation)
        
        #
        self.item_pixmap.setFlag(QGraphicsItem.ItemIsFocusable)
        
        # Placement aux coordonnées x = 10 et y = 10
        self.item_pixmap.setPos(10, 10)
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) L'oeuvre est à l'origine placée aux coordonnées 10 (en x), 10 (en y), dans la vue\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        # ---------------------------------------------------------------------
        # Commenté car cela ne fonctionne pas bien pour l'instant !!!
        # ---------------------------------------------------------------------
        """
        # Si l'utilisateur déplace l'item avec la souris
        # --> la liste se remplit à chaque mouvement de
        # l'item avec la souris ...
        if len(liste_position_PixmapItem) > 0 :
            # Les derniers placements sont insérés dans la liste
            self.item_pixmap.setPos(liste_position_PixmapItem[-1][0], liste_position_PixmapItem[-1][1])
        """
        # ---------------------------------------------------------------------
        
        # ---------------------------------------------------------------------
        # Si l'utilisateur a sélectionné 'Apposer une ombre portée' le 
        # traitement de l'ombre portée se fait ... sinon pas d'ommbre portée.
        # ---------------------------------------------------------------------
        if lecture_ombre_portee_oeuvre == _("Ombre portée oeuvre") :
            # Appel de la fonction pour 
            # créer l'ombrage de l'oeuvre
            self.ombrage_oeuvre(self.item_pixmap)
        elif lecture_ombre_portee_oeuvre == "" :
            pass
        
        # Insère QGraphicsPixmapItem
        # dans la scène
        self.scene.addItem(self.item_pixmap)

        # Met en place le QGrphicsView dans la scène
        self.setScene(self.scene)
        
        # Appel de la fonction pre_selection_oeuvre
        self.pre_selection_oeuvre()

        # Affichage de la vue
        self.show()
        
        # ---------------------------------------------------
        # Drapeaux (important !)
        # ---------------------------------------------------
        # Drapeau pour l'encadrement classique de l'oeuvre 
        # (traitement image) ici toujours positionné à 0
        # dans cette fonction widget_image_oeuvre car le
        # traitement n'a pas encore eu lieu (il aura lieu 
        # quand l'utilisateur aura cliqué sur la bouton 
        # Appliquer et pour finir sélectionné l'oeuvre
        # en question)
        self.drapeau_cadre_noir_classique = 0
        # Drapeau pour la pose d'un sous verre avec
        # bordure (traitement image) positionné à 0 ...
        self.drapeau_sous_verre_avec_bordure = 0


    def pre_selection_oeuvre(self) :
        """ Fonction de repérage (et association de l'item widget 
        avec le chemin de l'image traitée) ... en vue de pouvoir 
        ensuite capturer l'image/item sélectionné et ... faire le 
        traitement (si demandé) de (par exemple) la pose d'un 
        cadre noir """
        
        try :
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            # ------------------------------------------------------
            # Lecture du data fichier (rep. temporaire) ...
            # ------------------------------------------------------
            pre_lecture_selection_image = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_image_oeuvre_murexpo_', 'data_chemin_image_oeuvre_murexpo')
            # ... avec appel de la méthode affichage
            image = "".join(pre_lecture_selection_image.affichage())
        except : pass

        # ---------------------------------------------------------------
        # La (Pré) Liste de collecte des images des oeuvres 
        # insérées dans la scène se remplit au fur et à 
        # mesure que l'utilisateur charge des images ...
        self.liste_pre_collecte_images_oeuvres.append(image)
        # ---------------------------------------------------------------
        # Une caractéristique est que l'image image_oeuvre.png
        # (présente dans le répertoire temporaire) est aussi
        # insérée, ... il faut donc l'éliminer en détectant
        # le chemin du répertoire temporaire ... Résultat
        # en chargeant (par exemple) 3 images :
        # ['/home/angelo/... chemin .../andy_warhol_marilyn_rose_et_jaune_1962.jpg', 
        # '/home/angelo/... chemin .../anish_kapoor_installation_avec_des_pigments_et_des_formes_variees.jpg', 
        # '/home/angelo/... chemin .../annette_messager_et_range_ta_chambre_2007_2009.jpg']
        self.liste_images_oeuvres = [fichier for fichier in self.liste_pre_collecte_images_oeuvres if self.chemin_tempo_system_exploit not in fichier]
        # ---------------------------------------------------------------
        # Collecte des items pixmap déployés. A chaque fois que 
        # l'utilisateur déploye (dans la scène ... avec le bouton
        # Ajouter l'image à la vue) une image (de l'oeuvre) chargée 
        # à partir de l'onglet Fichier ouvrir, cette liste se remplit 
        # avec l'item pixmap concerné.
        self.liste_collecte_item_pixmap.append(self.item_pixmap)
        # ---------------------------------------------------------------
        # La liste des oeuvres chargées et des items pixmap sont fondues.
        # Le widget affiché dans la scène est associé à l'image chargée.
        # Le résultat sera sous la forme d'une liste de tuples.
        # ---------------------------------------------------------------
        # Par exemple (pour 3 images chargées et déployées), cela donne :
        # [(<fabrique_murexpo_gui.MonGraphiquePixmapItemOeuvre object at 0x7f3276875288>, 
        # '/home/angelo/.../andy_warhol_marilyn_rose_et_jaune_1962.jpg'), 
        # (<fabrique_murexpo_gui.MonGraphiquePixmapItemOeuvre object at 0x7f32768753a8>, 
        # '/home/angelo/.../anish_kapoor_installation_avec_des_pigments_et_des_formes_variees.jpg'), 
        # (<fabrique_murexpo_gui.MonGraphiquePixmapItemOeuvre object at 0x7f32768754c8>, 
        # '/home/angelo/.../annette_messager_et_range_ta_chambre_2007_2009.jpg')]
        self.liste_fondu_images_oeuvres_et_item_pixmap_PRE = list(zip(self.liste_collecte_item_pixmap, self.liste_images_oeuvres))
        # ---------------------------------------------------------------
        # Pour la suite ... une liste de tuples est immutable ... et
        # elle ne supporte pas l'indexation (les tuples en fait ne
        # supportent pas l'indexation). Cette nouvelle liste convertit
        # les tuples en listes (en fait il s'agira maintenant d'une
        # liste de listes) ... et convertit aussi la valeur des items
        # en string. Cette liste sera maintenant du style :
        # [['0', '<fabrique_murexpo_gui.MonGraphiquePixmapItemOeuvre object at 0x7f88d007ce58>', 
        # '/home/angelo/.../anish_kapoor_installation_avec_des_pigments_et_des_formes_variees.jpg'], 
        # ['1', '<fabrique_murexpo_gui.MonGraphiquePixmapItemOeuvre object at 0x7f88d007cf78>', 
        # '/home/angelo/.../andy_warhol_marilyn_rose_et_jaune_1962.jpg'], 
        # ['2', '<fabrique_murexpo_gui.MonGraphiquePixmapItemOeuvre object at 0x7f88c8ed00d8>', 
        # '/home/angelo/.../annette_messager_et_range_ta_chambre_2007_2009.jpg']
        self.liste_fondu_images_et_pixmap = [["{}".format(index), "{}".format(parc[0]), parc[1]] for index, parc in enumerate(self.liste_fondu_images_oeuvres_et_item_pixmap_PRE)]
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Chemin image (oeuvre) chargée : {}\n").format(image))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        ####################################################################################### ESSAI
        # ---------------------------------------------------------------
        # Retour de toutes les images chargées au moment de 
        # la création de l'exposition
        # ---------------------------------------------------------------
        # Récupération des chemins (des images) présents 
        # dans la liste self.liste_fondu_images_et_pixmap
        liste_chemims_oeuvres = [img[2] for img in self.liste_fondu_images_et_pixmap]
        # Boucle pour injecter les chemins ...
        for chemins in liste_chemims_oeuvres :
            self.capture_oeuvres_chargees.liste_capter_oeuvres_chargees('{}\n'.format(chemins))
        # -----------------
        # Ecriture des chemins dans le fichier tempo
        # data_capture_oeuvres_chargees_murexpo
        # -----------------
        # --> supression automatique des doublons dans la classe
        # CapterToutesOeuvresChargees (capture_oeuvres_chargees_murexpo.py)
        self.capture_oeuvres_chargees.liste_ecrire_oeuvres_chargees()
        # ---------------------------------------------------------------
        ####################################################################################### ESSAI


    def selection_changee(self) :
        """ Fonction servant à récupérer la sélection (image 
        oeuvre) dans la scène. En fait quand l'utilisateur 
        sélectionne une image (après l'avoir chargée), s'il 
        clique sur l'image en question ; elle est sélectionnée 
        ... puis le chemin de l'image est écrit dans le rep. 
        tempo ... ainsi que l'image elle-même (sous la forme
        d'un fichier PNG). L'image peut aussi être 
        redimensionnée ici (si cela s'avère nécessaire) """
        
        try :
            # Récupération de l'item sélectionné par 
            # l'utilisateur (l'image) dans la scène
            self.selection_courante = self.scene.selectedItems()
            
            print(_("Item image sélectionné dans la vue :"), self.selection_courante)
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Item image sélectionné dans la vue : {}\n").format(self.selection_courante[0]))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
            # Récupération de l'item sélectionné sous la forme d'un string, voilà le
            # contenu (par exemple) pour une sélection :
            # <fabrique_murexpo_gui.MonGraphiquePixmapItemOeuvre object at 0x7fa440f72c18>
            data = "{}".format(self.selection_courante[0])
        except : return

        try :
            # -------------------------------------------------------------
            # Boucle servant à récupérer le chemin de l'image (l'oeuvre) 
            # au travers de l'item sélectionné par l'utilisateur. La 
            # liste self.liste_fondu_images_et_pixmap vient de la fonction 
            # pre_selection_oeuvre
            # -------------------------------------------------------------
            for parc in self.liste_fondu_images_et_pixmap :
                # Comparaison entre l'item sélectionné (le widget de l'oeuvre sélectionnée)
                # et ce qui est présent dans la liste de fondu (variable contenue dans la 
                # fonction : pre_selection_oeuvre) :
                # [[index_1, 'widget_1, 'chemin_img_1'], [index_2, 'widget_2, 'chemin_img_2'], 
                # [index_3, 'widget_3, 'chemin_img_3'],[..., ..., ...], ...]
                # On compare le widget présent ds la liste self.liste_fondu_images_et_pixmap, 
                # c'est à dire --> parc[1] et le widget sélectionné ici --> data (présent plus 
                # haut)
                if parc[1] == data :
                    # -------------------------------------------------- 
                    # Récup du chemin de l'image sélectionnée : parc[2]
                    # et ensuite ouverture de l'image avec Pillow
                    # -------------------------------------------------- 
                    im = Image.open(parc[2])
                    # Conversion de l'image en PNG et sauvegarde ds le rep. tempo
                    image_selectionnee = im.convert("RGB").save(os.path.join(self.chemin_tempo_system_exploit, 'image_oeuvre_selectionnee.png'))
                    # Ecriture du chemin (fichier texte) dans le rep. tempo
                    ecr_selection = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_selection_oeuvre_ds_scene_murexpo_", "data_selection_oeuvre_ds_scene_murexpo", ".txt", parc[2])
                    
                    # --------------------------------------------------------------
                    # On vérifie si l'utilisateur a modifié la taille de l'image
                    # (dans l'onglet Configuration > Traitement intermédiaire)
                    # --------------------------------------------------------------
                    # On liste le contenu du répertoire temporaire ...
                    liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
                    # Comprehension-liste pour récupérer (ou non) le fichier
                    # data_taille_image_traitement_intermediaire du rep. tempo
                    # --> si le fichier a été crée
                    liste_presence_fichier_taille_img = [contenu for contenu in liste_contenu_tmp if 'data_taille_image_traitement_intermediaire' in contenu]
                    # Si le fichier existe (la liste contiendra 1 élément) ...
                    if len(liste_presence_fichier_taille_img) > 0 :
                        # Utilisation de la classe Tempo_lecture pour la lecture
                        # des données dans le répertoire (dit) temporaire
                        rep_tempo_lecture_taille = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_taille_image_traitement_intermediaire_', 'data_taille_image_traitement_intermediaire')
                        # ... avec appel de la méthode affichage
                        rep_tempo_aff_taille = rep_tempo_lecture_taille.affichage()[0]
                        # On récupère la valeur de la taille de l'image ...
                        taille_image = int(rep_tempo_aff_taille)
                        # ==============================================================
                        # Adaptation de la taille de l'image pour le traitement
                        # interne de l'image : image_oeuvre_selectionnee.png
                        # ==============================================================
                        # Appel du module de redimensionnement ... pour afficher les 
                        # images d'oeuvres plus rapidement (fonction avec return)
                        # --> pas besoin de return ici
                        redimensionnement_image_oeuvre(parc[2], taille_image, os.path.join(self.chemin_tempo_system_exploit, 'image_oeuvre_selectionnee.png'))
                        # ==============================================================
                        
                        # ---------------------------------------------------------------
                        # Retour commandes ....
                        # ---------------------------------------------------------------
                        # Récupération des commandes pour le terminal interne de MurExpo
                        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Chemin image (oeuvre) sélectionné dans la vue : {}\n").format(parc[2]))
                        # Ecriture des commandes dans le fichier
                        # tempo data_retour_commandes_murexpo
                        self.capter_commandes_murexpo.liste_ecrire_commandes()
                        # ---------------------------------------------------------------
                        
                    # Autrement si le fichier n'existe pas ...
                    elif len(liste_presence_fichier_taille_img) == 0 :
                        # ... une valeur par défaut est utilisée
                        taille_image = 950
                        # ==============================================================
                        # Adaptation de la taille de l'image pour le traitement
                        # interne de l'image : image_oeuvre_selectionnee.png
                        # ==============================================================
                        # Appel du module de redimensionnement ... pour afficher les 
                        # images d'oeuvres plus rapidement (fonction avec return)
                        # --> pas besoin de return ici
                        redimensionnement_image_oeuvre(parc[2], taille_image, os.path.join(self.chemin_tempo_system_exploit, 'image_oeuvre_selectionnee.png'))
                        # ==============================================================
                        
                        # ---------------------------------------------------------------
                        # Retour commandes ....
                        # ---------------------------------------------------------------
                        # Récupération des commandes pour le terminal interne de MurExpo
                        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Chemin image (oeuvre) sélectionné dans la vue : {}\n").format(parc[2]))
                        # Ecriture des commandes dans le fichier
                        # tempo data_retour_commandes_murexpo
                        self.capter_commandes_murexpo.liste_ecrire_commandes()
                        # ---------------------------------------------------------------
                    
                    # Ecriture du chemin (fichier texte) dans le rep. tempo
                    # --------------------------------------------------
                    # Ecriture de l'index dans un fichier (afin de 
                    # pouvoir sélectionner la bonne occurence pour
                    # l'image pour la gestion du reflet [le 
                    # morcellement/découpe de l'image du mur]). Suite
                    # de la procédure dans le module traitement_
                    # image_oeuvre (traitement avec Pillow.)
                    # --------------------------------------------------
                    ecr_index = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_selection_index_oeuvre_ds_scene_murexpo_", "data_selection_index_oeuvre_ds_scene_murexpo", ".txt", parc[0])

            # *******************************************************
            # Attention à ce stade (quand aucun traitement des 
            # images : Cadre noir classique, Sous verre avec
            # bordure, Toile peinte, ... n'a été lancé) l'index
            # des images est bien géré ... mais dès qu'un 
            # traitement de l'image sélectionnée est activé
            # (avec le bouton Appliquer ... et dès que 
            # l'utilisateur aura sélectionné l'image résultante),
            # la liste self.liste_fondu_images_et_pixmap sera en 
            # surcharge (les éléments avant le traitement Cadre 
            # noir classique, Sous verre, bordure, Toile 
            # peinte, ... + les éléments après le traitement),
            # la liste liste self.liste_fondu_images_et_pixmap
            # aura donc doublé de volume ... donc juste en 
            # dessous (Sélection des bons index ...) on va
            # faire en sorte de retouver le bon index de 
            # l'image sélectionnée a la fin du traitement (Cadre
            # noir classique, Sous verre avec bordure, Toile peinte,
            # ... ). Ceci pour le traitement des reflets sur les 
            # oeuvres.
            # *******************************************************
    
            # -----------------------------------------------------
            # Sélection des bons index après avoir opéré les
            # traitements Cadre noir classique, Sous verre avec
            # bordure, Toile peinte, ...
            # -----------------------------------------------------
            # Après avoir cliqué sur le bouton Appliquer et 
            # avoir sélectionné le widget de l'oeuvre concernée, 
            # le Drapeau pour "Cadre noir classique" est passé 
            # à la valeur 1. Cette condition vérifie si ...
            if self.drapeau_cadre_noir_classique == 1 :
                # La liste est re-scannée ...
                for p in self.liste_fondu_images_et_pixmap :
                    # On compare le widget présent ds la 
                    # liste et le widget sélectionné
                    if p[1] == data :
                        # On ré-ecrit l'index dans le fichier temporaire ...
                        # Attention la valeur de l'index sera l'index 
                        # divisé par 2 (pour les raisons expliquées plus haut)
                        index = int(int(p[0]) / 2)
                        ecr_index = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_selection_index_oeuvre_ds_scene_murexpo_", "data_selection_index_oeuvre_ds_scene_murexpo", ".txt", str(index))
                        print("")
                        print(_('INDEX Drapeau self.drapeau_cadre_noir_classique RECALCULE :'), index)
                        print("")
            # Ici le drapeau "Sous verre avec bordure" est
            # passé à la valeur 1 ...
            if self.drapeau_sous_verre_avec_bordure == 1 :
                for p in self.liste_fondu_images_et_pixmap :
                    if p[1] == data :
                        index = int(int(p[0]) / 2)
                        ecr_index = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_selection_index_oeuvre_ds_scene_murexpo_", "data_selection_index_oeuvre_ds_scene_murexpo", ".txt", str(index))
                        print("")
                        print(_('INDEX Drapeau self.drapeau_sous_verre_avec_bordure RECALCULE :'), index)
                        print("")
        except : pass
        
        
    def traitement_de_l_oeuvre(self) :
        """ Fonction pour le lancement du traitement 
        des oeuvres (par exemple : mise en place d'un 
        cadre noir classique, sous verre, bordure, 
        d'une mise en toile (entoilage), ... de l'oeuvre) """
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action/Traitement) Appliquer les changements sur l'oeuvre sélectionnée\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        # L'item est d'abord supprimé
        # (indispensable)
        for item in self.scene.selectedItems() :
            
            self.scene.removeItem(item)
            del item
           
        try :
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            # ------------------------------------------------------
            # Lecture du data fichier (rep. temporaire) ...
            # ------------------------------------------------------
            # RECUPERATION DU CHOIX DU MEDIA SELECTIONNE PAR 
            # L'UTILISATEUR (Cadre noir classique, Sous verre avec 
            # bordure, Toile peinte, ...)
            pre_lecture_choix_media = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_choix_media_oeuvre_murexpo_', 'data_choix_media_oeuvre_murexpo')
            # ... avec appel de la méthode affichage
            media = "".join(pre_lecture_choix_media.affichage())
        except : pass
        
        # Appel de la classe Traitement_Image_Oeuvre
        traitement_oeuvre = Traitement_Image_Oeuvre()
        
        # ----------------------------------------------
        # Application du traitement suivant le choix
        # de l'utilisateur (appel des méthodes de 
        # classe correspondantes)
        # ----------------------------------------------
        if media == _('Photo brute') : pass
        if media == _('Cadre noir classique') :
            # Appel de la méthode cadre_noir_classique
            # de la classe Traitement_Image_Oeuvre
            traitement_oeuvre.cadre_noir_classique()
        if media == _('Sous verre, bordure') :
            # Appel de la méthode sous_verre_avec_bordure
            # de la classe Traitement_Image_Oeuvre
            traitement_oeuvre.sous_verre_avec_bordure()
        if media == _('Toile peinte') : pass
        
        # ---------------------------------------------------------------------
        # Récupération de l'image traitée dans le rép. tempo (avec Pillow)
        # Il s'agit de l'image qui a été sélectionnée par l'utilisateur
        image = os.path.join(self.chemin_tempo_system_exploit, 'image_oeuvre_selectionnee.png')
        # ---------------------------------------------------------------------

        # Appel de la fonction widget_image_oeuvre -->
        # fonction qui sert à insérer chaque item (chaque
        # image chargée par l'utilisateur) dans la scène.
        # Gestion du pixmap et son redimensionnement, appel 
        # de la classe dérivée MonGraphiquePixmapItemOeuvre
        # (avec comme argument le pixmap), gestion de l'ombrage
        # ... et affichage de la scène
        self.widget_image_oeuvre(image)
        
        # ---------------------------------------------------
        # Drapeaux
        # ---------------------------------------------------
        # Condition essentielle pour le changement des 
        # valeurs des Drapeaux (important !)
        # ---------------------------------------------------
        # Si l'utilisateur a fait le choix Cadre noir classique ...
        if media == _('Cadre noir classique') :
            # Drapeau pour l'encadrement classique de l'oeuvre 
            # (traitement image) ici maintenant positionné à 1
            # ... car l'utilisateur a appliqué le traitement
            # (il a cliqué sur la bouton Appliquer et pour finir 
            # sélectionné l'oeuvre en question)
            self.drapeau_cadre_noir_classique = 1
        # Si l'utilisateur a fait le choix Cadre noir classique ...
        if media == _('Sous verre, bordure') :
            # Drapeau pour la mise en place d'un sous verre avec 
            # bordure (traitement image) ici maintenant positionné
            # à 1
            self.drapeau_sous_verre_avec_bordure = 1
            
            
    def fonction_panneau(self) :
        """ Fonction de gestion du panneau/cartel """
        
        # Dé-sélectionne les images
        # des oeuvres (les pointillés)
        self.scene.clearSelection()
        
        # ****************************************
        # Drapeau pour vérifier si l'utilisateur
        # a modifié les couleurs du panneau/
        # cartel dans l'onglet Choix couleur
        # (suite du traitement dans l'exception
        # en dessous ... et à la suite dans le
        # Bloc de traitement des panneaux et 
        # des testes associés ... plus bas)
        # ----------------------------------------
        # La valeur est d'abord positionnée à 1
        drapeau_choix_couleur_panneau = 1
        # ****************************************
        
        # -----------------------------------------------------------------
        # L'item est d'abord supprimé (indispensable) par exemple
        # lors d'une mise à jour d'un panneau/cartel
        # -----------------------------------------------------------------
        # Attention tout de même (!!!) : 
        # --> pour mettre à jour un panneau il faut d'abord cliquer
        # sur le bouton "Panneau" (le panneau est crée), vérifier 
        # que le panneau (précédemment crée) est bien sélectionné 
        # et cliquer (de nouveau) sur le bouton "Panneau"
        # --> pour créer un nouveau panneau , il faut d'abord vérifier 
        # que le panneau (précédemment crée) n'est pas sélectionné
        # ... et cliquer sur le bouton "Panneau"
        # -----------------------------------------------------------------
        for item in self.scene.selectedItems() :
            self.scene.removeItem(item)
            del item
        # -----------------------------------------------------------------
        
        # ------------------------------------------------------
        # --> Lexture du type de panneau/cartel. 
        # ------------------------------------------------------
        # Ceci est fait en dehors de l'exception juste 
        # en dessous car le fichier temporaire 
        # 'data_choix_type_de_panneau_cartel_murexpo' est 
        # crée à la base (avec la valeur "Panneau format 
        # carré, coins arrondis" ... et sans que 
        # l'utilisateur ait besoin de sélectionner un autre 
        # type de panneau/cartel dans la combo
        # ------------------------------------------------------
        pre_lecture_type_de_panneau = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_type_de_panneau_cartel_murexpo_", "data_choix_type_de_panneau_cartel_murexpo")
        # ... avec appel de la méthode affichage
        lecture_type_de_panneau = "".join(pre_lecture_type_de_panneau.affichage())
        type_de_panneau = lecture_type_de_panneau
        # ------------------------------------------------------
        
        # ------------------------------------------------------
        # --> Lexture de la largeur et de la hauteur des 
        # dimensions libres du panneau/cartel.
        # ------------------------------------------------------
        # Si l'utilisateur choisit l'item "Panneau coins 
        # arrondis avec redimensionnement libre" ou l'item 
        # "Panneau coins carrés avec redimensionnement libre"
        # dans la combo du type de panneau/cartel (et 
        # seulement ces sélections !)
        # ------------------------------------------------------
        # Ceci est fait en dehors de l'exception juste 
        # en dessous car les fichiers temporaires 
        # 'data_largeur_redim_libre_panneau_cartel_murexpo' et
        # 'data_hauteur_redim_libre_panneau_cartel_murexpo' 
        # sont crées à la base.
        # ------------------------------------------------------
        #
        pre_lecture_largeur_redim_libre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_redim_libre_panneau_cartel_murexpo_", "data_largeur_redim_libre_panneau_cartel_murexpo")
        # ... avec appel de la méthode affichage
        lecture_largeur_redim_libre = "".join(pre_lecture_largeur_redim_libre.affichage())
        largeur_redim_libre = int(lecture_largeur_redim_libre)
        #
        pre_lecture_hauteur_redim_libre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_hauteur_redim_libre_panneau_cartel_murexpo_", "data_hauteur_redim_libre_panneau_cartel_murexpo")
        # ... avec appel de la méthode affichage
        lecture_hauteur_redim_libre = "".join(pre_lecture_hauteur_redim_libre.affichage())
        hauteur_redim_libre = int(lecture_hauteur_redim_libre)
        # ------------------------------------------------------
        
        try :
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            # ------------------------------------------------------
            # --> Lecture de la couleur du panneau/cartel
            # ------------------------------------------------------
            pre_lecture_couleur_panneau = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_couleur_panneau_cartel_murexpo_", "data_couleur_panneau_cartel_murexpo")
            # ... avec appel de la méthode affichage
            lecture_couleur_panneau = "".join(pre_lecture_couleur_panneau.affichage())
            # Récupération des données, par exemple
            # ['120', '136', '145' '224']
            couleur_panneau = lecture_couleur_panneau.split('/')
            rouge = int(couleur_panneau[0])
            vert = int(couleur_panneau[1])
            bleu = int(couleur_panneau[2])
            canal_alpha = int(couleur_panneau[3])
            # **********************************
            # Le drapeau du choix de la couleur
            # du panneau prend ici la valeur 1
            drapeau_choix_couleur_panneau = 1
            # ------------------------------------------------------
            # --> Lecture de la largeur du contour du panneau/cartel
            # ------------------------------------------------------
            pre_lecture_largeur_contour_panneau = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_contour_panneau_cartel_murexpo_", "data_largeur_contour_panneau_cartel_murexpo")
            # ... avec appel de la méthode affichage
            lecture_largeur_contour_panneau = "".join(pre_lecture_largeur_contour_panneau.affichage())
            largeur_contour_panneau = int(lecture_largeur_contour_panneau)
            # ------------------------------------------------------
            # --> Lecture de l'arrondi du panneau/cartel (en
            # cas de sélection coins arrondis uniquement)
            # ------------------------------------------------------
            pre_lecture_arrondi_panneau = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_arrondi_panneau_cartel_murexpo_", "data_arrondi_panneau_cartel_murexpo")
            # ... avec appel de la méthode affichage
            lecture_arrondi_panneau = "".join(pre_lecture_arrondi_panneau.affichage())
            arrondi_panneau = int(lecture_arrondi_panneau)
            # ------------------------------------------------------
            # --> Lecture de la valeur de l'ombrage (si
            # l'utilisateur a coché la case, cela veut dire
            # oui je veux utiliser l'ombrage, donc la valeur
            # retournée sera 1, dans le cas contraire ce sera 0)
            # ------------------------------------------------------
            pre_lecture_ombrage_panneau_ou_texte = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_panneau_ou_texte_murexpo_", "data_ombrage_panneau_ou_texte_murexpo")
            # ... avec appel de la méthode affichage
            lecture_ombrage_panneau_ou_texte = "".join(pre_lecture_ombrage_panneau_ou_texte.affichage())
            # L'élément est converti en int mais aurait pû rester en string
            ombrage_panneau_ou_texte = int(lecture_ombrage_panneau_ou_texte)
            # ------------------------------------------------------
        except :
            # ****************************************
            # Si l'utilisateur n'a pas sélectionné la
            # couleur du panneau dans l'onglet Choix
            # couleur, le drapeau prend la valeur 0 
            # (voir en dessous pour la suite)
            drapeau_choix_couleur_panneau = 0
            # ------------------------------------------------------
            # Le fichier data_largeur_contour_panneau_cartel_murexpo 
            # est préalablement écrit dans le répertoire temporaire 
            # (même si l'utilisateur n'a pas modifié la valeur du 
            # spinbox "Contour") ... donc on l'ouvre pour récupérer 
            # la largeur du contour
            pre_lecture_largeur_contour_panneau = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_largeur_contour_panneau_cartel_murexpo_", "data_largeur_contour_panneau_cartel_murexpo")
            # ... avec appel de la méthode affichage
            lecture_largeur_contour_panneau = "".join(pre_lecture_largeur_contour_panneau.affichage())
            largeur_contour_panneau = int(lecture_largeur_contour_panneau)
            # ------------------------------------------------------
            # Exception pour contrôler l'ouverture du fichier tempo
            # data_arrondi_panneau_cartel_murexpo .... pour
            # récupérer la valeur de l'arrondi (le fichier tempo
            # est crée uniquement quand l'utilisateur choisit une
            # des entrées correspondant aux arrondis dans la combo)
            try :
                # ------------------------------------------------------
                # --> Lecture de l'arrondi du panneau/cartel (en
                # cas de sélection coins arrondis uniquement)
                # ------------------------------------------------------
                pre_lecture_arrondi_panneau = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_arrondi_panneau_cartel_murexpo_", "data_arrondi_panneau_cartel_murexpo")
                # ... avec appel de la méthode affichage
                lecture_arrondi_panneau = "".join(pre_lecture_arrondi_panneau.affichage())
                arrondi_panneau = int(lecture_arrondi_panneau)
            except :
                # Si le fichier tempo n'existe pas 
                # une valeur par défaut est affectée
                arrondi_panneau = 20
            # ------------------------------------------------------
            # Exception pour contrôler l'ouverture du fichier tempo
            # data_ombrage_panneau_ou_texte_murexpo ... pour
            # récupérer la valeur de l'ombre portée (ou pas) ... le
            # fichier tempo est crée uniquement quand l'utilisateur
            # coche la case de l'ombrage
            try :
                # ------------------------------------------------------
                # --> Lecture de la valeur de l'ombrage (si
                # l'utilisateur a coché la case, cela veut dire
                # oui je veux utiliser l'ombrage, donc la valeur
                # retournée sera 1, dans le cas contraire ce sera 0)
                # ------------------------------------------------------
                pre_lecture_ombrage_panneau_ou_texte = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_panneau_ou_texte_murexpo_", "data_ombrage_panneau_ou_texte_murexpo")
                # ... avec appel de la méthode affichage
                lecture_ombrage_panneau_ou_texte = "".join(pre_lecture_ombrage_panneau_ou_texte.affichage())
                # L'élément est converti en int mais aurait pû rester en string
                ombrage_panneau_ou_texte = int(lecture_ombrage_panneau_ou_texte)
            except :
                # ------------------------------------------------------
                # Si l'utilisateur n'a pas coché la case de l'ombrage
                ombrage_panneau_ou_texte = 0
                # ------------------------------------------------------
            
        # ----------------------------------------------------------
        # Bloc de traitement des panneaux et des testes associés
        # ----------------------------------------------------------
        # Si le drapeau est positionné à 1, 
        # cela veut dire que l'utilisateur
        # a modifié les couleurs dans l'onglet
        # Choix couleur ...
        # ****************************************
        if drapeau_choix_couleur_panneau == 1 :
            # Si l'utilisateur a sélectionné (dans la combo) un item de type coins arrondis ...
            if type_de_panneau in [_("Panneau format carré, coins arrondis"), \
                                   _("Panneau format paysage, coins arrondis"), \
                                   _("Panneau format portrait, coins arrondis"), \
                                   _("Panneau panoramique, coins arrondis")] :
                # Les dimensions (largeur et hauteur) sont redéfinies (en fonction du type d'item choisi)
                if type_de_panneau == _("Panneau format carré, coins arrondis" ): largeur, hauteur = 400, 400
                elif type_de_panneau == _("Panneau format paysage, coins arrondis") : largeur, hauteur = 400, 260
                elif type_de_panneau == _("Panneau format portrait, coins arrondis") : largeur, hauteur = 260, 400
                elif type_de_panneau == _("Panneau panoramique, coins arrondis") : largeur, hauteur = 400, 90
                # Le panneau est généré ici avec les bonnes options ...
                #contour = QPen(Qt.black, largeur_contour_panneau)
                contour = QPen(QColor(rouge, vert, bleu, canal_alpha), largeur_contour_panneau)
                chemin = QPainterPath()
                chemin.addRoundedRect(QRectF(10, 10, largeur, hauteur), arrondi_panneau, arrondi_panneau)
                rectangle = self.scene.addPath(chemin, pen = contour, brush = QBrush(QColor(rouge, vert, bleu, canal_alpha)))
            # Autrement si l'utilisateur a sélectionné (dans la combo) un item de type coins carrés ...
            elif type_de_panneau in [_("Panneau format carré, coins carrés"), \
                                     _("Panneau format paysage, coins carrés"), \
                                     _("Panneau format portrait, coins carrés"), \
                                     _("Panneau panoramique, coins carrés")] :
                # Les dimensions (largeur et hauteur) sont redéfinies (en fonction du type d'item choisi)
                if type_de_panneau == _("Panneau format carré, coins carrés") : largeur, hauteur = 400, 400
                elif type_de_panneau == _("Panneau format paysage, coins carrés") : largeur, hauteur = 400, 260
                elif type_de_panneau == _("Panneau format portrait, coins carrés") : largeur, hauteur = 260, 400
                elif type_de_panneau == _("Panneau panoramique, coins carrés") : largeur, hauteur = 400, 90
                # Le panneau est généré ici avec les bonnes options ...
                #contour = QPen(Qt.black, largeur_contour_panneau)
                contour = QPen(QColor(rouge, vert, bleu, canal_alpha), largeur_contour_panneau)
                chemin = QPainterPath()
                chemin.addRect(QRectF(10, 10, largeur, hauteur))
                rectangle = self.scene.addPath(chemin, pen = contour, brush = QBrush(QColor(rouge, vert, bleu, canal_alpha)))
            # Autrement si l'utilisateur a sélectionné (dans la combo) l'item "Panneau coins arrondis avec redimensionnement libre" ...
            elif type_de_panneau in [_("Panneau coins arrondis avec redimensionnement libre")] :
                # Le panneau est généré ici avec les bonnes options ...
                contour = QPen(QColor(rouge, vert, bleu, canal_alpha), largeur_contour_panneau)
                chemin = QPainterPath()
                chemin.addRoundedRect(QRectF(10, 10, largeur_redim_libre, hauteur_redim_libre), arrondi_panneau, arrondi_panneau)
                rectangle = self.scene.addPath(chemin, pen = contour, brush = QBrush(QColor(rouge, vert, bleu, canal_alpha)))
            # Autrement si l'utilisateur a sélectionné (dans la combo) l'item "Panneau coins carrés avec redimensionnement libre" ...
            elif type_de_panneau in [_("Panneau coins carrés avec redimensionnement libre")] :
                # Le panneau est généré ici avec les bonnes options ...
                contour = QPen(QColor(rouge, vert, bleu, canal_alpha), largeur_contour_panneau)
                chemin = QPainterPath()
                chemin.addRect(QRectF(10, 10, largeur_redim_libre, hauteur_redim_libre))
                rectangle = self.scene.addPath(chemin, pen = contour, brush = QBrush(QColor(rouge, vert, bleu, canal_alpha)))
        # ****************************************
        # Si le drapeau est positionné à 0, 
        # cela veut dire que l'utilisateur
        # n'a pas a modifié les couleurs dans 
        # l'onglet Choix couleur ...
        # ****************************************
        if drapeau_choix_couleur_panneau == 0 :
            # Attribution du panneau/cartel. Nous sommes ici dans le cas typique du 1er
            # du 1er démarrage de MurExpo (quand l'utilisateur n'a modifié aucune donnée)
            # Si l'utilisateur a sélectionné (dans la combo) un item de type coins arrondis ...
            if type_de_panneau in [_("Panneau format carré, coins arrondis"), \
                                   _("Panneau format paysage, coins arrondis"), \
                                   _("Panneau format portrait, coins arrondis"), \
                                   _("Panneau panoramique, coins arrondis")] :
                # Les dimensions (largeur et hauteur) sont redéfinies (en fonction du type d'item choisi)
                if type_de_panneau == _("Panneau format carré, coins arrondis") : largeur, hauteur = 400, 400
                elif type_de_panneau == _("Panneau format paysage, coins arrondis") : largeur, hauteur = 400, 260
                elif type_de_panneau == _("Panneau format portrait, coins arrondis") : largeur, hauteur = 260, 400
                elif type_de_panneau == _("Panneau panoramique, coins arrondis") : largeur, hauteur = 400, 90
                # Le panneau est généré ici avec les bonnes options ...
                #contour = QPen(Qt.black, largeur_contour_panneau)
                contour = QPen(QColor(120, 136, 145, 160), largeur_contour_panneau)
                chemin = QPainterPath()
                chemin.addRoundedRect(QRectF(10, 10, largeur, hauteur), arrondi_panneau, arrondi_panneau)
                rectangle = self.scene.addPath(chemin, pen = contour, brush = QBrush(QColor(120, 136, 145, 160)))
            # Autrement si l'utilisateur a sélectionné (dans la combo) un item de type coins carrés ...
            elif type_de_panneau in [_("Panneau format carré, coins carrés"), \
                                     _("Panneau format paysage, coins carrés"), \
                                     _("Panneau format portrait, coins carrés"), \
                                     _("Panneau panoramique, coins carrés")] :
                # Les dimensions (largeur et hauteur) sont redéfinies (en fonction du type d'item choisi)
                if type_de_panneau == _("Panneau format carré, coins carrés") : largeur, hauteur = 400, 400
                elif type_de_panneau == _("Panneau format paysage, coins carrés") : largeur, hauteur = 400, 260
                elif type_de_panneau == _("Panneau format portrait, coins carrés") : largeur, hauteur = 260, 400
                elif type_de_panneau == _("Panneau panoramique, coins carrés") : largeur, hauteur = 400, 90
                # Le panneau est généré ici avec les bonnes options ...
                #contour = QPen(Qt.black, largeur_contour_panneau)
                contour = QPen(QColor(120, 136, 145, 160), largeur_contour_panneau)
                chemin = QPainterPath()
                chemin.addRect(QRectF(10, 10, largeur, hauteur))
                rectangle = self.scene.addPath(chemin, pen = contour, brush = QBrush(QColor(120, 136, 145, 160)))
            # Autrement si l'utilisateur a sélectionné (dans la combo) l'item "Panneau coins arrondis avec redimensionnement libre" ...
            elif type_de_panneau in [_("Panneau coins arrondis avec redimensionnement libre")] :
                # Le panneau est généré ici avec les bonnes options ...
                contour = QPen(QColor(120, 136, 145, 160), largeur_contour_panneau)
                chemin = QPainterPath()
                chemin.addRoundedRect(QRectF(10, 10, largeur_redim_libre, hauteur_redim_libre), arrondi_panneau, arrondi_panneau)
                rectangle = self.scene.addPath(chemin, pen = contour, brush = QBrush(QColor(120, 136, 145, 160)))
            # Autrement si l'utilisateur a sélectionné (dans la combo) l'item "Panneau coins carrés avec redimensionnement libre" ...
            elif type_de_panneau in [_("Panneau coins carrés avec redimensionnement libre")] :
                # Le panneau est généré ici avec les bonnes options ...
                contour = QPen(QColor(120, 136, 145, 160), largeur_contour_panneau)
                chemin = QPainterPath()
                chemin.addRect(QRectF(10, 10, largeur_redim_libre, hauteur_redim_libre))
                rectangle = self.scene.addPath(chemin, pen = contour, brush = QBrush(QColor(120, 136, 145, 160)))
        # -----------------------------------------------------------------------------------------------------------
        
        # -------------------------------------------------------------------
        # Appel de la fonction de l'ombrage (si la case est sélectionnée)
        # -------------------------------------------------------------------
        try :
            # Appel de la fonction de l'ombrage ...
            if ombrage_panneau_ou_texte == 1 : self.ombrage_oeuvre(rectangle)
            elif ombrage_panneau_ou_texte == 0 : pass
        except : pass
        # -------------------------------------------------------------------        

        # Positionnement du panneau ...
        rectangle.setPos(12, 12)
        
        # Ajout de l'item (panneau) à la scène
        self.scene.addItem(rectangle)

        # Permet la mise à jour
        QApplication.processEvents()
        
        # L'item rectangle (le panneau généré dans la
        # scène) est sélectionnable et peut être déplacé
        rectangle.setFlag(QGraphicsItem.ItemIsMovable)
        rectangle.setFlag(QGraphicsItem.ItemIsSelectable)
   
            
    def fonction_texte(self) :
        """ Fonction de gestion du texte (pour les panneaux/cartels) """
        
        # Dé-sélectionne les images
        # des oeuvres (les pointillés)
        self.scene.clearSelection()
        
        # -----------------------------------------------------------------
        # L'item est d'abord supprimé (indispensable) par exemple
        # lors d'une mise à jour du texte
        # -----------------------------------------------------------------
        # Attention tout de même (!!!) : 
        # --> pour mettre à jour un texte il faut d'abord écrire le
        # texte dans la zone de texte (le QTextEdit), ensuite cliquer 
        # sur le bouton "Texte", modifier le texte précédemment écrit
        # dans la zone de texte et cliquer de nouveau sur le bouton
        # "Texte", ... le texte en question sera bien modifié (!)
        # --> pour créer un nouveau texte (à la suite d'une mise à
        # jour ou pas ...) il faut écrire le nouveau texte (dans la 
        # zone de texte ... et en ayant effacé l'ancien texte !),
        # vérifier que l'ancienne sélection n'est pas sélectionnée (!!!)
        # et pour finir cliquer sur le bouton "Texte"
        # -----------------------------------------------------------------
        for item in self.scene.selectedItems() :
            self.scene.removeItem(item)
            del item
        # -----------------------------------------------------------------
        
        try :
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            # ------------------------------------------------------
            # --> Lecture du type de la police (sélectionnée
            # par l'utilisateur dans l'onglet Choix police)
            pre_lecture_type_police = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_police_de_caractere_choisie_murexpo_", "data_police_de_caractere_choisie_murexpo")
            # ... avec appel de la méthode affichage
            lecture_type_police = "".join(pre_lecture_type_police.affichage())
            # Récupération des données, par exemple
            # ['aakar', 'medium', '48']
            type_police = lecture_type_police.split(',')
            # ------------------------------------------------------
            # --> Lecture de la valeur de l'ombrage (si
            # l'utilisateur a coché la case, cela veut dire
            # oui je veux utiliser l'ombrage, donc la valeur
            # retournée sera 1, dans le cas contraire ce sera 0)
            # ------------------------------------------------------
            pre_lecture_ombrage_panneau_ou_texte = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_panneau_ou_texte_murexpo_", "data_ombrage_panneau_ou_texte_murexpo")
            # ... avec appel de la méthode affichage
            lecture_ombrage_panneau_ou_texte = "".join(pre_lecture_ombrage_panneau_ou_texte.affichage())
            # L'élément est converti en int mais aurait pû rester en string
            ombrage_panneau_ou_texte = int(lecture_ombrage_panneau_ou_texte)
            # ------------------------------------------------------
        except :
            # ------------------------------------------------------
            # Si l'utilisateur n'a pas 
            # sélectionné de police dans
            # l'onglet Choix police, ce 
            # sera la police par défaut
            # qui sera utilisée
            type_police = ['Liberation Sans', 12]
            # ------------------------------------------------------
            # Si l'utilisateur n'a pas coché la case de l'ombrage
            ombrage_panneau_ou_texte = 0
            # ------------------------------------------------------
            
        try :
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            # ------------------------------------------------------
            # --> Lecture du texte écrit par l'utilisateur 
            # dans le QTextEdit ...
            pre_lecture_texte_utilisateur = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_contenu_texte_utilisateur_panneau_cartel_murexpo_", "data_contenu_texte_utilisateur_panneau_cartel_murexpo")
            # ... avec appel de la méthode affichage
            lecture_texte_utilisateur = "".join(pre_lecture_texte_utilisateur.affichage())
        except :
            # Si l'utilisateur n'a pas entré de texte dans la zone 
            # de texte, un texte par défaut fait office de texte
            lecture_texte_utilisateur = _("Vous devez écrire un texte dans la zone de texte !!!")
  
        try :
            # Attribution du texte entré (dans le QTextEdit) par l'utilisateur
            # et utilisation des élements de la police (la police elle-même et
            # sa taille) ... pour créer l'item
            self.texte = self.scene.addText(lecture_texte_utilisateur, QFont(type_police[0], int(type_police[2])))
        except :
            # Attribution du texte entré (dans le QTextEdit) par l'utilisateur
            # et utilisation d'une fonte par défaut. Nous sommes ici dans le
            # cas typique du 1er démarrage de MurExpo (quand l'utilisateur
            # n'a modifié aucune donnée quand à la configuration de la police)
            self.texte = self.scene.addText(lecture_texte_utilisateur, QFont(type_police[0], int(type_police[1]))) #QFont())
            
        # -------------------------------------------------------------------
        # Appel de la fonction de l'ombrage (si la case est sélectionnée)
        # -------------------------------------------------------------------
        try :
            # Appel de la fonction de l'ombrage ...
            if ombrage_panneau_ou_texte == 1 : self.ombrage_oeuvre(self.texte)
            elif ombrage_panneau_ou_texte == 0 : pass
        except : pass
        # -------------------------------------------------------------------

        # Positionnement du texte ...
        self.texte.setPos(12, 12)
        
        # --------------------------
        # Mise en couleur du texte
        # --------------------------
        try :
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            # ------------------------------------------------------
            # Si l'utilisateur a sélectionné une couleur 
            # pour le texte dans l'onglet Choix couleur ...
            pre_lecture_couleur_police = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_couleur_police_de_caractere_murexpo_", "data_couleur_police_de_caractere_murexpo")
            # ... avec appel de la méthode affichage
            lecture_couleur_police = "".join(pre_lecture_couleur_police.affichage())
            # Récupération des données, par exemple
            # ['aakar', 'medium', '48']
            couleur_police = lecture_couleur_police.split('/')
            # Application de la couleur du texte (défini par 
            # l'utilisateur) dans l'onglet Choix couleur
            self.texte.setDefaultTextColor(QColor(int(couleur_police[0]), int(couleur_police[1]), int(couleur_police[2]), int(couleur_police[3])))
        # Autrement ...
        except :
            self.texte.setDefaultTextColor(QColor(0, 0, 0, 255))
        
        # Ajout de l'item texte à la scène
        self.scene.addItem(self.texte)
        
        # Permet la mise à jour
        QApplication.processEvents()
        
        # L'item texte (le panneau généré dans la scène)
        # est sélectionnable et peut être déplacé
        self.texte.setFlag(QGraphicsItem.ItemIsMovable)
        self.texte.setFlag(QGraphicsItem.ItemIsSelectable)
    
    
    def fonction_ajout_image_alpha_map(self) :
        """ Fonction servant à insérer l'item (l'alpha map 
        sélectionné dans la combo) dans la scène. Gestion 
        du pixmap et son redimensionnement, appel de la 
        classe dérivée MonGraphiquePixmapItemOeuvre (avec 
        comme argument le pixmap) et affichage de la scène """
        
        # ------------------------------------------------------
        # --> Lecture de la valeur de l'ombrage (si
        # l'utilisateur a coché la case, cela veut dire
        # oui je veux utiliser l'ombrage, donc la valeur
        # retournée sera 1, dans le cas contraire ce sera 0)
        # ------------------------------------------------------
        pre_lecture_ombrage_image_alpha_map = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_ombrage_image_alpha_map_murexpo_", "data_ombrage_image_alpha_map_murexpo")
        # ... avec appel de la méthode affichage
        lecture_ombrage_image_alpha_map = "".join(pre_lecture_ombrage_image_alpha_map.affichage())
        # L'élément est converti en int mais aurait pû rester en string
        ombrage_image_alpha_map = int(lecture_ombrage_image_alpha_map)       
        
        # Le pixmap récupère l'image qui se trouve dans le répertoire temporaire
        # Cette image 'image_ajout_perso_cadre_ou_accessoire.png' a préalablement
        # été sauvée dans le rep. tempo au moment de sa sélection dans la combo
        pixmap = QPixmap(os.path.join(self.chemin_tempo_system_exploit, 'image_ajout_perso_cadre_ou_accessoire.png'))
        
        # Redimensionnement du pixmap ... pour affichage immédiat dans la vue (largeur = 400 pixels)
        pixmap = pixmap.scaled(400, 400, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        
        # Insère l'image dans le QGraphicsPixmapItem (pour 
        # le mur d'exposition) .--> Appel de la classe 
        # MonGraphiquePixmapItemOeuvre avec comme argument 
        # le pixmap
        self.item_pixmap = MonGraphiquePixmapItemOeuvre(pixmap)
        
        # Amélioration de la qualité
        self.item_pixmap.setTransformationMode(Qt.SmoothTransformation)
        
        #
        self.item_pixmap.setFlag(QGraphicsItem.ItemIsFocusable)
        
        # Placement aux coordonnées x = 10 et y = 10
        self.item_pixmap.setPos(10, 10)
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) L'image en alpha map est à l'origine placée aux coordonnées 10 (en x), 10 (en y), dans la vue\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        # -------------------------------------------------------------------
        # Appel de la fonction de l'ombrage (si la case est sélectionnée)
        # -------------------------------------------------------------------
        try :
            # Appel de la fonction de l'ombrage ...
            if ombrage_image_alpha_map == 1 : self.ombrage_oeuvre(self.item_pixmap)
            elif ombrage_image_alpha_map == 0 : pass
        except : pass
        # -------------------------------------------------------------------
        
        # Insère QGraphicsPixmapItem dans la scène
        self.scene.addItem(self.item_pixmap)

        # Met en place le QGrphicsView dans la scène
        self.setScene(self.scene)

        # Affichage de la vue
        self.show()
        
    
    def supprimer_image_oeuvre(self) :
        """ Fonction pour supprimer une image sélectionnée.
        Merci à musicamante de m'avoir aidé, voir ici :
        https://stackoverflow.com/questions/65051515/
        delete-a-selected-qgraphicspixmapitem-in-a-scene """
        
        for item in self.scene.selectedItems() :
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Suppression de l'item sélectionné, c'est à dire {} de la vue\n").format(item))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
            # Supression de l'item
            self.scene.removeItem(item)
            
            # L'item supprimé est inséré dans la liste ; liste 
            # self.item_oeuvre_supprime (au départ vide) 
            # positionnée dans l'init de la classe MonGraphiqueVueMur
            self.item_oeuvre_supprime.append(str(item))
            
            # Suite supression
            del item
            
        # ---------------------------------------------------------
        # Exception pour éviter l'erreur : 
        # Traceback (most recent call last):
        # File "Chemin...\fabrique_murexpo_gui.py", line 6406, in supprimer_image_oeuvre
        # for element in self.liste_fondu_images_et_pixmap :
        # AttributeError: 'MonGraphiqueVueMur' object has no attribute 'liste_fondu_images_et_pixmap'
        # dans le cas suivant : 
        # 1) je démarre MurExpo, j'ajoute un mur (page 1)
        # 2) je passe les pages 2 et 3, je passe directement à la page 4.
        # 3) je rajoute un personnage (ou un autre objet) ... et ds la foulée
        # je le supprime ... avec l'exception, plus d'erreur
        # ---------------------------------------------------------    
        try :
            # ---------------------------------------------------------
            # Partie de code servant à faire le liens entre l'item 
            # supprimé et son chemin (le chemin de l'item supprimé)
            # ---------------------------------------------------------
            # La liste self.liste_fondu_images_et_pixmap est définie
            # et utilisée dans la fonction : pre_selection_oeuvre
            # --> elle contient toutes les images chargées sur le mur
            # ---------------------------------------------------------
            liste_oeuvres_si_supression = []
            # Boucle pour identifier les éléments supprimés
            for element in self.liste_fondu_images_et_pixmap :
                # Si l'élément à l'indice 1 est dans la liste ...
             if element[1] in self.item_oeuvre_supprime :
                  # On récupère (uniquement) les chemins
                   # des images qui ont été supprimés
                   liste_oeuvres_si_supression.append(element[2])
        except : pass

        # ---------------------------------------------------------------
        # Retour de toutes les images supprimées au moment de 
        # la création de l'exposition
        # ---------------------------------------------------------------
        # Boucle pour écrire (dans le fichier tempo) les 
        # chemins des images supprimées du mur d'expo
        for chemins_supp in liste_oeuvres_si_supression :
            self.capture_oeuvres_supprimees.liste_capter_oeuvres_supprimees('{}\n'.format(chemins_supp))
        # -----------------
        # Ecriture des chemins dans le fichier tempo
        # data_capture_oeuvres_supprimees_murexpo
        # -----------------
        # --> supression automatique des doublons dans la classe
        # CapterToutesOeuvresSupprimees (capture_oeuvres_supprimees_murexpo.py)
        self.capture_oeuvres_supprimees.liste_ecrire_oeuvres_supprimees()
        # ---------------------------------------------------------------
     

    def ombrage_oeuvre(self, item_pixmap) :
        """ Fonction pour mettre en place l'ombrage
        de l'item pixmap. Fonction appelée dans la 
        fonction widget_image_oeuvre """
        
        # ------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture 
        # des données dans le répertoire (dit) temporaire
        # ------------------------------------------------------
        # Lecture du rayon de l'espompage
        pre_lecture_rayon_estompage = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_rayon_estompe_ombre_oeuvre_murexpo_', 'data_rayon_estompe_ombre_oeuvre_murexpo')
        # ... avec appel de la méthode affichage
        lecture_rayon_estompage = "".join(pre_lecture_rayon_estompage.affichage())
        # Lecture du décalage horizontal de l'ombre
        pre_lecture_decal_h_ombre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_decalage_horizontal_ombre_oeuvre_murexpo_', 'data_decalage_horizontal_ombre_oeuvre_murexpo')
        # ... avec appel de la méthode affichage
        lecture_decal_h_ombre = "".join(pre_lecture_decal_h_ombre.affichage())
        # Lecture du décalage vertical de l'ombre
        pre_lecture_decal_v_ombre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_decalage_vertical_ombre_oeuvre_murexpo_', 'data_decalage_vertical_ombre_oeuvre_murexpo')
        # ... avec appel de la méthode affichage
        lecture_decal_v_ombre = "".join(pre_lecture_decal_v_ombre.affichage())
        
        try :
            # -----------------------------------------------
            # Ajout d'une ombre portée à l'image
            # le widget contenant l'image) et ce 
            # avec QGraphicsDropShadowEffect
            shadow = QGraphicsDropShadowEffect()
            # Réglage du rayon de l'estompage de l'ombre
            shadow.setBlurRadius(int(lecture_rayon_estompage))
            # Décalage horizontal de l'ombre
            shadow.setXOffset(int(lecture_decal_h_ombre))
            # Décalage vertical de l'ombre
            shadow.setYOffset(int(lecture_decal_v_ombre))
            # Affectation
            item_pixmap.setGraphicsEffect(shadow)
            # -----------------------------------------------
        except :
            # -----------------------------------------------
            # Ajout d'une ombre portée à l'image
            # le widget contenant l'image) et ce 
            # avec QGraphicsDropShadowEffect
            shadow = QGraphicsDropShadowEffect()
            # Réglage du rayon de l'estompage de l'ombre
            shadow.setBlurRadius(13) # Rayon de l'estompage (blur). Par défaut réglé à 13
            # Décalage horizontal de l'ombre
            shadow.setXOffset(7) # Par défaut régélé à 7
            # Décalage vertical de l'ombre
            shadow.setYOffset(7) # Par défaut régélé à 7
            # Affectation
            item_pixmap.setGraphicsEffect(shadow)
            # -----------------------------------------------
        
            
    def relever_item(self) :
        """ Fonction pour faire passer l'image sélectionnée 
        au dessus des autres images (relever la sélection) """
        
        for item in self.scene.selectedItems() :
            # z est égal à 0.0 par défaut
            z = item.zValue()
            # On rajoute 1 pour relever ...
            item.setZValue(z+1)
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Item {} relevé dans la vue\n").format(item))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------


    def rabaisser_item(self) :
        """ Fonction pour faire passer l'image sélectionnée 
        en dessous des autres images (rabaisser la sélection) 
        Attention la sélection doit être relevée avant de 
        pouvoir être rabaissée """
        
        for item in self.scene.selectedItems() :
            # z est égal à 0.0 par défaut
            z = item.zValue()
            # Si z est égal ou 
            # au dessus de 1 
            if z >= 1 :
                # On enlève 1 à la 
                # valeur de z
                item.setZValue(z-1)
                
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Item {} rabaissé dans la vue\n").format(item))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
        
            
    def tourner_vers_la_droite(self) :
        """ Fonction pour faire tourner vers la droite
        la sélection. Tourne de 1° vers la droite.
        Syntaxe trouvée ici : https://stackoverflow.com/
        questions/30670137/how-to-rotate-selected-
        qgraphicsitems-around-the-selection-center """
        
        for item in self.scene.selectedItems() :
            item.setRotation(item.rotation()+1)
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Item {} tourné vers la droite d'une valeur de {} degrés dans la vue\n").format(item, int(item.rotation()+1)))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
	

    def tourner_vers_la_gauche(self) :
        """ Fonction pour faire tourner vers la gauche
        la sélection. Tourne de 1° vers la gauche.
        Syntaxe trouvée ici : https://stackoverflow.com/
        questions/30670137/how-to-rotate-selected-
        qgraphicsitems-around-the-selection-center """
        
        for item in self.scene.selectedItems() :
            item.setRotation(item.rotation()-1)
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Item {} tourné vers la gauche d'une valeur de {} degrés dans la vue\n").format(item, int(item.rotation()-1)))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
      
    
    def redimensionner_selection(self, valeur) :
        """ Fonction qui permet de redimensionner 
        l'image sélectionnée ... la valeur utilisée
        (définie par le slider Redimensionner la
        sélection) est du type float la métohode 
        setScale utilise des float """

        for item in self.scene.selectedItems() :
            # La valeur de redimensionnement 
            # doit être un float
            item.setScale(valeur/1000)
            self.update()
             
    # ****************************** Gestion des oeuvres ******************************
            
          
    # ------------------------------------------------ A garder pour l'instant ---------------
    def demi_perpective(self) : 
        """ """
        for item in self.scene.selectedItems() :
            matrix = QTransform()
            matrix.rotate(45, Qt.YAxis)
            item.setTransform(matrix)
    # ------------------------------------------------ A garder pour l'instant ---------------


    def repertoire_chemin_configuration(self, chem_tempo_1, chem_tempo_2) :
        ''' Fonction de mise en place du répertoire de chargement par défaut 
        des images (par ex ici pour la sauvegarde de l'exposition sous la
        forme d'une image), si modifs faites par l'utilisateur dans l'onglet
        Configuration > Chemins (chargement et sauvegarde) '''
        
        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
              
        # ----------
        # Si le chargement du répertoire a déjà été modifié
        # ... Boucle de vérification ...
        # ----------
        for contenu in liste_contenu_tmp :
            # Si le fichier tempo concerné est bien dedans ...
            if chem_tempo_1 in contenu :
                # ------------------------------------------------------- #
                # Utilisation de la classe Tempo_lecture pour la lecture
                # des données dans le répertoire (dit) temporaire
                # ------------------------------------------------------- #
                rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), chem_tempo_1, chem_tempo_2)
                # ... avec appel de la méthode affichage
                rep_tempo_chemin = rep_tempo_lecture.affichage()[0]
                # S'il y a bien une donnée récupérée ...
                # celle-ci est affectée
                if rep_tempo_chemin :
                    rep_defaut = rep_tempo_chemin
                    return rep_defaut       
            
            
    def sauvegarder_image_entiere_expo(self) :
        """ Fonction pour sauvegarder le mur (entier) 
        d'exposition ... et avec les oeuvres si elles
        sont déployées sur le mur """
        
        # --------------------------------------------------------------------
        # Problème réglé mais ... si une autre solution plus intéressante ...
        # --------------------------------------------------------------------
        # Voir ici éventuellement pour résoudre le problème de débordement ...
        # --> le hors-champ à l'enregistrement ...
        # https://stackoverflow.com/questions/53004835/how-to-render-a-part-of-qgraphicsscene-and-save-it-as-image-file-pyqt5
        # --------------------------------------------------------------------
        # Voir aussi :
        # https://stackoverflow.com/questions/8372045/why-are-png-images-rendered-from-a-qgraphicsscene-being-incorrectly-offset
        # --------------------------------------------------------------------

        # -----------------------------------------------------------------------
        # Solution trouvée ici : 
        # https://stackoverflow.com/questions/52760130/pyqt-5-qpainter-returns-false-while-rendering-qgraphicsscene-to-a-qimage
        # -----------------------------------------------------------------------
        # C'est ici la résolution de la scène (en l'occurrence la résolution 
        # de l'image du mur). Par exemple avec une image de mur de 1920 x 1080, 
        # voilà le résultat :
        # self.scene.sceneRect() PyQt5.QtCore.QRectF(-0.5, -0.5, 1920.5, 1080.5)
        # ... et avec une image de mur de 2716 x 1450, voilà le résultat :
        # self.scene.sceneRect() PyQt5.QtCore.QRectF(-0.5, -0.5, 2716.5, 1450.5)
        # -----------------------------------------------------------------------
        scene_rectangle_f = self.scene.sceneRect()
        # -----------------------------------------------------------------------
        print("self.scene.sceneRect()", self.scene.sceneRect()) # A garder !
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Rectangle de la scène : {}\n").format(self.scene.sceneRect()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        # Dé-sélectionne les images
        # des oeuvres (les pointillés)
        self.scene.clearSelection()

        # Le rectangle de la scène est ouvert dans un QImage (avec
        # comme dimensions celles de l'image du mur chargé)
        img = QImage(QSize(self.scene.sceneRect().width(), self.scene.sceneRect().height()), QImage.Format_ARGB32_Premultiplied) # Ou QImage.Format_ARGB32_Premultiplied ? # QImage.Format_RGB888

        # Repaint dans un QPainter
        painter = QPainter(img)
        # ----------------------------------------------------------------------------------------
        # C'est ensuite géré avec un QRectF
        # ----------------------------------------------------------------------------------------
        # Ici toutes les situations sont gérées ; on imagine que une oeuvre ajoutée dans l'expo
        # déborde de l'image du mur --> hors-champ (en fait si elle déborde par le haut), voilà
        # ce que pourra afficher self.scene.sceneRect().x(), self.scene.sceneRect().y(), 
        # self.scene.sceneRect().width() et self.scene.sceneRect().height() :
        # 0.0 -135.67889630078832 2520.0 855.6788963007883 ... on voit bien le - (moins). En fait
        # c'est le contenu de self.scene.sceneRect(), qui affiche : PyQt5.QtCore.QRectF(0.0, 
        # -135.67889630078832, 2520.0, 855.6788963007883) ... le moins indique que l'image du mur
        # a été déplacée vers le bas (comme l'oeuvre a débordé vers le haut). Maintenant une expo
        # avec aucune oeuvre qui déborde : 0.0 0.0 2520.0 720.0 
        # ... le retour de self.scene.sceneRect() : PyQt5.QtCore.QRectF(0.0, 0.0, 2520.0, 720.0)
        # ----------------------------------------------------------------------------------------
        self.scene.render(painter, target=QRectF(self.scene.sceneRect().x(), self.scene.sceneRect().y(), self.scene.sceneRect().width(), self.scene.sceneRect().height()), source=scene_rectangle_f) # ESSAI !!!
        painter.end()

        # Sauvegarde de l'image dans le répertoire temporaire
        # (en vue d'une utilisation ultérieure)
        try :
            sauvegarde = img.save(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
        except :
            sauvegarde = img.save(os.getcwd()+os.sep+'sauvegarde_image_scene_finale.png')
            
        # --------------------------------------------------------------
        # Post-traitement de l'image sauvegardée dans le rep. tempo
        # --------------------------------------------------------------
        # Réouverture de l'image préalablement sauvegardée et
        # traitement avec Pillow --> Dans certaines situations 
        # l'image est mal enregistrée (rajout d'artefacts disgracieux). 
        # En la redécoupant (crop de Pillow) aux bonnes dimensions
        # (2520 x 720), l'image est bien sauvegardée correctement
        im_temp = Image.open(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
        im_temp.crop((0, 0, 2520, 720)).save(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
        # --------------------------------------------------------------
        
        # ------------------------------
        # Appel des données récoltées dans la fonction :
        # repertoire_chemin_configuration (voir la condition
        # juste en dessous !)
        # ------------------------------        
        # Si le répertoire par défaut de chargement des images 
        # a été modifié par l'utilisateur dans l'onglet 
        # Configuration > Chemins (chargement et sauvegarde), 
        # les données récoltées par la fonction : 
        # repertoire_chemin_configuration ... sont utilisées 
        # (comme chemin par défaut) lors de la sauvegarde dans 
        # la boîte de dialogue ... autrement c'est le répertoire
        # de l'utilisateur qui est utilisé ...
        try :
            rep_defaut_chargement = self.repertoire_chemin_configuration("data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo")
        except :
            rep_defaut_chargement = os.path.expanduser('~')
        # ------------------------------
            
        try :
            # Sauvegarde en tant qu'image sur le disque dur La boîte d'enregistrement
            enregistrer_image = QFileDialog.getSaveFileName(self, _("Enregistrer l'image de l'exposition"), rep_defaut_chargement, _("Image de l'exposition (.png) (*.png)"))[0]
            # 
            if not QFileInfo(enregistrer_image).suffix(): enregistrer_image += ".png"
            
            # Récup du chemin et nom de fichier
            self.enregistrer_fichier_image_exposition = str(enregistrer_image)
            
            # Permet d'enregistrer définitivement l'image (important !)
            img.save(self.enregistrer_fichier_image_exposition)
    
            print(_("Chemin de l'enregistrement de l'image d'exposition :"), self.enregistrer_fichier_image_exposition)
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Image de l'exposition sauvegardée ici (chemin + nom de fichier) : {}\n").format(self.enregistrer_fichier_image_exposition))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
            # Le chemin d'enregistrement est écrit dans le répertoire temporaire
            ecr_tempo_chemin_image_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chemin_enregistrement_image_exposition_murexpo_", "data_chemin_enregistrement_image_exposition_murexpo", ".txt", self.enregistrer_fichier_image_exposition)
            
            # --------------------------------------------------------------
            # Post-traitement
            # --------------------------------------------------------------
            # Réouverture de l'image préalablement enregistrée et
            # traitement avec Pillow --> Dans certaines situations 
            # l'image est mal enregistrée (rajout d'artefacts disgracieux). 
            # En la redécoupant (crop de Pillow) aux bonnes dimensions
            # (2520 x 720), l'image est bien enregistrée correctement
            im = Image.open(self.enregistrer_fichier_image_exposition)
            im.crop((0, 0, 2520, 720)).save(self.enregistrer_fichier_image_exposition)
            # --------------------------------------------------------------
            
            # ------------------------------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture des données 
            # dans le répertoire (dit) temporaire
            pre_lecture_index_exportation = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_fin_exportation_exposition_image_murexpo_', 'data_fin_exportation_exposition_image_murexpo')
            # ... avec appel de la méthode affichage
            lecture_index_exportation = "".join(pre_lecture_index_exportation.affichage())
            # ------------------------------------------------------------------------------
            # Lecture de la valeur du fichier tempo. 
            # data_fin_exportation_exposition_image_murexpo,
            # dans un 1er temps positionné à '0' dans la
            # classe Fabrique (plus haut) (c'est en qque
            # sorte un drapeau écrit ds un fichier tempo).
            if lecture_index_exportation == '0' :
                # Le fichier tempo. est ensuite ré-écrit
                # (valeur '1') ... à la toute fin ici
                # (prouvant que l'exportation de 
                # l'exposition sous la forme d'une image)
                # a bien eu lieu. --> Ré-utilisation de ce 
                # fichier tempo dans la classe
                # MonGraphiqueVueMur (dans la fonction
                # afficher_barre_progression_et_bout_visuel_image)
                ecr_tempo_drapeau_fin_export_expo_img = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_fin_exportation_exposition_image_murexpo_", "data_fin_exportation_exposition_image_murexpo", ".txt", '1')
            # ------------------------------------------------------------------------------
            
        except :
            return
        
    
    def sauvegarder_gif_anime(self) :
        """ Fonction pour sauvegarder le mur (entier) 
        d'exposition ... et avec les oeuvres si elles
        sont déployées sur le mur. Cette fonction
        (qui reprend les éléments de la fonction :
        sauvegarder_image_entiere_expo) sert à gérer
        la sauvegarde de l'export en gif animé """

        # -----------------------------------------------------------------------
        # Solution trouvée ici : 
        # https://stackoverflow.com/questions/52760130/pyqt-5-qpainter-returns-false-while-rendering-qgraphicsscene-to-a-qimage
        # -----------------------------------------------------------------------
        # C'est ici la résolution de la scène (en l'occurrence la résolution 
        # de l'image du mur). Par exemple avec une image de mur de 1920 x 1080, 
        # voilà le résultat :
        # self.scene.sceneRect() PyQt5.QtCore.QRectF(-0.5, -0.5, 1920.5, 1080.5)
        # ... et avec une image de mur de 2716 x 1450, voilà le résultat :
        # self.scene.sceneRect() PyQt5.QtCore.QRectF(-0.5, -0.5, 2716.5, 1450.5)
        # -----------------------------------------------------------------------
        scene_rectangle_f = self.scene.sceneRect()
        # -----------------------------------------------------------------------
        print("self.scene.sceneRect()", self.scene.sceneRect()) # A garder !
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Rectangle de la scène : {}\n").format(self.scene.sceneRect()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        # Dé-sélectionne les images
        # des oeuvres (les pointillés)
        self.scene.clearSelection()

        # Le rectangle de la scène est ouvert dans un QImage (avec
        # comme dimensions celles de l'image du mur chargé)
        img = QImage(QSize(self.scene.sceneRect().width(), self.scene.sceneRect().height()), QImage.Format_ARGB32_Premultiplied) # Ou QImage.Format_ARGB32_Premultiplied ? # QImage.Format_RGB888

        # Repaint dans un QPainter
        painter = QPainter(img)
        # ----------------------------------------------------------------------------------------
        # C'est ensuite géré avec un QRectF
        # ----------------------------------------------------------------------------------------
        # Ici toutes les situations sont gérées ; on imagine que une oeuvre ajoutée dans l'expo
        # déborde de l'image du mur --> hors-champ (en fait si elle déborde par le haut), voilà
        # ce que pourra afficher self.scene.sceneRect().x(), self.scene.sceneRect().y(), 
        # self.scene.sceneRect().width() et self.scene.sceneRect().height() :
        # 0.0 -135.67889630078832 2520.0 855.6788963007883 ... on voit bien le - (moins). En fait
        # c'est le contenu de self.scene.sceneRect(), qui affiche : PyQt5.QtCore.QRectF(0.0, 
        # -135.67889630078832, 2520.0, 855.6788963007883) ... le moins indique que l'image du mur
        # a été déplacée vers le bas (comme l'oeuvre a débordé vers le haut). Maintenant une expo
        # avec aucune oeuvre qui déborde : 0.0 0.0 2520.0 720.0 
        # ... le retour de self.scene.sceneRect() : PyQt5.QtCore.QRectF(0.0, 0.0, 2520.0, 720.0)
        # ----------------------------------------------------------------------------------------
        self.scene.render(painter, target=QRectF(self.scene.sceneRect().x(), self.scene.sceneRect().y(), self.scene.sceneRect().width(), self.scene.sceneRect().height()), source=scene_rectangle_f) # ESSAI !!!
        painter.end()

        # Sauvegarde de l'image dans le répertoire temporaire
        # (en vue d'une utilisation ultérieure)
        try :
            sauvegarde = img.save(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
        except :
            sauvegarde = img.save(os.getcwd()+os.sep+'sauvegarde_image_scene_finale.png')
            
        # --------------------------------------------------------------
        # Post-traitement de l'image sauvegardée dans le rep. tempo
        # --------------------------------------------------------------
        # Réouverture de l'image préalablement sauvegardée et
        # traitement avec Pillow --> Dans certaines situations 
        # l'image est mal enregistrée (rajout d'artefacts disgracieux). 
        # En la redécoupant (crop de Pillow) aux bonnes dimensions
        # (2520 x 720), l'image est bien sauvegardée correctement
        im_temp = Image.open(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
        im_temp.crop((0, 0, 2520, 720)).save(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
        # --------------------------------------------------------------
        
        # ------------------------------
        # Appel des données récoltées dans la fonction :
        # repertoire_chemin_configuration (voir la condition
        # juste en dessous !)
        # ------------------------------        
        # Si le répertoire par défaut de chargement des images 
        # a été modifié par l'utilisateur dans l'onglet 
        # Configuration > Chemins (chargement et sauvegarde), 
        # les données récoltées par la fonction : 
        # repertoire_chemin_configuration ... sont utilisées 
        # (comme chemin par défaut) lors de la sauvegarde dans 
        # la boîte de dialogue ... autrement c'est le répertoire
        # de l'utilisateur qui est utilisé ...
        try :
            rep_defaut_chargement = self.repertoire_chemin_configuration("data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo")
        except :
            rep_defaut_chargement = os.path.expanduser('~')
        # ------------------------------
        
        # ------------------------------
        # Sauvegarde en tant que gif animé sur le disque dur La boîte d'enregistrement
        enregistrer_gif_anime = QFileDialog.getSaveFileName(self, _("Enregistrer le gif animé"), rep_defaut_chargement, _("Gif animé (.gif) (*.gif)"))[0]
        # 
        if not QFileInfo(enregistrer_gif_anime).suffix(): enregistrer_gif_anime += ".gif"
        
        # Récup du chemin et nom de fichier
        self.enregistrer_fichier_gif_anime = str(enregistrer_gif_anime)
        # ------------------------------
        
        # Le chemin d'enregistrement est écrit dans le fichier tempo data_chemin_enregistrement_gif_anim_murexpo
        ecr_chemin_enregistrement_gif_anim = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chemin_enregistrement_gif_anim_murexpo_", "data_chemin_enregistrement_gif_anim_murexpo", ".txt", self.enregistrer_fichier_gif_anime)

        # ====================================================================

        # -----------------------------------------------------
        # Appel de la classe Fenetre_barre_Gif_Anime 
        # contenue dans le fichier gif_anime_thread.py
        # -----------------------------------------------------
        # 2ème partie du traitement (qui se déroule dans
        # dans le fichier gif_anime_thread.py qui contient
        # les classes GifAnimeThread et Fenetre_barre_Gif_Anime)
        # -----------------------------------------------------
        self.barre_gif_anim = Fenetre_barre_Gif_Anime()
        
        # ====================================================================

        
    def image_de_l_exposition(self) :
        """ Fonction qui affiche une image de 
        l'exposition (mur + oeuvres(s)) ...
        en cours d'accrochage ... ou au moment 
        de la sauvegarde """
        
        try :
            # -----------------------------------------------------------------------
            # Solution trouvée ici : 
            # https://stackoverflow.com/questions/52760130/pyqt-5-qpainter-returns-false-while-rendering-qgraphicsscene-to-a-qimage
            # -----------------------------------------------------------------------
            # C'est ici la résolution de la scène (en l'occurrence la résolution 
            # de l'image du mur). Par exemple avec une image de mur de 1920 x 1080, 
            # voilà le résultat :
            # self.scene.sceneRect() PyQt5.QtCore.QRectF(-0.5, -0.5, 1920.5, 1080.5)
            # ... et avec une image de mur de 2716 x 1450, voilà le résultat :
            # self.scene.sceneRect() PyQt5.QtCore.QRectF(-0.5, -0.5, 2716.5, 1450.5)
            # -----------------------------------------------------------------------
            scene_rectangle_f = self.scene.sceneRect()
            # -----------------------------------------------------------------------
            print("self.scene.sceneRect()", self.scene.sceneRect())
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Rectangle de la scène : {}\n").format(self.scene.sceneRect()))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
            # Dé-sélectionne les images
            # des oeuvres (les pointillés)
            self.scene.clearSelection()
            
            # Le rectangle de la scène est ouvert dans un QImage (avec
            # comme dimensions celles de l'image du mur chargée)
            img = QImage(QSize(self.scene.sceneRect().width(), self.scene.sceneRect().height()), QImage.Format_ARGB32_Premultiplied) # Ou QImage.Format_ARGB32_Premultiplied ? # QImage.Format_RGB888
    
            # Repaint dans un QPainter
            painter = QPainter(img)
            # ----------------------------------------------------------------------------------------
            # C'est ensuite géré avec un QRectF
            # ----------------------------------------------------------------------------------------
            # Ici toutes les situations sont gérées ; on imagine que une oeuvre ajoutée dans l'expo
            # déborde de l'image du mur --> hors-champ (en fait si elle déborde par le haut), voilà
            # ce que pourra afficher self.scene.sceneRect().x(), self.scene.sceneRect().y(), 
            # self.scene.sceneRect().width() et self.scene.sceneRect().height() :
            # 0.0 -135.67889630078832 2520.0 855.6788963007883 ... on voit bien le - (moins). En fait
            # c'est le contenu de self.scene.sceneRect(), qui affiche : PyQt5.QtCore.QRectF(0.0, 
            # -135.67889630078832, 2520.0, 855.6788963007883) ... le moins indique que l'image du mur
            # a été déplacée vers le bas (comme l'oeuvre a débordé vers le haut). Maintenant une expo
            # avec aucune oeuvre qui déborde : 0.0 0.0 2520.0 720.0 
            # ... le retour de self.scene.sceneRect() : PyQt5.QtCore.QRectF(0.0, 0.0, 2520.0, 720.0)
            # ----------------------------------------------------------------------------------------
            self.scene.render(painter, target=QRectF(self.scene.sceneRect().x(), self.scene.sceneRect().y(), self.scene.sceneRect().width(), self.scene.sceneRect().height()), source=scene_rectangle_f)
            painter.end()
    
            # Sauvegarde de l'image dans le répertoire temporaire
            # (en vue d'une utilisation ultérieure)
            try :
                sauvegarde = img.save(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
            except :
                sauvegarde = img.save(os.getcwd()+os.sep+'sauvegarde_image_scene_finale.png')
        except : pass
    
        # --------------------------------------------------------------
        # Post-traitement de l'image sauvegardée dans le rep. tempo
        # --------------------------------------------------------------
        # Réouverture de l'image préalablement sauvegardée et
        # traitement avec Pillow --> Dans certaines situations 
        # l'image est mal enregistrée (rajout d'artefacts disgracieux). 
        # En la redécoupant (crop de Pillow) aux bonnes dimensions
        # (2520 x 720), l'image est bien sauvegardée correctement
        im_temp = Image.open(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
        im_temp.crop((0, 0, 2520, 720)).save(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png')
        # --------------------------------------------------------------

    
    def affichage_du_logo(self) :
        """ Fonction servant à afficher le logo de MurExpo """
        
        # Maj scène et vue QGraphicsView
        self.scene.clear()
        self.items().clear()

        # ------------------------------------
        # Détection de la taille de l'image 
        # du logo (pour harmoniser la vue)
        # ------------------------------------
        # Ouverture de l'image du logo
        im = Image.open(chemin_ressources(os.path.join('ressources_murexpo', 'logo_murexpo_general_664x280.png')))
        largeur, hauteur = im.size

        # Récupération textuelle de l'image du logo dans le rep. ressources_murexpo
        images = chemin_ressources(os.path.join('ressources_murexpo', 'logo_murexpo_general_664x280.png'))
        
        # Le pixmap récupère l'image ...
        pixmap = QPixmap(images)

        # Insère l'image dans le QGraphicsPixmapItem (le logo
        # --> Appel de la classe MonGraphiquePixmapItemMur 
        # avec comme argument le pixmap
        item = MonGraphiquePixmapItemMur(pixmap)

        # Amélioration de la qualité
        item.setTransformationMode(Qt.SmoothTransformation)

        # Insère QGraphicsPixmapItem
        # dans la scène
        self.scene.addItem(item)

        # Met en place le QGrphicsView dans la scène
        self.setScene(self.scene)

        # ------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture 
        # des données dans le répertoire (dit) temporaire
        # ------------------------------------------------------
        # Lecture de la résolution utilisée
        # (ou sélectionnée par l'utilisateur)
        pre_lecture_resolution = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_desktop_murexpo_", "data_resolution_desktop_murexpo")
        # ... avec appel de la méthode affichage
        lecture_resolution = "".join(pre_lecture_resolution.affichage())
        # Récupération des données,
        # par exemple ['1707', '960']
        resolution = lecture_resolution.split('x')

        # Adapter la vue (le QGraphicsView) pour qu'il soit totalement
        # visible --> L'image déployée s'adapte à la taille de la vue
        self.fitInView(item, Qt.KeepAspectRatio)
        
        # Conditions d'affichage du logo
        # suivant la résolution utilisée
        if 681 <= int(resolution[1]) <= 750 : y = 0
        if 751 <= int(resolution[1]) <= 768 : y = 24
        if 769 <= int(resolution[1]) <= 800 : y = 40
        if 801 <= int(resolution[1]) <= 850 : y = 60
        if 851 <= int(resolution[1]) <= 900 : y = 80
        if 901 <= int(resolution[1]) <= 950 : y = 100
        if 951 <= int(resolution[1]) <= 1000 : y = 114
        if 1001 <= int(resolution[1]) <= 1050 : y = 118
        if 1051 <= int(resolution[1]) <= 1100 : y = 124
        if 1101 <= int(resolution[1]) <= 1200 : y = 128
        
        # La valeur de x reste à 0
        x = 0
        # Affichage du logo dans la vue ...
        self.setSceneRect(-x, -y, self.viewport().width(), self.viewport().height())
        self.centerOn(0, 0)

        # Affichage de la vue
        self.show()
 

# ------------------------------------------------ A garder pour l'instant ---------------
'''
class RecupCheminOeuvre() :
    """ Classe pour récupérer le chemin de l'oeuvre sélectionnée """

    # Appel de la classe pour récupérer le chemin du 
    # répertoire temporaire selon le système d'exploitation
    chem_rep_tempo = Chemin_Rep_Tempo()
    # Le chemin est retourné (return ds la fonction de la classe)
    chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
    
    # Utilisation de la classe Tempo_lecture pour la lecture 
    # des données dans le répertoire (dit) temporaire
    pre_lecture_chemin_image = Tempo_lecture(chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_image_oeuvre_murexpo_', 'data_chemin_image_oeuvre_murexpo')
    # ... avec appel de la méthode affichage
    lecture_chemin_image = "".join(pre_lecture_chemin_image.affichage())
        
    def __str__(self) :
        
        return "{}".format(self.lecture_chemin_image)
'''  

'''
        # ====================================================== #
        # On va chercher l'image finale de prévisualisation 
        # générée directement dans le répertoire dit temporaire
        # ====================================================== #
        try :
            liste_image_finale = glob.glob(self.chemin_tempo_system_exploit+os.sep+'pdf_png_apsc_final.png')
        except :
            liste_image_finale = glob.glob(os.getcwd()+os.sep+'pdf_png_apsc_final.png') 
        # Affichage d'une page d'information lors du 1er démarrage d'APSC
        # (et par exemple si l'image /tmp/pdf_png_apsc_final.png est 
        # inéxistante).
        if len(liste_image_finale) == 0 :
            # (--> Affichage avec la fonction chemin_ressources  contenue dans : 
            # config_apsc/chemin_data_ressources.py)
            liste_image_finale.append(chemin_ressources('ressources_murexpo'+os.sep+'vue_page_inexistante.png'))
        # ====================================================== #
'''    
# ------------------------------------------------ A garder pour l'instant ---------------
