#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

import os


class CapterToutesOeuvresSupprimees(object) :
    """ Classe pour capter les images 
    (oeuvres) supprimées dans MurExpo """
    
    # Liste des actions 
    # et traitement qui 
    # va être remplie 
    # au fur et à mesure ...
    liste_evolue = []
    
    def __init__(self):
        super(CapterToutesOeuvresSupprimees, self).__init__()
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        
    def liste_capter_oeuvres_supprimees(self, item) :
        # Remplissage de la liste
        self.liste_evolue.append(item)
        # Elimination/supression des doublons (par l'intermédiaire d'un dictionnaire)
        self.liste_evolue = list(dict.fromkeys(self.liste_evolue))

        
    def liste_ecrire_oeuvres_supprimees(self) :
        # Ecriture des commandes dans le fichier data_retour_commandes_murexpo
        ecr_tempo_oeuvres_supprimees = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_capture_oeuvres_supprimees_murexpo_", "data_capture_oeuvres_supprimees_murexpo", ".txt", "".join(self.liste_evolue))
        
        # =========================================================================
        
        # -------------------------------------------------------------------------
        # Suppression des images présentes dans le répertoire temporaire quand 
        # l'utilisateur les supprime dur mur d'exposition ...
        # -------------------------------------------------------------------------
        # Si l'utilisateur supprime 2 images la liste self.liste_evolue contiendra
        # les 2 images supprimées, par exemple (en fin de compte) :
        # ['chemin .../image_0001.jpg\n', 'chemin .../image_0002.png\n']
        # On s'apperçoit que chaque élément de la liste se termine par un caractère 
        # de saut de ligne --> \n
        # -------------------------------------------------------------------------
        # Cette liste ici ne garde que le nom des images ... et par la même 
        # occasion élimine le caractère de saut à la ligne.
        liste_evolue_nom_image_uniquement = [os.path.basename(img)[:-1] for img in self.liste_evolue]
        # -------------------------------------------------------------------------
        # Supression des images dans le répertoire temporaire (préalablement 
        # éliminées du mur d'exposition)
        # -------------------------------------------------------------------------
        # Parcours des fichiers du répertoire temporaire ...
        for fich in os.listdir(self.chemin_tempo_system_exploit) :
            # Si le fichier (image) se trouve dans la 
            # liste liste_evolue_nom_image_uniquement ...
            if fich in liste_evolue_nom_image_uniquement :
                # ... le fichier (l'image) en question est supprimée ...
                os.remove(os.path.join(self.chemin_tempo_system_exploit, fich))
                
        # =========================================================================
