#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture
# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

# Imports PyQt5 ------------------------------------------------------------
from PyQt5.QtCore import QDir, pyqtSignal, QLocale
from PyQt5.QtWidgets import QTabWidget, QTreeView, QWidget, QVBoxLayout, \
                            QFileDialog, QPushButton, QAbstractItemView, \
                            QMainWindow, QApplication
# --------------------------------------------------------------------------
                            
import sys, os, platform

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------

from os import path


class FichierOuvrir(QFileDialog) :
    """ QFileDialog surclassé pour que la boîte de 
    chargement des images soit intégrée dans le 
    tabwidget de chargement """
    
    # =========================================================================
    # Création du signal en vue d'afficher automatiquement l'onglet 
    # ... . 1ère partie. Un grand merci à eyllanesc !!!
    # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
    # qtabwidget-defined-in-the-main-class-in-another-class#63065558
    customSignal_choix_oeuvre = pyqtSignal(int)
    # =========================================================================

    def __init__(self, parent=None):
        super(FichierOuvrir, self).__init__(parent)
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------

        # ---------------------------------------------------------------------
        # Ecriture du choix par défaut du QRadioButton dans le répertoire
        # temporaire
        # ---------------------------------------------------------------------
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        # ---------------------------------------------------------------------
        
        self.setOption(self.DontUseNativeDialog, True)
        self.setFileMode(self.ExistingFiles)
        self.setNameFilter(_("Images (*.png *.xpm *.jpg *.jpeg *.JPG *.JPEG *.gif *.tga)"))
        
        # En option
        #self.setViewMode(self.List)

        # ********************************************************************************
        
        # --------------------------------------------------------------------------------
        # Les fichiers data_chem_charg_rep_... et data_chem_sauv_rep_... (contenus dans
        # le rep. tampon) sont préalablement crées par la classe :
        # Configuration_Repertoires_Chargement contenue dans murexpo_gui.py, il n'y aura
        # donc pas d'erreur en appelant la fonction repertoire_chemin_configuration (!!!)
        # --------------------------------------------------------------------------------
        # Appel de la fonction repertoire_chemin_configuration et affectation de la donnée
        # retournée :
        rep_defaut_chargement = self.repertoire_chemin_configuration("data_chem_charg_rep_images_murexpo_", "data_chem_charg_rep_images_murexpo")
        # La boîte de dialogue de chargement se mete à jour et le répertoire  
        # de chargement (le chemin défini de chargement) se met en place ...
        self.setDirectory(rep_defaut_chargement)
        
        # ********************************************************************************
        
        bouton = self.findChildren(QPushButton)
        # ***********************************
        # Récupère le texte du bouton (par ex: Ouvrir)
        intitule_bouton = bouton[0].text().lower()
        # ***********************************
        self.ouvertureBouton = [x for x in bouton if intitule_bouton in str(x.text()).lower()][0]
        # ***********************************
        self.ouvertureBouton.clicked.disconnect()
        self.ouvertureBouton.clicked.connect(self.openClicked)
        self.tree = self.findChild(QTreeView)

        # Sélection simple
        self.tree.setSelectionMode(QAbstractItemView.SingleSelection)
        
        # ---------------------------------------------------- Rendre dispos (ou indispos) par réglages ?
        # Permet de cacher l'entête
        #self.tree.setHeaderHidden(True) # Important !
        
        # On enlève la colonne 
        # à l'index 1, 2 et 3
        #self.tree.hideColumn(1)
        #self.tree.hideColumn(2)
        #self.tree.hideColumn(3)
        # ---------------------------------------------------- Rendre dispos (ou indispos) par réglages ?
        
        # Permet de sélectionner les fichiers à la souris
        selmodele = self.tree.selectionModel()
        selmodele.selectionChanged.connect(self.selection_activee)
        
    
    def repertoire_chemin_configuration(self, chem_tempo_1, chem_tempo_2) :
        ''' Fonction de mise en place du répertoire de chargement par défaut 
        des images (par exemple les images des oeuvres) '''
        
        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)

        # ----------
        # Si le chargement du répertoire a déjà été modifié
        # ... Boucle de vérification ...
        # ----------
        for contenu in liste_contenu_tmp :
            # Si le fichier tempo concerné est bien dedans ...
            if chem_tempo_1 in contenu :
                
                # ------------------------------------------------------- #
                # Utilisation de la classe Tempo_lecture pour la lecture
                # des données dans le répertoire (dit) temporaire
                # ------------------------------------------------------- #
                rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), chem_tempo_1, chem_tempo_2)
                # ... avec appel de la méthode affichage
                rep_tempo_chemin = rep_tempo_lecture.affichage()[0]
                # S'il y a bien une donnée récupérée ...
                # celle-ci est affectée
                if rep_tempo_chemin :
                    rep_defaut = rep_tempo_chemin
                    return rep_defaut
    

    def selection_activee(self) :
        """ Fichier sélectionné à l'aide de la souris. 
        Appel de la fonction capture_fichier. Cas
        ici sous GNU/Linux quand le simple clic est 
        activé (dans le système) à la place du double 
        clic pour la sélection """
        
        self.capture_fichier()
        

    def openClicked(self) :
        """ Fichier capturé quand l'utilisateur 
        clique sur le bouton Ouvrir (ou Open selon les
        cas). Appel de la fonction capture_fichier """
        
        self.capture_fichier()
        
        # =========================================================================
        # Uniquement quand l'utilisateur clique sur le bouton pour terminer
        # la sélection !
        # =========================================================================
        # Utilisation du signal pour afficher automatiquement l'onglet ... . 
        # 2ème partie ==> le signal est émis pour l'id 0 (du QTabWidget défini 
        # [les onglets] dans le script main (murexpo_gui.py)). A voir aussi
        # Une ligne de code par rapport à cela se trouve dans murexpo_gui.py. 
        # Un grand merci à eyllanesc !!!
        # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
        # qtabwidget-defined-in-the-main-class-in-another-class#63065558
        self.customSignal_choix_oeuvre.emit(0)
        # =========================================================================
        
        
    def capture_fichier(self) :
        """ Fonction de capture du fichier dans le QTreeView """
        
        # PyQt5.QtCore.QModelIndex
        modeleIndex = self.tree.selectionModel().selectedIndexes()
        # Liste qui va accueillir
        # la sélection du fichier
        liste_fichier = []
        for i in modeleIndex :
            if i.column() == 0 :
                liste_fichier.append(os.path.join(str(self.directory().absolutePath()), str(i.data())))
        # En fin de compte le fichier sélectionné 
        # est une chaîne de caractères        
        self.fichier_selectionne = "".join(liste_fichier)
        
        # Détection pour savoir si le fichier 
        # est un fichier (donc une image) ...
        if os.path.isfile(self.fichier_selectionne) :

            # --------------------------------------------
            # Le chemin de l'image est donc écrit dans 
            # le rep. temporaire ...
            # --------------------------------------------
            ecr_tempo_chemin_image = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chemin_image_oeuvre_murexpo_", "data_chemin_image_oeuvre_murexpo", ".txt", self.fichier_selectionne)
        
            print("self.fichier_selectionne", self.fichier_selectionne)
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Image sélectionnée dans la boîte de chargement de l'onglet Chargement des images : {}\n").format(self.fichier_selectionne))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
   
        #return self.fichier_selectionne
