#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Merci à eyllanesc pour son aide est pour cette classe ... import
# --> pour les icônes à droite dans le bouton
# --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
from push_button_icone_droite import PushButtonIconeDroite
# --> pour les icônes à gauche dans le bouton
from push_button_icone_gauche import PushButtonIconeGauche

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture
# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Traitement des oeuvres avec Pillow (par exemple
# l'oeuvre est encadrée ou entoilée)
from traitement_image_oeuvre import Traitement_Image_Oeuvre

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

import os, glob, imageio

# Import de Pillow
try : import Image
except ImportError : from PIL import Image

# Imports PyQt5 -----------------------------------------------------------------------
from PyQt5.QtWidgets import QVBoxLayout, QHBoxLayout, QGroupBox, QPushButton, QGridLayout, \
                            QLabel, QDesktopWidget, QApplication, QTextEdit, QSlider, \
                            QSpinBox, QCheckBox, QPushButton, QDialog, QProgressBar, QLineEdit, QSizePolicy
from PyQt5.QtCore import QThread, pyqtSlot, QSize, Qt, pyqtSignal, QLocale, QRect, QPoint
from PyQt5.QtGui import QPixmap, QIcon, QImage, QMovie
# --------------------------------------------------------------------------------------

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------

# -------------------------------------------------------------
# *** 2ème partie pour le traitement du gif animé
# --> la 1ère partie se trouve dans fabrique_murexpo_gui.py
# ... dans la fonction sauvegarder_gif_anime (contenue
# dans la classe MonGraphiqueVueMur)
# -------------------------------------------------------------

class GifAnimeThread(QThread):
    """ Partie du Thread qui s'exécute en tâche de fond 
    (dont le code exécuté ---> dans la fonction run) """
    
    # création du signal
    fini = pyqtSignal(bool) # signal pour la fin du thread
    
    def __init__(self, parent=None):
        super().__init__(parent)
 
        self.stop = False # drapeau pour exécuter une demande d'arrêt
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
       
        
    def run(self):
        """ Partie qui s'exécute en tâche de fond """
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()

        # ------------------------------
        # Appel des données récoltées dans la fonction :
        # repertoire_chemin_configuration (voir la condition
        # juste en dessous !)
        # ------------------------------        
        # Si le répertoire par défaut de chargement des images 
        # a été modifié par l'utilisateur dans l'onglet 
        # Configuration > Chemins (chargement et sauvegarde), 
        # les données récoltées par la fonction : 
        # repertoire_chemin_configuration ... sont utilisées 
        # (comme chemin par défaut) lors de la sauvegarde dans 
        # la boîte de dialogue ... autrement c'est le répertoire
        # de l'utilisateur qui est utilisé ...
        try :
            rep_defaut_chargement = self.repertoire_chemin_configuration("data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo")
        except :
            rep_defaut_chargement = os.path.expanduser('~')
        # ------------------------------

        # --------------------------------------------------------------------------------
        # Lecture du fichier tempo. data_choix_decalage_gif_anime_murexpo
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_choix_decalage_gif_anim = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_choix_decalage_gif_anime_murexpo_', 'data_choix_decalage_gif_anime_murexpo')
        # ... avec appel de la méthode affichage
        # variable ==> decalage
        decalage = "".join(pre_lecture_choix_decalage_gif_anim.affichage())
        decalage = int(decalage)
        
        # Attribution du nombre d'images (approximatif) à traiter
        # comparativement au à la valeur de décalage (en pixels)
        if decalage == 5 : nbre_img = 401
        elif decalage == 10 : nbre_img = 201
        elif decalage == 20 : nbre_img = 101
        elif decalage == 25 : nbre_img = 81
        
        # --------------------------------------------------------------------------------
        # Lecture du fichier data_choix_fps_gif_anime_murexpo
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_choix_frequence_gif_anim = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_choix_fps_gif_anime_murexpo_', 'data_choix_fps_gif_anime_murexpo')
        # ... avec appel de la méthode affichage
        # variable ==> frequence
        frequence = "".join(pre_lecture_choix_frequence_gif_anim.affichage())
        frequence = int(frequence)
        
        # Appel de la classe Traitement_Image_Oeuvre
        traitement_gif = Traitement_Image_Oeuvre()
        # Fonction decalage_points_gif_anim contenue
        # dans la classe Traitement_Image_Oeuvre.
        # Pour récupérer les coordonnées des points
        # pour la découpe (crop)
        coordonnees = traitement_gif.decalage_points_gif_anim()
        
        # --------------------------------------------------------------
        # Réouverture image ...
        # Découpe (crop) avec Pillow
        # --------------------------------------------------------------
        # Ouverture de l'image de l'exposition au format png, conversion en RGB et sauvegarde en JPEG
        image_png = Image.open(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.png').convert('RGB').save(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.jpg')
        # Ouverture de l'image de l'exposition convertie en JPEG (cela va diminuer très 
        # significativement le poids des images dans la suite du traitement)
        image = Image.open(self.chemin_tempo_system_exploit+os.sep+'sauvegarde_image_scene_finale.jpg')
        for n, coord in enumerate(coordonnees) :
            image.crop(coord).save(os.path.join(self.chemin_tempo_system_exploit, 'img_decoup_coord_gif_'+str(n+1).zfill(5)+'.jpg'))

        # Liste contenant toutes les images qui viennent d'être découpées (crop)
        liste_images_decoupee_pour_gif = glob.glob(os.path.join(self.chemin_tempo_system_exploit, 'img_decoup_coord_gif_*'))
        liste_images_decoupee_pour_gif.sort()
        # On tranche la liste (on ne garde que les éléments
        # entre l'indice 0 et le nombre d'éléments nbre_img)
        liste_images_decoupee_pour_gif = liste_images_decoupee_pour_gif[0:nbre_img]
        
        # --------------------------------------------------------------------------------
        # Lecture du fichier tempo. data_chemin_enregistrement_gif_anim_murexpo
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_chemin_sauv_gif_anim = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_enregistrement_gif_anim_murexpo_', 'data_chemin_enregistrement_gif_anim_murexpo')
        # ... avec appel de la méthode affichage
        # variable ==> decalage
        self.enregistrer_fichier_gif_anime = "".join(pre_lecture_chemin_sauv_gif_anim.affichage())
        
        # --------------------------------------------------------
        # Traitement du gif animé et enregistrement
        # --------------------------------------------------------
        try :
            
            # ----------------------------------------------------
            # Fabrication du gif animé avec imageio
            liste_images_sortie = []
            for n, i in enumerate(liste_images_decoupee_pour_gif) :
                liste_images_sortie.append(imageio.imread(i))
            # On crée le gif animé avec imageio (loop=0 ... en boucle normalement)
            imageio.mimsave(self.enregistrer_fichier_gif_anime, liste_images_sortie, fps=frequence, loop=0)
            # ----------------------------------------------------
            
            print(_("Chemin de l'enregistrement du gif animé :"), self.enregistrer_fichier_gif_anime)
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Gif animé sauvegardé ici (chemin + nom de fichier) : {}\n").format(self.enregistrer_fichier_gif_anime))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
            
            # Le chemin d'enregistrement est écrit dans le répertoire temporaire
            ecr_tempo_chemin_gif_anime = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chemin_enregistrement_gif_anime_murexpo_", "data_chemin_enregistrement_gif_anime_murexpo", ".txt", self.enregistrer_fichier_gif_anime)
            
        except : pass
        # --------------------------------------------------------
        
        # Signale que la code exécuté 
        # dans run vient de se terminer    
        self.fini.emit(self.stop)


class Fenetre_barre_Gif_Anime(QDialog):
    """ La partie graphique du thread """

    def __init__(self, parent=None):
        super(Fenetre_barre_Gif_Anime,self).__init__(parent)
        
        ## QApplication.ProcessEvents() (si nécessaire)
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()

        # --------------------------------------------------------------------------------
        # Lecture du fichier tempo. data_chemin_enregistrement_gif_anim_murexpo
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_chemin_sauv_gif_anim = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_enregistrement_gif_anim_murexpo_', 'data_chemin_enregistrement_gif_anim_murexpo')
        # ... avec appel de la méthode affichage
        self.enregistrer_fichier_gif_anime = "".join(pre_lecture_chemin_sauv_gif_anim.affichage())
        
        self.setWindowTitle("MurExpo : avancement du traitement")
        self.resize(500, 100)

        # --------------------------------------------------------------------------------
        # Lecture du fichier tempo. data_type_de_resolution_fenetre_principale_murexpo
        # ... afin de savoir si le fichier contient "Résolution programmée" ou bien
        # "Résolution au choix"
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_type_de_resolution = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_type_de_resolution_fenetre_principale_murexpo_', 'data_type_de_resolution_fenetre_principale_murexpo')
        # ... avec appel de la méthode affichage
        self.type_de_resolution = "".join(pre_lecture_type_de_resolution.affichage())
        
        # -----------------------------------------------------------------
        # Positionnement du QDialog ... à la fin du traitement. Ceci
        # fait en sorte que cette boîte de dialogue se positionne bien
        # au milieu ... en largeur et en hauteur
        # -----------------------------------------------------------------
        # Par défaut --> Résolution programmée
        if self.type_de_resolution == "Résolution programmée" :
            # Placement du QDialog
            geom = self.frameGeometry()
            centre = QDesktopWidget().availableGeometry().center()
            geom.moveCenter(centre)
            self.move(geom.topLeft())
        # Si l'utilisateur choisit Résolution au choix dans :
        # Configuration > Résolution
        elif self.type_de_resolution == "Résolution au choix" :
            # Le fichier data_resolution_desktop_murexpo 
            # (dans le rep. tempo) est ouvert (lecture)
            pre_lecture_resolution_desktop = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_resolution_desktop_murexpo_', 'data_resolution_desktop_murexpo')
            # La donnée contenue ds le fichier est récupérée (elle est
            # sous la forme [155x100], par exemple, puis elle est splitée.
            self.resolution_desktop = "".join(pre_lecture_resolution_desktop.affichage()).split('x')
            # Placement du QDialog
            geom = self.frameGeometry()
            # La boîte de dialogue est placée bien au milieu ...
            geom.moveCenter(QPoint(int(self.resolution_desktop[0]) / 2, int(self.resolution_desktop[1]) / 2))
            self.move(geom.topLeft())
        # -----------------------------------------------------------------

        # ---------------------------------------------------------------------------
        # Les boutons pour le lancement du thread et d'arrêt anticipé du thread
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe --> elle est ici : affichage_apsc/push_button_icone_gauche.py ...
        # en fait le texte dans le bouton est centré et l'icône positionné à gauche.
        # En fait j'ai un peu modifié la classe ...
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : config_apsc/chemin_data_ressources.py)
        icone_lancer_traitement = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_lancer_le_traitement_murexpo_256x256.png')))
        #icone_arreter_traitement = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_arreter_le_traitement_murexpo_256x256.png')))
        # Bouton de lancement du thread
        self.depart = PushButtonIconeGauche("Lancer le traitement", icon = icone_lancer_traitement, iconSize = QSize(16, 16))
        self.depart.clicked.connect(self.lancement)
        # ----------------------------------------------------------------------------
        # Bouton pour jouer l'animation gif
        self.bout_play_gif_anime = QPushButton("▶")
        self.bout_play_gif_anime.setStyleSheet('''QPushButton {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }'''
            '''QPushButton:pressed {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #6190F2, stop: 1 #fdfbf7);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }''')
        self.bout_play_gif_anime.setToolTip(_("Permet de lire le gif animé"))
        # Le bouton est rendu invisible
        self.bout_play_gif_anime.hide()
        # Bouton pour arrêter l'animation gif
        self.bout_stop_gif_anime = QPushButton("⏹")
        self.bout_stop_gif_anime.setStyleSheet('''QPushButton {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }'''
            '''QPushButton:pressed {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #6190F2, stop: 1 #fdfbf7);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }''')
        self.bout_stop_gif_anime.setToolTip(_("Permet d'arrêter la lecture du gif animé"))
        # Le bouton est rendu invisible
        self.bout_stop_gif_anime.hide()
        # ----------------------------------------------------------------------------
        # Le label pour visualiser le gif animé
        self.label_vision_gif_anime = QLabel()
        # Alignement
        self.label_vision_gif_anime.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.label_vision_gif_anime.setAlignment(Qt.AlignCenter)
        # Résolution du label (la zone de visualisation
        # du gif animé) : 120 x 120 pix
        self.label_vision_gif_anime.setGeometry(QRect(0, 0, 313, 450))
        self.label_vision_gif_anime.setMaximumSize(QSize(313, 450))
        self.label_vision_gif_anime.setFixedSize(313, 450)
        # Couleur (grise) du label par défaut
        self.label_vision_gif_anime.setStyleSheet("""background-color: rgb(100, 100, 100);""")
        # Le label de visionnage de l'animation 
        # gif est d'abor rendu invisible
        self.label_vision_gif_anime.hide()
        # ----------------------------------------------------------------------------
        # Le label pour préciser qu'il s'agit du chemin d'enregistrement :
        self.label_chemin_d_enregistrement = QLabel(_("Votre animation gif a été enregistrée ici :"))
        # Alignement
        self.label_chemin_d_enregistrement.setAlignment(Qt.AlignCenter)
        # Le label est rendu invisible
        self.label_chemin_d_enregistrement.hide()
        # ----------------------------------------------------------------------------
        # La ligne pour afficher le chemin 
        # d'enregistrement de l'animation gif
        self.ligne_chem_enregistr_anim_gif = QLineEdit()
        # On attribue le chemin d'enregistrement du gif animé dans la ligne d'édition
        self.ligne_chem_enregistr_anim_gif.setText(self.enregistrer_fichier_gif_anime)
        # Positionné en lecture seule
        self.ligne_chem_enregistr_anim_gif.setReadOnly(True)
        # La ligne pour afficher le chemin est rendue invisible
        self.ligne_chem_enregistr_anim_gif.hide()
        # ----------------------------------------------------------------------------
        # Barre de progression (en boucle ; voir plus 
        # bas ds la fonction lancement) pour montrer 
        # que c'est en cours de traitement
        self.progress = QProgressBar()
        # La barre est centrée
        self.progress.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        # La barre est d'abord 
        # rendue invisible
        self.progress.hide()
        # ---------------------------------------------------------------------------
        # Le bouton pour fermer la boîte de dialogue
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe --> elle est ici : affichage_apsc/push_button_icone_gauche.py ...
        # en fait le texte dans le bouton est centré et l'icône positionné à gauche.
        # En fait j'ai un peu modifié la classe ...
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage avec la fonction chemin_ressources contenue dans : config_apsc/chemin_data_ressources.py)
        icone_fermer = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_png_fermeture.png')))
        # Bouton pour fermer la boîte de dialogue
        self.bouton_fermer = PushButtonIconeGauche("Fermer la boîte de dialogue", icon = icone_fermer, iconSize = QSize(16, 16))
        self.bouton_fermer.clicked.connect(self.close)
        # Le bouton est d'abord 
        # rendu invisible
        self.bouton_fermer.hide()
        # ----------------------------------------------------------------------------
 
        # Positionne les widgets dans la fenêtre
        self.posit = QGridLayout()
        self.posit.addWidget(self.depart, 0, 0)
        self.posit.addWidget(self.progress, 1, 0)
        self.posit.addWidget(self.label_vision_gif_anime, 2, 0)
        self.posit.addWidget(self.bout_play_gif_anime, 3, 0)
        self.posit.addWidget(self.bout_stop_gif_anime, 4, 0)
        self.posit.addWidget(self.label_chemin_d_enregistrement, 5, 0)
        self.posit.addWidget(self.ligne_chem_enregistr_anim_gif, 6, 0)
        self.posit.addWidget(self.bouton_fermer, 7, 0)
        self.setLayout(self.posit)
 
        # initialisation variable d'instance de classe
        self.calcul_thread_gif_anime = None
        
        # Lancer l'exécution (permet notamment
        # de rendre la fenêtre du QDialog modale
        # c'est à dire que la fenêtre en question
        # reste active par dessus ...)
        self.exec_()
        

    @pyqtSlot(bool)
    def lancement(self, ok=False):
        """
        Lancement de l'opération longue dans le thread
        """
        if self.calcul_thread_gif_anime==None or not self.calcul_thread_gif_anime.isRunning():

            # Affichage de la barre de
            # progression (devient visible)
            self.progress.show()
            # La barre est en boucle 
            # (mouvements de gauche à droite)
            self.progress.setRange(0, 0)
            
            # initialise l'opération longue dans le thread
            self.calcul_thread_gif_anime = GifAnimeThread()
            # prépare la réception du signal de fin
            self.calcul_thread_gif_anime.fini.connect(self.stop)
            # lance le thread (mais ne l'attend pas avec .join!!!)
            self.calcul_thread_gif_anime.start()


    @pyqtSlot(bool)
    def stop(self, fin_anormale=False):
        """
        Lancé quand le thread se termine
        """
        if fin_anormale:
            pass
        else:
            # La fenêtre est redimensionnée
            self.resize(330, 640)
            
            # -----------------------------------------------------------------
            # Positionnement du QDialog ... à la fin du traitement. Ceci
            # fait en sorte que cette boîte de dialogue se positionne bien
            # au milieu ... en largeur et en hauteur
            # -----------------------------------------------------------------
            # Par défaut --> Résolution programmée
            if self.type_de_resolution == "Résolution programmée" :
                # Placement du QDialog
                geom = self.frameGeometry()
                centre = QDesktopWidget().availableGeometry().center()
                geom.moveCenter(centre)
                self.move(geom.topLeft())
            # Si l'utilisateur choisit Résolution au choix dans :
            # Configuration > Résolution
            elif self.type_de_resolution == "Résolution au choix" :
                # Le fichier data_resolution_desktop_murexpo 
                # (dans le rep. tempo) est ouvert (lecture)
                pre_lecture_resolution_desktop = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_resolution_desktop_murexpo_', 'data_resolution_desktop_murexpo')
                # La donnée contenue ds le fichier est récupérée (elle est
                # sous la forme [155x100], par exemple, puis elle est splitée.
                self.resolution_desktop = "".join(pre_lecture_resolution_desktop.affichage()).split('x')
                # Placement du QDialog
                geom = self.frameGeometry()
                # La boîte de dialogue est placée bien au milieu ...
                geom.moveCenter(QPoint(int(self.resolution_desktop[0]) / 2, int(self.resolution_desktop[1]) / 2))
                self.move(geom.topLeft())
            # -----------------------------------------------------------------
            
            # Le bouton de lancement du 
            # traitement est rendu invisible ici
            self.depart.hide()
            
            # La barre de progression est 
            # de nouveau rendue invisible
            self.progress.hide()
            
            # Les widgets redeviennent visibles
            self.label_vision_gif_anime.show()
            self.bout_play_gif_anime.show()
            self.bout_stop_gif_anime.show()
            self.label_chemin_d_enregistrement.show()
            self.ligne_chem_enregistr_anim_gif.show()
            self.bouton_fermer.show()
            
            # ================================================================
            # Signaux
            # ================================================================
            # Signal pour le bouton pour jouer (visualiser) le gif animé
            self.bout_play_gif_anime.clicked.connect(self.fonction_jouer_gif_anime)
            # Signal pour le bouton pour arrêter (... de visualiser) le gif animé
            self.bout_stop_gif_anime.clicked.connect(self.fonction_stop_gif_anime)
            
            
    def closeEvent(self, event):
        """
        Lancé à la fermeture de la fenêtre quelqu'en soit la méthode
        """
        # si le thread est en cours d'exécution, on l'arrête (brutalement!)
        if self.calcul_thread_gif_anime!=None and self.calcul_thread_gif_anime.isRunning():
            self.calcul_thread_gif_anime.terminate()
        # et on accepte la fermeture de la fenêtre dans tous les cas
        event.accept()


    def fonction_jouer_gif_anime(self) :
        """" Fonction qui met en place le QMovie et joue le gif animé """
        
        # QMovie
        self.animation_gif = QMovie(self.enregistrer_fichier_gif_anime)
        # Redimensionner l'animation gif (au départ : 500 x 720) pour qu'elle
        # puisse être visionnée en 313 x 450 ... dans la visionneuse
        self.animation_gif.setScaledSize(QSize().scaled(313, 450, Qt.IgnoreAspectRatio))
        # L'animation gif est visionnée dans le label au travers de QMovie    
        self.label_vision_gif_anime.setMovie(self.animation_gif)
        
        # On démarre l'animation
        self.animation_gif.start()
        
        
    def fonction_stop_gif_anime(self) :
        """" Fonction qui stoppe le gif animé """
        self.animation_gif.stop()
        
