#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture
# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

import os, sys, platform

# Imports PyQt5 -------------------------------------------------------
from PyQt5.QtWidgets import QFileDialog
# ---------------------------------------------------------------------

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------


class Murexpo_Conversion_Html() :
    """ Classe pour convertir les données en html """
    
    def __init__(self, liste_parametres_export) :
        super().__init__()
        
        self.liste_parametres_export = liste_parametres_export

        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # ===========================================
        # HTML ; écriture de l'entête (html et CSS)
        # ===========================================
        #
        html_entete = '''<html><head><title></title>'''+'''\n'''+'''<style type="text/css">'''+'''\n'''+'''body { background-color:#ffffff;color:#000000 } table { align:center;border-collapse:collapse;border-style:solid;border-width:1px;border-color:black;font-size:7pt;font-family: "DejaVu Sans" } td { border: 1px solid black; } p { line-height : 20px; } div { line-height : 20px; }'''+'''\n'''+'''</style>'''+'''\n'''+'''</head>'''+'''\n'''+'''<body><meta http-equiv="Content-Type" content="text/html; charset=UTF-8">'''+'''\n'''

        # ---------------------------
        # On liste le contenu du répertoire temporaire
        contenu_rep_tempo = os.listdir(self.chemin_tempo_system_exploit)
        # Liste pour capturer l'image sauvegarde_image_scene_finale.png si elle existe dans le rep. tempo
        liste_image_scene_finale = [os.path.join(self.chemin_tempo_system_exploit, visuel) for visuel in contenu_rep_tempo if "sauvegarde_image_scene_finale.png" in visuel]
        # Si l'image existe elle est écrite 
        # dans le fichier html
        # -------------------------------------------------------
        # La résolution initiale du mur d'exposition est de 
        # 2520 x 720, mais ici (dans le futur fichier html et le
        # fichier PDF) l'image sera affichée en 1600 x 457 afin
        # qu'on puisse la voir en entier dans le fichier PDF
        # -------------------------------------------------------
        if len(liste_image_scene_finale) == 1 :
            # Lecture du texte de présentation de l'exposition
            html_presentation_exposition = '''<div align="center"><img src="'''+os.path.basename(liste_image_scene_finale[0])+'''" width="1600" height="457" /></div><p align="center"><b>''' + liste_parametres_export[0][0] + '''</b></p>''' + '''\n'''
        # Si l'image n'existe pas un message 
        # (d'erreur) est écrit dans le fichier html
        elif len(liste_image_scene_finale) == 0 :
            # Lecture du texte de présentation de l'exposition
            html_presentation_exposition = '''<p align="center"><b><u>Problème (!) :</u> votre mur d'exposition (avec l'accrochage des oeuvres) n'apparaît pas ici car vous n'avez pas cliqué sur le bouton Visuel au cours de la construction de l'exposition. Fermez MurExpo et recommencez svp !</b></p>''' + '''\n'''
        # ---------------------------
            
        # Le contenu de liste_parametres_export est du style :
        # ----------------------------------------------------
        # [["Explications sur l'exposition en question ..."], 
        # ['image_001.jpg', "Explications sur l'oeuvre 1 ...", 0, (300, 300)], 
        # ['image_002.jpg', "Explications sur l'oeuvre 2 ...", 1, (300, 218)], 
        # ['image_003.jpg', "Explications sur l'oeuvre 3 ...", 2, (300, 150)], 
        # ['image_004.jpg', "Explications sur l'oeuvre 4 ...", 3, (300, 450)], 
        # ['image_005.jpg', "Explications sur l'oeuvre 5 ...", 4, (300, 228)]]
        
        # =====================================
        # Gestion des oeuvres et commentaires
        # =====================================
        liste_oeuvres = ['''<tr>'''+'''\n'''+'''<td width="2%">'''+'''<div align="center"><img src="'''+oeuvre[0]+'''" width="'''+str(oeuvre[3][0])+'''"'''+''' height="'''
        +str(oeuvre[3][1])+'''" /></div>'''+'''</td>'''+'''\n'''+'''<td width="98%"><div align="left"><font color="#000000" size="3" family="DejaVu Sans"><u>Commentaire :</u> '''+oeuvre[1]+'''</b></font><br /></div></td></tr>''' for oeuvre in liste_parametres_export[1:]]
        # Tous les éléments récoltés dans la liste (toutes les
        # vignettes et les commentaires sur les oeuvres) sont
        # mises une variable de code html
        oeuvres_et_com = ''.join(liste_oeuvres)
        
        # ===========================================
        # HTML ; affichage des images/oeuvres + commentaire
        # ===========================================
        # Si la séquence possède des données pour les références 
        # artistiques, le tableau est construit avec ces données, ...
        #if len(ref_art_com_et_img) > 0 :
        html_oeuvres_et_com = '''<table width="100%" cellpadding="10" cellspacing="10">'''+'''\n'''+str(oeuvres_et_com)+'''\n'''+'''</table>'''+'''\n'''
        
        # ===========================================
        # HTML ; écriture de la fin de la page
        # ===========================================
        html_body_html_final = '''</body></html>'''
            
        # Le html total (l'entête du html et la fermeture du head et du html sont rajoutés)
        # --------------------------------------------------------------------------------------
        # marge_haute agit comme marge tout en haut de la page (pour que les marges du tout 
        # 1er tableau tout en haut soient mieux affichées). C'est modulaire quel que soit la 
        # rubrique affichée tout en haut de la page. marge_haute --> '' ou '<br />' selon
        # le thème choisi.
        # --------------------------------------------------------------------------------------
        self.html_total = html_entete + html_presentation_exposition + html_oeuvres_et_com + html_body_html_final

        # --------------------------------------------------------------------- #
        # Utilisation de la classe Tempo_ecriture
        # pour l'écriture des données dans le 
        # répertoire (dit) temporaire
        # ===================================================================== #
        # Ecriture du fichier html total (sous forme de fichier texte)
        # dans le répertoire temporaire
        # --------------------------------------------------------------------- #
        # L'écriture ici du html_total dans le répertoire de sauvegarde ... 
        # indispensable ici dans le fichier seq_prof_gui.py (voir le commentaire
        # : Détection du fichier html temporaire [--> html_total 
        # de la prépa prof]) dans exec_pdfkit_seq_prof_gui.py
        # ===================================================================== #
        # Appel de la classe Tempo_ecriture
        ecr_tempo_html = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_exposition_html_murexpo_", "data_exposition_html_murexpo", ".txt", self.html_total)
        # --------------------------------------------------------------------- # 
        
        # ================================================= #
        # Sous Windows (au moment de la conversion en html)
        # des problèmes avec les caractères accentuées
        # apparaissent, on a donc besoin d'appeler la 
        # fonction de parcing des accents (remplace le 
        # caractère accentué par son équivalent en html)
        # ================================================= #
        # Sous windows
        if platform.system() == 'Windows' :
            self.fonction_parcing_accents_html()
        
        # --------------------------------------------------------------------- #
        # Appel de la classe Tempo_ecriture pour écriture des données avec extension html
        ecr_extension_html = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "index_exposition_html_", "index_exposition_html", ".html", self.html_total)
        # --------------------------------------------------------------------- #
        

    def fonction_parcing_accents_html(self) :
        """ Cette fonction remplace le caractère 
        accentué par son équivalent en html 
        --> sous windows uniquement """
        
        self.html_total = self.html_total.replace("à", "&agrave;").replace("â", "&acirc;").replace("ä", "&auml;").replace("é", "&eacute;").replace("è", "&egrave;").replace("ê", "&ecirc;").replace("ë", "&euml;").replace("ç", "&ccedil;").replace("î", "&icirc;").replace("ï", "&iuml;").replace("ô", "&ocirc;").replace("ö", "&ouml;").replace("ù", "&ugrave;").replace("û", "&ucirc;").replace("ü", "&uuml;")
