#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture
# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Merci à eyllanesc pour son aide est pour cette classe ... import
# --> pour les icônes à droite dans le bouton
# --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
from push_button_icone_gauche import PushButtonIconeGauche

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

# Import de la classe d'exportation des données en Html
# La page html est ainsi générée
from murexpo_conversion_html import Murexpo_Conversion_Html

# Ligne de séparation verticale (disposée entre les groupes de widgets)
from ligne_horizontale_ligne_verticale import Ligne_Verticale

# Import de la fonction de recherche du navigateur web installé
# pour afficher la documentation dans un onglet du navigateur
# (après coup ici utilisation du module webbrowser)
from recherche_webbrowser_linux import recherche_installation_navigateur_web_linux

import sys, os, shutil, glob, pdfkit, platform, subprocess, platform, webbrowser

from zipfile import ZipFile

# PyMuPDF (installé avec pip3)
# Traitement de toutes les pages 
# pour visualiser la transformation 
# des PDF vers les images PNG
import fitz

# Pillow
try : import Image
except ImportError : from PIL import Image

# Imports PyQt5 -------------------------------------------------------
from PyQt5.QtWidgets import (QWidget, QPushButton, QLabel, QWidget, \
    QHBoxLayout, QVBoxLayout, QApplication, QListWidget, QTextEdit, \
    QListWidgetItem, QLayout, QLineEdit, QFileDialog, QMessageBox, \
    QDialog, QDesktopWidget, QProgressBar, QGridLayout, QLineEdit, \
    QSpinBox, QCheckBox)
from PyQt5.QtCore import (QThread, pyqtSlot, QSize, Qt, QFileInfo, \
    pyqtSignal, QPoint)
from PyQt5.QtGui import QIcon
# ---------------------------------------------------------------------

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------


def traitement_image_pour_visionnage() :
    """ Fonction pour faire le traitement des images 
    résultantes de la conversion html vers PDF. En
    fait une bordure grise (de 10 pixels) est crée
    sur les côtés de chaque image ... ensuite les
    images des pages du PDF sont assemblées sur la
    hauteur pour imiter les plus possible une 
    visualisation comme dans une lecteur PDF """
    
    # Appel de la classe pour récupérer le chemin du 
    # répertoire temporaire selon le système d'exploitation
    chem_rep_tempo = Chemin_Rep_Tempo()
    # Le chemin est retourné (return ds la fonction de la classe)
    chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
    
    # Remise à jour de la liste après renommage
    try :
        images_temporaires = glob.glob(chemin_tempo_system_exploit+os.sep+'pdf_png_murexpo*.png')
        images_temporaires.sort()
    except :
        images_temporaires = glob.glob(os.getcwd()+os.sep+'pdf_png_murexpo*.png')

    # ------ Ajout d'une bordure grise de chaque côté de l'image ------
    for n, i in enumerate(images_temporaires) :
        # Ouverture image
        im = Image.open(i)
        # Vérification de la résolution
        # (en pixels) de l'image
        w, h = im.size
        # Création d'une nouvelle image en ajoutant une épaisseur de 10 pixels (fois 2)
        img_bordure = Image.new('RGB', (w + (2 * 10), h + (2 * 10)), (100, 100, 100))
        # On colle l'épaisseur à l'image
        img_bordure.paste(im, (10, 10))
        # Sauvegarde
        img_bordure.save(os.path.join(chemin_tempo_system_exploit, 'pdf_png_murexpo_'+str(n+1).zfill(5)+'.png'), 'PNG')

    try :
        images_temporaires = glob.glob(chemin_tempo_system_exploit+os.sep+'pdf_png_murexpo*.png')
        images_temporaires.sort()
    except :
        images_temporaires = glob.glob(os.getcwd()+os.sep+'pdf_png_murexpo*.png')
    # -----------------------------------------------------------------

    # =================================================================================
    # Traitement des images ... assemblage des images (représentant 
    # les différentes pages du PDF) sur la hauteur. Visualisation.
    # Trouvé ici :
    # https://stackoverflow.com/questions/53876007/how-to-vertically-merge-two-images
    # =================================================================================
    
    imgs = [Image.open(i) for i in images_temporaires]

    # If you're using an older version of Pillow, you might have to use .size[0] instead of .width
    # and later on, .size[1] instead of .height
    min_img_width = min(i.width for i in imgs)

    total_height = 0
    for i, img in enumerate(imgs):
        # If the image is larger than the minimum width, resize it
        if img.width > min_img_width:
            imgs[i] = img.resize((min_img_width, int(img.height / img.width * min_img_width)), Image.ANTIALIAS)
        total_height += imgs[i].height

    # I have picked the mode of the first image to be generic. You may have other ideas
    # Now that we know the total height of all of the resized images, we know the height of our final image
    img_merge = Image.new(imgs[0].mode, (min_img_width, total_height))
    y = 0
    for img in imgs:
        img_merge.paste(img, (0, y))

        y += img.height
    img_merge.save(chemin_tempo_system_exploit+os.sep+'pdf_png_final.png')


class Murexpo_Export_Html(QWidget):
    """ Classe qui s'occupe de la mise de l'interface / dispositif de 
    traitement de la sortie html (et le traitement html lui-même ... 
    et aussi la sortie en PDF !!!) """
    
    def __init__(self):
        super().__init__()
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # CHemin du répertoire courant
        # (répertoire racine --> ds
        # lequel se trouve le script
        # principal ... INDISPENSABLE !
        # utilisé dans la fonction :
        # enregistrement)
        self.chemin_rep_courant = os.getcwd()

        # Appel de la fonction 
        # listbox_interface
        self.listbox_interface()

        # -----------------------
        # Dictionnaire pour afficher les oeuvres dans 
        # l'interface (et capter le chemin des oeuvres)
        # --> QTextEdit avec setHtml
        # -----------------------
        self.dico_chemin_oeuvres = {}
        # -----------------------
        # Dictionnaire des commentaires des concernant 
        # les images/oeuvres --> QTextEdit
        # -----------------------
        self.dico_commmentaires_oeuvres = {}
        # -----------------------
        # Dictionnaire des QPushButton dans l'interface
        # -----------------------
        self.dico_bout_connect_com = {}
        # -----------------------
        # Dictionnaire pour le label de 
        # redimensionnement des vignettes
        # -----------------------
        self.label_redim_vignettes = {}
        # -----------------------
        # Dictionnaire pour le redimensionnement 
        # des futures vignettes (html et pdf)
        # -----------------------
        self.dico_spin_largeur_img = {}

        # -----------------------
        # Liste pour chargement des données 
        # (oeuvres à prendre en compte)
        # -----------------------
        self.liste_widgets_oeuvres = []
        # -----------------------
        # Liste pour les explications
        # (au niveau général)
        # concernant l'exposition
        # -----------------------
        self.liste_renseignements_expo = []
        # -----------------------
        # Liste pour les valeurs concernant le 
        # redimensionnement des vignettes (html et pdf)
        # -----------------------
        self.liste_largeur_img = []

        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        
    def listbox_interface(self) :
        """ Fonction de préaffichage des widgets """
        
        icone_aff_ajout_outils = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_plus_murexpo_256x256.png')))
        
        # Bouton pour afficher la rangée de widgets (se trouvant dans la listbox)
        self.bouton_affichage_outils_capture = PushButtonIconeGauche(_("Afficher les outils de sortie en html/pdf"), icon = icone_aff_ajout_outils, iconSize = QSize(16, 16))
        self.bouton_affichage_outils_capture.setFixedWidth(400)
        self.bouton_affichage_outils_capture.setToolTip(_("Charge (et affiche) les outils d'exportation en html/pdf"))
        
        # ==========================================================================
        # SIGNAL
        # ==========================================================================
        self.bouton_affichage_outils_capture.clicked.connect(self.widgets_interface)
        # ==========================================================================
        
        # Le bouton est montré/visible
        self.bouton_affichage_outils_capture.show()

        # Création de la listbox
        self.listbox = QListWidget()
        
        self.listbox.setStyleSheet("""QListWidget{background: rgb(150, 150, 150);}""")
        
        # La lstbox est dans 
        # un 1er temps cachée
        self.listbox.hide()

        vbox = QVBoxLayout()
        vbox.addWidget(self.bouton_affichage_outils_capture, alignment=Qt.AlignCenter)
        vbox.addWidget(self.listbox) # Pas d'alignement pour la listbox, c'est normal
        #vbox.setStretch(0,1)
        #vbox.setStretch(1,4)
        
        # Alignement au centre sur la hauteur
        vbox.setAlignment(Qt.AlignVCenter)

        self.setLayout(vbox)


    def widgets_interface(self) :
        """ Fonction d'affichage des widgets """
        
        # Mise en place de deux lignes verticales de 
        # séparation entre qques groupes de widgets
        widget_ligne_verticale_1 = Ligne_Verticale()
        widget_ligne_verticale_2 = Ligne_Verticale()
        
        # ====================================================================== #
        # Mise à jour des commentaires sur le contexte de l'exposition 
        # et les oeuvres (départ 2)
        # ====================================================================== #
        # Récupération du fichier temporaire data_commentaires_expo_oeuvres_sortie_html_pdf (et son chemin)
        # --> dans ce fichier sont écrits (à la fin de la fonction recup_donnees_oeuvres) tous les
        # commentaires par rapport à l'exposition et par rapport aux oeuvres. Le chemin du fichier est
        # inséré dans une liste.
        recup_fichier_com_expo_oeuvre_html_pdf = glob.glob(os.path.join(self.chemin_tempo_system_exploit, 'data_commentaires_expo_oeuvres_sortie_html_pdf*'))
        # Si le fichier est bien présent, c'est à dire que la liste 
        # contient bien un élément ... on récupére le fichier tempo
        if len(recup_fichier_com_expo_oeuvre_html_pdf) == 1 :
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_data_com_expo_oeuvre_html_pdf = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_commentaires_expo_oeuvres_sortie_html_pdf_', 'data_commentaires_expo_oeuvres_sortie_html_pdf')
            # ... avec appel de la méthode affichage
            lecture_data_com_expo_oeuvre_html_pdf = "".join(pre_lecture_data_com_expo_oeuvre_html_pdf.affichage())
            # On supprime les sauts à la ligne 
            data_com_expo_oeuvre_html_pdf = lecture_data_com_expo_oeuvre_html_pdf.split("\n")
            # On récupère le commentaire sur le contexte de l'oeuvre (string)
            data_com_expo_uniquement_html_pdf = data_com_expo_oeuvre_html_pdf[0]
            # On récupère les commentaires sur les oeuvres uniquement sans le
            # commentaire sur le contexte de l'exposition
            self.data_com_oeuvres_uniquement_html_pdf = data_com_expo_oeuvre_html_pdf[1:]
        # ====================================================================== #
        
        try :
            # --------------------------------------------------------------
            # Récup de la largeur des vignettes (défini dans l'onglet
            # Export HTML/PDF) au niveau de la configuartion (onglet
            # Configuration)
            # --------------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_largeur_vignettes = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_taille_vignettes_html_pdf_', 'data_taille_vignettes_html_pdf')
            # ... avec appel de la méthode affichage
            lecture_largeur_vignettes = "".join(pre_lecture_largeur_vignettes.affichage())
            # Conversion en int
            largeur_vignettes = int(lecture_largeur_vignettes)
        except :
            # La valeur est définie
            # à 200 pixels par défaut
            largeur_vignettes = 200
            
        # Affichage d'un message d'info sur comment 
        # procéder --> Boîte de dialogue Oui|Non
        boite = QMessageBox()
        boite.setIcon(QMessageBox.Question)
        boite.setText(_("L'exportation en html ou pdf de votre exposition ne pourra s'effectuer correctement <b><font color='red'>que si vous avez visualisé l'exposition à la fin de l'accrochage des oeuvres (clic sur le bouton Visuel)</font></b>.<br /><b><font color='red'>N'oubliez</font> donc <font color='red'>pas de visualiser le mur d'exposition à chaque modification !</font></b><br /><b>Etes-vous d'accord avec cela ?</b>"))
        boite.setWindowTitle(_("Répondez par Oui ou par Non"))
        boite.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        retour = boite.exec()
        # Si la réponse est Oui le traitement se fait ...
        if retour == QMessageBox.Yes :
        
            # La listbox est maintenant 
            # montrée (!!!)
            self.listbox.show()
    
            # Le QWidget qui acceuille 
            # la listbox
            self.widget = QWidget()
    
            hbox = QHBoxLayout()
            
            # Création des QTextEdit pour expliquer les
            # tenants et aboutissants de l'exposition
            self.textedit_com_exposition = QTextEdit()
            self.textedit_com_exposition.setMaximumWidth(430)
            self.textedit_com_exposition.setMinimumWidth(430)
            self.textedit_com_exposition.setMaximumHeight(60)
            self.textedit_com_exposition.setMinimumHeight(60)

            # ====================================================================== #
            # Mise à jour des commentaires sur le contexte de l'exposition et les
            # oeuvres (récup et affichage du commentaire pour le contexte de l'expo)
            # ====================================================================== #
            # Si le fichier tempo est bien présent, c'est à dire que la liste 
            # contient bien un élément ...
            if len(recup_fichier_com_expo_oeuvre_html_pdf) == 1 : 
                # on affiche la partie réservée ...
                self.textedit_com_exposition.setText(data_com_expo_uniquement_html_pdf)
            else :
                # ... autrement on dispose un texte pour expliquer à
                # l'utilisateur comment utiliser ce champ de texte
                self.textedit_com_exposition.setText("Expliquez ici les raisons de cette exposition (où, quand, comment). Veuillez supprimer ce texte et insérer le votre !")
            # ====================================================================== #
            
            # Insertion dans le QHBoxLayout
            hbox.addWidget(self.textedit_com_exposition)
            hbox.addWidget(widget_ligne_verticale_1)
            
            # ------------------------------------------------------------------
            # La liste self.liste_renseignements_expo récupère le QTextEdit
            # pour le commentaire (les explications de l'exposition)
            self.liste_renseignements_expo.append(self.textedit_com_exposition)
            # ------------------------------------------------------------------
    
            # --------------------------------------------------------------
            # Récup de toutes les images chargées sur le mur d'expo
            # (pendant l'accrochage)
            # --------------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_chemins_images_chargees = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_capture_oeuvres_chargees_murexpo_', 'data_capture_oeuvres_chargees_murexpo')
            # ... avec appel de la méthode affichage
            lecture_chemins_images_chargees = "".join(pre_lecture_chemins_images_chargees.affichage())
            # Split et insertion dans une liste des données
            liste_images_chargees = lecture_chemins_images_chargees.split('\n')
            # Dernière liste pour éliminer les éléments indésirables (par ex: '')
            self.liste_ttes_oeuvres_chargees = [i for i in liste_images_chargees if i != '']

            try :
                # --------------------------------------------------------------
                # Récup des images supprimées (si l'utilisateur en a 
                # supprimé pendant l'accrochage)
                # --------------------------------------------------------------
                # Utilisation de la classe Tempo_lecture pour la lecture 
                # des données dans le répertoire (dit) temporaire
                pre_lecture_chemins_images_supprimees = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_capture_oeuvres_supprimees_murexpo_', 'data_capture_oeuvres_supprimees_murexpo')
                # ... avec appel de la méthode affichage
                lecture_chemins_images_supprimees = "".join(pre_lecture_chemins_images_supprimees.affichage())
                # Split et insertion dans une liste des données
                liste_images_supprimees = lecture_chemins_images_supprimees.split('\n')
                # Dernière liste pour éliminer les éléments indésirables (par ex: '')
                liste_ttes_oeuvres_supprimees = [i for i in liste_images_supprimees if i != '']
            except :
                liste_ttes_oeuvres_supprimees = []
                
            # ------------------------------------------------------------
            # Liste finales des oeuvres à traiter
            # ------------------------------------------------------------
            self.liste_des_oeuvres = list(set(self.liste_ttes_oeuvres_chargees) - set(liste_ttes_oeuvres_supprimees))

            # -------------------------------------------------------------
            # Ré-Ordonnancement de la liste ...
            # -------------------------------------------------------------
            # On remet les images dans le même ordre de chargement (dans
            # le même ordre comme elles ont été chargées par l'utilisateur)
            # -------------------------------------------------------------
            # Dans la comprehension list ... deux boucles pour récupérer l'ordre
            # de chargement ... avec comparaison des fichiers (images)
            liste_oeuvres_en_ordre = [parc_1 for parc_1 in self.liste_ttes_oeuvres_chargees for parc_2 in self.liste_des_oeuvres if parc_1 == parc_2]
            # self.liste_des_oeuvres est vidée
            self.liste_des_oeuvres.clear()
            # La liste self.liste_des_oeuvres prend la
            # valeur de la liste liste_oeuvres_en_ordre
            # ... car la liste self.liste_des_oeuvres
            # est traitée juste en dessous ...
            self.liste_des_oeuvres = liste_oeuvres_en_ordre
            # -------------------------------------------------------------

            # Boucle principale ...
            for n, chemin_images in enumerate(self.liste_des_oeuvres) :
                
                # Création des QTextEdit pour afficher les 
                # oeuvres (et capter le chemin des oeuvres)
                self.dico_chemin_oeuvres[n] = QTextEdit()
                #self.fonte(self.dico_chemin_oeuvres[n]) # Fontion pout l'attribution de la fonte
                self.dico_chemin_oeuvres[n].setMaximumWidth(120)
                self.dico_chemin_oeuvres[n].setMinimumWidth(120)
                self.dico_chemin_oeuvres[n].setMaximumHeight(60)
                self.dico_chemin_oeuvres[n].setMinimumHeight(60)
                # --------------------------------------------------------------------
                # Appel de la fonction de redimensionnement des images (vignettage)
                # La dimension en largeur est de 90 pixels (nouv_w). Dimension
                # affichée dans l'interface graphique de MurExpo au moment de
                # l'enregistrement en html
                self.redim_img(chemin_images, 90)
                # Affichage de l'image dans le QTextEdit
                self.dico_chemin_oeuvres[n].setHtml('<center><img src="{}" width="{}" height="{}" title="{}" />'.format(chemin_images, 90, self.calcHauteur_img, chemin_images))
                # --------------------------------------------------------------------
                # Placement en lecture seule (vignette de l'oeuvre et chemin)
                self.dico_chemin_oeuvres[n].setReadOnly(True)
            
                # Création des QTextEdit pour renseigner 
                # les commentaires sur chaque oeuvre
                self.dico_commmentaires_oeuvres[n] = QTextEdit()
                #self.fonte(self.dico_commmentaires_oeuvres[n]) # Fontion pout l'attribution de la fonte
                self.dico_commmentaires_oeuvres[n].setMaximumWidth(430)
                self.dico_commmentaires_oeuvres[n].setMinimumWidth(430)
                self.dico_commmentaires_oeuvres[n].setMaximumHeight(60)
                self.dico_commmentaires_oeuvres[n].setMinimumHeight(60)
                # Les différents QTextEdit déployés 
                # sont placés en lecture seule
                #self.dico_commmentaires_oeuvres[n].setReadOnly(True)
                
                # Insertion d'un texte pour expliquer à l'utilisateur comment utiliser ce champ de texte
                self.dico_commmentaires_oeuvres[n].setText("<b>Il vous est fortement conseillé d'écrire votre commentaire ici une fois toutes les oeuvres modifiées (ajouter/supprimer), ... de même pour la taille de la future vignette !</b> Expliquez ici les renseignements et commentaires sur cette 'oeuvre'. Veuillez supprimer ce texte et insérer le votre !")

                # QSpinBox de redimensionnement des 
                # vignettes (pour html et pdf)
                self.dico_spin_largeur_img[n] = QSpinBox()
                self.dico_spin_largeur_img[n].setFixedWidth(100)
                self.dico_spin_largeur_img[n].setRange(50, 500)
                self.dico_spin_largeur_img[n].setSingleStep(1)
                
                # Affichage de chaque valeur numérique (spinbox redimensionnement)
                # [:-1] pour omettre le caractère de saut à la ligne --> \n
                try :
                    self.dico_spin_largeur_img[n].setValue(int(rep_tempo_aff_valeur[:-1][n]))
                except : pass
                
                # On liste le contenu du répertoire temporaire ...
                liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)

                # --------------------------------------------------------
                # Exception pour contrôler la permanence du
                # choix du slider et aussi du choix du spinbox
                # --------------------------------------------------------
                try :
                    # ----------
                    # Si Taille de la vignette (en pixels) a déjà
                    # été modifiéé .. Boucle de vérification ...
                    # ----------
                    for contenu in liste_contenu_tmp :
                        # Si le fichier tempo concerné est bien dedans ...
                        if 'data_taille_vignettage_export_html_pdf' in contenu :
                            # Utilisation de la classe Tempo_lecture pour la lecture
                            # des données dans le répertoire (dit) temporaire
                            rep_tempo_lecture_val = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_taille_vignettage_export_html_pdf_', 'data_taille_vignettage_export_html_pdf')
                            # ... avec appel de la méthode affichage
                            # -------------------------------------------
                            # rep_tempo_aff_valeur est ici une liste (!)
                            # -------------------------------------------
                            rep_tempo_aff_valeur = rep_tempo_lecture_val.affichage()
                            try :
                                # Affichage de la valeur numérique pour le spinbox. [:-1] 
                                # pour omettre le caractère de saut à la ligne --> \n
                                self.dico_spin_largeur_img[n].setValue(int(rep_tempo_aff_valeur[:-1][n]))
                            except : pass
                # Si le fichier tempo n'existe pas ...
                except :
                    # Attribution de la valeur dans le spinbox ...
                    self.dico_spin_largeur_img[n].setValue(200)

                # =========================================================================================================
                # Gestion des spinbox de redimensionnement (vignettes dans les pages html ou pages pdf) en statique ...
                # c'est à dire que le dictionnaire faisant référence au spinbox pointe vers une fonction dédiée
                # =========================================================================================================
                # Signaux spécifiques pour le dictionnaire faisant référence au spinbox. On peut utiliser 20 oeuvres
                # ... c'est à dire que l'on a la possibilité de redimensionner 20 vignettes (et pas une de plus !)
                # ---------------------------------------------------------------------------------------------------------
                if n == 0 : self.dico_spin_largeur_img[0].valueChanged.connect(self.fonction_spin_chang_taille_0_html_pdf)
                if n == 1 : self.dico_spin_largeur_img[1].valueChanged.connect(self.fonction_spin_chang_taille_1_html_pdf)
                if n == 2 : self.dico_spin_largeur_img[2].valueChanged.connect(self.fonction_spin_chang_taille_2_html_pdf)
                if n == 3 : self.dico_spin_largeur_img[3].valueChanged.connect(self.fonction_spin_chang_taille_3_html_pdf)
                if n == 4 : self.dico_spin_largeur_img[4].valueChanged.connect(self.fonction_spin_chang_taille_4_html_pdf)
                if n == 5 : self.dico_spin_largeur_img[5].valueChanged.connect(self.fonction_spin_chang_taille_5_html_pdf)
                if n == 6 : self.dico_spin_largeur_img[6].valueChanged.connect(self.fonction_spin_chang_taille_6_html_pdf)
                if n == 7 : self.dico_spin_largeur_img[7].valueChanged.connect(self.fonction_spin_chang_taille_7_html_pdf)
                if n == 8 : self.dico_spin_largeur_img[8].valueChanged.connect(self.fonction_spin_chang_taille_8_html_pdf)
                if n == 9 : self.dico_spin_largeur_img[9].valueChanged.connect(self.fonction_spin_chang_taille_9_html_pdf)
                if n == 10 : self.dico_spin_largeur_img[10].valueChanged.connect(self.fonction_spin_chang_taille_10_html_pdf)
                if n == 11 : self.dico_spin_largeur_img[11].valueChanged.connect(self.fonction_spin_chang_taille_11_html_pdf)
                if n == 12 : self.dico_spin_largeur_img[12].valueChanged.connect(self.fonction_spin_chang_taille_12_html_pdf)
                if n == 13 : self.dico_spin_largeur_img[13].valueChanged.connect(self.fonction_spin_chang_taille_13_html_pdf)
                if n == 14 : self.dico_spin_largeur_img[14].valueChanged.connect(self.fonction_spin_chang_taille_14_html_pdf)
                if n == 15 : self.dico_spin_largeur_img[15].valueChanged.connect(self.fonction_spin_chang_taille_15_html_pdf)
                if n == 16 : self.dico_spin_largeur_img[16].valueChanged.connect(self.fonction_spin_chang_taille_16_html_pdf)
                if n == 17 : self.dico_spin_largeur_img[17].valueChanged.connect(self.fonction_spin_chang_taille_17_html_pdf)
                if n == 18 : self.dico_spin_largeur_img[18].valueChanged.connect(self.fonction_spin_chang_taille_18_html_pdf)
                if n == 19 : self.dico_spin_largeur_img[19].valueChanged.connect(self.fonction_spin_chang_taille_19_html_pdf)
                # ---------------------------------------------------------------------------------------------------------
                # Appel de la fonction de redimensionnement des images (vignettage)
                # On va conserver ces dimensions et les injecter dans la liste :  
                # self.liste_oeuvres_donnees_finales dans la fonction :
                # recup_donnees_oeuvres (--> dimension affichée dans le fichier html)
                # ---------------------------------------------------------------------------------------------------------
                # Ici ce sont les images de la page html (ou pdf) qui sont impactés
                # ---------------------------------------------------------------------------------------------------------
                try : 
                    if n == 0 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[0].value())
                    if n == 1 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[1].value())
                    if n == 2 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[2].value())
                    if n == 3 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[3].value())
                    if n == 4 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[4].value())
                    if n == 5 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[5].value())
                    if n == 6 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[6].value())
                    if n == 7 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[7].value())
                    if n == 8 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[8].value())
                    if n == 9 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[9].value())
                    if n == 10 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[10].value())
                    if n == 11 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[11].value())
                    if n == 12 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[12].value())
                    if n == 13 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[13].value())
                    if n == 14 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[14].value())
                    if n == 15 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[15].value())
                    if n == 16 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[16].value())
                    if n == 17 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[17].value())
                    if n == 18 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[18].value())
                    if n == 19 : redimension_img = self.redim_img(chemin_images, self.dico_spin_largeur_img[19].value())
                except : redimension_img = self.redim_img(chemin_images, int(200))
                # =========================================================================================================
                
                # Label de renseignement pour le redimensionnement des vignettes
                self.label_redim_vignettes[n] = QLabel(_("Taille de la future vignette : "))

                # ====================================================================== #
                # Mise à jour des commentaires par rapport à chaque oeuvre déployée
                # dans le QTextEdit concerné (récup et affichage du commentaire)
                # ====================================================================== #
                # Si le fichier tempo est bien présent, c'est à dire que la liste 
                # contient bien un élément ...
                if len(recup_fichier_com_expo_oeuvre_html_pdf) == 1 :
                    # ------------------------------------------------------
                    # Exception pour contrôler si une erreur survient ...
                    # Exemple de manip qui pourrait déclencher une erreur :
                    # 1) Je charge 3 images, j'enregistre en pdf
                    # 2) Je supprime une des 3 images et je met à jour
                    # en cliquant sur le bouton je mise à jour (la flèche
                    # en boucle), puis j'enregistre en pdf
                    # 3) Je rajoute 2 images, je met à jour (la flèche en
                    # boucle), puis j'enregistre en pdf ==> Sans l'exception
                    # NameError et IndexError, cela générait une erreur :
                    # AttributeError: 'Murexpo_Export_Html' object has no 
                    # attribute 'data_com_oeuvres_uniquement_html_pdf' ...
                    # puis ensuite une autre erreur : IndexError: list 
                    # index out of range
                    # ------------------------------------------------------
                    try :
                        # Quand on recharge/rajoute une oeuvre une oeuvre 
                        # on peut se retrouver dans la situation qu'il
                        # n'y ait pas de commentaire de prévu pour cette
                        # image, ... donc ...
                        if self.data_com_oeuvres_uniquement_html_pdf[n] == "" :
                            # ...  un commentaire est mis d'office ...
                            # (insertion d'un texte pour expliquer à 
                            # l'utilisateur comment utiliser ce ...)
                            self.data_com_oeuvres_uniquement_html_pdf[n] = "<b>Il vous est fortement conseillé d'écrire votre commentaire ici une fois toutes les oeuvres modifiées (ajouter/supprimer), ... de même pour la taille de la future vignette !</b> Expliquez ici les renseignements et commentaires sur cette 'oeuvre'. Veuillez supprimer ce texte et insérer le votre !"
                        #
                        self.dico_commmentaires_oeuvres[n].setText(self.data_com_oeuvres_uniquement_html_pdf[n])
                    except (NameError, IndexError) :
                        pass
                else :
                    # Insertion d'un texte pour expliquer à l'utilisateur comment utiliser ce champ de texte
                    self.dico_commmentaires_oeuvres[n].setText("<b>Il vous est fortement conseillé d'écrire votre commentaire ici une fois toutes les oeuvres modifiées (ajouter/supprimer), ... de même pour la taille de la future vignette !</b> Expliquez ici les renseignements et commentaires sur cette 'oeuvre'. Veuillez supprimer ce texte et insérer le votre !")
    
                # --------------------------------------------------
                # Insertion dans le QHBoxLayout
                hbox.addWidget(self.dico_chemin_oeuvres[n])
                hbox.addWidget(self.dico_commmentaires_oeuvres[n])
                hbox.addWidget(self.label_redim_vignettes[n])
                hbox.addWidget(self.dico_spin_largeur_img[n])
                hbox.addStretch()
                hbox.setSizeConstraint(QLayout.SetFixedSize)
                
                # ----------------------------------------------------------------------
                # Copie des images des oeuvres dans le répertoire temporaire
                # (et ce afin d'afficher + tard les images correctement)
                shutil.copy2(chemin_images, self.chemin_tempo_system_exploit)
                # ----------------------------------------------------------------------
                
                # Capture les widgets dans la liste, par exemple :
                #
                # [['image_001.jpg', <PyQt5.QtWidgets.QTextEdit object at 0x7f859bc7a318>, 0, (300, 300)], 
                # ['image_002.jpg', <PyQt5.QtWidgets.QTextEdit object at 0x7f859bc7a438>, 1, (300, 218)], 
                # ['image_003.jpg', <PyQt5.QtWidgets.QTextEdit object at 0x7f859bc7a558>, 2, (300, 150)], 
                # ['image_004.jpg', <PyQt5.QtWidgets.QTextEdit object at 0x7f859bc7a678>, 3, (300, 450)], 
                # ['image_005.jpg', <PyQt5.QtWidgets.QTextEdit object at 0x7f859bc7a798>, 4, (300, 228)]]
                # 
                self.liste_widgets_oeuvres.append([os.path.basename(chemin_images), self.dico_commmentaires_oeuvres[n], n, redimension_img])

            # 2ème ligne de séparation verticale
            # (entre les groupes de widgets)
            hbox.addWidget(widget_ligne_verticale_2)

            # ----------------------------------------------------
            icone_commun = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
            
            # Le bouton permet de valider les
            # textes entrés par l'utilisateur
            self.bouton_valider_html = QPushButton(_("Enregistrer html"))
            self.bouton_valider_html.setFixedWidth(150)
            self.bouton_valider_html.setIcon(QIcon(icone_commun))
            self.bouton_valider_html.setToolTip(_("Pour enregistrer le fichier html"))
            # Insertion dans le QHBoxLayout
            hbox.addWidget(self.bouton_valider_html)
            self.bouton_valider_html.setStyleSheet('''QPushButton {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }'''
            '''QPushButton:pressed {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #6190F2, stop: 1 #fdfbf7);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }''')
            # ----------------------------------------------------
            icone_maj = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_revenir_aux_reglages_par_defaut_murexpo_256x256.png')))
            # Le bouton permet de mettre à jour les 
            # oeuvres (après par ex en avoir rajouté)
            bouton_maj = QPushButton("")
            bouton_maj.setFixedWidth(50)
            bouton_maj.setIcon(QIcon(icone_maj))
            bouton_maj.setToolTip(_("Permet de mettre à jour après des modifications (par exemple en rajoutant,\nen supprimant des oeuvres ou bien en modifiant la taille des futures vignettes)"))
            # Insertion dans le QHBoxLayout
            hbox.addWidget(bouton_maj)
            bouton_maj.setStyleSheet('''QPushButton {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fbc602, stop: 1 #fb3e02);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #fbc602;
            padding: 3px; }'''
            '''QPushButton:pressed {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #f7ed8b, stop: 1 #fb8a02 );
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #fbc602;
            padding: 3px; }''')
            # ----------------------------------------------------
            # Le bouton permet de valider les
            # textes entrés par l'utilisateur
            self.bouton_valider_pdf = QPushButton(_("Enregistrer pdf"))
            self.bouton_valider_pdf.setFixedWidth(150)
            self.bouton_valider_pdf.setIcon(QIcon(icone_commun))
            self.bouton_valider_pdf.setToolTip(_("Pour enregistrer dans un fichier PDF"))
            # Insertion dans le QHBoxLayout
            hbox.addWidget(self.bouton_valider_pdf)
            self.bouton_valider_pdf.setStyleSheet('''QPushButton {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }'''
            '''QPushButton:pressed {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #6190F2, stop: 1 #fdfbf7);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }''')
            
            icone_aide = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_aide_version_murexpo_256x256.png')))
            # Le bouton d'aide
            self.bouton_aide = QPushButton(_(""))
            self.bouton_aide.setFixedWidth(50)
            self.bouton_aide.setIcon(QIcon(icone_aide))
            self.bouton_aide.setToolTip(_("Pour montrer la démarche d'utilisation"))
            # Insertion dans le QHBoxLayout
            hbox.addWidget(self.bouton_aide)
            self.bouton_aide.setStyleSheet('''QPushButton {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }'''
            '''QPushButton:pressed {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #6190F2, stop: 1 #fdfbf7);
            border-style: solid;
            border-width: 2px;
            border-radius: 8px;
            border-color: #9BB7F0;
            padding: 3px; }''')
            
            self.widget.setLayout(hbox)
    
            # Ligne de widgets
            row = self.widget
            
            # Création d'un item du QListWidget
            itemN = QListWidgetItem()
            
            itemN.setSizeHint(row.sizeHint())
            # Ajout de l'item au QListWidget
            self.listbox.addItem(itemN)
            # Ajout de la ligne de widgets au QListWidget
            self.listbox.setItemWidget(itemN, row)
            
            # Le bouton pour afficher la rangée de 
            # widgets est supprimé (devient invisible)
            self.bouton_affichage_outils_capture.hide()

            # =====================================================
            # Signaux
            # ---------- Récup des données textuelles
            self.dico_chemin_oeuvres[n].textChanged.connect(self.recup_donnees_oeuvres)
            self.dico_commmentaires_oeuvres[n].textChanged.connect(self.recup_donnees_oeuvres)
            self.bouton_valider_html.released.connect(self.fonction_export_html)
            self.bouton_valider_pdf.released.connect(self.fonction_export_pdf)
            # Mise à jour (bouton maj) si modifs (par ex des oeuvres)
            bouton_maj.released.connect(self.fonction_mise_a_jour)
            # Afficher l'aide en ouvrant un onglet dans le navigateur web
            self.bouton_aide.released.connect(self.fonction_aide)
            # =====================================================

        # Si la réponse est Non ... le traitement ne se fait pas ...
        elif retour == QMessageBox.No : return

    
    # ------ Fonctions pour le fonctionnement/traitement (en statique) des spinbox de redimensionnement des vignettes ------ Début -------
    
    def fonction_spin_chang_taille_0_html_pdf(self) :
        """ Fonction de changement de taille pour le widget à l'indice 0. 
        En fait c'est pour le 1er spinbox de redimensionnement des vignettes. 
        Les autres fonctions en dessous correspondent aux (éventuels) 
        spinboxes aux indices 1, 2, 3, ... 20 (20 maximum) """
        
        # Liste pour la mise en place des valeurs dans la liste et ce pour le remplissage
        # du fichier data_taille_vignettage_export_html_pdf contenu dans le rep. tempo
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        # Redimensionnement des vignettes dans les pages html (ou pdf)
        redimension_img = self.redim_img(self.liste_des_oeuvres[0], self.dico_spin_largeur_img[0].value())
        # Ecriture préalable de la donnée dans le rep. tampon ... 
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_1_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[1], self.dico_spin_largeur_img[1].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_2_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[2], self.dico_spin_largeur_img[2].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_3_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[3], self.dico_spin_largeur_img[3].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_4_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[4], self.dico_spin_largeur_img[4].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_5_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[5], self.dico_spin_largeur_img[5].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_6_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[6], self.dico_spin_largeur_img[6].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_7_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[7], self.dico_spin_largeur_img[7].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_8_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[8], self.dico_spin_largeur_img[8].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_9_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[9], self.dico_spin_largeur_img[9].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_10_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[10], self.dico_spin_largeur_img[10].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_11_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[11], self.dico_spin_largeur_img[11].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_12_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  str(self.dico_spin_largeur_img[12].value())+'\n', '200\n', '200\n', '200\n', '200\n', '200\n', \
                                  '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[12], self.dico_spin_largeur_img[12].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_13_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  str(self.dico_spin_largeur_img[12].value())+'\n', str(self.dico_spin_largeur_img[13].value())+'\n', \
                                  '200\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[13], self.dico_spin_largeur_img[13].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_14_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  str(self.dico_spin_largeur_img[12].value())+'\n', str(self.dico_spin_largeur_img[13].value())+'\n', \
                                  str(self.dico_spin_largeur_img[14].value())+'\n', '200\n', '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[14], self.dico_spin_largeur_img[14].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_15_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  str(self.dico_spin_largeur_img[12].value())+'\n', str(self.dico_spin_largeur_img[13].value())+'\n', \
                                  str(self.dico_spin_largeur_img[14].value())+'\n', str(self.dico_spin_largeur_img[15].value())+'\n', \
                                  '200\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[15], self.dico_spin_largeur_img[15].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_16_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  str(self.dico_spin_largeur_img[12].value())+'\n', str(self.dico_spin_largeur_img[13].value())+'\n', \
                                  str(self.dico_spin_largeur_img[14].value())+'\n', str(self.dico_spin_largeur_img[15].value())+'\n', \
                                  str(self.dico_spin_largeur_img[16].value())+'\n', '200\n', '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[16], self.dico_spin_largeur_img[16].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_17_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  str(self.dico_spin_largeur_img[12].value())+'\n', str(self.dico_spin_largeur_img[13].value())+'\n', \
                                  str(self.dico_spin_largeur_img[14].value())+'\n', str(self.dico_spin_largeur_img[15].value())+'\n', \
                                  str(self.dico_spin_largeur_img[16].value())+'\n', str(self.dico_spin_largeur_img[17].value())+'\n', \
                                  '200\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[17], self.dico_spin_largeur_img[17].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_18_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  str(self.dico_spin_largeur_img[12].value())+'\n', str(self.dico_spin_largeur_img[13].value())+'\n', \
                                  str(self.dico_spin_largeur_img[14].value())+'\n', str(self.dico_spin_largeur_img[15].value())+'\n', \
                                  str(self.dico_spin_largeur_img[16].value())+'\n', str(self.dico_spin_largeur_img[17].value())+'\n', \
                                  str(self.dico_spin_largeur_img[18].value())+'\n', '200\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[18], self.dico_spin_largeur_img[18].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
    
    
    def fonction_spin_chang_taille_19_html_pdf(self) :
        self.liste_largeur_img = [str(self.dico_spin_largeur_img[0].value())+'\n', str(self.dico_spin_largeur_img[1].value())+'\n', \
                                  str(self.dico_spin_largeur_img[2].value())+'\n', str(self.dico_spin_largeur_img[3].value())+'\n', \
                                  str(self.dico_spin_largeur_img[4].value())+'\n', str(self.dico_spin_largeur_img[5].value())+'\n', \
                                  str(self.dico_spin_largeur_img[6].value())+'\n', str(self.dico_spin_largeur_img[7].value())+'\n', \
                                  str(self.dico_spin_largeur_img[8].value())+'\n', str(self.dico_spin_largeur_img[9].value())+'\n', \
                                  str(self.dico_spin_largeur_img[10].value())+'\n', str(self.dico_spin_largeur_img[11].value())+'\n', \
                                  str(self.dico_spin_largeur_img[12].value())+'\n', str(self.dico_spin_largeur_img[13].value())+'\n', \
                                  str(self.dico_spin_largeur_img[14].value())+'\n', str(self.dico_spin_largeur_img[15].value())+'\n', \
                                  str(self.dico_spin_largeur_img[16].value())+'\n', str(self.dico_spin_largeur_img[17].value())+'\n', \
                                  str(self.dico_spin_largeur_img[18].value())+'\n', str(self.dico_spin_largeur_img[19].value())+'\n']
        redimension_img = self.redim_img(self.liste_des_oeuvres[19], self.dico_spin_largeur_img[19].value())
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(self.liste_largeur_img))
        
    # ------ Fonctions pour le fonctionnement/traitement (en statique) des spinbox de redimensionnement des vignettes ------ Fin ---------
    
    
    def fonction_aide(self) :
        """ Cette fonction affiche l'aide (par l'intermédiaire d'un 
        onglet dans le navigateur). Utilisation du module webbrowser """
        
        # Sous Windows
        if platform.system() == 'Windows' :
            navigateur = webbrowser.get('windows-default')
        # Sous MacOS
        elif platform.system() == 'Darwin' :
            try :
                navigateur = webbrowser.get('macosx')
            except : navigateur = webbrowser.get('safari')
        # Sous GNU/Linux
        elif platform.system() == 'Linux' :
            # Appel de la fonction 
            # recherche_installation_navigateur_web_linux
            # présente ds le module recherche_webbrowser_linux
            recherche_nav = recherche_installation_navigateur_web_linux()
            navigateur = webbrowser.get(recherche_nav)
            
        # On ouvre le lien de la doc
        navigateur.open('https://murexpo.org/me/exportation-en-html-ou-en-pdf')

    
    def fonction_mise_a_jour(self) :
        """ Fonction permettant de mettre à jour la ligne de 
        widgets et ainsi (après coup) pouvoir ré-Appliquer 
        les changements """

        # Si aucune ligne de widgets n'a été 
        # sélectionnée par l'utilisateur ...
        if self.listbox.currentRow() == -1 :
            # -------------------------------
            # Captation de la ligne de widgets
            # -------------------------------
            row = self.listbox.count() - 1
        # Autrement c'est la ligne de widgets 
        # sélectionnée par l'utilisateur ...
        else :
            #  ... qui est supprimée
            # -------------------------------
            # Captation de la ligne de widgets
            # -------------------------------
            row = self.listbox.currentRow()
    
        # Sélection de l'item (ligne de widgets)
        item = self.listbox.takeItem(row)
        
        # Controle pour que index du row ne soit pas 
        # négatif sur bouton supprimer(Out of range)
        if row < 0 : return
        
        # Supression de la ligne de widgets
        del item
        
        # ===============================================================
        # Re-création des dictionnaires pour les widgets
        # ===============================================================
        # -----------------------
        # Dictionnaire ...
        # -----------------------
        self.dico_chemin_oeuvres = {}
        # -----------------------
        # Dictionnaire des commentaires des concernant les images/oeuvres
        # -----------------------
        self.dico_commmentaires_oeuvres = {}
        # -----------------------
        # Dictionnaire des QPushButton de ...
        # -----------------------
        self.dico_bout_connect_com = {}
        # -----------------------
        # Liste pour chargement des données 
        # (oeuvres à prendre en compte)
        # -----------------------
        self.liste_widgets_oeuvres = []
        # -----------------------
        # Liste pour les explications
        # (au niveau général)
        # concernant l'exposition
        # -----------------------
        self.liste_renseignements_expo = []
        # ===============================================================
        
        # ---------------------------------------------------
        # Très important aussi (!) La liste de récupération
        # des données est vidée ... elle va contenir 
        # uniquementles les fichiers mis à jour (ajout et
        # suppression si l'utilisateur choisit de modifier
        # les images du mur d'exposition)
        try :
            self.liste_oeuvres_donnees_finales.clear()
        except : pass
        # ---------------------------------------------------
        
        # --------------------------------
        # On relance la fonction de 
        # création de la ligne de widgets
        self.widgets_interface()
        # --------------------------------
        

    def recup_donnees_oeuvres(self) :
        """ Fonction de récupération des données """
        
        # ==============================================
        # self.liste_widgets_oeuvres reçoit les 
        # données entrées dans les QTextEdit.
        # --> La donnée à l'indice 0 (dans chaque 
        # sous liste pour la gestion des images) ...
        # c'est le chemin de chargement de chaque image
        # ==============================================
        self.liste_oeuvres_donnees_finales = [[donnees[0], str(donnees[1].toPlainText()), donnees[2], donnees[3]] for donnees in self.liste_widgets_oeuvres]

        # Récupération des données textuelles de la liste self.liste_renseignements_expo
        liste_explications_expo = [str(data_explications_expo.toPlainText()) for data_explications_expo in self.liste_renseignements_expo]
        
        # Insertion à l'indice 0 des explications de l'exposition
        # on aura donc des données finales du style :
        # [["Explications sur l'exposition en question ..."], 
        # ['image_001.jpg', "Explications sur l'oeuvre 1 ...", 0, (300, 300)], 
        # ['image_002.jpg', "Explications sur l'oeuvre 2 ...", 1, (300, 218)], 
        # ['image_003.jpg', "Explications sur l'oeuvre 3 ...", 2, (300, 150)], 
        # ['image_004.jpg', "Explications sur l'oeuvre 4 ...", 3, (300, 450)], 
        # ['image_005.jpg', "Explications sur l'oeuvre 5 ...", 4, (300, 228)]]
        self.liste_oeuvres_donnees_finales.insert(0, liste_explications_expo)

        # **********************************************************************
        
        # Liste permettant de mettre en relation le nom des oeuvres 
        # et le commentaire correspondant, par exemple :
        # [['thomas_barbey_rack_n_roll.jpg', "Expliquez ici les renseignements et 
        # commentaires sur cette 'oeuvre'. Veuillez supprimer ce texte et insérer 
        # le votre !"], ['tom_wesselmann_still_life_20_vue_01.jpg', 'Tom Wesselmann 
        # c'est chouette !'], ['dans_flavin_untitled_to_donna_5a_1971.jpg', 
        # "Expliquez ici les renseignements et commentaires sur cette 'oeuvre'. 
        # Veuillez supprimer ce texte et insérer le votre !"]]
        liste_oeuvres_et_com = [[parcours_1[0], parcours_1[1]] for parcours_1 in self.liste_oeuvres_donnees_finales[1:]]

        # ====================================================================== #
        # Mise à jour des commentaires sur le contexte de l'exposition 
        # et les oeuvres (départ 1)
        # ====================================================================== #
        # Liste permettant de sélectionner (théoriquement) les bons commentaires
        # sur les oeuvres en comparant le nom des oeuvres dans la liste
        # liste_oeuvres_et_com (que l'index qui mentionne les oeuvres) et la
        # partie contenant le nom des oeuvres dans la liste 
        # self.liste_oeuvres_donnees_finales. Cela permet de mettre en lumière 
        # le commentaire qui correspond (MAIS LA CA MARCHE PAS ENCORE 
        # CORRECTEMENT ! SI UTILISE POUR LES COMMENTAIRES De CHAQUE OEUVRE !)
        liste_explications_expo_et_oeuvres = [parc_1[1]+"\n" for parc_1 in liste_oeuvres_et_com for parc_2 in self.liste_oeuvres_donnees_finales[1:] if parc_1[0] == parc_2[0]]
        # Insertion du commentaire sur le contexte de l'oeuvre (POUR
        # L'INSTANT IL N4Y AURA QUE Ce COMMENTAIRE QUI POURRA ETRE
        # RECHARGE ET MIS A JOUR)
        liste_explications_expo_et_oeuvres.insert(0, "".join(liste_explications_expo)+"\n")
        # Ecriture de tous les commentaires (dans le rep tempo) malgré tout !
        # MEME SI (dans la fonction widgets_interface) LES COMMENTAIRES SUR
        # CHAQUE OEUVRE NE SONT PAS UTILISES A CE NIVEAU
        ecr_tempo_commentaires_expo_et_oeuvre = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_commentaires_expo_oeuvres_sortie_html_pdf_", "data_commentaires_expo_oeuvres_sortie_html_pdf", ".txt", "".join(liste_explications_expo_et_oeuvres))
        # ====================================================================== #
        
        # **********************************************************************


    def fonction_export_html(self) :
        """ Fonction dédiée à l'exportation --> récupération des
        données, appel de la classe pour construire la page html 
        et enregistrement (appel de la boîte de dialogue 
        d'enregistrement et création de l'archive zip """
        
        # Appel de la fonction de 
        # recupération des données
        self.recup_donnees_oeuvres()

        # Appel de la classe pour faire le traitement html 
        self.export_html = Murexpo_Conversion_Html(self.liste_oeuvres_donnees_finales)
        
        # Appel de la fonction d'entregistrement du
        # zip contenant le fichier html et les images
        self.enregistrement_zip_html()
        
        # Si le fichier zip (qui vient
        # d'être enregistré) existe
        if os.path.exists(self.enregistrer_html_zip) :
            # -----------------------------------------------------
            # Appel de la classe Fenetre_barre_sortie_html_vers_img
            # -----------------------------------------------------
            self.barre_html_vers_img = Fenetre_barre_sortie_html_vers_img()  
        
        
    def fonction_export_pdf(self) :
        """ Fonction dédiée à l'exportation sous la forme 
        d'un document PDF récupération des données, appel 
        de la classe pour construire la page html et 
        enregistrement et conversion en fichier PDF """
        
        # Appel de la fonction de 
        # recupération des données
        self.recup_donnees_oeuvres()

        # Appel de la classe pour faire le traitement html 
        self.export_html = Murexpo_Conversion_Html(self.liste_oeuvres_donnees_finales)
        
        # Appel de la fonction 
        # d'entregistrement du
        # fichier pdf
        self.enregistrement_pdf()
        
        # ---------------------------------------------------------
        # Appel de la classe Fenetre_barre_sortie_html_pdf_vers_pdf
        # ---------------------------------------------------------
        self.barre_html_vers_pdf = Fenetre_barre_sortie_html_pdf_vers_pdf()

        
    def redim_img(self, chemin, nouv_w) :
        ''' Fonction de redimensionnement des images '''

        # Redimensionnnement de l'image pour affichage dans le QTextEdit

        # Largeur de la future vignette
        self.nouv_w = nouv_w
        # Ouverture de l'image
        obImg = Image.open(chemin)
        # Recup dimensions de l'image
        w, h = obImg.size
        # Calcul du ratio de l'image
        ratio = float(w)/float(h)
        # Calcul de future hauteur avec les dimensions donnees par l'utilisateur
        self.calcHauteur_img = int(float(self.nouv_w)/ratio)

        # Retourne le tuple : (largeur, hauteur) de chaque image
        self.tuple_dim_img_vignette = (self.nouv_w, self.calcHauteur_img)
        
        return self.tuple_dim_img_vignette
    
    
    def enregistrement_zip_html(self) :
        """ Fonction d'enregistrement/export de l'archive 
        zip contenant le fichier html et les images """
        
        # ------------------------------
        # Appel des données récoltées dans la fonction :
        # repertoire_chemin_configuration (voir la condition
        # juste en dessous !)
        # ------------------------------        
        # Si le répertoire par défaut de chargement du fichier
        # html a été modifié par l'utilisateur dans l'onglet 
        # Configuration > Chemins (chargement et sauvegarde), 
        # les données récoltées par la fonction : 
        # repertoire_chemin_configuration ... sont utilisées 
        # (comme chemin par défaut) lors de la sauvegarde dans 
        # la boîte de dialogue ... autrement c'est le répertoire
        # de l'utilisateur qui est utilisé ...
        try :
            rep_defaut_chargement = self.repertoire_chemin_configuration("data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo")
        except :
            rep_defaut_chargement = os.path.expanduser('~')
        # ------------------------------

        # Sauvegarde en html sur le disque dur La boîte d'enregistrement
        enregistrer_html = QFileDialog.getSaveFileName(self, _("Enregistrer le html dans un fichier zip"), rep_defaut_chargement, _("Fichier zip (.zip) (*.zip)"))[0]

        if not QFileInfo(enregistrer_html).suffix(): enregistrer_html += ".zip"
        
        # Récup du chemin et nom de fichier
        self.enregistrer_html_zip = str(enregistrer_html)

        print(_("Chemin de l'enregistrement du fichier html en zip :"), self.enregistrer_html_zip)
        
        # Ecriture du chemin du code html zippé et enregistré
        ecr_tempo_chem_fichier_zip = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chemin_sauv_html_zip_", "data_chemin_sauv_html_zip", ".txt", self.enregistrer_html_zip)

        # On liste le contenu du répertoire temporaire
        contenu_rep_tempo = os.listdir(self.chemin_tempo_system_exploit)
        # On vérifie la présence du fuchier html dans le rep. tempo
        liste_fichier_html = [os.path.join(self.chemin_tempo_system_exploit, html) for html in contenu_rep_tempo if "index_exposition_html" in html]
        # On vérifie si l'image finale du mur d'exposition est bien dans le rep. tempo
        liste_fichier_mur_d_exposition = [os.path.join(self.chemin_tempo_system_exploit, image_mur) for image_mur in contenu_rep_tempo if "sauvegarde_image_scene_finale" in image_mur]
        # Liste des oeuvres importées et accrochées sur le mur d'exposition
        liste_oeuvres_uniquement = [os.path.join(self.chemin_tempo_system_exploit, oeuvre[0]) for oeuvre in self.liste_oeuvres_donnees_finales[1:]]

        # ------------------------------------------------------------
        # Appel de la fonction pour la création de l'archive
        # zip (archive zip qui va contenir le fichier html,
        # le mur d'exposition avec les oeuvres accrochées 
        # dessus et les oeuvres en miniatures ... appel
        # des arguments ...)
        # ------------------------------------------------------------
        self.fonction_creation_zip(self.chemin_tempo_system_exploit, \
                              self.enregistrer_html_zip, \
                              liste_oeuvres_uniquement, \
                              liste_fichier_html, \
                              liste_fichier_mur_d_exposition)

        # =======================================
        # On se repositionne dans le répertoire 
        # courant (TRES TRES IMPORTANT !)
        # =======================================
        os.chdir(self.chemin_rep_courant)
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Info) Exportation en html. Chemin de l'enregistrement du fichier html en zip : : {}\n").format(self.enregistrer_html_zip))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
                
    def fonction_creation_zip(self, chemin_tempo, enregistrer_html_zip, l_oeuvres, l_html, l_mur_accrochage) :
        """ Fonction de création de l'archive zip """
    
        # On se positionne dans le répertoire temporaire
        # (indispensable pour le "zippage")
        os.chdir(chemin_tempo)
        # Création de l'archive zip
        with ZipFile(enregistrer_html_zip, 'w') as zipobj :
            for obj in l_oeuvres :
                # On ajoute les oeuvres à l'archive zip
                zipobj.write(os.path.basename(obj))
            # Si la liste contenant (théoriquement) le
            # fichier html contient bien le fichier
            if len(l_html) == 1 :
                # Le fichier html est ajouté à l'archive
                zipobj.write(os.path.basename(l_html[0]))
            # Si la liste contenant (théoriquement) le
            # mur d'exposition contient bien l'image
            if len(l_mur_accrochage) == 1 :
                # L'image est ajoutée à l'archive
                zipobj.write(os.path.basename(l_mur_accrochage[0]))
        

    def repertoire_chemin_configuration(self, chem_tempo_1, chem_tempo_2) :
        ''' Fonction de mise en place du répertoire de chargement par défaut 
        des images (par ex ici pour la sauvegarde de l'exposition sous la
        forme d'une image), si modifs faites par l'utilisateur dans l'onglet
        Configuration > Chemins (chargement et sauvegarde) '''
        
        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
              
        # ----------
        # Si le chargement du répertoire a déjà été modifié
        # ... Boucle de vérification ...
        # ----------
        for contenu in liste_contenu_tmp :
            # Si le fichier tempo concerné est bien dedans ...
            if chem_tempo_1 in contenu :
                # ------------------------------------------------------- #
                # Utilisation de la classe Tempo_lecture pour la lecture
                # des données dans le répertoire (dit) temporaire
                # ------------------------------------------------------- #
                rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), chem_tempo_1, chem_tempo_2)
                # ... avec appel de la méthode affichage
                rep_tempo_chemin = rep_tempo_lecture.affichage()[0]
                # S'il y a bien une donnée récupérée ...
                # celle-ci est affectée
                if rep_tempo_chemin :
                    rep_defaut = rep_tempo_chemin
                    return rep_defaut
                
          
    def enregistrement_pdf(self) :
        """ Fonction d'enregistrement/export du fichier html en pdf """
        
        # ------------------------------
        # Appel des données récoltées dans la fonction :
        # repertoire_chemin_configuration (voir la condition
        # juste en dessous !)
        # ------------------------------        
        # Si le répertoire par défaut de chargement du fichier
        # html a été modifié par l'utilisateur dans l'onglet 
        # Configuration > Chemins (chargement et sauvegarde), 
        # les données récoltées par la fonction : 
        # repertoire_chemin_configuration ... sont utilisées 
        # (comme chemin par défaut) lors de la sauvegarde dans 
        # la boîte de dialogue ... autrement c'est le répertoire
        # de l'utilisateur qui est utilisé ...
        try :
            rep_defaut_chargement = self.repertoire_chemin_configuration("data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo")
        except :
            rep_defaut_chargement = os.path.expanduser('~')
        # ------------------------------

        # Sauvegarde en pdf sur le disque dur La boîte d'enregistrement
        enregistrer_pdf = QFileDialog.getSaveFileName(self, _("Enregistrer l'exposition sous la forme d'un fichier PDF"), rep_defaut_chargement, _("Fichier pdf (.pdf) (*.pdf)"))[0]

        if not QFileInfo(enregistrer_pdf).suffix(): enregistrer_pdf += ".pdf"
        
        # Récup du chemin et nom de fichier
        self.enregistrer_pdf = str(enregistrer_pdf)

        print(_("Chemin de l'enregistrement du fichier pdf :"), self.enregistrer_pdf)
        
        # Ecriture du chemin du code html zippé et enregistré
        ecr_tempo_chem_fichier_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chemin_sauv_pdf_", "data_chemin_sauv_pdf", ".txt", self.enregistrer_pdf)

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Info) Exportation sous la forme d'un fichier PDF. Chemin de l'enregistrement en PDF : {}\n").format(self.enregistrer_pdf))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


class CalculThreadHtmlImg(QThread):
    ''' Partie du Thread qui s'exécute en tâche de fond 
    (dont le code exécuté ---> dans la fonction run) '''
    
    # création du signal
    fini = pyqtSignal(bool) # signal pour la fin du thread
 
    def __init__(self, parent=None):
        super().__init__(parent)
 
        self.stop = False # drapeau pour exécuter une demande d'arrêt
        
    def recherche_installation_wkhtmltoimg_linux(self, log) :
        """ Fonction de recherche du chemin de wkhtmltoimgage 
        ... si l'application n'est pas contenue dans 
        l'arborescence de MurExpo sous GNU/Linux. Cette fonction 
        est très utile si MurExpo est installé sur le disque dur 
        sous GNU/Linux et qu'elle est préalablement compilée avec 
        le script shell murexpo_build.sh """
        # Liste pour récupérer les chemins
        liste_chem_wkhtmltoimage = []
        # Recherche du chemin sur 3 niveaux
        # par exemple :
        # /usr/bin/wkhtmltoimage ... ou ...
        # /usr/local/bin/wkhtmltoimage ... ou ...
        # ... /usr/...autre chemin
        for n in range(3) :
            # On ne recherche qu'à partir de /usr/...
            recherche = glob.glob("/usr"+n*"/*"+"/"+log)
            for rech in recherche :
                # Il faut obligatoirement qu'il y ait 
                # un répertoire bin dans le chemin ...
                if 'bin' in rech :
                    liste_chem_wkhtmltoimage.append(rech)
        # On ne renvoie que la 1ère occurrence de la liste
        return liste_chem_wkhtmltoimage[0]
         
    
    def run(self):
        """ Partie qui s'exécute en tâche de fond """
        
        try :
            
            # Appel de la classe pour récupérer le chemin du 
            # répertoire temporaire selon le système d'exploitation
            chem_rep_tempo = Chemin_Rep_Tempo()
            # Le chemin est retourné (return ds la fonction de la classe)
            chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
            
            # ---------------------------------------------------------------------------------------
            # Chemin du fichier html source (dans le répertoire temporaire)
            fichier_html = os.path.join(chemin_tempo_system_exploit, "index_exposition_html_.html")
            # Chemin de l'image jpeg après conversion
            image_sortie = os.path.join(chemin_tempo_system_exploit, "html_vers_image.jpg")
            # ---------------------------------------------------------------------------------------
        
            # Sous windows
            if platform.system() == 'Windows' :
                # Sous windows permet de récupérer le fichier html (même avec le module tempfile dans le rep. tempo)
                fich_html_temp_windows = [fichier for fichier in os.listdir(chemin_tempo_system_exploit) if os.path.splitext(fichier)[1] == '.html']
                
                # Sous windows wkhtmltoimage.exe se trouve à la racine
                chem_wkhtmltoimage_windows = os.path.join(os.getcwd(), "wkhtmltoimage.exe")

                # La commande ...
                commande = "{} --quiet --format jpeg --enable-local-file-access {} {}".format(chem_wkhtmltoimage_windows, os.path.join(chemin_tempo_system_exploit, fich_html_temp_windows[0]), os.path.join(chemin_tempo_system_exploit, "html_vers_image.jpg"))
                
                # Traitement avec subprocess
                subprocess.call(commande, shell = True)

            # Sous GNU/Linux
            elif platform.system() == 'Linux' :
                # ----------------------------------------------------------------------------------------------------
                # Le chemin de wkhtmltoimage sera complètement différent dans le cas où MurExpo est compilé avec 
                # pyinstaller
                # (Cas 1) --> le wkhtmltoimage exécuté sera celui contenu dans l'arborescence de MurExpo (version
                # onefile et onedir)
                # (Cas 2) --> /usr/bin/wkhtmltoimage ... avec le script shell murexpo_build.sh (création d'un 
                # paquet deb)
                # ----------------------------------------------------------------------------------------------------
                # Cas 1 
                chem_wkhtmltoimage_linux_1 = os.path.join(os.getcwd(), "wkhtmltox_linux", "wkhtmltoimage")
                # Cas 2
                chem_wkhtmltoimage_linux_2 = self.recherche_installation_wkhtmltoimg_linux('wkhtmltoimage')
                
                # Si wkhtmltoimage est bien dans l'arborescence de MurExpo ...
                if os.path.exists(chem_wkhtmltoimage_linux_1) == True :
                    # La commande ...
                    commande = "{} --quiet --format jpeg {} {}".format(chem_wkhtmltoimage_linux_1, os.path.join(chemin_tempo_system_exploit, "index_exposition_html_.html"), os.path.join(chemin_tempo_system_exploit, "html_vers_image.jpg"))
                # Si wkhtmltoimage n'est pas dans l'arborescence de MurExpo ... c'est le 
                # wkhtmltoimage installé sur le dd qui va prendre le relais pour 
                # faire les traitements
                # --> il faut que wkhtmltoimage soit bien installé sur le dd !
                else :
                    # La commande ...
                    commande = "{} --quiet --format jpeg {} {}".format(chem_wkhtmltoimage_linux_2, os.path.join(chemin_tempo_system_exploit, "index_exposition_html_.html"), os.path.join(chemin_tempo_system_exploit, "html_vers_image.jpg"))
                    
                # Traitement avec subprocess
                subprocess.call(commande, shell = True)

            # Sous MacOS
            elif platform.system() == 'Darwin' :
                # Le chemin sous MacOS
                chem_wkhtmltoimage_macos = os.path.join(os.getcwd(), "wkhtmltox_macos", "wkhtmltoimage")
                # La commande ...
                commande = "{} --quiet --format jpeg --enable-local-file-access {} {}".format(chem_wkhtmltoimage_macos, os.path.join(chemin_tempo_system_exploit, "index_exposition_html_.html"), os.path.join(chemin_tempo_system_exploit, "html_vers_image.jpg"))
                
                # Traitement avec subprocess
                subprocess.call(commande, shell = True)

            # Signale que la code exécuté 
            # dans run vient de se terminer    
            self.fini.emit(self.stop)
            
        except : return
        
        
    def arreter(self):
        """ Pour arrêter avant la fin normale d'exécution du thread """
        self.stop = True


class Fenetre_barre_sortie_html_vers_img(QDialog):
    ''' La partie graphique du thread '''

    def __init__(self, parent=None):
        super(Fenetre_barre_sortie_html_vers_img, self).__init__(parent)
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        ## QApplication.ProcessEvents() (si nécessaire)
        
        self.setWindowTitle("MurExpo : Traitement html")
        # Dimensionnement de la fenêtre
        self.resize(500, 80)

        # --------------------------------------------------------------------------------
        # Lecture du fichier tempo. data_type_de_resolution_fenetre_principale_murexpo
        # ... afin de savoir si le fichier contient "Résolution programmée" ou bien
        # "Résolution au choix"
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_type_de_resolution = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_type_de_resolution_fenetre_principale_murexpo_', 'data_type_de_resolution_fenetre_principale_murexpo')
        # ... avec appel de la méthode affichage
        self.type_de_resolution = "".join(pre_lecture_type_de_resolution.affichage())
        
        # -----------------------------------------------------------------
        # Positionnement du QDialog ... à la fin du traitement. Ceci
        # fait en sorte que cette boîte de dialogue se positionne bien
        # au milieu ... en largeur et en hauteur
        # -----------------------------------------------------------------
        # Par défaut --> Résolution programmée
        if self.type_de_resolution == "Résolution programmée" :
            # Placement du QDialog
            geom = self.frameGeometry()
            centre = QDesktopWidget().availableGeometry().center()
            geom.moveCenter(centre)
            self.move(geom.topLeft())
        # Si l'utilisateur choisit Résolution au choix dans :
        # Configuration > Résolution
        elif self.type_de_resolution == "Résolution au choix" :
            # Le fichier data_resolution_desktop_murexpo 
            # (dans le rep. tempo) est ouvert (lecture)
            pre_lecture_resolution_desktop = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_resolution_desktop_murexpo_', 'data_resolution_desktop_murexpo')
            # La donnée contenue ds le fichier est récupérée (elle est
            # sous la forme [155x100], par exemple, puis elle est splitée.
            self.resolution_desktop = "".join(pre_lecture_resolution_desktop.affichage()).split('x')
            # Placement du QDialog
            geom = self.frameGeometry()
            # La boîte de dialogue est placée bien au milieu ...
            geom.moveCenter(QPoint(int(self.resolution_desktop[0]) / 2, int(self.resolution_desktop[1]) / 2))
            self.move(geom.topLeft())
            
        # ------------------------------------------------
        # Création des QTextEdit pour afficher les 
        # oeuvres (et capter le chemin des oeuvres)
        self.visionneuse_image_html = QTextEdit()
        self.visionneuse_image_html.setMaximumWidth(1200)
        self.visionneuse_image_html.setMinimumWidth(1200)
        self.visionneuse_image_html.setMaximumHeight(700)
        self.visionneuse_image_html.setMinimumHeight(700)
        # Le QTextEdit est caché par défaut
        self.visionneuse_image_html.hide()
        
        # Les boutons pour le lancement du thread et d'arrêt anticipé du thread
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe --> elle est ici : affichage_apsc/push_button_icone_gauche.py ...
        # en fait le texte dans le bouton est centré et l'icône positionné à gauche.
        # En fait j'ai un peu modifié la classe ...
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : config_apsc/chemin_data_ressources.py)
        icone_lancer_traitement = QIcon(chemin_ressources(os.path.join('ressources_apsc', 'icone_lancer_le_traitement_apsc_256x256.png')))
        icone_arreter_traitement = QIcon(chemin_ressources(os.path.join('ressources_apsc', 'icone_arreter_le_traitement_apsc_256x256.png')))
        # Bouton de lancement du thread
        self.depart = PushButtonIconeGauche(_("Lancer le traitement"), icon = icone_lancer_traitement, iconSize = QSize(16, 16))
        self.depart.clicked.connect(self.lancement)
        # ----------------------------------------------------------------------------
        # Ligne d'édition pour afficher le chemin où a
        # été enregistré le fichier html une fois zippé
        self.ligne_chemin_enregistrement_html_zip = QLineEdit()
        # La ligne d'édition est cachée par défaut
        self.ligne_chemin_enregistrement_html_zip.hide()
        # ----------------------------------------------------------------------------
        # (--> Affichage avec la fonction chemin_ressources contenue dans : config_apsc/chemin_data_ressources.py)
        icone_fermer = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_png_fermeture.png')))
        # Bouton pour fermer la boîte de dialogue
        self.bouton_fermer = PushButtonIconeGauche(_("Fermer la boîte de dialogue"), icon = icone_fermer, iconSize = QSize(16, 16))
        self.bouton_fermer.clicked.connect(self.close)
        # Le bouton est d'abord 
        # rendu invisible
        self.bouton_fermer.hide()
        
        # ----------------------------------------------------------------------------
        # Barre de progression (en boucle ; voir plus 
        # bas ds la fonction lancement) pour montrer 
        # que c'est en cours de traitement
        self.progress = QProgressBar()
        # La barre est centrée
        self.progress.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        # La barre est d'abord 
        # rendue invisible
        self.progress.hide()
 
        # crée du QLabel pour intégrer le throbber
        self.barre = QLabel()
        # redimenssionne
        self.barre.resize(QSize(500, 30))
        # interdit le redimenssionnement à la souris
        self.barre.setFixedSize(self.barre.width(), self.barre.height()) 
        # option: met un fond transparent
        self.barre.setStyleSheet("background-color:rgba(255, 255, 255, 0);") 
        # le contenu du QLabel sera au milieu (hor. et vert.)
        self.barre.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
 
        # Positionne les widgets dans la fenêtre
        self.posit = QGridLayout()
        self.posit.addWidget(self.visionneuse_image_html, 0, 0)
        self.posit.addWidget(self.depart, 1, 0)
        self.posit.addWidget(self.progress, 2, 0)
        self.posit.addWidget(self.ligne_chemin_enregistrement_html_zip, 3, 0)
        self.posit.addWidget(self.bouton_fermer, 4, 0)
        self.setLayout(self.posit)
 
        # initialisation variable d'instance de classe
        self.calcul_thread_html_img = None
        
        # Lancer l'exécution (permet notamment
        # de rendre la fenêtre du QDialog modale
        # c'est à dire que la fenêtre en question
        # reste active par dessus ...)
        self.exec_()

    @pyqtSlot(bool)
    def lancement(self, ok=False):
        """
        Lancement de l'opération longue dans le thread
        """
        
        if self.calcul_thread_html_img == None or not self.calcul_thread_html_img.isRunning():

            # Affichage de la barre de
            # progression (devient visible)
            self.progress.show()
            # La barre est en boucle 
            # (mouvements de gauche à droite)
            self.progress.setRange(0, 0)
            
            # initialise l'opération longue dans le thread
            self.calcul_thread_html_img = CalculThreadHtmlImg()
            # prépare la réception du signal de fin
            self.calcul_thread_html_img.fini.connect(self.stop)
            # lance le thread (mais ne l'attend pas avec .join!!!)
            self.calcul_thread_html_img.start()
    
    @pyqtSlot(bool)
    def arreter(self, ok=False):
        """
        Pour arrêter avant la fin
        """
        
        if self.calcul_thread_html_img!=None and self.calcul_thread_html_img.isRunning():
            self.calcul_thread_html_img.arreter()
            # Ferme la fenêtre (le QDialog
            # dans lequel se trouve le throbber)
            self.close()
 
    @pyqtSlot(bool)
    def stop(self, fin_anormale=False):
        """
        Lancé quand le thread se termine
        """
        
        # Dimensionnement de la fenêtre
        self.resize(1200, 720)
        
        if fin_anormale : pass
        else :
            # -----------------------------------------------------------------
            # Positionnement du QDialog ... à la fin du traitement. Ceci
            # fait en sorte que cette boîte de dialogue se positionne bien
            # au milieu ... en largeur et en hauteur
            # -----------------------------------------------------------------
            # Par défaut --> Résolution programmée
            if self.type_de_resolution == "Résolution programmée" :
                # Placement du QDialog
                geom = self.frameGeometry()
                centre = QDesktopWidget().availableGeometry().center()
                geom.moveCenter(centre)
                self.move(geom.topLeft())
            # Si l'utilisateur choisit Résolution au choix dans :
            # Configuration > Résolution
            elif self.type_de_resolution == "Résolution au choix" :
                # Le fichier data_resolution_desktop_murexpo 
                # (dans le rep. tempo) est ouvert (lecture)
                pre_lecture_resolution_desktop = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_resolution_desktop_murexpo_', 'data_resolution_desktop_murexpo')
                # La donnée contenue ds le fichier est récupérée (elle est
                # sous la forme [155x100], par exemple, puis elle est splitée.
                self.resolution_desktop = "".join(pre_lecture_resolution_desktop.affichage()).split('x')
                # Placement du QDialog
                geom = self.frameGeometry()
                # La boîte de dialogue est placée bien au milieu ...
                geom.moveCenter(QPoint(int(self.resolution_desktop[0]) / 2, int(self.resolution_desktop[1]) / 2))
                self.move(geom.topLeft())
                
            # --------------------------------------------------------------------
            # Chemin de l'image à visionner (il s'agit de l'image temporaire ; résultat 
            # de la conversion de la page html en image JPEG)
            #chemin_image = os.path.join(self.chemin_tempo_system_exploit, "html_vers_image.jpg")
            
            chemin_image = ''.join(glob.glob(os.path.join(self.chemin_tempo_system_exploit, "html_vers_image*.jpg")))
            # --------------------------------------------------------------------
            # Appel de la fonction de redimensionnement des images (vignettage)
            # La dimension en largeur est de 100 pixels (nouv_w)
            self.redim_img(chemin_image, 1600)
            # Affichage de l'image dans le QTextEdit
            self.visionneuse_image_html.setHtml('<center><img src="{}" width="{}" height="{}" title="{}" />'.format(chemin_image, 1600, self.calcHauteur_img, chemin_image))
            # --------------------------------------------------------------------
            # Placement en lecture seule (vignette de l'oeuvre et chemin)
            self.visionneuse_image_html.setReadOnly(True)
            # Le QTextEdit est montré
            self.visionneuse_image_html.show()
            
            # Le bouton de lancement du 
            # traitement est rendu invisible ici
            self.depart.hide()
            
            # La barre de progression est 
            # de nouveau rendue invisible
            self.progress.hide()
            
            # ----------------------------------------------------------------
            # Lecture du chemin d'enregistrement du fichier html zippé
            # ----------------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_chemin_html_zip = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_sauv_html_zip_', 'data_chemin_sauv_html_zip')
            # ... avec appel de la méthode affichage
            lecture_chemin_html_zip = "".join(pre_lecture_chemin_html_zip.affichage())
            # ----------------------------------------------------------------
            
            # La ligne dédition est montrée
            self.ligne_chemin_enregistrement_html_zip.show()
            # La ligne d'édition affiche le chemin d'enregistrement du fichier html zippé
            self.ligne_chemin_enregistrement_html_zip.setText(_("Votre fichier html (zippé) a été enregistré ici : {}").format(lecture_chemin_html_zip))
            # Placement en lecture seule
            self.ligne_chemin_enregistrement_html_zip.setReadOnly(True)
            
            # Le bouton pour fermer
            # la fenêtre est montré
            self.bouton_fermer.show()
            
            
    def closeEvent(self, event):
        """
        Lancé à la fermeture de la fenêtre quelqu'en soit la méthode
        """
        
        # Si le thread est en cours d'exécution, on l'arrête (brutalement!)
        if self.calcul_thread_html_img!=None and self.calcul_thread_html_img.isRunning():
            self.calcul_thread_html_img.terminate()
        # Et on accepte la fermeture de la fenêtre dans tous les cas
        event.accept()
        
        
    def redim_img(self, chemin, nouv_w) :
        ''' Fonction de redimensionnement des images '''

        # Redimensionnnement de l'image pour affichage dans le QTextEdit

        # Largeur de la future vignette
        self.nouv_w = nouv_w
        # Ouverture de l'image
        obImg = Image.open(chemin)
        # Recup dimensions de l'image
        w, h = obImg.size
        # Calcul du ratio de l'image
        ratio = float(w)/float(h)
        # Calcul de future hauteur avec les dimensions donnees par l'utilisateur
        self.calcHauteur_img = int(float(self.nouv_w)/ratio)

        # Retourne le tuple : (largeur, hauteur) de chaque image
        self.tuple_dim_img_vignette = (self.nouv_w, self.calcHauteur_img)
        
        return self.tuple_dim_img_vignette


class CalculThreadHtmlPDF(QThread):
    ''' Partie du Thread qui s'exécute en tâche de fond 
    (dont le code exécuté ---> dans la fonction run) '''
    
    # création du signal
    fini = pyqtSignal(bool) # signal pour la fin du thread
 
    def __init__(self, parent=None):
        super().__init__(parent)
 
        self.stop = False # drapeau pour exécuter une demande d'arrêt

  
    def recherche_installation_wkhtmltopdf_linux(self, log) :
        """ Fonction de recherche du chemin de wkhtmltopdf ... si 
        l'application n'est pas contenue dans l'arborescence de 
        MurExpo sous GNU/Linux. Cette fonction est très utile si 
        MurExpo est installé sur le disque dur sous GNU/Linux et 
        qu'elle est préalablement compilée avec le script shell
        murexpo_build.sh"""
        # Liste pour récupérer les chemins
        liste_chem_wkhtmltopdf = []
        # Recherche du chemin sur 3 niveaux
        # par exemple :
        # /usr/bin/wkhtmltopdf ... ou ...
        # /usr/local/bin/wkhtmltopdf ... ou ...
        # ... /usr/...autre chemin
        for n in range(3) :
            # On ne recherche qu'à partir de /usr/...
            recherche = glob.glob("/usr"+n*"/*"+"/"+log)
            for rech in recherche :
                # Il faut obligatoirement qu'il y ait 
                # un répertoire bin dans le chemin ...
                if 'bin' in rech :
                    liste_chem_wkhtmltopdf.append(rech)
        # On ne renvoie que la 1ère occurrence de la liste
        return liste_chem_wkhtmltopdf[0]
  
    
    def run(self):
        """ Partie qui s'exécute en tâche de fond """
        
        try :
            
            # Appel de la classe pour récupérer le chemin du 
            # répertoire temporaire selon le système d'exploitation
            chem_rep_tempo = Chemin_Rep_Tempo()
            # Le chemin est retourné (return ds la fonction de la classe)
            chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
    
            # --------------------------------------------------------
            # Récup du chemin d'enregistrement du fichier PDF
            # --------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_chemin_pdf = Tempo_lecture(chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_sauv_pdf_', 'data_chemin_sauv_pdf')
            # ... avec appel de la méthode affichage
            chemin_pdf = "".join(pre_lecture_chemin_pdf.affichage())
    
            # Options
            #option = {'quiet': '', 'encoding': 'UTF-8', 'page-size': 'A4', 'dpi': 400, 'orientation': 'Landscape', 'footer-right': '[page]', 'footer-line': '', 'footer-font-size': '8', 'footer-spacing': '2', 'footer-left': "Réalisé avec MurExpo (https://me.murexpo.org)", 'custom-header': [('Accept-Encoding', 'gzip')], 'no-outline': None, 'margin-top': '7mm',  'margin-left': '7mm', 'margin-right': '7mm', 'margin-bottom': '7mm'}
            option = {'quiet': '', 'encoding': 'UTF-8', 'page-size': 'A4', 'dpi': 400, 'orientation': 'Landscape', 'custom-header': [('Accept-Encoding', 'gzip')], 'no-outline': None, 'margin-top': '7mm',  'margin-left': '7mm', 'margin-right': '7mm', 'margin-bottom': '7mm'}
            
            # Sous windows
            if platform.system() == 'Windows' :
                
                # Sous windows permet de récupérer le fichier html (même avec le module tempfile dans le rep. tempo)
                fich_html_temp_windows = [fichier for fichier in os.listdir(chemin_tempo_system_exploit) if os.path.splitext(fichier)[1] == '.html']
                fichier_html_windows = os.path.join(chemin_tempo_system_exploit, fich_html_temp_windows[0])
                
                # --------------------------------------- fonctionne pas ----------------
                # Chemin de wkhtmltopdf sous windows
                #chem_wkhtmltopdf_windows = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))+os.sep+"wkhtmltopdf.exe"
                # --------------------------------------- fonctionne pas ----------------
    
                # Sous windows wkhtmltoimage.exe se trouve à la racine
                chem_wkhtmltopdf_windows = os.path.join(os.getcwd(), "wkhtmltopdf.exe")
    
                config_windows = pdfkit.configuration(wkhtmltopdf = chem_wkhtmltopdf_windows)
                # Exécution
                pdfkit.from_file(fichier_html_windows, chemin_pdf, configuration = config_windows, options = option)
    
            # Sous GNU/Linux
            elif platform.system() == 'Linux' :
                
                # Chemin du fichier html source (dans le répertoire temporaire)
                fichier_html_linux = os.path.join(chemin_tempo_system_exploit, "index_exposition_html_.html")
                
                # ----------------------------------------------------------------------------------------------------
                # Le chemin de wkhtmltopdf sera complètement différent dans le cas où MurExpo est compilé avec 
                # pyinstaller
                # (Cas 1) --> le wkhtmltopdf exécuté sera celui contenu dans l'arborescence de MurExpo (version
                # onefile et onedir)
                # (Cas 2) --> /usr/bin/wkhtmltopdf ... avec le script shell murexpo_build.sh (création d'un 
                # paquet deb)
                # ----------------------------------------------------------------------------------------------------
                # Cas 1
                chem_wkhtmltopdf_linux_1 = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))+os.sep+'wkhtmltox_linux'+os.sep+"bin"+os.sep+"wkhtmltopdf"
                # Cas 2
                # Appel de la fonction de recherche du chemin d'install de wkhtmltopdf
                chem_wkhtmltopdf_linux_2 = self.recherche_installation_wkhtmltopdf_linux('wkhtmltopdf')
                # Si wkhtmltopdf est bien dans l'arborescence de MurExpo ...
                if os.path.exists(chem_wkhtmltopdf_linux_1) == True :
                    config_linux = pdfkit.configuration(wkhtmltopdf=chem_wkhtmltopdf_linux_1)
                # Si wkhtmltopdf n'est pas dans l'arborescence de MurExpo ... c'est le 
                # wkhtmltopdf qui va prendre le relais pour faire les traitements
                # --> il faut que wkhtmltopdf soit bien installé sur le dd !
                else :
                    config_linux = pdfkit.configuration(wkhtmltopdf=chem_wkhtmltopdf_linux_2)
                # Exécution
                pdfkit.from_file(fichier_html_linux, chemin_pdf, configuration = config_linux, options = option)
                
            # Sous MacOSX
            elif platform.system() == 'Darwin' :

                # Chemin du fichier html source (dans le répertoire temporaire)
                fichier_html_macos = os.path.join(chemin_tempo_system_exploit, "index_exposition_html_.html")
                
                # Chemin de wkhtmltopdf sous MacOs --> ne fonctionne pas !
                #chem_wkhtmltopdf_macos = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))+os.sep+'wkhtmltox_macos'+os.sep+"wkhtmltopdf"

                # wkhtmltopdf se trouve dans le rep. wkhtmltox_macos
                chem_wkhtmltopdf_macos = os.path.join(os.getcwd(), "wkhtmltox_macos", "wkhtmltopdf")
                
                # La commande ... on passe ici par subprocess ...
                # --------------------------------------------------------------------------------
                # Dans la version 12.0.6 de wkhtmltopdf (celle accessible pour MurExpo pour MacOS)
                # il faut utiliser l'option --enable-local-file-access pour pouvoir utiliser les
                # fichier locaux. Les autres options qui vont bien sont présentes
                # --------------------------------------------------------------------------------
                commande = "{} --quiet --enable-local-file-access --dpi 400 --encoding 'utf-8' --page-size A4 -O landscape -L 7mm -R 7mm -T 7mm -B 7mm {} {}".format(chem_wkhtmltopdf_macos, os.path.join(chemin_tempo_system_exploit, "index_exposition_html_.html"), os.path.join(chemin_tempo_system_exploit, chemin_pdf))
                
                # Traitement avec subprocess
                subprocess.call(commande, shell = True)
                
            # =====================================================
            # Visionnement
            # =====================================================
            
            # -----------------------------------------------------------------------
            # Suppression des précédentes images (PDF vers PNG) ... Prenons
            # un exemple : j'accroche 4 images sur le mur d'exposition,
            # je fais le traitemet pour une sortie PDF (j'enregistre le PDF).
            # Je constate que les 4 images ont bien été disposées (les unes
            # en dessous des autres) dans le tableau ... là je constate aussi
            # que la sortie PDF (donc la sortie PNG pour le visionnement)
            # se déploye sur 2 pages. Ensuite je supprime une image et je
            # refais le traitement pour une sortie PDF (maintenant la sortie
            # PDF se déploye sur une seule page) ... et là dans au visionnement
            # (la sortie PNG) on a bien qu'une seule page. Auparavant (sans 
            # ce code) on avait une page résiduelle comportant l'image
            # précedemment supprimée. ==> En fait les images PNG pdf_png_murexpo...
            # sont supprimées (si elles existent), puis recrées dans la foulée
            # -----------------------------------------------------------------------
            # Liste des images (issues d'une conversion PDF vers image PNG)
            liste_pdf_png_img = glob.glob(os.path.join(chemin_tempo_system_exploit, 'pdf_png_murexpo*'))
            # Si des images sont détectées (c'est à dire 
            # que la liste contient des éléments) ...
            if len(liste_pdf_png_img) > 0 :
                # ... on parcourt la liste des
                # images pdf_png_murexpo ...
                for img in liste_pdf_png_img :
                    # ... et on les supprime
                    os.remove(img)
            # Si aucune image pdf_png_murexpo...
            # n'existe, on passe sans rien faire
            else : pass
            
            # -----------------------------------------------------------------------
            # On enclanche le chemin (par ex : /home/angelo/.murexpo/pdf_png_murexpo)
            img_visionnement = os.path.join(chemin_tempo_system_exploit, 'pdf_png_murexpo')
            
            # On ouvre le fichier PDF
            doc_pdf = fitz.open(chemin_pdf)
            # Boucle pour contrôler toutes 
            # les pages du fichier PDF
            for n in range(doc_pdf.pageCount) :
                # On traite chaque page
                page_pdf = doc_pdf.loadPage(n)
                # Traitement de l'image convertie
                pix = page_pdf.getPixmap(alpha=False, matrix=fitz.Matrix(1.35, 1.35))
                # Sauvegarde de l'image PNG
                pix.writePNG(img_visionnement+"_"+str(n+1).zfill(5)+".png")
            
            # Appel de la fonction d'affiche de l'image 
            # de visionnement finale (toutes les images 
            # des pages sont assemblées de haut en bas)
            traitement_image_pour_visionnage()
    
            # Signale que la code exécuté 
            # dans run vient de se terminer    
            self.fini.emit(self.stop)
            
        except : return
        
        
    def arreter(self):
        """ Pour arrêter avant la fin normale d'exécution du thread """
        self.stop = True


class Fenetre_barre_sortie_html_pdf_vers_pdf(QDialog):
    ''' La partie graphique du thread '''

    def __init__(self, parent=None):
        super(Fenetre_barre_sortie_html_pdf_vers_pdf, self).__init__(parent)
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        ## QApplication.ProcessEvents() (si nécessaire)
        
        self.setWindowTitle("MurExpo : Traitement PDF")
        # Dimensionnement de la fenêtre
        self.resize(500, 80)

        # --------------------------------------------------------------------------------
        # Lecture du fichier tempo. data_type_de_resolution_fenetre_principale_murexpo
        # ... afin de savoir si le fichier contient "Résolution programmée" ou bien
        # "Résolution au choix"
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_type_de_resolution = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_type_de_resolution_fenetre_principale_murexpo_', 'data_type_de_resolution_fenetre_principale_murexpo')
        # ... avec appel de la méthode affichage
        self.type_de_resolution = "".join(pre_lecture_type_de_resolution.affichage())
        
        # -----------------------------------------------------------------
        # Positionnement du QDialog ... à la fin du traitement. Ceci
        # fait en sorte que cette boîte de dialogue se positionne bien
        # au milieu ... en largeur et en hauteur
        # -----------------------------------------------------------------
        # Par défaut --> Résolution programmée
        if self.type_de_resolution == "Résolution programmée" :
            # Placement du QDialog
            geom = self.frameGeometry()
            centre = QDesktopWidget().availableGeometry().center()
            geom.moveCenter(centre)
            self.move(geom.topLeft())
        # Si l'utilisateur choisit Résolution au choix dans :
        # Configuration > Résolution
        elif self.type_de_resolution == "Résolution au choix" :
            # Le fichier data_resolution_desktop_murexpo 
            # (dans le rep. tempo) est ouvert (lecture)
            pre_lecture_resolution_desktop = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_resolution_desktop_murexpo_', 'data_resolution_desktop_murexpo')
            # La donnée contenue ds le fichier est récupérée (elle est
            # sous la forme [155x100], par exemple, puis elle est splitée.
            self.resolution_desktop = "".join(pre_lecture_resolution_desktop.affichage()).split('x')
            # Placement du QDialog
            geom = self.frameGeometry()
            # La boîte de dialogue est placée bien au milieu ...
            geom.moveCenter(QPoint(int(self.resolution_desktop[0]) / 2, int(self.resolution_desktop[1]) / 2))
            self.move(geom.topLeft())
            
        # ------------------------------------------------
        # Création des QTextEdit pour afficher les 
        # oeuvres (et capter le chemin des oeuvres)
        self.visionneuse_image_pdf = QTextEdit()
        self.visionneuse_image_pdf.setMaximumWidth(1200)
        self.visionneuse_image_pdf.setMinimumWidth(1200)
        self.visionneuse_image_pdf.setMaximumHeight(700)
        self.visionneuse_image_pdf.setMinimumHeight(700)
        # Le QTextEdit est caché par défaut
        self.visionneuse_image_pdf.hide()
        
        # Les boutons pour le lancement du thread et d'arrêt anticipé du thread
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe --> elle est ici : affichage_apsc/push_button_icone_gauche.py ...
        # en fait le texte dans le bouton est centré et l'icône positionné à gauche.
        # En fait j'ai un peu modifié la classe ...
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : config_apsc/chemin_data_ressources.py)
        icone_lancer_traitement = QIcon(chemin_ressources(os.path.join('ressources_apsc', 'icone_lancer_le_traitement_apsc_256x256.png')))
        icone_arreter_traitement = QIcon(chemin_ressources(os.path.join('ressources_apsc', 'icone_arreter_le_traitement_apsc_256x256.png')))
        # Bouton de lancement du thread
        self.depart = PushButtonIconeGauche(_("Lancer le traitement"), icon = icone_lancer_traitement, iconSize = QSize(16, 16))
        self.depart.clicked.connect(self.lancement)
        # ----------------------------------------------------------------------------
        # Ligne d'édition pour afficher le chemin 
        # où a été enregistré le fichier pdf
        self.ligne_chemin_enregistrement_pdf = QLineEdit()
        # La ligne d'édition est cachée par défaut
        self.ligne_chemin_enregistrement_pdf.hide()
        # ----------------------------------------------------------------------------
        # (--> Affichage avec la fonction chemin_ressources contenue dans : config_apsc/chemin_data_ressources.py)
        icone_fermer = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_png_fermeture.png')))
        # Bouton pour fermer la boîte de dialogue
        self.bouton_fermer = PushButtonIconeGauche(_("Fermer la boîte de dialogue"), icon = icone_fermer, iconSize = QSize(16, 16))
        self.bouton_fermer.clicked.connect(self.close)
        # Le bouton est d'abord 
        # rendu invisible
        self.bouton_fermer.hide()
        
        # ----------------------------------------------------------------------------
        # Barre de progression (en boucle ; voir plus 
        # bas ds la fonction lancement) pour montrer 
        # que c'est en cours de traitement
        self.progress = QProgressBar()
        # La barre est centrée
        self.progress.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        # La barre est d'abord 
        # rendue invisible
        self.progress.hide()
 
        # crée du QLabel pour intégrer le throbber
        self.barre = QLabel()
        # redimenssionne
        self.barre.resize(QSize(500, 30))
        # interdit le redimenssionnement à la souris
        self.barre.setFixedSize(self.barre.width(), self.barre.height()) 
        # option: met un fond transparent
        self.barre.setStyleSheet("background-color:rgba(255, 255, 255, 0);") 
        # le contenu du QLabel sera au milieu (hor. et vert.)
        self.barre.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
 
        # Positionne les widgets dans la fenêtre
        self.posit = QGridLayout()
        self.posit.addWidget(self.visionneuse_image_pdf, 0, 0)
        self.posit.addWidget(self.depart, 1, 0)
        self.posit.addWidget(self.progress, 2, 0)
        self.posit.addWidget(self.ligne_chemin_enregistrement_pdf, 3, 0)
        self.posit.addWidget(self.bouton_fermer, 4, 0)
        self.setLayout(self.posit)
 
        # initialisation variable d'instance de classe
        self.calcul_thread_html_pdf = None
        
        # Lancer l'exécution (permet notamment
        # de rendre la fenêtre du QDialog modale
        # c'est à dire que la fenêtre en question
        # reste active par dessus ...)
        self.exec_()

    @pyqtSlot(bool)
    def lancement(self, ok=False):
        """
        Lancement de l'opération longue dans le thread
        """
        
        if self.calcul_thread_html_pdf == None or not self.calcul_thread_html_pdf.isRunning():

            # Affichage de la barre de
            # progression (devient visible)
            self.progress.show()
            # La barre est en boucle 
            # (mouvements de gauche à droite)
            self.progress.setRange(0, 0)
            
            # initialise l'opération longue dans le thread
            self.calcul_thread_html_pdf = CalculThreadHtmlPDF()
            # prépare la réception du signal de fin
            self.calcul_thread_html_pdf.fini.connect(self.stop)
            # lance le thread (mais ne l'attend pas avec .join!!!)
            self.calcul_thread_html_pdf.start()
    
    @pyqtSlot(bool)
    def arreter(self, ok=False):
        """
        Pour arrêter avant la fin
        """
        
        if self.calcul_thread_html_pdf != None and self.calcul_thread_html_pdf.isRunning():
            self.calcul_thread_html_pdf.arreter()
            # Ferme la fenêtre (le QDialog
            # dans lequel se trouve le throbber)
            self.close()
 
    @pyqtSlot(bool)
    def stop(self, fin_anormale=False):
        """
        Lancé quand le thread se termine
        """
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # Dimensionnement de la fenêtre
        self.resize(1200, 720)
        
        if fin_anormale : pass
        else :
            # -----------------------------------------------------------------
            # Positionnement du QDialog ... à la fin du traitement. Ceci
            # fait en sorte que cette boîte de dialogue se positionne bien
            # au milieu ... en largeur et en hauteur
            # -----------------------------------------------------------------
            # Par défaut --> Résolution programmée
            if self.type_de_resolution == "Résolution programmée" :
                # Placement du QDialog
                geom = self.frameGeometry()
                centre = QDesktopWidget().availableGeometry().center()
                geom.moveCenter(centre)
                self.move(geom.topLeft())
            # Si l'utilisateur choisit Résolution au choix dans :
            # Configuration > Résolution
            elif self.type_de_resolution == "Résolution au choix" :
                # Le fichier data_resolution_desktop_murexpo 
                # (dans le rep. tempo) est ouvert (lecture)
                pre_lecture_resolution_desktop = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_resolution_desktop_murexpo_', 'data_resolution_desktop_murexpo')
                # La donnée contenue ds le fichier est récupérée (elle est
                # sous la forme [155x100], par exemple, puis elle est splitée.
                self.resolution_desktop = "".join(pre_lecture_resolution_desktop.affichage()).split('x')
                # Placement du QDialog
                geom = self.frameGeometry()
                # La boîte de dialogue est placée bien au milieu ...
                geom.moveCenter(QPoint(int(self.resolution_desktop[0]) / 2, int(self.resolution_desktop[1]) / 2))
                self.move(geom.topLeft())
               
            # Permet de récupérer le fichier PNG nommé pdf_png_murexpo (même avec le 
            # module tempfile dans le rep. tempo sous windows) après conversion avec
            # pyMuPDF (import fitz) du fichier PDF vers des images PNG
            fich_png_visio_pdf_vers_png = [fichier for fichier in os.listdir(chemin_tempo_system_exploit) if fichier[0:13] == 'pdf_png_final']
            pdf_vers_image_png = os.path.join(chemin_tempo_system_exploit, fich_png_visio_pdf_vers_png[0])

            # --------------------------------------------------------------------
            # Appel de la fonction de redimensionnement des images (vignettage)
            # La dimension en largeur est de 100 pixels (nouv_w)
            self.redim_img(pdf_vers_image_png, 1170)
            # Affichage de l'image dans le QTextEdit
            self.visionneuse_image_pdf.setHtml('<center><img src="{}" width="{}" height="{}" title="{}" />'.format(pdf_vers_image_png, 1170, self.calcHauteur_img, pdf_vers_image_png))
            # --------------------------------------------------------------------
            # Placement en lecture seule (vignette de l'oeuvre et chemin)
            self.visionneuse_image_pdf.setReadOnly(True)
            # Le QTextEdit est montré
            self.visionneuse_image_pdf.show()

            # Le bouton de lancement du 
            # traitement est rendu invisible ici
            self.depart.hide()
            
            # La barre de progression est 
            # de nouveau rendue invisible
            self.progress.hide()
            
            # ----------------------------------------------------------------
            # Lecture du chemin d'enregistrement du fichier PDF
            # ----------------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            pre_lecture_chemin_pdf = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_chemin_sauv_pdf_', 'data_chemin_sauv_pdf')
            # ... avec appel de la méthode affichage
            lecture_chemin_pdf = "".join(pre_lecture_chemin_pdf.affichage())
            # ----------------------------------------------------------------
            
            # La ligne dédition est montrée
            self.ligne_chemin_enregistrement_pdf.show()
            # La ligne d'édition affiche le chemin d'enregistrement du fichier html zippé
            self.ligne_chemin_enregistrement_pdf.setText(_("Votre fichier PDF a été enregistré ici : {}").format(lecture_chemin_pdf))
            # Placement en lecture seule
            self.ligne_chemin_enregistrement_pdf.setReadOnly(True)
            
            # Le bouton pour fermer
            # la fenêtre est montré
            self.bouton_fermer.show()
            
            
    def closeEvent(self, event):
        """
        Lancé à la fermeture de la fenêtre quelqu'en soit la méthode
        """
        
        # Si le thread est en cours d'exécution, on l'arrête (brutalement!)
        if self.calcul_thread_html_pdf != None and self.calcul_thread_html_pdf.isRunning():
            self.calcul_thread_html_pdf.terminate()
        # Et on accepte la fermeture de la fenêtre dans tous les cas
        event.accept()
        
        
    def redim_img(self, chemin, nouv_w) :
        ''' Fonction de redimensionnement des images '''

        # Redimensionnnement de l'image pour affichage dans le QTextEdit

        # Largeur de la future vignette
        self.nouv_w = nouv_w
        # Ouverture de l'image
        obImg = Image.open(chemin)
        # Recup dimensions de l'image
        w, h = obImg.size
        # Calcul du ratio de l'image
        ratio = float(w)/float(h)
        # Calcul de future hauteur avec les dimensions donnees par l'utilisateur
        self.calcHauteur_img = int(float(self.nouv_w)/ratio)

        # Retourne le tuple : (largeur, hauteur) de chaque image
        self.tuple_dim_img_vignette = (self.nouv_w, self.calcHauteur_img)
        
        return self.tuple_dim_img_vignette

