#!/usr/bin/python3
# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------


# Atelier de fabrique (mise en place du mur d'exposition, 
# disposition des oeuvres et enregistrement)
from fabrique_murexpo_gui import Fabrique

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Ligne horizonatle de séparation à placer avant le bouton final
from ligne_horizontale_ligne_verticale import Ligne_Horizontale

# Redémarrage de MurExpo après certaines modifications, 
# par exemple le changement de résolution
from redemarrage_suite_modifs import redemarrer

# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture
# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Surclassement de QFileDialog (associé à
# QTreeView) pour charger l'image de l'oeuvre
from fichier_ouvrir import FichierOuvrir

#from widget_arborescence import AppelArbre

# Merci à eyllanesc pour son aide est pour cette classe ... import
# --> pour les icônes à droite dans le bouton
# --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
from push_button_icone_droite import PushButtonIconeDroite
# --> pour les icônes à gauche dans le bouton
from push_button_icone_gauche import PushButtonIconeGauche

# Visionnage de l'image, gif animé, image du fichier 
# html ou vidéo, pendant ... ou à la fin du traitement
from visionnage_murexpo_gui import Visionnage

# Sélection de la police par un widget perso
from selection_police import SelectionPolice
# Sélection de la couleur par un widget perso
from selection_couleur import SelectionCouleur

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

# Import de Pillow
try : import Image
except ImportError : from PIL import Image

# Imports PyQt5 -----------------------------------------------------------------------
from PyQt5.QtCore import (Qt, QT_VERSION_STR, PYQT_VERSION_STR, QLocale, QTranslator, QLibraryInfo, QRect, 
                          QSize, QFileInfo, QTimer, pyqtSignal, QDir)
from PyQt5.QtWidgets import (QWidget, QGroupBox, QGridLayout, QTextEdit, QMainWindow, QTabWidget, QWidget, 
                             QVBoxLayout, QAction, QApplication, QStyleFactory, QFileDialog, QDialog, QSpinBox, 
                             QLabel, QLineEdit, QPushButton, QFrame, QComboBox, QListWidget, QListWidgetItem, 
                             QHBoxLayout, QColorDialog, QMessageBox, QCheckBox, QScrollArea, QDesktopWidget,
                             QSpacerItem, QSizePolicy, QFontDialog, QSlider, QTextBrowser) ### QTextBrowser ESSAI !!!
from PyQt5.QtGui import (QIcon, QFont, QPalette, QColor)
# ------------------------------------------------------------------------------------

import sys, os, platform, pickle, shutil, glob, subprocess

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    print(locale)
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------

# Numéro de version de MurExpo
__version__ = "0.6.8"


class Configuration_General(QDialog) :
    ''' Classe principale d'affichage de la fenêtre de configuration '''
    
    def __init__(self, parent):
        super(Configuration_General, self).__init__(parent)

        # Boîte d'onglets
        self.tab_widget_config = QTabWidget()

        # Appel des classes contenant le groupement de widgets
        self.win_wid_config_1 = Configuration_Repertoires_Chargement(self)
        self.win_wid_config_2 = Configuration_Resolution(self)
        self.win_wid_config_3 = Configuration_Mode_Affichage_Onglets(self)
        self.win_wid_config_4 = Configuration_Traitement_Intermediaire(self)
        self.win_wid_config_5 = StylesInterface(self)
        self.win_wid_config_6 = Reinitialiser(self)
        
        # Le widget
        widget = QWidget()

        self.tab_widget_config.addTab(self.win_wid_config_1, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), _("Chemins (chargement et sauvegarde)"))
        self.tab_widget_config.addTab(self.win_wid_config_2, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), _("Résolution"))
        self.tab_widget_config.addTab(self.win_wid_config_3, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), _("Mode d'affichage des onglets"))
        self.tab_widget_config.addTab(self.win_wid_config_4, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), _("Traitement intermédiaire"))        
        self.tab_widget_config.addTab(self.win_wid_config_5, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), _("Style/Apparence"))
        # L'onglet est affiché uniquement sous GNU/Linux ou sous MacOS
        if platform.system() in ['Linux', 'Darwin'] :
            self.tab_widget_config.addTab(self.win_wid_config_6, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png'))), "Réinitialisation")
        
        # Taille de l'icône dans le QTabWidget
        self.tab_widget_config.setIconSize(QSize(16, 16))
        
        # Avec le StyleShett comme ceci ça fonctionne bien !
        self.tab_widget_config.setStyleSheet("""QTabWidget::tab-bar {alignment: center;}""")

        # VBox.
        layout = QVBoxLayout()
        layout.addWidget(self.tab_widget_config)
        
        self.setLayout(layout)


class Configuration_Repertoires_Chargement(QWidget) :
    ''' Classe qui affiche la configuration du chargement des répertoires '''

    def __init__(self, parent=None):
        super(Configuration_Repertoires_Chargement, self).__init__(parent)
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # -------------------------------------------------------------- #
        # Appel du contenu des fonctions de captage des fichiers tempo
        # -------------------------------------------------------------- #
        # Répertoire de chargement des images (oeuvres à déposer sur le mur d'exposition par exemple)
        retour_tempo_charg_images = self.lecture_contenu_tempo_rep("data_chem_charg_rep_images_murexpo_", "data_chem_charg_rep_images_murexpo")
        # Répertoire de sauvegarde de l'exposition (mur + oeuvres) sous la forme d'une image (panoramique)
        retour_tempo_sauv_panoramiq_img_expo = self.lecture_contenu_tempo_rep("data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo")
        # Répertoire de sauvegarde de l'exposition (mur + oeuvres) sous la forme d'un gif animé
        retour_tempo_sauv_gif_anime_expo = self.lecture_contenu_tempo_rep("data_chem_sauv_rep_gif_anime_murexpo_", "data_chem_sauv_rep_gif_anime_murexpo")
        # Répertoire de sauvegarde de l'exposition (mur + oeuvres) sous la forme d'un fichier html
        retour_tempo_sauv_html_expo = self.lecture_contenu_tempo_rep("data_chem_sauv_rep_expo_html_murexpo_", "data_chem_sauv_rep_expo_html_murexpo")
        # Répertoire de sauvegarde de l'exposition (mur + oeuvres) sous la forme d'une vidéo
        retour_tempo_sauv_video_expo = self.lecture_contenu_tempo_rep("data_chem_charg_rep_expo_video_murexpo_", "data_chem_charg_rep_expo_video_murexpo")
        
        # ======================================
        # Les labels
        # ======================================
        #
        label_chem_charg_images = QLabel(_("Chargement des images (chemin de chargement) :"))
        label_chem_sauv_panoramiq_img_expo = QLabel(_("Sauvegarde (sous la forme d'une image) de l'exposition :"))
        label_chem_sauv_gif_anime_expo = QLabel(_("Sauvegarde (sous la forme d'un gif animé) de l'exposition :"))
        label_chem_sauv_html_expo = QLabel(_("Sauvegarde (sous la forme d'un fichier html et PDF) de l'exposition :"))
        label_chem_sauv_video_expo = QLabel(_("Sauvegarde (sous la forme d'une vidéo) de l'exposition :"))
        
        # --------------------------------------
        # Certains labels sont pour l'instant 
        # grisés (car pas encore mis en service)
        # --------------------------------------
        label_chem_sauv_video_expo.setEnabled(False)
        
        # ======================================
        # Les QLineEdit
        # ======================================
        # Les lignes de saisie (affichage des chemins)
        # --> 1 --------------------------------------
        self.ligne_aff_charg_images = QLineEdit()
        self.fonte(self.ligne_aff_charg_images) # Fontion pour l'attribution de la fonte
        # Affichage d'une information ...
        if retour_tempo_charg_images :
            self.ligne_aff_charg_images.setText(retour_tempo_charg_images)
        else : 
            # Appel de la fonction pour l'écriture des infos dans la QLineEdit
            # ... et l'écriture des bonnes infos dans le fichier tempo concerné
            self.parametres_defaut_chem_img()
        # En lecture seule
        self.ligne_aff_charg_images.setReadOnly(True)
        # --> 2 --------------------------------------
        self.ligne_aff_sauv_panoramiq_img_expo = QLineEdit()
        self.fonte(self.ligne_aff_sauv_panoramiq_img_expo) # Fontion pour l'attribution de la fonte
        # Affichage d'une information ...
        if retour_tempo_sauv_panoramiq_img_expo :
            self.ligne_aff_sauv_panoramiq_img_expo.setText(retour_tempo_sauv_panoramiq_img_expo)
        else :
            # Appel de la fonction pour l'écriture des infos dans la QLineEdit
            # ... et l'écriture des bonnes infos dans le fichier tempo concerné
            self.parametres_defaut_panoramiq_img()
        # En lecture seule
        self.ligne_aff_sauv_panoramiq_img_expo.setReadOnly(True)
        # --> 3 --------------------------------------
        self.ligne_aff_sauv_gif_anime_expo = QLineEdit()
        self.fonte(self.ligne_aff_sauv_gif_anime_expo) # Fontion pour l'attribution de la fonte
        # Affichage d'une information ...
        if retour_tempo_sauv_gif_anime_expo :
            self.ligne_aff_sauv_gif_anime_expo.setText(retour_tempo_sauv_gif_anime_expo)
        else :
            # Appel de la fonction pour l'écriture des infos dans la QLineEdit
            # ... et l'écriture des bonnes infos dans le fichier tempo concerné
            self.parametres_defaut_gif_anim()
        # En lecture seule
        self.ligne_aff_sauv_gif_anime_expo.setReadOnly(True)
        # --> 4 --------------------------------------
        self.ligne_aff_sauv_html_expo = QLineEdit()
        self.fonte(self.ligne_aff_sauv_html_expo) # Fontion pour l'attribution de la fonte
        # Affichage d'une information ...
        if retour_tempo_sauv_html_expo :
            self.ligne_aff_sauv_html_expo.setText(retour_tempo_sauv_html_expo)
        else :
            # Appel de la fonction pour l'écriture des infos dans la QLineEdit
            # ... et l'écriture des bonnes infos dans le fichier tempo concerné
            self.parametres_defaut_html()
        # En lecture seule
        self.ligne_aff_sauv_html_expo.setReadOnly(True)
        # --> 5 --------------------------------------
        self.ligne_aff_sauv_video_expo = QLineEdit()
        self.fonte(self.ligne_aff_sauv_video_expo) # Fontion pour l'attribution de la fonte
        # Affichage d'une information ...
        if retour_tempo_sauv_video_expo :
            self.ligne_aff_sauv_video_expo.setText(retour_tempo_sauv_video_expo)
        else :
            # Appel de la fonction pour l'écriture des infos dans la QLineEdit
            # ... et l'écriture des bonnes infos dans le fichier tempo concerné
            self.parametres_defaut_video()
        # En lecture seule
        self.ligne_aff_sauv_video_expo.setReadOnly(True)
        
        # --------------------------------------
        # Certaines QLineEdit sont pour l'instant 
        # grisées (car pas encore mis en service)
        # --------------------------------------
        self.ligne_aff_sauv_video_expo.setEnabled(False)
        
        # ======================================
        # Les boutons
        # ======================================
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe --> elle est ici : affichage_apsc/push_button_icone_gauche.py ... 
        # Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : config_apsc/chemin_data_ressources.py)
        # ----------------------------------------------------------------------------
        icone_dossier = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_png_dossier_murexpo_256x256.png')))
        #
        bout_charg_chem_images = PushButtonIconeGauche(_("Ouvrir"), icon = icone_dossier, iconSize = QSize(16, 16))
        bout_charg_chem_images.setFixedWidth(120)
        bout_charg_chem_images.setToolTip(_("Définir, par défaut, le chemin de chargement des images."))
        bout_sauv_chem_panoramiq_img_expo = PushButtonIconeGauche(_("Ouvrir"), icon = icone_dossier, iconSize = QSize(16, 16))
        bout_sauv_chem_panoramiq_img_expo.setFixedWidth(120)
        bout_sauv_chem_panoramiq_img_expo.setToolTip(_("Définir, par défaut, le chemin de sauvegarde de l'image de l'exposition."))
        bout_sauv_chem_gif_anime_expo = PushButtonIconeGauche("Ouvrir", icon = icone_dossier, iconSize = QSize(16, 16))
        bout_sauv_chem_gif_anime_expo.setFixedWidth(120)
        bout_sauv_chem_gif_anime_expo.setToolTip(_("Définir, par défaut, le chemin de sauvegarde de l'exposition sous la forme d'un gif animé."))
        bout_sauv_chem_html_expo = PushButtonIconeGauche(_("Ouvrir"), icon = icone_dossier, iconSize = QSize(16, 16))
        bout_sauv_chem_html_expo.setFixedWidth(120)
        bout_sauv_chem_html_expo.setToolTip(_("Définir, par défaut, le chemin de sauvegarde de l'exposition sous la forpme d'un fichier html."))
        bout_sauv_chem_video_expo = PushButtonIconeGauche(_("Ouvrir"), icon = icone_dossier, iconSize = QSize(16, 16))
        bout_sauv_chem_video_expo.setFixedWidth(120)
        bout_sauv_chem_video_expo.setToolTip(_("Définir, par défaut, le chemin de l'exposition sous la forme d'une vidéo."))
        
        icone_revenir_chemin_par_defaut = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_revenir_aux_reglages_par_defaut_murexpo_256x256.png')))
        #
        bout_revenir_param_par_defaut_chem_img = PushButtonIconeGauche("", icon = icone_revenir_chemin_par_defaut, iconSize = QSize(16, 16))
        bout_revenir_param_par_defaut_chem_img.setFixedWidth(26)
        bout_revenir_param_par_defaut_chem_img.setToolTip(_("Revenir au chemin par défaut."))
        bout_revenir_param_par_defaut_panoramiq_img = PushButtonIconeGauche("", icon = icone_revenir_chemin_par_defaut, iconSize = QSize(16, 16))
        bout_revenir_param_par_defaut_panoramiq_img.setFixedWidth(26)
        bout_revenir_param_par_defaut_panoramiq_img.setToolTip(_("Revenir au chemin par défaut."))
        bout_revenir_param_par_defaut_gif_anim = PushButtonIconeGauche("", icon = icone_revenir_chemin_par_defaut, iconSize = QSize(16, 16))
        bout_revenir_param_par_defaut_gif_anim.setFixedWidth(26)
        bout_revenir_param_par_defaut_gif_anim.setToolTip(_("Revenir au chemin par défaut."))
        bout_revenir_param_par_defaut_html = PushButtonIconeGauche("", icon = icone_revenir_chemin_par_defaut, iconSize = QSize(16, 16))
        bout_revenir_param_par_defaut_html.setFixedWidth(26)
        bout_revenir_param_par_defaut_html.setToolTip(_("Revenir au chemin par défaut."))
        bout_revenir_param_par_defaut_video = PushButtonIconeGauche("", icon = icone_revenir_chemin_par_defaut, iconSize = QSize(16, 16))
        bout_revenir_param_par_defaut_video.setFixedWidth(26)
        bout_revenir_param_par_defaut_video.setToolTip(_("Revenir au chemin par défaut."))
        
        # ----------------------------------------------------------------------------
        # Bouton pour le redémarrage après le modifs des chemins
        # ----------------------------------------------------------------------------
        # Icône pour le bouton de changement des chemins
        icone_affichage = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
        # Bouton pour redémarrer le logiciel après un changement
        # de répertoire (chargement et sauvegarde)
        self.bout_redemarrer = PushButtonIconeGauche(_("Appliquer les changements"), icon = icone_affichage, iconSize = QSize(16, 16))
        self.bout_redemarrer.setFixedHeight(32)
        #self.bout_redemarrer.setMaximumWidth(250)
        #self.bout_redemarrer.setMinimumWidth(250)
        # ToolTip pour l'information --> application 
        # du changement de répertoire
        self.bout_redemarrer.setToolTip(_("Appliquer les modifications après re-définition des chemins de chargement et/ou sauvegarde"))
        # ----------------------------------------------------------------------------
        
        # --------------------------------------
        # Certains boutons sont pour l'instant 
        # grisés (car pas encore mis en service)
        # --------------------------------------
        bout_sauv_chem_video_expo.setEnabled(False)
        bout_revenir_param_par_defaut_video.setEnabled(False)

        #
        grid_1 = QGridLayout()
        grid_1.setSpacing(10)
        
        # Placement des widgets dans la grille 1
        grid_1.addWidget(label_chem_charg_images, 0, 0)
        grid_1.addWidget(self.ligne_aff_charg_images, 0, 1)
        grid_1.addWidget(bout_charg_chem_images, 0, 2)
        grid_1.addWidget(bout_revenir_param_par_defaut_chem_img, 0, 3)    
        grid_1.addWidget(label_chem_sauv_panoramiq_img_expo, 1, 0)
        grid_1.addWidget(self.ligne_aff_sauv_panoramiq_img_expo, 1, 1)
        grid_1.addWidget(bout_sauv_chem_panoramiq_img_expo, 1, 2)
        grid_1.addWidget(bout_revenir_param_par_defaut_panoramiq_img, 1, 3)
        grid_1.addWidget(label_chem_sauv_gif_anime_expo, 2, 0)
        grid_1.addWidget(self.ligne_aff_sauv_gif_anime_expo, 2, 1)
        grid_1.addWidget(bout_sauv_chem_gif_anime_expo, 2, 2) 
        grid_1.addWidget(bout_revenir_param_par_defaut_gif_anim, 2, 3)
        grid_1.addWidget(label_chem_sauv_html_expo, 3, 0)
        grid_1.addWidget(self.ligne_aff_sauv_html_expo, 3, 1)
        grid_1.addWidget(bout_sauv_chem_html_expo, 3, 2)
        grid_1.addWidget(bout_revenir_param_par_defaut_html, 3, 3)
        grid_1.addWidget(label_chem_sauv_video_expo, 4, 0)
        grid_1.addWidget(self.ligne_aff_sauv_video_expo, 4, 1)
        grid_1.addWidget(bout_sauv_chem_video_expo, 4, 2)
        grid_1.addWidget(bout_revenir_param_par_defaut_video, 4, 3)
        # Appel de la classe Ligne_Horizontale définie
        # dans ligne_horizontale_ligne_verticale.py
        # (classe importée --> voir plus haut)
        grid_1.addWidget(Ligne_Horizontale(), 6, 1)
        # Bouton pour appliquer les changements
        # l'emplacement 7, 1
        grid_1.addWidget(self.bout_redemarrer, 7, 1)
        
        # Alignement
        grid_1.setAlignment(Qt.AlignVCenter)
        grid_1.setAlignment(Qt.AlignHCenter)
        #
        self.setLayout(grid_1)
        
        # Signal pour chaque bouton (Ouvrir)
        bout_charg_chem_images.clicked.connect(self.rep_chem_images)
        bout_sauv_chem_panoramiq_img_expo.clicked.connect(self.rep_chem_panoramiq_img_expo)
        bout_sauv_chem_gif_anime_expo.clicked.connect(self.rep_chem_gif_anime_expo)
        bout_sauv_chem_html_expo.clicked.connect(self.rep_chem_html_expo)
        bout_sauv_chem_video_expo.clicked.connect(self.rep_chem_video_expo)
        # Signal pour chaque bouton pour revenir aux paramètres par défaut
        bout_revenir_param_par_defaut_chem_img.clicked.connect(self.parametres_defaut_chem_img)
        bout_revenir_param_par_defaut_panoramiq_img.clicked.connect(self.parametres_defaut_panoramiq_img)
        bout_revenir_param_par_defaut_gif_anim.clicked.connect(self.parametres_defaut_gif_anim)
        bout_revenir_param_par_defaut_html.clicked.connect(self.parametres_defaut_html)
        bout_revenir_param_par_defaut_video.clicked.connect(self.parametres_defaut_video)
        # Signal pour appliquer à la fin des modifs de
        # chemins --> le logiciel redémarre quand on
        # clique sur le bouton et les changements
        # deviennent effectifs
        # -----------------------------
        # Appel de la fonction redemarrer contenue
        # dans le module redemarrage_suite_modifs 
        # (voir import plus haut).
        self.bout_redemarrer.clicked.connect(redemarrer)
        # =====================================================
        
        
    def fonte(self, widget) :
        '''
        Fonction d'attribution de la fonte dans les QLineEdit et les QTextWidget
        '''

        f = QFont()
        #f.setFamily('DejaVu Sans')
        f.setFamily('Liberation Sans')
        #f.setBold(True)
        f.setFixedPitch(True)
        f.setPointSize(9)
        widget.setFont(f)


    def lecture_contenu_tempo_rep(self, chem_tempo_1, chem_tempo_2) :
        ''' Fonction pour récupérer le chemin exact
        (déjà sélectionné par l'utilisateur ; par 
        l'intermédiaire de la boîte de dialogue) 
        concernant la séquence du professeur '''
        
        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
        # Boucle de vérification ...
        for contenu in liste_contenu_tmp :
            # Si le fichier tempo concerné est bien dedans ...
            if chem_tempo_1 in contenu :
                # ------------------------------------------------------- #
                # Utilisation de la classe Tempo_lecture pour la lecture
                # des données dans le répertoire (dit) temporaire
                # ------------------------------------------------------- #
                rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), chem_tempo_1, chem_tempo_2)
                # L'exception sert à ne pas produire d'erreur et à faire 
                # en sorte que la boîte de dialogue deConfiguration > 
                # Réglages divers s'ouvre quand même si l'utilisateur a 
                # annulé le chargement d'un chemin (Sélection d'un
                # répertoire ... et bouton Annuler au lieu de Ok)
                try :
                    # ... avec appel de la méthode affichage
                    rep_tempo_chemin = rep_tempo_lecture.affichage()[0]
                    # S'il y a bien une donnée récupérée ...
                    if rep_tempo_chemin :
                        # ... la donnée est retournée
                        return rep_tempo_chemin
                except : pass
    
        
    def rep_chem_images(self) : # 1
        ''' Boîte de dialogue pour le chemin (répertoire)
        de chargement des images (en général) et écriture
        du chemin sélectionné par l'utilisateur dans le 
        répertoire temporaire '''
        
        # Boîte de dialogue d'ouverture du répertoire
        self.ouv_Reps_Charg_images = QFileDialog.getExistingDirectory(None, _('Sélectionner un répertoire:'), os.path.expanduser('~'), QFileDialog.ShowDirsOnly)
        
        # Le chemin s'affiche dans le QLineEdit
        self.ligne_aff_charg_images.setText(self.ouv_Reps_Charg_images)
        
        # --------------------------------------------------------------------- #
        # Utilisation de la classe Tempo_ecriture
        # pour l'écriture des données dans le 
        # répertoire (dit) temporaire
        # ===================================================================== #
        # Ecriture du chemin du répertoire chargé par l'utilisateur (sous 
        # forme de fichier texte) dans le répertoire temporaire
        # --------------------------------------------------------------------- #
        # Cette donnée sera utilisée quand l'utilisateur chargera des images
        # (oeuvres) à charger et glisser sur le mur d'exposition par la boîte 
        # de dialogue d'ouverture.
        # ===================================================================== #
        # Appel de la classe Tempo_ecriture
        ecr_tempo_charg_chem_rep_images = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_charg_rep_images_murexpo_", "data_chem_charg_rep_images_murexpo", ".txt", self.ouv_Reps_Charg_images)
        # --------------------------------------------------------------------- #

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Chemin de chargement des images : {}\n").format(self.ouv_Reps_Charg_images))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def rep_chem_panoramiq_img_expo(self) : # 2
        ''' Boîte de dialogue pour le chemin (répertoire)
        d'enregistrement de l'image de l'exposition et 
        écriture du chemin sélectionné par l'utilisateur 
        dans le répertoire temporaire '''
        
        # Boîte de dialogue d'ouverture du répertoire
        self.ouv_Reps_Sauv_panoramiq_img_expo = QFileDialog.getExistingDirectory(None, _('Sélectionner un répertoire:'), os.path.expanduser('~'), QFileDialog.ShowDirsOnly)

        # Le chemin s'affiche dans le QLineEdit
        self.ligne_aff_sauv_panoramiq_img_expo.setText(self.ouv_Reps_Sauv_panoramiq_img_expo)
        
        # Appel de la classe Tempo_ecriture
        ecr_tempo_sauv_chem_rep_panoramiq_img_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo", ".txt", self.ouv_Reps_Sauv_panoramiq_img_expo)

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Chemin d'enregistrement des images : {}\n").format(self.ouv_Reps_Sauv_panoramiq_img_expo))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def rep_chem_gif_anime_expo(self) : # 3
        ''' Boîte de dialogue pour le chemin (répertoire)
        d'enregistrement de l'exposition comme un gif animé 
        et écriture du chemin sélectionné par l'utilisateur 
        dans le répertoire temporaire '''
        
        # Boîte de dialogue d'ouverture du répertoire
        self.ouv_Reps_Sauv_gif_anime_expo = QFileDialog.getExistingDirectory(None, _('Sélectionner un répertoire:'), os.path.expanduser('~'), QFileDialog.ShowDirsOnly)

        # Le chemin s'affiche dans le QLineEdit
        self.ligne_aff_sauv_gif_anime_expo.setText(self.ouv_Reps_Sauv_gif_anime_expo)
        
        # Appel de la classe Tempo_ecriture
        ecr_tempo_sauv_chem_rep_gif_anime_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_gif_anime_murexpo_", "data_chem_sauv_rep_gif_anime_murexpo", ".txt", self.ouv_Reps_Sauv_gif_anime_expo)

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Chemin d'enregistrement des gif animés : {}\n").format(self.ouv_Reps_Sauv_gif_anime_expo))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def rep_chem_html_expo(self) : # 4
        ''' Boîte de dialogue pour le chemin (répertoire)
        d'enregistrement de l'exposition dans un fichier 
        html et écriture du chemin sélectionné par 
        l'utilisateur dans le répertoire temporaire '''
        
        # Boîte de dialogue d'ouverture du répertoire
        self.ouv_Reps_Sauv_html_expo = QFileDialog.getExistingDirectory(None, _('Sélectionner un répertoire:'), os.path.expanduser('~'), QFileDialog.ShowDirsOnly)

        # Le chemin s'affiche dans le QLineEdit
        self.ligne_aff_sauv_html_expo.setText(self.ouv_Reps_Sauv_html_expo)
        
        # Appel de la classe Tempo_ecriture
        ecr_tempo_sauv_chem_rep_html_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_expo_html_murexpo_", "data_chem_sauv_rep_expo_html_murexpo", ".txt", self.ouv_Reps_Sauv_html_expo)

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Chemin d'enregistrement de l'exposition sous la forme d'un fichier html : {}\n").format(self.ouv_Reps_Sauv_html_expo))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def rep_chem_video_expo(self) : # 5
        ''' Boîte de dialogue pour le chemin (répertoire)
        d'enregistrement de l'exposition dans un fichier 
        vidéo et écriture du chemin sélectionné par 
        l'utilisateur dans le répertoire temporaire '''
        
        # Boîte de dialogue d'ouverture du répertoire
        self.ouv_Reps_Charg_video_expo = QFileDialog.getExistingDirectory(None, _('Sélectionner un répertoire:'), os.path.expanduser('~'), QFileDialog.ShowDirsOnly)

        # Le chemin s'affiche dans le QLineEdit
        self.ligne_aff_sauv_video_expo.setText(self.ouv_Reps_Charg_video_expo)
        
        # Appel de la classe Tempo_ecriture
        ecr_tempo_charg_chem_rep_video_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_expo_video_murexpo_", "data_chem_sauv_rep_expo_video_murexpo", ".txt", self.ouv_Reps_Charg_video_expo)

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Chemin d'enregistrement de l'exposition sous la forme d'une vidéo : {}\n").format(self.ouv_Reps_Charg_video_expo))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


    def parametres_defaut_chem_img(self) :
        ''' Revenir (ou adopter) aux/les paramètres par défaut ...
        (affichage des bonnes infos dans la QLineEdit concernée
        et écriture des infos dans le fichier tempo concerné ... '''
        
        if platform.system() == 'Linux' or platform.system() == 'Darwin' : 
            # Sous GNU/Linux ou MacOS ; /home/utilisateur est inscrit dans la QLineEdit
            self.ligne_aff_charg_images.setText("{}".format(QDir.homePath()))
            # /home/utilisateur est préalablement écrit dans le fichier tempo ...
            ecr_tempo_charg_chem_rep_imgages = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_charg_rep_images_murexpo_", "data_chem_charg_rep_images_murexpo", ".txt", "{}".format(QDir.homePath()))
        if platform.system() == 'Windows' :
            # Sous Windows, c'est le répertoire racine du système qui est inscrit ...
            self.ligne_aff_charg_images.setText("{}".format(QDir.rootPath()))
            # Le répertoire racine du système est préalablement écrit dans le fichier tempo ...
            ecr_tempo_charg_chem_rep_images = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_charg_rep_images_murexpo_", "data_chem_charg_rep_images_murexpo", ".txt", "{}".format(QDir.rootPath()))
        
        
    def parametres_defaut_panoramiq_img(self) :
        ''' Revenir (ou adopter) aux/les paramètres par défaut ...
        (affichage des bonnes infos dans la QLineEdit concernée
        et écriture des infos dans le fichier tempo concerné ... '''
        
        if platform.system() == 'Linux' or platform.system() == 'Darwin' :
            # Sous GNU/Linux ou MacOS ; /home/utilisateur est inscrit dans la QLineEdit
            self.ligne_aff_sauv_panoramiq_img_expo.setText("{}".format(QDir.homePath()))
            # /home/utilisateur est préalablement écrit dans le fichier tempo ...
            ecr_tempo_sauv_chem_rep_panoramiq_img_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo", ".txt", "{}".format(QDir.homePath()))
        if platform.system() == 'Windows' :
            # Sous Windows, c'est le répertoire racine du système qui est inscrit ...
            self.ligne_aff_sauv_panoramiq_img_expo.setText("{}".format(QDir.rootPath()))
            # Le répertoire racine du système est préalablement écrit dans le fichier tempo ...
            ecr_tempo_sauv_chem_rep_panoramiq_img_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_img_panoramique_expo_murexpo_", "data_chem_sauv_rep_img_panoramique_expo_murexpo", ".txt", "{}".format(QDir.rootPath()))
        
        
    def parametres_defaut_gif_anim(self) :
        ''' Revenir (ou adopter) aux/les paramètres par défaut ...
        (affichage des bonnes infos dans la QLineEdit concernée
        et écriture des infos dans le fichier tempo concerné ... '''
        
        if platform.system() == 'Linux' or platform.system() == 'Darwin' :
            # Sous GNU/Linux ou MacOS ; /home/utilisateur est inscrit dans la QLineEdit
            self.ligne_aff_sauv_gif_anime_expo.setText("{}".format(QDir.homePath()))
            # /home/utilisateur est préalablement écrit dans le fichier tempo ...
            ecr_tempo_sauv_chem_rep_gif_anime_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_gif_anime_murexpo_", "data_chem_sauv_rep_gif_anime_murexpo", ".txt", "{}".format(QDir.homePath()))
        if platform.system() == 'Windows' :
            # Sous Windows, c'est le répertoire racine du système qui est inscrit ...
            self.ligne_aff_sauv_gif_anime_expo.setText("{}".format(QDir.rootPath()))
            # Le répertoire racine du système est préalablement écrit dans le fichier tempo ...
            ecr_tempo_sauv_chem_rep_gif_anime_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_gif_anime_murexpo_", "data_chem_sauv_rep_gif_anime_murexpo", ".txt", "{}".format(QDir.rootPath()))
        
        
    def parametres_defaut_html(self) :
        ''' Revenir (ou adopter) aux/les paramètres par défaut ...
        (affichage des bonnes infos dans la QLineEdit concernée
        et écriture des infos dans le fichier tempo concerné ... '''
        
        if platform.system() == 'Linux' or platform.system() == 'Darwin' :
            # Sous GNU/Linux ou MacOS ; /home/utilisateur est inscrit dans la QLineEdit
            self.ligne_aff_sauv_html_expo.setText("{}".format(QDir.homePath()))
            # /home/utilisateur est préalablement écrit dans le fichier tempo ...
            ecr_tempo_sauv_chem_rep_html_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_expo_html_murexpo_", "data_chem_sauv_rep_expo_html_murexpo", ".txt", "{}".format(QDir.homePath()))
        if platform.system() == 'Windows' :
            # Sous Windows, c'est le répertoire racine du système qui est inscrit ...
            self.ligne_aff_sauv_html_expo.setText("{}".format(QDir.rootPath()))
            # Le répertoire racine du système est préalablement écrit dans le fichier tempo ...
            ecr_tempo_sauv_chem_rep_html_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_expo_html_murexpo_", "data_chem_sauv_rep_expo_html_murexpo", ".txt", "{}".format(QDir.rootPath()))
       
        
    def parametres_defaut_video(self) :
        ''' Revenir (ou adopter) aux/les paramètres par défaut ...
        (affichage des bonnes infos dans la QLineEdit concernée
        et écriture des infos dans le fichier tempo concerné ... '''
        
        if platform.system() == 'Linux' or platform.system() == 'Darwin' :
            # Sous GNU/Linux ou MacOS ; /home/utilisateur est inscrit dans la QLineEdit
            self.ligne_aff_sauv_video_expo.setText("{}".format(QDir.homePath()))
            # /home/utilisateur est préalablement écrit dans le fichier tempo ...
            ecr_tempo_charg_chem_rep_video_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_expo_video_murexpo_", "data_chem_sauv_rep_expo_video_murexpo", ".txt", "{}".format(QDir.homePath()))
        if platform.system() == 'Windows' :
            # Sous Windows, c'est le répertoire racine du système qui est inscrit ...
            self.ligne_aff_sauv_video_expo.setText("{}".format(QDir.rootPath()))
            # Le répertoire racine du système est préalablement écrit dans le fichier tempo ...
            ecr_tempo_charg_chem_rep_video_expo = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_chem_sauv_rep_expo_video_murexpo_", "data_chem_sauv_rep_expo_video_murexpo", ".txt", "{}".format(QDir.rootPath()))


class Configuration_Resolution(QWidget):
    ''' Classe pour la configuration de la résolution '''
    
    def __init__(self, parent=None):
        super(Configuration_Resolution, self).__init__(parent)
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # ----------------------------------------------------------------------------
        # Cases à cocher pour le choix de la résolution 
        # de la fenêtre principale de MurExpo         
        self.case_resolution_programmee = QCheckBox(_("Résolution programmée"))
        self.case_resolution_programmee.setFixedHeight(32)
        self.case_resolution_au_choix = QCheckBox(_("Résolution au choix"))
        self.case_resolution_au_choix.setFixedHeight(32)
        # La case Résolution automatique
        # est cochée par défaut
        self.case_resolution_programmee.setChecked(True)
        # ----------------------------------------------------------------------------
        # Combo pour le choix de la résolution
        # ----------------------------------------------------------------------------
        # Si on est sous windows ...
        if platform.system() == 'Windows' :
            # Testé fonctionnel sous windows 7/10 ...
            self.liste_choix_resolution = ["1200 x 720", "1200 x 800", "1210 x 681", "1216 x 811", "1224 x 768", "1301 x 732", 
                                           "1334 x 800", "1344 x 840", "1360 x 768", "1366 x 768", "1368 x 912", "1397 x 786", 
                                           "1415 x 794", "1444 x 900", "1444 x 1080", "1463 x 823", "1477 x 831", "1500 x 1000", 
                                           "1517 x 853", "1544 x 900", "1564 x 1043", "1644 x 900", "1683 x 947", "1700 x 800", 
                                           "1750 x 1080", "1768 x 992", "1776 x 1000", "1807 x 1016", "1829 x 1029", "1920 x 911", 
                                           "1920 x 1080", "1920 x 1200"]
        # Si on est sous GNU/Linux ou MacOS ...
        elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
            # Testé fonctionnel sous GNU/Linux ...
            self.liste_choix_resolution = ["1200 x 720", "1200 x 800", "1210 x 681", "1216 x 811", "1224 x 768", "1280 x 720", "1280 x 768", 
                                           "1280 x 800", "1280 x 960", "1280 x 1024", "1301 x 732", "1334 x 800", "1344 x 840", "1360 x 768", 
                                           "1366 x 768", "1368 x 912", "1397 x 786", "1415 x 794", "1444 x 900", "1444 x 1080", "1463 x 823", 
                                           "1477 x 831", "1500 x 1000", "1504 x 1003", "1517 x 853", "1536 x 864", "1536 x 960", "1544 x 900", 
                                           "1564 x 1043", "1644 x 900", "1683 x 947", "1700 x 800", "1707 x 960", "1750 x 1080", "1768 x 992", 
                                           "1776 x 1000", "1807 x 1016", "1829 x 1029", "1920 x 1080", "1920 x 1200"]
        self.combo_choix_resolution = QComboBox()
        # La liste du menu se fait par
        # l'intermédiaire d'un QComBoBox
        for c_resolution in self.liste_choix_resolution :
            self.combo_choix_resolution.addItem(c_resolution)
        self.combo_choix_resolution.setFixedWidth(150)
        self.combo_choix_resolution.setFixedHeight(32)
        self.combo_choix_resolution.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        # La combo est cachée
        self.combo_choix_resolution.hide()                                                                        
        # ----------------------------------------------------------------------------
        # Bouton pour le redémarrage après le changement de résolution
        # ----------------------------------------------------------------------------
        # Icône pour le bouton de changement de résolution
        icone_affichage = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
        # Bouton pour redémarrer le logiciel après un changement
        # de résolution (choix : Résolution au choix)
        self.bout_redemarrer = PushButtonIconeGauche(_("Appliquer les changements"), icon = icone_affichage, iconSize = QSize(16, 16))
        self.bout_redemarrer.setMaximumWidth(250)
        self.bout_redemarrer.setMinimumWidth(250)
        self.bout_redemarrer.setFixedHeight(32)
        # ToolTip pour l'information --> application 
        # du changement de résolution
        self.bout_redemarrer.setToolTip(_("Appliquer le changement de résolution"))
        
        # Ecriture "Résolution programmée" dans le répertoire temporaire
        ecr_tempo_choix_type_de_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_type_de_resolution_fenetre_principale_murexpo_", "data_type_de_resolution_fenetre_principale_murexpo", ".txt", _("Résolution programmée"))
        
        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
        
        # ========================================================
        # Exception pour contrôler la permanence du choix du type
        # de résolution, c'est à dire "Résolution programmée" ou
        # "Résolution au choix" par l'intermédiaire des cases à 
        # cocher ... et du choix de la résolution (par exemple
        # 1440 x 900) par l'utilisateur dans la combo
        # ========================================================
        try :
            # -------------------------------------
            # Si le type de résolution a déjà été
            # modifié ... Boucle de vérification.
            # -------------------------------------
            for contenu in liste_contenu_tmp :
                # Si le fichier tempo concerné (c'est à dire :
                # data_resolution_choisie_fenetre_principale_murexpo
                # est bien dedans ...
                if 'data_resolution_choisie_fenetre_principale_murexpo' in contenu :
                    print(_("Type de résolution : Résolution au choix"))
                    # Utilisation de la classe Tempo_lecture pour la lecture
                    # des données dans le répertoire (dit) temporaire (lecture
                    # de la résolution choisie, par exemple : 1440 x 900) ...
                    # et ce ds data_resolution_choisie_fenetre_principale_murexpo
                    rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_resolution_choisie_fenetre_principale_murexpo_', 'data_resolution_choisie_fenetre_principale_murexpo')
                    # ... avec appel de la méthode affichage
                    rep_tempo_resolution = rep_tempo_lecture.affichage()[0]
                    # Si le fichier tempo contient bien 
                    # des données (et n'est pas vide)
                    # le traitement se poursuit ...
                    if rep_tempo_resolution != "" : 
                        print(_("Résolution sélectionnée :"), rep_tempo_resolution) # A garder !!!
                        # Repérage de l'index de la combo correspondant à l'item préalablement sélectionné
                        # (et sauvegardé dans le fichier data_resolution_choisie_fenetre_principale_murexpo,
                        # se trouvant dans le répertoire temporaire) ... 
                        index = self.combo_choix_resolution.findText(rep_tempo_resolution, Qt.MatchFixedString)
                        # Attribution de l'index correspondant. La combo s'affichera avec
                        # l'item capturé dans le fichier (du répertoire temporaire) :
                        # data_resolution_choisie_fenetre_principale_murexpo
                        self.combo_choix_resolution.setCurrentIndex(index)
                        # La case "Résolution programmée" est décochée (!)
                        self.case_resolution_programmee.setChecked(False)
                        # La case "Résolution au choix" est cochée (!)
                        self.case_resolution_au_choix.setChecked(True)
                        # La combo du choix de la résolution est montrée
                        self.combo_choix_resolution.show()
                        # Ecriture "Résolution au choix" dans le rép. tempo
                        # --> data_type_de_resolution_fenetre_principale_murexpo
                        ecr_tempo_choix_type_de_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_type_de_resolution_fenetre_principale_murexpo_", "data_type_de_resolution_fenetre_principale_murexpo", ".txt", _("Résolution au choix"))
                    
        # Ici le fichier peut exister ... MAIS ...
        except : 
            # Si la case "Résolution programmée" est cochée ...
            if self.case_resolution_programmee.isChecked() :
                print(_("Type de résolution : Résolution programmée"))
                # Ecriture "Résolution programmée" dans le répertoire temporaire
                ecr_tempo_choix_type_de_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_type_de_resolution_fenetre_principale_murexpo_", "data_type_de_resolution_fenetre_principale_murexpo", ".txt", _("Résolution programmée"))
                # Ecriture de "" (c'est à dire une donnée vide) dans le rep. 
                # tempo parce que l'utilisateur a choisi de fonctionner avec 
                # la "Résolution programmée" (--> Important d'écrire ici ... 
                # à ce niveau !)
                ecr_tempo_choix_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_choisie_fenetre_principale_murexpo_", "data_resolution_choisie_fenetre_principale_murexpo", ".txt", "")
                # La case "Résolution programmée" est cochée (!)
                self.case_resolution_programmee.setChecked(True)
                # La case "Résolution au choix" est décochée (!)
                self.case_resolution_au_choix.setChecked(False)
                # La combo du choix de la résolution est cachée
                self.combo_choix_resolution.hide()

        # -------------------------------------------------------
        grid_1 = QGridLayout()
        grid_1.setSpacing(4)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        grid_1.addWidget(self.case_resolution_programmee, 0, 0)
        grid_1.addWidget(self.case_resolution_au_choix, 0, 1)
        grid_1.addWidget(self.combo_choix_resolution, 0, 2)
        grid_1.addWidget(self.bout_redemarrer, 0, 3)

        # Alignement/centrage sur la hauteur des widgets
        grid_1.setAlignment(Qt.AlignVCenter)
        grid_1.setAlignment(Qt.AlignHCenter)

        #
        self.setLayout(grid_1)
        # -------------------------------------------------------

        # =====================================================
        # Signaux
        # =====================================================
        # Les cases à cocher QCheckBox (utilisation de lambda
        # car il y a un argument dans la fonction appelée)
        # 1 Fonction pour chaque QCheckBox
        # -----------------------------------------
        # Signaux pour les cases pour la résolution ("Résolution programmée" ou "Résolution au choix")
        self.case_resolution_programmee.toggled.connect(lambda : self.fonction_resolution_programmee(self.case_resolution_programmee))
        self.case_resolution_au_choix.toggled.connect(lambda : self.fonction_resolution_choisie(self.case_resolution_au_choix))
        # Signal du combobox du choix de la résolution (par exemple 1366 x 768)
        self.combo_choix_resolution.currentIndexChanged.connect(self.fonction_choix_resolution)
        # Signal pour appliquer le changement de résolution
        # --> le logiciel redémarre pour opérer la nouvelle
        # résolution (celle choisie dans la combo pour le
        # choix Résolution au choix ou re-sélectionnat la 
        # case à cocher Résolution programmée)
        # -----------------------------
        # Appel de la fonction redemarrer contenue
        # dans le module redemarrage_suite_modifs 
        # (voir import plus haut).
        self.bout_redemarrer.clicked.connect(redemarrer)
        # =====================================================
        
        
    def fonction_resolution_programmee(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) pour la résolution 
        programmée a été cochée """
        
        if c.isChecked() == True :
            # La case "Résolution au choix" est décochée (!)
            self.case_resolution_au_choix.setChecked(False)
            # Le texte est pris en compte
            resolution_programmee = str(c.text())
            # Ecriture de "Résolution programmée" dans le fichier tempo. 
            # ATENTION !!! : "Résolution &programmée" risque d'être écrit ... donc 
            # il faudra tester "Résolution programmée" ou "Résolution &programmée"
            # ou bien encore "&Résolution programmée" !!!
            ecr_tempo_choix_type_de_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_type_de_resolution_fenetre_principale_murexpo_", "data_type_de_resolution_fenetre_principale_murexpo", ".txt", resolution_programmee)

            # La combo ... est cachée
            self.combo_choix_resolution.hide()
            
            # Ecriture de "" (c'est à dire un vide) dans le rep. temp parce que 
            # l'utilisateur a choisi de fonctionner avec la "Résolution programmée"
            ecr_tempo_choix_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_choisie_fenetre_principale_murexpo_", "data_resolution_choisie_fenetre_principale_murexpo", ".txt", "")
        
        
    def fonction_resolution_choisie(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) pour la résolution 
        au choix a été cochée """
        
        if c.isChecked() == True :
            # La case "Résolution programmée" est décochée (!)
            self.case_resolution_programmee.setChecked(False)
            # Le texte est pris en compte
            resolution_choisie = str(c.text())
            # Ecriture de "Résolution au choix" dans le fichier tempo. 
            # ATENTION !!! : "Résolution &au choix" risque d'être écrit ... donc 
            # il faudra tester "Résolution au choix" ou "Résolution &au choix"
            ecr_tempo_choix_type_de_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_type_de_resolution_fenetre_principale_murexpo_", "data_type_de_resolution_fenetre_principale_murexpo", ".txt", resolution_choisie)
        
            # La combo ... est cachée
            self.combo_choix_resolution.show()
            
            # Ecriture du choix de la résolution choisie dans le répertoire
            # temporaire (--> Important d'écrire ici ... à ce niveau !)
            ecr_tempo_choix_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_choisie_fenetre_principale_murexpo_", "data_resolution_choisie_fenetre_principale_murexpo", ".txt", self.combo_choix_resolution.currentText())
        

    def fonction_choix_resolution(self) :
        """ Fonction servant à écrire (dans le répertoire temporaire) 
        le choix de la résolution choisie dans la combo """
        
        # Ecriture du choix de la résolution choisie dans le répertoire temporaire
        ecr_tempo_choix_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_choisie_fenetre_principale_murexpo_", "data_resolution_choisie_fenetre_principale_murexpo", ".txt", self.combo_choix_resolution.currentText())


class Configuration_Mode_Affichage_Onglets(QWidget) :
    ''' Classe qui affiche la config du mode d'affichage des onglets '''
    
    def __init__(self, parent=None):
        super(Configuration_Mode_Affichage_Onglets, self).__init__(parent)
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # ======================================
        # Le label
        # ======================================
        label_combo_mode_sel_onglet_general_conf = QLabel(_("Mode d'affichage :"))
        
        # ====================================================================
        # Combobox de sélection de l'apparence des onglets
        # ====================================================================
        # Création du combo pour sélection de l'entrée
        # principale du réglage de l'apprence des onglets
        self.combo_apparence_des_onglets = QComboBox()
        self.combo_apparence_des_onglets.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 2px")
        # ------------------------------
        # Dimensionnement de la combo
        self.combo_apparence_des_onglets.setMaximumWidth(350)
        self.combo_apparence_des_onglets.setMinimumWidth(350)
        
        # Liste des icônes
        # ------------------------------
        # (--> Affichage avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        # ------------------------------
        liste_icones = [QIcon(chemin_ressources('ressources_murexpo'+os.sep+'icone_rubriques_combobox_apsc_256x256.png')), QIcon(chemin_ressources('ressources_apsc'+os.sep+'icone_rubriques_combobox_apsc_256x256.png')),
                        QIcon(chemin_ressources('ressources_murexpo'+os.sep+'icone_rubriques_combobox_apsc_256x256.png'))]
        
        # Liste des sélections
        liste_selection_combo_app_onglets = [_('Texte uniquement'), _('Texte et icônes'), _('Icônes uniquement')]
        # Attribution ...
        for icones, rubriques in zip(liste_icones, liste_selection_combo_app_onglets) :
            self.combo_apparence_des_onglets.addItem(icones, rubriques)
            
        # ----------------------------------------------------------------------------
        # Bouton pour le redémarrage après le modifs des chemins
        # ----------------------------------------------------------------------------
        # Icône pour le bouton de changement des chemins
        icone_affichage = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
        # Bouton pour redémarrer le logiciel après un changement
        # de répertoire (chargement et sauvegarde)
        self.bout_redemarrer = PushButtonIconeGauche(_("Appliquer les changements"), icon = icone_affichage, iconSize = QSize(16, 16))
        self.bout_redemarrer.setFixedHeight(32)
        self.bout_redemarrer.setMaximumWidth(250)
        self.bout_redemarrer.setMinimumWidth(250)
        # ToolTip pour l'information --> application du changement d'apparence des onglets
        self.bout_redemarrer.setToolTip(_("Appliquer les modifications après re-définition de l'aspect visuel des onglets"))
        # ----------------------------------------------------------------------------
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()

        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
        
        # --------------------------------------------------------
        # Exception pour contrôler la permanence du choix de
        # l'item ('Texte uniquement', 'Texte et icônes' ou 
        # 'Icônes uniquement') choisi par l'utilisateur dans
        # la combo
        # --------------------------------------------------------
        try :
            # ----------
            # Si l'apparence des onglets a déjà été modifiéé
            # ... Boucle de vérification ...
            # ----------
            for contenu in liste_contenu_tmp :
                # Si le fichier tempo concerné est bien dedans ...
                if 'data_affichage_onglets_murexpo' in contenu :
                    # Utilisation de la classe Tempo_lecture pour la lecture
                    # des données dans le répertoire (dit) temporaire
                    rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_affichage_onglets_murexpo_', 'data_affichage_onglets_murexpo')
                    # ... avec appel de la méthode affichage
                    rep_tempo_aff = rep_tempo_lecture.affichage()[0]
                    # Repérage de l'index de la combo correspondant à l'item préalablement 
                    # sélectionné (et sauvegardé dans le fichier data_affichage_onglets_apsc_ ...
                    # se trouvant dans le répertoire temporaire) ... 
                    index = self.combo_apparence_des_onglets.findText(rep_tempo_aff, Qt.MatchFixedString)
                    # Attribution de l'index correspondant. La combo s'affichera
                    # avec l'item capturé dans le fichier data_affichage_onglets_murexpo_ ...
                    self.combo_apparence_des_onglets.setCurrentIndex(index)
                    
                    # ---------------------------------------------------------------
                    # Retour commandes ....
                    # ---------------------------------------------------------------
                    # Récupération des commandes pour le terminal interne de MurExpo
                    self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Mode d'affichage des onglets : {}\n").format(rep_tempo_aff))
                    # Ecriture des commandes dans le fichier
                    # tempo data_retour_commandes_murexpo
                    self.capter_commandes_murexpo.liste_ecrire_commandes()
                    # ---------------------------------------------------------------
                    
        # Si le fichier tempo n'existe pas ...
        except : 
            # ... il est écrit (avec l'item : Texte uniquement)
            ecr_tempo_sauv_aff_onglets = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_affichage_onglets_murexpo_", "data_affichage_onglets_murexpo", ".txt", _('Texte uniquement'))
            
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Affichage de l'onglet : Texte uniquement\n"))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ====================================================================
        
        #
        grid_1 = QGridLayout()
        grid_1.setSpacing(10)
        
        # Placement des widgets dans la grille 1
        grid_1.addWidget(label_combo_mode_sel_onglet_general_conf, 0, 0)
        grid_1.addWidget(self.combo_apparence_des_onglets, 0, 1)
        grid_1.addWidget(self.bout_redemarrer, 0, 2)
        
        # Alignement vertical et horizontal
        grid_1.setAlignment(Qt.AlignVCenter)
        grid_1.setAlignment(Qt.AlignHCenter)
        
        #
        self.setLayout(grid_1)
        
        # ==========================================================
        # Signaux
        # ==========================================================
        # Signal pour la combobox de l'affichage des rubriques
        self.combo_apparence_des_onglets.currentIndexChanged.connect(self.affichage_rubriques)
        # Signal pour appliquer à la fin des modifs de
        # chemins --> le logiciel redémarre quand on
        # clique sur le bouton et les changements
        # deviennent effectifs
        # -----------------------------
        # Appel de la fonction redemarrer contenue
        # dans le module redemarrage_suite_modifs 
        # (voir import plus haut).
        self.bout_redemarrer.clicked.connect(redemarrer)
        # ==========================================================
        
        
    def affichage_rubriques(self) :
        ''' Fonction pour sélectionner et sauver dans le
        répertoire remporaire le choix de l'utilisateur 
        quant au mode d'affichage de l'onglet '''
        
        texte_combo = str(self.combo_apparence_des_onglets.currentText())

        # Appel de la classe Tempo_ecriture
        ecr_tempo_sauv_aff_onglets = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_affichage_onglets_murexpo_', 'data_affichage_onglets_murexpo', ".txt", texte_combo)


class Configuration_Traitement_Intermediaire(QWidget) :
    ''' Classe servant à la modification du réglage de la
    valeur (de la largeur, de la hauteur, ou les deux ... 
    selon les cas) ... définie par l'appel de la fonction 
    redimensionnement_image_oeuvre (contenue dans le script 
    redimensionnement_image_oeuvre.py) ... et appelée à deux 
    reprises dans la classe MonGraphiqueVueMur (contenue 
    dans fabrique_murexpo_gui.py). En fait il s'agit du
    réglage de la taille des images : image_oeuvre.png et :
    image_oeuvre_selectionnee.png crées au cours du 
    traitement dans le répertoire temporaire). C'est la
    valeur de la taille (w, h, ou w x h selon les cas) et
    ce pour diminuer les temps de traitement (pour une 
    photo de 4026 x 2876, par exemple, le temps de 
    traitement peut être long). Avec ce réglage 
    (défini en paramètre dans la fonction 
    redimensionnement_image_oeuvre) les images
    (image_oeuvre.png et image_oeuvre_selectionnee.png) 
    peuvent être redimensionnées si la taille la + grde
    (entre w et h) dépasse la valeur réglée ici dans le
    slider et/ou le spinbox ... et diminuera incontestablement
    les temps de rendu (bien entendu + la taille réglée est 
    basse + le rendu sera court ... mais en contrepartie +
    la qualité de rendu sera basse) '''
    
    def __init__(self, parent=None):
        super(Configuration_Traitement_Intermediaire, self).__init__(parent)
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # ======================================
        # Le label
        # ======================================
        label_taille_image = QLabel(_("Taille de l'image (en pixels) :"))
        
        # ====================================================================
        # Slider et Spinbox connectés ensemble (voir fonctions :
        # fonction_spin_changement_taille_img_inter et
        # fonction_slider_changement_taille_img_inter
        # ====================================================================
        # Création du sslider pour modifier la taille ...
        self.slider_taille_img_inter = QSlider()
        self.slider_taille_img_inter.setOrientation(Qt.Horizontal)
        self.slider_taille_img_inter.setFixedWidth(350)
        self.slider_taille_img_inter.setMinimum(600)
        self.slider_taille_img_inter.setMaximum(1600)
        self.slider_taille_img_inter.setValue(950)
        self.slider_taille_img_inter.setSingleStep(1)
        # Le tooltip d'info
        self.slider_taille_img_inter.setToolTip(_("Permet de définir la taille des images pendant le traitement interne de MurExpo.\nUne valeur plus petite va accélérer le temps de traitement (au détriment de la\nqualité du rendu). Une valeur trop grande va augmenter la qualité, mais en\ncontrepartie le temps de rendu sera plus long."))

        # Création du spinbox pour modifier la taille ...
        self.spinbox_taille_img_inter = QSpinBox()
        self.spinbox_taille_img_inter.setFixedWidth(65)
        self.spinbox_taille_img_inter.setRange(600, 1600)
        self.spinbox_taille_img_inter.setValue(950)
        self.spinbox_taille_img_inter.setSingleStep(1)
        # Le toolTip d'info
        self.spinbox_taille_img_inter.setToolTip(_("Permet de définir la taille des images pendant le traitement interne de MurExpo.\nUne valeur plus petite va accélérer le temps de traitement (au détriment de la\nqualité du rendu). Une valeur trop grande va augmenter la qualité, mais en\ncontrepartie le temps de rendu sera plus long."))
            
        # ----------------------------------------------------------------------------
        # Bouton pour appliquer les modifs
        # ----------------------------------------------------------------------------
        # Icône pour le bouton ...
        icone_affichage = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
        self.bout_appliquer = PushButtonIconeGauche(_("Appliquer"), icon = icone_affichage, iconSize = QSize(16, 16))
        self.bout_appliquer.setFixedHeight(32)
        self.bout_appliquer.setMaximumWidth(200)
        self.bout_appliquer.setMinimumWidth(200)
        # ToolTip pour l'information --> application 
        # de l'écriture ds le rep. tempo
        self.bout_appliquer.setToolTip(_("Appliquer les modifications"))
        # ----------------------------------------------------------------------------
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()

        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)

        # --------------------------------------------------------
        # Exception pour contrôler la permanence du
        # choix du slider et aussi du choix du spinbox
        # --------------------------------------------------------
        try :
            # ----------
            # Si Taille de l'image (en pixels) a déjà
            # été modifiéé .. Boucle de vérification ...
            # ----------
            for contenu in liste_contenu_tmp :
                # Si le fichier tempo concerné est bien dedans ...
                if 'data_taille_image_traitement_intermediaire' in contenu :
                    # Utilisation de la classe Tempo_lecture pour la lecture
                    # des données dans le répertoire (dit) temporaire
                    rep_tempo_lecture_val = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_taille_image_traitement_intermediaire_', 'data_taille_image_traitement_intermediaire')
                    # ... avec appel de la méthode affichage
                    rep_tempo_aff_valeur = rep_tempo_lecture_val.affichage()[0]
                    # Attribution de la valeur dans le slider ...
                    self.slider_taille_img_inter.setValue(int(rep_tempo_aff_valeur))
                    # Attribution de la valeur dans le spinbox ...
                    self.spinbox_taille_img_inter.setValue(int(rep_tempo_aff_valeur))
        # Si le fichier tempo n'existe pas ...
        except : 
            # Ecriture du réglage (le spinbox) dans le répertoire temporaire
            ecr_tempo_taille_img_trait_inter = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_image_traitement_intermediaire_", "data_taille_image_traitement_intermediaire", ".txt", '1200')
        # --------------------------------------------------------
        
        grid_1 = QGridLayout()
        grid_1.setSpacing(10)
        
        # Placement des widgets dans la grille 1
        grid_1.addWidget(label_taille_image, 0, 0, alignment=Qt.AlignCenter)
        grid_1.addWidget(self.slider_taille_img_inter, 0, 1, alignment=Qt.AlignCenter)
        grid_1.addWidget(self.spinbox_taille_img_inter, 0, 2, alignment=Qt.AlignCenter)
        grid_1.addWidget(self.bout_appliquer, 0, 3, alignment=Qt.AlignCenter)
        
        # Alignement vertical et horizontal
        grid_1.setAlignment(Qt.AlignVCenter)
        grid_1.setAlignment(Qt.AlignHCenter)
        
        #
        self.setLayout(grid_1)

        # =====================================================
        # Signaux
        # =====================================================
        # Signal pour le changement de taille de l'image
        # intermédiaire (traitement dans MurExpo) dans le spinbox
        self.spinbox_taille_img_inter.valueChanged.connect(self.fonction_spin_changement_taille_img_inter)
        # Signal pour le changement de taille de l'image
        # intermédiaire (traitement dans MurExpo) dans le slider
        self.slider_taille_img_inter.valueChanged.connect(self.fonction_slider_changement_taille_img_inter)
        # Signal pour rendre effectif le changement de 
        # taille (résolution) de l'image pour le traitement 
        # intermédiaire dans MurExpo
        self.bout_appliquer.clicked.connect(self.fonction_appliquer_taille_img_inter)
        # =====================================================
        
        
    def fonction_spin_changement_taille_img_inter(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox de la Taille de l'image (en pixels) 
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """
        
        # Le spinbox est conecté au slider (la valeur 
        # du spinbox est affectée dans le slider)
        self.slider_taille_img_inter.setValue(self.spinbox_taille_img_inter.value())
        
        # Ecriture préalable de la donnée dans le rep. tampon ... même si l'action du bouton 'Appliquer' la ré-écrit ensuite
        ecr_tempo_taille_img_trait_inter = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_image_traitement_intermediaire_", "data_taille_image_traitement_intermediaire", ".txt", str(self.spinbox_taille_img_inter.value()))


    def fonction_slider_changement_taille_img_inter(self) :
        """ Fonction pour écrire le changement de valeur 
        du slider de la Taille de l'image (en pixels) 
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """
        
        # Le slider est conecté au spinbox (la valeur 
        # du slider est affectée dans le spinbox)
        self.spinbox_taille_img_inter.setValue(self.slider_taille_img_inter.value()) 
        
        # Ecriture préalable de la donnée dans le rep. tampon ... même si l'action du bouton 'Appliquer' la ré-écrit ensuite
        ecr_tempo_taille_img_trait_inter = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_image_traitement_intermediaire_", "data_taille_image_traitement_intermediaire", ".txt", str(self.slider_taille_img_inter.value()))
  

    def fonction_appliquer_taille_img_inter(self) :
        """ Appliquer les changements """
        
        # Ecriture de la donnée dans le rep. tampon
        ecr_tempo_taille_img_trait_inter = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_image_traitement_intermediaire_", "data_taille_image_traitement_intermediaire", ".txt", str(self.spinbox_taille_img_inter.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Changement de la taille de l'image intermédiaire (traitement interne du logiciel) : {}\n").format(self.spinbox_taille_img_inter.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


class Configuration_Taille_Vignettes_Html_Pdf(QWidget) :
    ''' Classe servant à ... '''
    
    def __init__(self, parent=None):
        super(Configuration_Taille_Vignettes_Html_Pdf, self).__init__(parent)
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # ======================================
        # Le label
        # ======================================
        label_taille_vignettes_html_pdf = QLabel(_("Taille des vignettes (en pixels) :"))
        
        # ====================================================================
        # Slider et Spinbox connectés ensemble (voir fonctions :
        # fonction_spin_changement_taille_vignettes et
        # fonction_slider_changement_taille_vignettes
        # ====================================================================
        # Création du spinbox pour modifier la taille ...
        self.slider_taille_vignettes_html_pdf = QSlider()
        self.slider_taille_vignettes_html_pdf.setOrientation(Qt.Horizontal)
        self.slider_taille_vignettes_html_pdf.setFixedWidth(350)
        self.slider_taille_vignettes_html_pdf.setMinimum(50)
        self.slider_taille_vignettes_html_pdf.setMaximum(500)
        self.slider_taille_vignettes_html_pdf.setValue(200)
        self.slider_taille_vignettes_html_pdf.setSingleStep(1)
        # Le tooltip d'info
        self.slider_taille_vignettes_html_pdf.setToolTip(_("Permet de définir la largeur des vignettes contenues dans le fichier html/pdf."))

        # Création du spinbox pour modifier la taille ...
        self.spinbox_taille_vignettes_html_pdf = QSpinBox()
        self.spinbox_taille_vignettes_html_pdf.setFixedWidth(65)
        self.spinbox_taille_vignettes_html_pdf.setRange(50, 500)
        self.spinbox_taille_vignettes_html_pdf.setValue(200)
        self.spinbox_taille_vignettes_html_pdf.setSingleStep(1)
        # Le toolTip d'info
        self.spinbox_taille_vignettes_html_pdf.setToolTip(_("Permet de définir la largeur des vignettes contenues dans le fichier html/pdf."))
            
        # ----------------------------------------------------------------------------
        # Bouton pour appliquer les modifs
        # ----------------------------------------------------------------------------
        # Icône pour le bouton ...
        icone_affichage = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
        self.bout_appliquer = PushButtonIconeGauche(_("Appliquer"), icon = icone_affichage, iconSize = QSize(16, 16))
        self.bout_appliquer.setFixedHeight(32)
        self.bout_appliquer.setMaximumWidth(200)
        self.bout_appliquer.setMinimumWidth(200)
        # ToolTip pour l'information --> application 
        # de l'écriture ds le rep. tempo
        self.bout_appliquer.setToolTip(_("Appliquer les modifications"))
        # ----------------------------------------------------------------------------
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()

        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)

        # --------------------------------------------------------
        # Exception pour contrôler la permanence du
        # choix du slider et aussi du choix du spinbox
        # --------------------------------------------------------
        try :
            # ----------
            # Si Taille de la vignette (en pixels) a déjà
            # été modifiéé .. Boucle de vérification ...
            # ----------
            for contenu in liste_contenu_tmp :
                # Si le fichier tempo concerné est bien dedans ...
                if 'data_taille_vignettes_html_pdf' in contenu :
                    # Utilisation de la classe Tempo_lecture pour la lecture
                    # des données dans le répertoire (dit) temporaire
                    rep_tempo_lecture_val = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_taille_vignettes_html_pdf_', 'data_taille_vignettes_html_pdf')
                    # ... avec appel de la méthode affichage
                    rep_tempo_aff_valeur = rep_tempo_lecture_val.affichage()[0]
                    # Attribution de la valeur dans le slider ...
                    self.slider_taille_vignettes_html_pdf.setValue(int(rep_tempo_aff_valeur))
                    # Attribution de la valeur dans le spinbox ...
                    self.spinbox_taille_vignettes_html_pdf.setValue(int(rep_tempo_aff_valeur))
        # Si le fichier tempo n'existe pas ...
        except : 
            # Ecriture du réglage (le spinbox) dans le répertoire temporaire
            ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettes_html_pdf_", "data_taille_vignettes_html_pdf", ".txt", '200')
        # --------------------------------------------------------
        
        grid_1 = QGridLayout()
        grid_1.setSpacing(10)
        
        # Placement des widgets dans la grille 1
        grid_1.addWidget(label_taille_vignettes_html_pdf, 0, 0, alignment=Qt.AlignCenter)
        grid_1.addWidget(self.slider_taille_vignettes_html_pdf, 0, 1, alignment=Qt.AlignCenter)
        grid_1.addWidget(self.spinbox_taille_vignettes_html_pdf, 0, 2, alignment=Qt.AlignCenter)
        grid_1.addWidget(self.bout_appliquer, 0, 3, alignment=Qt.AlignCenter)
        
        # Alignement vertical et horizontal
        grid_1.setAlignment(Qt.AlignVCenter)
        grid_1.setAlignment(Qt.AlignHCenter)
        
        #
        self.setLayout(grid_1)

        # =====================================================
        # Signaux
        # =====================================================
        # Signal pour le changement de taille de l'image
        # intermédiaire (traitement dans MurExpo) dans le spinbox
        self.spinbox_taille_vignettes_html_pdf.valueChanged.connect(self.fonction_spin_changement_taille_vignettes_html_pdf)
        # Signal pour le changement de taille de l'image
        # intermédiaire (traitement dans MurExpo) dans le slider
        self.slider_taille_vignettes_html_pdf.valueChanged.connect(self.fonction_slider_changement_taille_vignettes_html_pdf)
        # Signal pour rendre effectif le changement de 
        # taille (résolution) de l'image pour le traitement 
        # intermédiaire dans MurExpo
        self.bout_appliquer.clicked.connect(self.fonction_appliquer_taille_vignettes_html_pdf)
        # =====================================================
        
        
    def fonction_spin_changement_taille_vignettes_html_pdf(self) :
        """ Fonction pour écrire le changement de valeur 
        du spinbox de la Taille de l'image (en pixels) 
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """
        
        # Le spinbox est conecté au slider (la valeur 
        # du spinbox est affectée dans le slider)
        self.slider_taille_vignettes_html_pdf.setValue(self.spinbox_taille_vignettes_html_pdf.value())
        
        # Ecriture préalable de la donnée dans le rep. tampon ... même si l'action du bouton 'Appliquer' la ré-écrit ensuite
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettes_html_pdf_", "data_taille_vignettes_html_pdf", ".txt", str(self.spinbox_taille_vignettes_html_pdf.value()))


    def fonction_slider_changement_taille_vignettes_html_pdf(self) :
        """ Fonction pour écrire le changement de valeur 
        du slider de la Taille de l'image (en pixels) 
        ... et écriture de la valeur dans le fichier 
        concerné dans le répertoire temporaire """
        
        # Le slider est conecté au spinbox (la valeur 
        # du slider est affectée dans le spinbox)
        self.spinbox_taille_vignettes_html_pdf.setValue(self.slider_taille_vignettes_html_pdf.value()) 
        
        # Ecriture préalable de la donnée dans le rep. tampon ... même si l'action du bouton 'Appliquer' la ré-écrit ensuite
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettes_html_pdf_", "data_taille_vignettes_html_pdf", ".txt", str(self.slider_taille_vignettes_html_pdf.value()))
  

    def fonction_appliquer_taille_vignettes_html_pdf(self) :
        """ Appliquer les changements """
        
        # Ecriture de la donnée dans le rep. tampon
        ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettes_html_pdf_", "data_taille_vignettes_html_pdf", ".txt", str(self.spinbox_taille_vignettes_html_pdf.value()))

        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Changement de la taille des vignettes au niveau de l'affichage dans l'exportation html/pdf : {}\n").format(self.spinbox_taille_vignettes_html_pdf.value()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------


class Retour_Commandes(QWidget) :
    ''' Classe le retour des commandes ... traitements 
    effectués par MurExpo --> (script : capter_commandes_murexpo, 
    classe : CapterCommandesMurExpo, dans le QTextEdit présent ici '''
    
    def __init__(self, parent=None):
        super(Retour_Commandes, self).__init__(parent)
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # Terminal dans un QTextEdit ...
        self.texte_edit_retour_commandes = QTextEdit()
        # Couleurs du fond (gris) et du texte (blanc)
        # Solution trouvée ici :
        # https://riverbankcomputing.com/pipermail/pyqt/2007-January/015237.html
        # L'avantage avec cette méthode est que la barre de défilement reste à
        # la couleur de l'interface et pas  du fond du QTextEdit
        palette = self.texte_edit_retour_commandes.palette()
        palette.setColor(QPalette.Active, QPalette.Text, QColor(255, 255, 255))
        palette.setColor(QPalette.Active, QPalette.Base, QColor(100, 100, 100))
        self.texte_edit_retour_commandes.setPalette(palette)
        self.texte_edit_retour_commandes.setStyleSheet("background-color: rgb(100, 100, 100);")
        self.texte_edit_retour_commandes.setTextColor(QColor(255, 255, 255))
        self.fonte(self.texte_edit_retour_commandes) # Fontion pour l'attribution de la fonte
        # Le QTextEdit est placé en lecture seule
        self.texte_edit_retour_commandes.setReadOnly(True)
        
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe 1 --> elle est ici : affichage_apsc/push_button_icone_droite.py ... 
        # en fait le texte dans le bouton est centré et l'icône positionné à droite.
        # J'ai un peu modifié la classe et adaptée. Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : config_apsc/chemin_data_ressources.py)
        icone_retour = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_gauche_murexpo_256x256.png')))
        #
        self.bout_maj = PushButtonIconeGauche(_("Mise à jour des commandes"), icon = icone_retour, iconSize = QSize(16, 16))
        self.bout_maj.setFixedWidth(300)
        self.bout_maj.setToolTip(_("Permet de mettre à jour l'affichage des commandes"))

        grid_1 = QGridLayout()
        grid_1.setSpacing(8)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        grid_1.addWidget(self.texte_edit_retour_commandes, 0, 0)
        # Alignement du bouton au centre
        grid_1.addWidget(self.bout_maj, 1, 0, alignment=Qt.AlignCenter)

        #
        self.setLayout(grid_1)
        
        # =====================================================
        # Signal
        # =====================================================
        # Signal pour faire la mise à jour des commandes
        # dans le QTextEdit des retours commandes
        self.bout_maj.clicked.connect(self.ecriture_commandes)
        
        # Afficher (ds un 1er temps) les commandes (et
        # ce avant leur mise à jour avec le bouton)
        self.ecriture_commandes()
        
        
    def ecriture_commandes(self) :
        """ Fonction d'affichage des commandes dans
        le terminal et mise à jour de celui-ci """
        
        # Utilisation de la classe Tempo_lecture pour la lecture 
        # des données dans le répertoire (dit) temporaire
        pre_lecture_retour_commandes = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_retour_commandes_murexpo_', 'data_retour_commandes_murexpo')
        # ... avec appel de la méthode affichage
        lecture_retour_commandes = "".join(pre_lecture_retour_commandes.affichage())
        
        # Texte du (terminal) inéré dans le QTextEdit
        self.texte_edit_retour_commandes.setText(lecture_retour_commandes)
        # Permet la mise à jour ...
        QApplication.processEvents()
        
        
    def fonte(self, widget) :
        ''' Fonction d'attribution de la fonte dans le QTextEdit'''

        f = QFont()
        f.setFamily('Liberation Sans')
        f.setFixedPitch(True)
        f.setPointSize(10)
        widget.setFont(f)
        
        
class Reinitialiser(QWidget) :
    ''' Classe pour réinitialiser complètement l'interface 
    de MurExpo (en vidant complètement le répertoire temporaire) '''
    
    def __init__(self, parent=None):
        super(Reinitialiser, self).__init__(parent)

        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()

        # Icône pour le bouton de changement de résolution
        icone_rei = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
        # Bouton pour redémarrer le logiciel ...
        self.bout_redemarrer = PushButtonIconeGauche(_("Réinitialiser la configuration"), icon = icone_rei, iconSize = QSize(16, 16))
        self.bout_redemarrer.setMaximumWidth(450)
        self.bout_redemarrer.setMinimumWidth(450)
        self.bout_redemarrer.setFixedHeight(32)
        # ToolTip pour l'information --> application 
        # du changement de résolution
        self.bout_redemarrer.setToolTip(_("Supprimer les fichiers temporaires et réninitialiser le logiciel"))

        # -------------------------------------------------------
        grid_1 = QGridLayout()
        grid_1.setSpacing(4)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        grid_1.addWidget(self.bout_redemarrer, 0, 0)

        # Alignement/centrage sur la hauteur des widgets
        grid_1.setAlignment(Qt.AlignVCenter)
        grid_1.setAlignment(Qt.AlignHCenter)
        
        #
        self.setLayout(grid_1)

        # =====================================================
        # Signaux
        # =====================================================
        # Appel de la fonction redemarrer contenue
        # dans le module redemarrage_suite_modifs 
        # (voir import plus haut).
        self.bout_redemarrer.clicked.connect(self.supprimer_et_reinitialiser)
        self.bout_redemarrer.clicked.connect(redemarrer)
        # =====================================================


    def supprimer_et_reinitialiser(self) :
        """ Fonction pour supprimer tous les fichiers 
        du rep. tempo et relancer le logiciel """

        # ----------------------------------------------------------------------------
        # Nettoyage du répertoire temporaire
        # ----------------------------------------------------------------------------
        # On liste le contenu du répertoire temporaire
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)

        # --------------------------------------------------------
        # Suppression des fichiers ...
        # --------------------------------------------------------
        for contenu in liste_contenu_tmp : 
            os.remove(os.path.join(self.chemin_tempo_system_exploit, contenu))    
        

class StylesInterface(QWidget) :
    """ Classe qui permet de sélectionner 
    le style de l'interface """
    
    def __init__(self, parent):
        super(StylesInterface, self).__init__(parent)
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # La comprehension liste de la capture des styles est lancée ...
        self.liste_styles = [style for style in QStyleFactory.keys() if style not in ["Fusion", 'bb10bright', 'bb10dark', 'gtk2', 'dsemilight', 'dsemidark', 'dlight', 'ddark']]
        
        # ----------------------------------------------------------------------------
        # Le label
        # ----------------------------------------------------------------------------
        label_apparence_style = QLabel(_("Style de l'interface :"))

        # ----------------------------------------------------------------------------
        # La combo
        # ----------------------------------------------------------------------------
        # Liste des styles présents ...
        self.combo_styles = QComboBox()
        self.combo_styles.setMinimumHeight(36) # 36
        self.combo_styles.setFixedWidth(350)
        # La liste du menu se fait par
        # l'intermédiaire d'un QComBoBox
        for styles in self.liste_styles :
            self.combo_styles.addItem(styles)
        self.combo_styles.setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #fdfbf7, stop: 1 #6190F2);border-style: solid;border-width: 2px;border-radius: 8px;border-color: #9BB7F0;padding: 3px")
        
        # ----------------------------------------------------------------------------
        # Cases à cocher pour afficher (oui ou non) l'interface grise
        # ----------------------------------------------------------------------------   
        self.case_interface_grise = QCheckBox(_("Interface grise"))
        # La case Ombre portée est cochée par défaut
        self.case_interface_grise.setChecked(False)                                                             

        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
        
        # ========================================================
        # Exception pour contrôler la permanence
        # de la case à cocher 'Interface grise'
        # ========================================================
        try :
            # -------------------------------------
            # Si la case a déjà été cochée ...
            # -------------------------------------
            for cont in liste_contenu_tmp :
                # Si le fichier tempo concerné (c'est à dire :
                # data_interface_grise_murexpo est bien dedans ...
                if 'data_interface_grise_murexpo' in cont :
                    # Utilisation de la classe Tempo_lecture pour la lecture
                    # des données dans le répertoire (dit) temporaire
                    rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_interface_grise_murexpo_', 'data_interface_grise_murexpo')
                    # ... avec appel de la méthode affichage
                    rep_tempo_interface_g = rep_tempo_lecture.affichage()[0]
                    # Si le fichier tempo contient '1', c'est à dire 
                    # que la case a préalablement déjà été cochée
                    if rep_tempo_interface_g == "1" :
                        # la case (le widget) est cochée ds l'interface
                        self.case_interface_grise.setChecked(True)
                    # Si le fichier tempo contient '0', c'est
                    # à dire que la case n'a pas été cochée 
                    # (cas typique du 1er démarrage de MurExpo)
                    elif rep_tempo_interface_g == "0" :
                        # la case (le widget) est décochée ds l'interface
                        self.case_interface_grise.setChecked(False)
                        
        # Si le fichier tempo concerné (c'est à 
        # dire data_interface_grise_murexpo) n'est 
        # pas dans le rep. tempo. ...
        except : 
            # La case est décochée ...
            self.case_interface_grise.setChecked(False)
            # Ecriture de la valeur 0 dans le fichier tempo.
            ecr_tempo_choix_interface_grise = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_interface_grise_murexpo_", "data_interface_grise_murexpo", ".txt", '0')

        # ----------------------------------------------------------------------------
        # Bouton pour appliquer les modifs
        # ----------------------------------------------------------------------------
        # Icône pour le bouton ...
        icone_affichage = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_changements_de_configuration_256x256.png')))
        self.bout_appliquer = PushButtonIconeGauche(_("Appliquer les changements"), icon = icone_affichage, iconSize = QSize(16, 16))
        self.bout_appliquer.setFixedHeight(32)
        self.bout_appliquer.setMaximumWidth(300)
        self.bout_appliquer.setMinimumWidth(300)
        # ToolTip pour l'information --> application 
        # de l'écriture ds le rep. tempo
        self.bout_appliquer.setToolTip(_("Appliquer les modifications"))
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()

        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
        
        # Liste pour vérifier la présence du fichier tempo. data_style_interface_choisi_murexpo
        liste_contenu = [contenu for contenu in liste_contenu_tmp if 'data_style_interface_choisi_murexpo' in contenu]
        
        # -----------------------------------------------------------------
        # Permanence ...
        # -----------------------------------------------------------------
        # Si le fichier data_style_interface_choisi_murexpo existe (c'est
        # à dire qu'il a préalablement été modifié par l'utilisateur).
        # --> L'utilisateur a déjà sélectionnée une entrée dans la combo
        # Configuration > Style/Apparence --> combo Style de l'interface
        # -----------------------------------------------------------------
        if len(liste_contenu) > 0 :
            # Utilisation de la classe Tempo_lecture pour la lecture
            # des données dans le répertoire (dit) temporaire
            rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_style_interface_choisi_murexpo_', 'data_style_interface_choisi_murexpo')
            # ... avec appel de la méthode affichage
            rep_tempo_aff = rep_tempo_lecture.affichage()[0]
            # Repérage de l'index de la combo correspondant à l'item préalablement 
            # sélectionné (et sauvegardé dans le fichier data_style_interface_choisi_murexpo ...
            # se trouvant dans le répertoire temporaire) ... 
            index = self.combo_styles.findText(rep_tempo_aff, Qt.MatchFixedString)
            # Attribution de l'index correspondant. La combo s'affichera
            # avec l'item capturé dans le fichier data_affichage_onglets_murexpo_ ...
            self.combo_styles.setCurrentIndex(index)
            # ...
            QApplication.setStyle(rep_tempo_aff)
            # Si la case Interface grise est cochée ...
            if self.case_interface_grise.isChecked() == True :
                # ... Appel de fonction de mise
                # en place de l'interface grise
                self.interface_grise()
        # -----------------------------------------------------------------
        # Si le fichier data_style_interface_choisi_murexpo n'existe pas,
        # c'est à dire que l'utilisateur n'a rien sélectionné dans :
        # Configuration > Style/Apparence --> combo Style de l'interface
        # -----------------------------------------------------------------
        elif len(liste_contenu) == 0 :
            # Si on est sous GNU/Linux
            if platform.system() == 'Linux' :
                # Si Oxygen est dans la liste ...
                if 'Oxygen' in self.liste_styles :
                    # Mise en place du style Oxygen
                    QApplication.setStyle('Oxygen')
                    style = 'Oxygen'
                    try :
                        # Repérage de l'index de la combo correspondant à l'item préalablement 
                        # sélectionné (et sauvegardé dans le fichier data_style_interface_choisi_murexpo ...
                        # se trouvant dans le répertoire temporaire) ... 
                        index = self.combo_styles.findText(style, Qt.MatchFixedString)
                        # Attribution de l'index correspondant. La combo s'affichera avec
                        # l'item capturé dans le fichier data_style_interface_choisi_murexpo
                        self.combo_styles.setCurrentIndex(index)
                    except : pass
                # Autrement si Oxygen n'est pas dans la liste et que Windows y est ...
                elif 'Oxygen' not in self.liste_styles and 'Windows' in self.liste_styles :
                    # Mise en place du style Windows
                    QApplication.setStyle('Windows')
                    style = 'Windows'
                    try :
                        index = self.combo_styles.findText(style, Qt.MatchFixedString)
                        self.combo_styles.setCurrentIndex(index)
                    except : pass
            # Si on est sous MacOS ...
            elif platform.system() == 'Darwin' :
                # Si Windows est dans la liste ...
                if 'Windows' in self.liste_styles :
                    # Mise en place du style Windows
                    QApplication.setStyle('Windows')
                    style = 'Windows'
                    try :
                        index = self.combo_styles.findText(style, Qt.MatchFixedString)
                        self.combo_styles.setCurrentIndex(index)
                    except : pass
                # Si Windows n'est pas dans la liste et MacStyle y est ...
                if 'Windows' not in self.liste_styles and 'MacStyle' in self.liste_styles :
                    # Mise en place du style MacStyle
                    QApplication.setStyle('MacStyle')
                    style = 'MacStyle'
                    try :
                        index = self.combo_styles.findText(style, Qt.MatchFixedString)
                        self.combo_styles.setCurrentIndex(index)
                    except : pass
            # Si on est sous Windows ...
            elif platform.system() == 'Windows' :
                # Si Windows est dans la liste ...
                if 'Windows' in self.liste_styles :
                    # Mise en place du style Windows
                    QApplication.setStyle('Windows')
                    style = 'Windows'
                    try :
                        index = self.combo_styles.findText(style, Qt.MatchFixedString)
                        self.combo_styles.setCurrentIndex(index)
                    except : pass
                # Si windowsvista est dans la liste ...
                if 'windowsvista' in self.liste_styles :
                    # Mise en place du style windowsvista
                    QApplication.setStyle('windowsvista')
                    style = 'windowsvista'
                    try :
                        index = self.combo_styles.findText(style, Qt.MatchFixedString)
                        self.combo_styles.setCurrentIndex(index)
                    except : pass

            # Ecriture du style dans le rep. tempo.
            ecr_tempo_sauv_style_interface = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_style_interface_choisi_murexpo_", "data_style_interface_choisi_murexpo", ".txt", style)
                                                       
        # -------------------------------------------------------
        grid_1 = QGridLayout()
        grid_1.setSpacing(8)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        grid_1.addWidget(label_apparence_style, 0, 0)
        grid_1.addWidget(self.combo_styles, 0, 1)
        grid_1.addWidget(self.case_interface_grise, 0, 2)
        grid_1.addWidget(self.bout_appliquer, 0, 3)

        # Alignement/centrage sur la hauteur des widgets
        grid_1.setAlignment(Qt.AlignVCenter)
        grid_1.setAlignment(Qt.AlignHCenter)
        
        #
        self.setLayout(grid_1)
        
        # ==========================================================
        # Signaux
        # ==========================================================
        # Signal pour la combobox de...
        self.combo_styles.currentIndexChanged.connect(self.affichage_styles)
        # -----------------------------------------
        # Signal pour la cases pour lea mise en place de l'interface grise
        self.case_interface_grise.toggled.connect(lambda : self.fonction_case_interface_grise(self.case_interface_grise))
        # -----------------------------------------
        # Signal pour appliquer à la fin des modifs de
        # chemins --> le logiciel redémarre quand on
        # clique sur le bouton et les changements
        # deviennent effectifs
        # -----------------------------
        # Appel de la fonction redemarrer contenue
        # dans le module redemarrage_suite_modifs 
        # (voir import plus haut).
        self.bout_appliquer.clicked.connect(redemarrer)
        # ==========================================================
        
        
    def affichage_styles(self) :
        ''' Fonction pour sélectionner et sauver dans le
        répertoire remporaire le choix de l'utilisateur 
        quant à l'affichage des style (et écriture ds le
        fichier tempo data_style_interface_choisi_murexpo '''
        
        texte_combo = str(self.combo_styles.currentText())

        # Appel de la classe Tempo_ecriture
        ecr_tempo_sauv_style_interface = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_style_interface_choisi_murexpo_", "data_style_interface_choisi_murexpo", ".txt", texte_combo)
        
        
    def fonction_case_interface_grise(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) pour l'interface 
        grise a été cochée """
        
        if c.isChecked() == True :
            # Ecriture de la valeur 1 dans le fichier tempo.
            ecr_tempo_choix_interface_grise = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_interface_grise_murexpo_", "data_interface_grise_murexpo", ".txt", '1')
            
        if c.isChecked() == False :
            # Ecriture de la valeur 0 dans le fichier tempo.
            ecr_tempo_choix_interface_grise = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_interface_grise_murexpo_", "data_interface_grise_murexpo", ".txt", '0')
        
        
    def interface_grise(self) :
        """ Fonction pour customizer l'interface
        (les couleurs de l'interface) """
        
        pal = QApplication.palette()
        pal.setColor(QPalette.Window, QColor(180, 180, 180)) # La couleur de fond de la fenêtre (interface)
        pal.setColor(QPalette.WindowText, Qt.black) # La couleur du texte de la fenêtre
        pal.setColor(QPalette.Base, QColor(145, 145, 145)) # Couleur du fond des widgets par ex un QTextEdit
        pal.setColor(QPalette.AlternateBase, QColor(145, 145, 145)) # La couleur de fond
        pal.setColor(QPalette.ToolTipBase, QColor(100, 100, 100)) # La couleur de fond du tooltip
        pal.setColor(QPalette.ToolTipText, Qt.white) # Le texte des tolltips
        pal.setColor(QPalette.Text, Qt.black) # Les textes dans les QTextEdit par ex.
        pal.setColor(QPalette.Button, QColor(145, 145, 145)) # Théoriquement les boutons
        pal.setColor(QPalette.ButtonText, Qt.black) # Le texte des boutons
        pal.setColor(QPalette.BrightText, Qt.blue)
        pal.setColor(QPalette.Link, Qt.blue)
        pal.setColor(QPalette.Highlight, QColor(100, 100, 100)) # Les lignes/items électionnés (par ex dans une combo)
        pal.setColor(QPalette.HighlightedText, Qt.white) # Les textes des lignes/items électionnés
        pal.setColor(QPalette.Active, QPalette.Button, QColor(145, 145, 145)) # En partie l'arrière-plan des cases à cocher
        pal.setColor(QPalette.Disabled, QPalette.ButtonText, QColor(140, 140, 140)) # Couleur du texte des boutons quand ils sont désactivés
        pal.setColor(QPalette.Disabled, QPalette.Text, QColor(100, 100, 100)) # Couleur du texte ds le fond des widgets quand ils sont désactivés
        pal.setColor(QPalette.Disabled, QPalette.WindowText, QColor(120, 120, 120)) # Couleur du texte ds le fond des widgets quand ils sont désactivés
        QApplication.setPalette(pal)
        
 
class Info(QWidget) :
    # Classe qui affiche les infos diverses et sur le logiciel
    def __init__(self, parent):
        super(Info, self).__init__(parent)

        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : config_apsc/chemin_data_ressources.py)
        txt_0 = '''<p align="center"><img src="'''+chemin_ressources('''ressources_murexpo'''+os.sep+'''logo_murexpo_general_664x280.png"''')+''' width="664" height="280"></p>'''

        txt_1 = _("<p><b><u>Version du logiciel :</u></b></p>")
        txt_2 = _("<b>MurExpo</b> version {0}").format(__version__)
        
        txt_3 = _("<p><b><u>Versions embarquées :</u></b></p>")
        
        txt_4 = _("L'application fonctionne avec Python {0}, PyQt {1} et Qt {2}").format(platform.python_version(), PYQT_VERSION_STR, QT_VERSION_STR)

        txt_5 = _("<p><b><u>Développement :</u></b></p>")
        txt_6 = _("Lama Angelo (professeur d'arts plastiques au collège Dunois à Caen).")
        
        #txt_7 = "<p><b><u>Remerciements (à) :</u></b></p>"
        #txt_8 = "* Eric Duponchel (professeur de technologie au collège Dunois à Caen), pour les quelques petits tests du logiciel pendant le développement."

        txt_9 = _("<p><b><u>Autres Renseignements (processeur, système d'exploitation, ...) :</u></b></p>")
        txt_10 = _("Type de processeur : {0}<br />").format(platform.processor())
        txt_11 = _("Système d'exploitation : {0}<br />").format(platform.system())
        txt_12 = _("Version du système : {0}<br />").format(platform.version())
        txt_13 = _("Version du noyau : {0}<br />").format(platform.release())
        
        # ----------------------------------------- #
        
        doc_aide_titre = _("<p><b># ========================================= #</b><br /><b>{0}DOCUMENTATION / AIDE</b><br /><b># ========================================= #</b></p>").format('&nbsp;'*18)
        
        # -------------------------------------------------------
        # Documentation texte
        # -------------------------------------------------------                
                           
        doc_aide_entete_texte =  _("<p><b><u>Documentation texte :</u></b></p>")             
              
        doc_aide_texte_001 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Exécuter la version onefile sous Windows :"), "https://murexpo.org/me/execution-de-la-version-onefile-sous-windows", _("Voir ici"))
        doc_aide_texte_002 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Exécuter la version onedir sous Windows :"), "https://murexpo.org/me/execution-de-la-version-onedir-sous-windows", _("Voir ici"))             
        doc_aide_texte_003 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Installer le logiciel sur le disque dur sous Windows :"), "https://murexpo.org/me/installation-sur-le-disque-dur-sous-windows", _("Voir ici"))
        doc_aide_texte_004 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Exécuter la version onefile sous MacOS :"), "https://murexpo.org/me/execution-de-la-version-onefile-sous-macos", _("Voir ici"))
        doc_aide_texte_005 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Exécuter la version onefile sous GNU/Linux :"), "https://murexpo.org/me/execution-de-la-version-onefile-sous-gnu-linux", _("Voir ici"))        
        doc_aide_texte_006 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Exécuter la version onedir GNU/Linux :"), "https://murexpo.org/me/execution-de-la-version-onedir-sous-gnu-linux", _("Voir ici"))
        doc_aide_texte_007 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Installer le paquet deb pour Ubuntu et ses dérivés (GNU/Linux) :"), "https://murexpo.org/me/installation-et-desinstallation-sur-le-disque-dur-sous-gnu-linux", _("Voir ici"))
        doc_aide_texte_008 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Installer le paquet deb pour Debian (GNU/Linux) :"), "https://murexpo.org/me/installation-et-desinstallation-sur-le-disque-dur-sous-gnu-linux", _("Voir ici")) 
        doc_aide_texte_009 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Installer le paquet rpm (GNU/Linux) :"), "https://murexpo.org/me/installation-et-desinstallation-sur-le-disque-dur-sous-gnu-linux", _("Voir ici"))
        doc_aide_texte_010 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Exporter l'exposition sous la forme d'une page HTML ou d'un document PDF :"), "https://murexpo.org/me/exportation-en-html-ou-en-pdf", _("Voir ici"))
        doc_aide_texte_011 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Erreur de l’affichage de l’aide ... solution :"), "https://murexpo.org/me/erreur-de-laffichage-de-laide-solution", _("Voir ici"))

        # -------------------------------------------------------
        # Documentation vidéo
        # ------------------------------------------------------- 

        doc_aide_entete_video =  _("<p><b><u>Documentation vidéo :</u></b></p>")
        doc_aide_video_001 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* MurExpo (version 0.6.0), présentation de l'application :"), "https://youtu.be/HxwfLagCnEY", _("Voir ici"))
        doc_aide_video_002 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Accrocher des oeuvres au mur d’exposition et l’agrémenter de divers accessoires (version 0.6.0) :"), "https://youtu.be/jAKRqQq1Kfo", _("Voir ici"))
        doc_aide_video_003 = '''{0} <a href="{1}">{2}</a><br />'''.format(_("* Exportation de l'exposition sous la forme d'un gif animé (version 0.6.4) :"), "https://youtu.be/pYu7w9nO0KY", _("Voir ici"))

        #
        self.texteBrowser = QTextBrowser()
        #
        self.texteBrowser.setHtml(txt_0+txt_1+txt_2+txt_3+txt_4+txt_5+txt_6
                                  +txt_9+txt_10+txt_11+txt_12+txt_13
                                  +doc_aide_titre
                                  +doc_aide_entete_texte
                                  +doc_aide_texte_001+doc_aide_texte_002
                                  +doc_aide_texte_003+doc_aide_texte_004
                                  +doc_aide_texte_005+doc_aide_texte_006
                                  +doc_aide_texte_007+doc_aide_texte_008
                                  +doc_aide_texte_009+doc_aide_texte_010
                                  +doc_aide_texte_011
                                  +doc_aide_entete_video
                                  +doc_aide_video_001+doc_aide_video_002
                                  +doc_aide_video_003)
        #
        self.texteBrowser.setOpenExternalLinks(True)
        
        # Couleurs du fond (gris) et du texte (blanc)
        # Solution trouvée ici :
        # https://riverbankcomputing.com/pipermail/pyqt/2007-January/015237.html
        # L'avantage avec cette méthode est que la barre de défilement reste à
        # la couleur de l'interface et pas  du fond du QTextEdit
        palette = self.texteBrowser.palette()
        palette.setColor(QPalette.Active, QPalette.Text, QColor(255, 255, 255))
        palette.setColor(QPalette.Active, QPalette.Base, QColor(100, 100, 100))
        self.texteBrowser.setPalette(palette)
        self.texteBrowser.setStyleSheet("background-color: rgb(100, 100, 100);")
        self.texteBrowser.setTextColor(QColor(255, 255, 255))
        self.fonte(self.texteBrowser) # Fontion pour l'attribution de la fonte
        # Le QTextEdit est placé en lecture seule
        self.texteBrowser.setReadOnly(True)
        
        #
        grid_1 = QGridLayout()
        grid_1.setSpacing(10)
        #
        grid_1.addWidget(self.texteBrowser, 0, 0)
        # Alignement
        grid_1.setAlignment(Qt.AlignVCenter)
        
        #
        self.setLayout(grid_1)
        

    def fonte(self, widget) :
        ''' Fonction d'attribution de la fonte dans le QTextEdit'''

        f = QFont()
        f.setFamily('Liberation Sans')
        f.setFixedPitch(True)
        f.setPointSize(10)
        widget.setFont(f)


class MUREXPO_GUI(QMainWindow):
    # Classe GUI principale
    def __init__(self):
        super(MUREXPO_GUI, self).__init__()
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # Si le répertoire temporaire n'existe pas ... il est crée
        # Spécialement utile dans le cas où le répertoire temporaire
        # se situe dans : /home/utilisateur/.murexpo ... sous GNU/Linux
        # ... ou dans /Users/utilisateur/.murexpo sous MacOS
        # ----------------------------------------------------------
        # Sous GNU/Linux ou sous MacOSX ...
        if platform.system() == 'Darwin' or platform.system() == 'Linux' :
            if not os.path.exists(self.chemin_tempo_system_exploit): os.mkdir(self.chemin_tempo_system_exploit)
            
        # ----------------------------------------------------------------------------
        # Nettoyage des images dans le répertoire temporaire
        # ----------------------------------------------------------------------------
        # On liste le contenu du répertoire temporaire
        contenu_rep_tempo = os.listdir(self.chemin_tempo_system_exploit)
        # On détecte toutes les images (avec extension : .jpg, .jpeg, .JPG, 
        # .JPEG, .png, .gif, .xpm, .tga) dans le répertoire temporaire
        liste = [os.path.join(self.chemin_tempo_system_exploit, images) for images in contenu_rep_tempo if os.path.splitext(images)[1] in ['.jpg', '.jpeg', '.JPG', '.JPEG', '.png', '.gif', '.xpm', '.tga']]
        # On supprime toutes les images ...
        try :
            for img in liste : os.remove(img)
        except : pass

        # ---------------------------------------------------------------------------------------------
        # Liste pour tester si le fichier temporaire data_capture_oeuvres_supprimees_murexpo existe
        # --> c'est le fichier qui recence si dans une précédente exécution de MurExpo, l'utilisateur
        # a supprimé des oeuvres (sur le mur d'expo), après les avoir chargées (utilisé en particulier)
        # dans l'export de l'exposition sous la forme d'un fichier html ou pdf.
        liste_oeuvres_supprimees = [os.path.join(self.chemin_tempo_system_exploit, supp) for supp in contenu_rep_tempo if "data_capture_oeuvres_supprimees_murexpo" in supp]
        # Si ce fichier tempo existe, il est supprimé à chaque démarrage
        if len(liste_oeuvres_supprimees) == 1 : os.remove(liste_oeuvres_supprimees[0])
        # ---------------------------------------------------------------------------------------------
        
        # ---------------------------------------------------------------------------------------------
        # Liste pour tester si le fichier temporaire data_commentaires_expo_oeuvres_sortie_html existe
        # --> c'est le fichier qui capture le commentaire entré dans les QTextEdit concernant
        # le contexte de l'exposition et les commentaires pour chaque ouvre. C'est pour la sortie 
        # html ou pdf.
        liste_commentaires = [os.path.join(self.chemin_tempo_system_exploit, com) for com in contenu_rep_tempo if "data_commentaires_expo_oeuvres_sortie_html" in com]
        # Si ce fichier tempo existe, il est supprimé à chaque démarrage
        if len(liste_commentaires) == 1 : os.remove(liste_commentaires[0])
        # ---------------------------------------------------------------------------------------------
        
        # ---------------------------------------------------------------------------------------------
        # Liste pour récupérer les largeurs d'images pour les vignettes (traitement html ou pdf)
        liste_vignettage = [os.path.join(self.chemin_tempo_system_exploit, largeur_v) for largeur_v in contenu_rep_tempo if "data_taille_vignettage_export_html_pdf" in largeur_v]
        # Création préalable d'une liste remplie de 20 éléments de '200'+'\n'
        liste_param_larg_vignettage = ["200"+'\n' for n in range(20)]
        # Si le fichier data_taille_vignettage_export_html_pdf
        # dans le répertoire. temporaire, il est crée ...
        if len(liste_vignettage) == 0 : 
            # Ecriture du fichier dans le répertoire temporaire
            # (--> data_taille_vignettage_export_html_pdf)
            ecr_tempo_taille_vignettes_html_pdf = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_taille_vignettage_export_html_pdf_", "data_taille_vignettage_export_html_pdf", ".txt", "".join(liste_param_larg_vignettage))
        elif len(liste_vignettage) == 1 : pass # Si le fichier existe déjà, on passe ...
        # ---------------------------------------------------------------------------------------------
    
        ##### ----------- Commenté pour l'instant -----------------------------------------
        """
        # ----------------------------------------------
        # Ecriture de ... dans le répertoire temporaire
        # ----------------------------------------------
        # 0 --> Non, 1 --> Oui (ici la case n'est pas cochée par défaut, donc 0 pour Non)
        ecr_tempo_case_termine = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_case_j_ai_termine_html_pdf_murexpo_", "data_case_j_ai_termine_html_pdf_murexpo", ".txt", '0')
        """
        ##### ----------- Commenté pour l'instant -----------------------------------------

        # ----------------------------------------------------------------------------

        # Version de PyQt5 et de Qt
        print("Python {0}, PyQt {1} et Qt {2}".format(platform.python_version(), PYQT_VERSION_STR, QT_VERSION_STR))
        print(" ")
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("Version de Python : {}\nVersion de PyQt : {}\nVersion de Qt : {}\n\n").format(platform.python_version(), PYQT_VERSION_STR, QT_VERSION_STR))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------

        # Titre
        self.setWindowTitle('MurExpo')
        
        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)

        # ========================================================================================
        # AFFICHAGE (ET RESOLUTION) --> Récupération de la Résolution ...
        # ========================================================================================
        
        # ------------------------------------------------------------
        # Trouvé ici : https://www.programcreek.com/python/example
        # /108102/PyQt5.QtWidgets.QDesktopWidget
        # ------------------------------------------------------------
        # Récupérer la résolution de l'écran (affichage en cours)
        resolution = QDesktopWidget().screenGeometry()
        # --------------------------------------------------------------------
        # Récupérer l'espace disponible en hauteur (sans la barre des tâches)
        # --> il s'agit de l'espace restant sans quand on enlève la taille de
        # la barre des tâches et autre ...
        espace_disponible_hauteur = QDesktopWidget().availableGeometry()
        print(_("ESPACE DISPONIBLE EN LARGEUR (sans barre des tâches) :"), espace_disponible_hauteur.width())
        print(_("ESPACE DISPONIBLE EN HAUTEUR (sans barre des tâches) :"), espace_disponible_hauteur.height())
        # Ecriture de l'espace disponible en hauteur (sans barre des tâches) dans le rep. tempo
        ecr_tempo_espace_dispo_hauteur = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_espace_dispo_haut_desktop_murexpo_", "data_espace_dispo_haut_desktop_murexpo", ".txt", "{}x{}".format(espace_disponible_hauteur.width(), espace_disponible_hauteur.height()))
        # --------------------------------------------------------------------
        #
        # Dimension x (largeur) et y (hauteur) pour l'affichage de la fenêtre 
        # principale de MurExpo. La dimension y est faite avec la dimension en
        # hauteur sans la barre des tâches (QDesktopWidget().availableGeometry(),
        # ... espace_disponible_hauteur.height())
        dim_x, dim_y = resolution.width(), espace_disponible_hauteur.height()
        #
        #dim_x, dim_y = 1440, 1080 # Pour Tests !
        #
        print(_("Résolution ecran :"), resolution.width(), resolution.height())
        # ========================================================================================
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Info) ESPACE DISPONIBLE EN LARGEUR (sans la barre des tâches) : {}\n(* Info) ESPACE DISPONIBLE EN HAUTEUR (sans la barre des tâches) : {}\n(* Info) Résolution initiale de l'écran : {} x {}\n").format(espace_disponible_hauteur.width(), espace_disponible_hauteur.height(), resolution.width(), resolution.height()))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        # --------------------------------------------------------
        # Boucle de vérification ... (et affichage de la 
        # fenêtre principale de MurExpo suivant la configuration
        # de l'utilisateur) ...
        # --------------------------------------------------------
        for contenu in liste_contenu_tmp :
            # ====================================================
            # Si le fichier tempo :
            # data_resolution_choisie_fenetre_principale_murexpo,
            # est bien dans le répertoire temporaire ...
            # ----------------------------------------------------
            # Fichier crée lors du 1er lancement de MurExpo ...
            # et accessible au 2ème lancement du logiciel
            # ====================================================
            if 'data_type_de_resolution_fenetre_principale_murexpo' in contenu :

                # Ouverture de fichier tempo qui vient d'être crée
                rep_tempo_lecture_choix_type_de_resolution = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_type_de_resolution_fenetre_principale_murexpo_', 'data_type_de_resolution_fenetre_principale_murexpo')

                # Récupération des données sous la forme d'une
                # liste ... avec appel de la méthode affichage
                rep_tempo_resolution_type_liste = rep_tempo_lecture_choix_type_de_resolution.affichage()

                # ----------------------------------------------------------------------
                # Lors du 2ème lancement de MurExpo (et si l'utilisateur n'a pas choisi
                # "Résolution au choix" dans la configuration de la Résolution), ...
                # la case "Résolution programmée" est cochée par défaut et la fenêtre
                # s'affiche en utilisant les dimensions dim_x et dim_y définies plus
                # haut dans la récupération de la résolution ... (appel de la fonction
                # affichage_fenetre avec comme arguments dim_x et dim_y)
                # ----------------------------------------------------------------------
                if rep_tempo_resolution_type_liste == [_('Résolution programmée')] \
                    or rep_tempo_resolution_type_liste == [_('Résolution &programmée')] \
                    or rep_tempo_resolution_type_liste == [_('&Résolution programmée')] :

                        # Appel de la fonction d'affichage 
                        # de l'interface (fenêtre)
                        self.affichage_fenetre(dim_x, dim_y)
                        
                # ----------------------------------------------------------------------
                # ... Autrement si l'utilisateur a choisi "Résolution au choix" ... et
                # qu'il a éventuellement fait le choix d'une résolution (ds la combo)
                # ----------------------------------------------------------------------
                elif rep_tempo_resolution_type_liste == [_('Résolution au choix')] \
                    or rep_tempo_resolution_type_liste == [_('&Résolution au choix')] \
                    or rep_tempo_resolution_type_liste == [_('Résolution &au choix')] \
                    or rep_tempo_resolution_type_liste == [_('Résolution au &choix')] :

                        # Utilisation de la classe Tempo_lecture pour la lecture
                        # des données dans le répertoire (dit) temporaire (lecture
                        # de la résolution choisie, par exemple : 1360 x 768) ...
                        # et ce ds data_resolution_choisie_fenetre_principale_murexpo
                        rep_tempo_lecture_resolution_choix = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_resolution_choisie_fenetre_principale_murexpo_', 'data_resolution_choisie_fenetre_principale_murexpo')
                        # ... avec appel de la méthode affichage (les données récoltées 
                        # sont ici ... sous la forme d'une chaîne de caractères)
                        rep_tempo_resolution_choix = rep_tempo_lecture_resolution_choix.affichage()[0]
                        # La chaîne est splitée et on ne conserve que la résolution ...
                        liste_resolution_recoltee = rep_tempo_resolution_choix.split(' x ')
                        
                        print(_("Résolution sélectionnée par l'utilisateur :"), liste_resolution_recoltee)
                        
                        # ---------------------------------------------------------------
                        # Retour commandes ....
                        # ---------------------------------------------------------------
                        # Récupération des commandes pour le terminal interne de MurExpo
                        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Résolution sélectionnée par l'utilisateur : {} x {}\n").format(liste_resolution_recoltee[0], liste_resolution_recoltee[1]))
                        # Ecriture des commandes dans le fichier
                        # tempo data_retour_commandes_murexpo
                        self.capter_commandes_murexpo.liste_ecrire_commandes()
                        # ---------------------------------------------------------------
                        
                        # --------------------------------------------
                        # Particulièrement important sous Windows !
                        # --------------------------------------------
                        # Redéfinition de la définition en largeur (x) et en hauteur (y)
                        dim_x, dim_y = int(liste_resolution_recoltee[0]), int(liste_resolution_recoltee[1])
                        
                        # Appel de la fonction d'affichage de l'interface (fenêtre)
                        # ... avec comme arguments (ce coup-ci) les éléments
                        # (largeur, hauteur) de la résolution sélectionnée
                        self.affichage_fenetre(dim_x, dim_y)
                
            # ----------------------------------------------------------------------
            # Au tout 1er lancement de MurExpo (le fichier tempo :
            # data_type_de_resolution_fenetre_principale_murexpo n'ayant pas 
            # encore été crée) la fonction affichage_fenetre est lancée avec
            # comme arguments dim_x et dim_y ...
            # ----------------------------------------------------------------------
            elif 'data_type_de_resolution_fenetre_principale_murexpo' not in contenu :

                self.affichage_fenetre(dim_x, dim_y)
        # --------------------------------------------------------
        
        # Appel de la fonction affichant le menu
        self.menus()
        
        # ============================================= #
        # Boîte d'onglets
        self.tab_widget = QTabWidget()
        
        # Appel des classes contenant le groupement de widgets
        self.win_widget_1 = Fabrique(self)
        self.win_widget_2 = FichierOuvrir(self)
        self.win_widget_3 = SelectionPolice(self)
        self.win_widget_4 = SelectionCouleur(self)
        self.win_widget_5 = Configuration_General(self)
        self.win_widget_6 = Visionnage(self)
        self.win_widget_7 = Retour_Commandes(self)
        self.win_widget_8 = Info(self)
        
        # Le widget
        widget = QWidget()

        # ********************************************************************** #
        
        # ------------------------------------------------------
        # Au tout 1er démarrage de MurExpo, le fichier temporaire :
        # data_affichage_onglets_murexpo ... n'existe pas encore.
        # Tant que l'utilisateur n'aura pas sélectionné le Mode
        # d'affichage des onglets (sélection de l'apparence 
        # visuelle des onglets) cela s'affichera en 'mode' texte
        # ------------------------------------------------------
        # Boucle de vérification ...
        for contenu in liste_contenu_tmp :
            # 
            if 'data_affichage_onglets_murexpo' not in contenu :
                # On rajoute chaque onglet ... en 'mode' texte
                self.tab_widget.addTab(self.win_widget_1, _("Conception et exportation"))
                self.tab_widget.addTab(self.win_widget_2, _("Chargement des images"))
                self.tab_widget.addTab(self.win_widget_3, _("Choix police"))
                self.tab_widget.addTab(self.win_widget_4, _("Choix couleur"))
                self.tab_widget.addTab(self.win_widget_5, _("Configuration"))
                self.tab_widget.addTab(self.win_widget_6, _("Visionnage"))
                self.tab_widget.addTab(self.win_widget_7, _("Retour commandes"))
                self.tab_widget.addTab(self.win_widget_8, _("Aide/Infos"))
                
                ###############################################
                #self.tab_widget.setCurrentIndex(3) ###########
                ###############################################
        
        # ====================================================================== #
        # Onglets ( --> mise en forme)
        # ====================================================================== #
        self.tab_widget.addTab(self.win_widget_1, _("Conception et exportation"))
        self.tab_widget.addTab(self.win_widget_2, _("Chargement des images"))
        self.tab_widget.addTab(self.win_widget_3, _("Choix police"))
        self.tab_widget.addTab(self.win_widget_4, _("Choix couleur"))
        self.tab_widget.addTab(self.win_widget_5, _("Configuration"))
        self.tab_widget.addTab(self.win_widget_6, _("Visionnage"))
        self.tab_widget.addTab(self.win_widget_7, _("Retour commandes"))
        self.tab_widget.addTab(self.win_widget_8, _("Aide/Infos"))
        
        # ====================================================================== #
        
        # ---------------------------------------------------------
        # Si l'apparence des onglets a déjà été modifié, c'est à
        # dire que l'utilisateur a déjà sélectionné le type
        # d'Apparence des onglets de sélection des rubriques, cela
        # s'affichera dans le 'mode' choisi par l'utilisateur
        # ---------------------------------------------------------
        # Boucle de vérification ...
        for contenu in liste_contenu_tmp :
            # Si le fichier tempo concerné est bien 
            # dans le répertoire temporaire ...
            if 'data_affichage_onglets_murexpo' in contenu :
                # ------------------------------------------------------- #
                # Utilisation de la classe Tempo_lecture pour la lecture
                # des données dans le répertoire (dit) temporaire
                # ------------------------------------------------------- #
                rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_affichage_onglets_murexpo_', 'data_affichage_onglets_murexpo')
                # ... avec appel de la méthode affichage
                rep_tempo_aff = rep_tempo_lecture.affichage()[0]
                # Si sélection en 'mode' Texte uniquement ...
                if rep_tempo_aff == _('Texte uniquement') :
                    self.tab_widget.addTab(self.win_widget_1, _("Conception et exportation"))
                    self.tab_widget.addTab(self.win_widget_2, _("Chargement des images"))
                    self.tab_widget.addTab(self.win_widget_3, _("Choix police"))
                    self.tab_widget.addTab(self.win_widget_4, _("Choix couleur"))
                    self.tab_widget.addTab(self.win_widget_5, _("Configuration"))
                    self.tab_widget.addTab(self.win_widget_6, _("Visionnage"))
                    self.tab_widget.addTab(self.win_widget_7, _("Retour commandes"))
                    self.tab_widget.addTab(self.win_widget_8, _("Aide/Infos"))
                # Si sélection en 'mode' Texte et icônes ...
                if rep_tempo_aff == _('Texte et icônes') :
                    # Les images (les icônes) sont importés par la fonction chemin_ressources contenue dans : chemin_data_ressources.py
                    self.tab_widget.addTab(self.win_widget_1, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_conception_et_exportation_murexpo_256x256.png'))), _("Conception et exportation"))
                    self.tab_widget.addTab(self.win_widget_2, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_chargement_des_images_murexpo_256x256.png'))), _("Chargement des images"))
                    self.tab_widget.addTab(self.win_widget_3, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_choix_police_murexpo_256x256.png'))), _("Choix police"))
                    self.tab_widget.addTab(self.win_widget_4, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_choix_couleur_murexpo_256x256.png'))), _("Choix couleur"))
                    self.tab_widget.addTab(self.win_widget_5, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_configuration_murexpo_256x256.png'))), _("Configuration"))
                    self.tab_widget.addTab(self.win_widget_6, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png'))), _("Visionnage"))
                    self.tab_widget.addTab(self.win_widget_7, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_terminal_murexpo_256x256.png'))), _("Retour commandes"))
                    self.tab_widget.addTab(self.win_widget_8, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_aide_version_murexpo_256x256.png'))), _("Aide/Infos"))
                    # Taille de l'icône dans le QTabWidget
                    self.tab_widget.setIconSize(QSize(24, 24))
                # Si sélection en 'mode' Icônes uniquement ...    
                if rep_tempo_aff == _('Icônes uniquement') :
                    # Les images (les icônes) sont importés par la fonction chemin_ressources contenue dans : chemin_data_ressources.py
                    self.tab_widget.addTab(self.win_widget_1, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_conception_et_exportation_murexpo_256x256.png'))), None)
                    self.tab_widget.addTab(self.win_widget_2, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_chargement_des_images_murexpo_256x256.png'))), None)
                    self.tab_widget.addTab(self.win_widget_3, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_choix_police_murexpo_256x256.png'))), None)
                    self.tab_widget.addTab(self.win_widget_4, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_choix_couleur_murexpo_256x256.png'))), None)
                    self.tab_widget.addTab(self.win_widget_5, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_configuration_murexpo_256x256.png'))), None)
                    self.tab_widget.addTab(self.win_widget_6, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png'))), None)
                    self.tab_widget.addTab(self.win_widget_7, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_terminal_murexpo_256x256.png'))), None)
                    self.tab_widget.addTab(self.win_widget_8, QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_aide_version_murexpo_256x256.png'))), None)
                    
                    # Taille de l'icône dans le QTabWidget
                    self.tab_widget.setIconSize(QSize(32, 32))
        
        # Onglets positionnés à gauche
        self.tab_widget.setStyleSheet("""QTabWidget::tab-bar {alignment: left;}""") # center

        # Commenté -------------------------------------
        """
        layout = QVBoxLayout(widget)
        layout.addWidget(self.tab_widget)
        self.setCentralWidget(widget)
        """
        # Commenté -------------------------------------
        
        # Remplacé par ---------------------------------
        layout = QVBoxLayout()
        layout.addWidget(self.tab_widget)
        widget.setLayout(layout)
        self.setCentralWidget(widget)
        # Remplacé par ---------------------------------
        
        # ********************************************************************** #
        
        # =======================================================================================
        # Signaux pour le QTabWidget : l'onglet pour ouvrir une image (oeuvre) et une fois 
        # l'oeuvre sélectionnée dans l'arborescence du dd ... rebasculer/retourner dans 
        # l'onglet de visionnage et de réglage des paramètres de l'oeuvre.
        # =======================================================================================
        # Un grand merci à eyllanesc, voir ici :
        # https://stackoverflow.com/questions/63065472/call-and-use-a-qtabwidget-defined-
        # in-the-main-class-in-another-class#63065558
        # ---------------------------------------------------------------------------------------
        # Connection des tables du QTabWidget (des onglets) en vue d'émettre un signal
        # pour que l'onglet Fichier ouvrir puisse s'afficher quand l'utilisateur clique
        # sur le bouton Ouvrir image (bouton présent dans le stacked 2 dans la classe 
        # Fabrique)
        # ---------------------------------------------------------------------------------------
        # A voir l'autre partie du code pour le signal dans fabrique_murexpo_gui.py
        # ---------------------------------------------------------------------------------------
        self.win_widget_1.customSignal_onglet_ouvrir_oeuvre.connect(self.tab_widget.setCurrentIndex)
        # ---------------------------------------------------------------------------------------
        # Connection des tables du QTabWidget (des onglets) en vue d'émettre un signal
        # pour que l'onglet Conception et exportation puisse s'afficher à la fin du 
        # chargement de l'image
        # ---------------------------------------------------------------------------------------
        # A voir l'autre partie du code pour le signal dans fichier_ouvrir.py
        # ---------------------------------------------------------------------------------------
        # self.win_widget_2 correspond à l'onglet Chargement des images
        self.win_widget_2.customSignal_choix_oeuvre.connect(self.tab_widget.setCurrentIndex)
        # ---------------------------------------------------------------------------------------
        # Connection des tables du QTabWidget (des onglets) en vue d'émettre un signal
        # pour que l'onglet Visionnage puisse s'afficher à la fin du chargement de l'image
        # ---------------------------------------------------------------------------------------
        # A voir l'autre partie du code pour le signal dans fabrique_murexpo_gui.py
        # ---------------------------------------------------------------------------------------
        # self.win_widget_1 correspond à l'onglet Conception et exportation
        self.win_widget_1.customSignal_onglet_visuel_exposition.connect(self.tab_widget.setCurrentIndex)
        # ---------------------------------------------------------------------------------------
        # Connection des tables du QTabWidget (des onglets) en vue d'émettre un signal
        # pour que l'onglet Conception et exportation puisse s'afficher à la fin du 
        # choix de la police
        # ---------------------------------------------------------------------------------------
        # A voir l'autre partie du code pour le signal dans selection_police.py
        # ---------------------------------------------------------------------------------------
        # self.win_widget_3 correspond à l'onglet Choix police
        self.win_widget_3.customSignal_onglet_retour_reglages.connect(self.tab_widget.setCurrentIndex)
        # ---------------------------------------------------------------------------------------
        # Connection des tables du QTabWidget (des onglets) en vue d'émettre un signal
        # pour que l'onglet Conception et exportation puisse s'afficher à la fin du 
        # chargement de l'image
        # ---------------------------------------------------------------------------------------
        # A voir l'autre partie du code pour le signal dans visionnage_murexpo_gui.py
        # ---------------------------------------------------------------------------------------
        # self.win_widget_5 correspond à l'onglet Visionnage
        self.win_widget_6.customSignal_onglet_retour_conception_exportation.connect(self.tab_widget.setCurrentIndex)
        # ---------------------------------------------------------------------------------------
        # Connection des tables du QTabWidget (des onglets) en vue d'émettre un signal
        # pour que l'onglet Conception et exportation puisse s'afficher à la fin du 
        # chargement de l'image
        # ---------------------------------------------------------------------------------------
        # A voir l'autre partie du code pour le signal dans visionnage_murexpo_gui.py
        # ---------------------------------------------------------------------------------------
        # self.win_widget_1 correspond à l'onglet Conception et exportation
        self.win_widget_1.customSignal_onglet_choix_police.connect(self.tab_widget.setCurrentIndex)
        # ---------------------------------------------------------------------------------------
        # Connection des tables du QTabWidget (des onglets) en vue d'émettre un signal
        # pour que l'onglet Choix couleur s'affiche quand l'utilisateur clique sur le
        # bouton Modifier les couleurs ... réglages des couleurs (pour le panneau/cartel 
        # et/ou la police  de caractère)
        # ---------------------------------------------------------------------------------------
        # A voir l'autre partie du code pour le signal dans visionnage_murexpo_gui.py
        # ---------------------------------------------------------------------------------------
        # self.win_widget_1 correspond à l'onglet Conception et exportation
        self.win_widget_1.customSignal_onglet_choix_couleur.connect(self.tab_widget.setCurrentIndex)
        # ---------------------------------------------------------------------------------------
        # Connection des tables du QTabWidget (des onglets) en vue d'émettre un signal
        # pour que l'onglet Conception et exportation puisse s'afficher à la fin du 
        # réglages des couleurs (pour le panneau/cartel et/ou la police  de caractère)
        # ---------------------------------------------------------------------------------------
        # A voir l'autre partie du code pour le signal dans visionnage_murexpo_gui.py
        # ---------------------------------------------------------------------------------------
        # self.win_widget_4 correspond à l'onglet Choix couleur
        self.win_widget_4.customSignal_onglet_retour_reglages_choix_couleur.connect(self.tab_widget.setCurrentIndex)
        # =======================================================================================
        
        
    def affichage_fenetre(self, largeur, hauteur) :
        """ Fonction pour l'affichage et le 
        calcul de la bonne résolution """
        
        # Ecriture de la résolution du bureau dans le rep. tempo
        ecr_tempo_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_desktop_murexpo_", "data_resolution_desktop_murexpo", ".txt", "{}x{}".format(largeur, hauteur))

        # ----------------------------
        # Logo/icône pour la fenêtre
        # ----------------------------
        icone_murexpo = chemin_ressources(os.path.join('ressources_murexpo', 'logo_murexpo_me_format_carre_256x256.png'))
        self.setWindowIcon(QIcon(icone_murexpo))

        # La taille minimum est fixée
        self.setMinimumSize(largeur, hauteur - 25) # -25 pour afficher toute la fenêtre
        
        # ------------------------------------------
        # Conditions particulières quant à la 
        # résolution de l'écran --> si la résolution
        # en x est supérieure à 1920, elle est 
        # ramenée à 1920 x 1080
        # ------------------------------------------
        if largeur > 1920 : 
            largeur, hauteur = 1920, 1080
            # Ré-ecriture de la résolution du bureau dans le rep. tempo
            ecr_tempo_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_desktop_murexpo_", "data_resolution_desktop_murexpo", ".txt", "{}x{}".format(largeur, hauteur))

        # ------------------------------------------
        # Si la résolution de l'écran n'est pas (en
        # x) à 1920 ou 1680 ou 1600 ou 1400 ou 1280
        # ... elle est définie à 1280 par défaut
        # ------------------------------------------
        if largeur not in [1920, 1829, 1807, 1792, 1776, 1768, 1750, 1722, 1707, 1700, 1683, 1680, 1654, 1644, 
                           1600, 1564, 1544, 1536, 1517, 1504, 1500, 1477, 1463, 1444, 1440, 1415, 1400, 1397, 
                           1368, 1366, 1360, 1344, 1334, 1301, 1280, 1224, 1216, 1210, 1200] :
            largeur, hauteur = 1280, 720
            # Ré-ecriture de la résolution du bureau dans le rep. tempo
            ecr_tempo_resolution = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_resolution_desktop_murexpo_", "data_resolution_desktop_murexpo", ".txt", "{}x{}".format(largeur, hauteur))

        # ------------------------------------
        # Dimensionnement et positionnement
        # de la fenêtre principale de MurExpo
        # ------------------------------------
        # Si on est sous windows ...
        if platform.system() == 'Windows' :
            # Mise en place de l'affichage définitif
            self.setGeometry(0, 25, largeur, hauteur - 25) # -25 pour afficher toute la fenêtre
        # Autrement sous GNU/Linux et sous MacOS ...
        else :
            self.setGeometry(0, 0, largeur, hauteur)
        
        # Enlève complètement la barre de titre (commenté) 
        #self.setWindowFlag(Qt.FramelessWindowHint)
        

    def menus(self) :
        
        # =================================================================== #
        # MENUS
        # =================================================================== #
        menubar = self.menuBar()

        # Les différents menus 
        menu_fichier = menubar.addMenu(_('&Quitter'))

        # -------------------
        ## Sous-menu Quitter
        # -------------------
        # (--> # Les images (les icônes) sont importés par la fonction chemin_ressources 
        # contenue dans : config_apsc/chemin_data_ressources.py)
        quitterAction = QAction(QIcon(chemin_ressources('ressources_murexpo'+os.sep+'icone_png_fermeture.png')), _('Quitter'), self)
        quitterAction.setShortcut('Ctrl+Q')
        quitterAction.setStatusTip(_("Quitter l'application."))
        
        #
        menu_fichier.addAction(quitterAction)

        # Signal de l'action du sous-menu Quitter
        quitterAction.triggered.connect(self.quitter)
    

    def quitter(self) : sys.exit(0)


if __name__ == '__main__':
    
    app = QApplication(sys.argv)
    ex = MUREXPO_GUI()
    ex.show()
    # ----- Garder commenté ------------------------------
    """
    # -------------------------------------------------- #
    # Voir les styles disponibles :
    styles = [style for style in QStyleFactory.keys()]
    print(styles)
    # Déployement des styles interface (sous les différents systèmes)
    if platform.system() == 'Windows':
        app.setStyle(QStyleFactory.create('windowsvista'))
        app.setPalette(QApplication.style().standardPalette())
    else :
        # Sous GNU/Linux ... sous PyQt5 uniquement 
        # versions les styles standards dépendent des 
        # de PyQt5
        if 'Oxygen' in styles :
            QApplication.setStyle('Oxygen')
        elif 'Oxygen' not in styles and 'Windows' in styles :
            QApplication.setStyle('Windows')
        elif ('Oxygen' and' Windows') not in styles and 'MacStyle' in styles :
            QApplication.setStyle('MacStyle')
        # Si Oxygen ne fait pas partie de la liste
        # c'est le 1er élément qui est utilisé
        # ... sous MacOSX il est pratiquement
        # sûr que ce soit le 1er élément qui soit 
        # utilisé
        else :
            # Changement après test fait sous
            # Linux Mint 19.1 ...
            # Si 'Windows' se trouve à l'index 0
            # on sélectionne l'index 1
            if styles[0] == 'Windows' :
                QApplication.setStyle(styles[1])
                print("* Utilisation du style {}".format(styles[1]))
            # Autrement on sélectionne l'index 0
            else :
                QApplication.setStyle(styles[0])
                print("* Utilisation du style {}".format(styles[0]))
    """
    # ----- Garder commenté ------------------------------
    # -------------------------------------------------- #
    # Traduction en français des boutons ...
    # -------------------------------------------------- #    
    # Assure la traduction automatique du conversationnel à la locale
    if platform.system() == 'Windows':
        locale = QLocale.system().name() 
        translator = QTranslator()
        reptrad = QLibraryInfo.location(QLibraryInfo.TranslationsPath)
        if getattr(sys, 'frozen', False) :
            # Le programme a été traité par PyInstaller
            reptrad = os.path.join(sys._MEIPASS, "PyQt5", "Qt", "Translations")
            translator.load("qtBase_" + locale, reptrad)
            app.installTranslator(translator)
    # -------------------------------------------------- #
    elif platform.system() == 'Linux':
        locale = QLocale.system().name()[:2]
        translator = QTranslator()
        reptrad = QLibraryInfo.location(QLibraryInfo.TranslationsPath)
        if getattr(sys, 'frozen', False) :
            # Le programme a été traité par PyInstaller
            reptrad = os.path.join(sys._MEIPASS, "PyQt5", "Qt", "Translations")
            translator.load("qt_" + locale, reptrad)
            app.installTranslator(translator)

    sys.exit(app.exec_())
