#!/usr/bin/python3
# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

# Import PyQt5 -------------------------------------------------------------
from PyQt5.QtCore import QLocale
# --------------------------------------------------------------------------

# Pillow
try : import Image
except ImportError : from PIL import Image

import os

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------


def redimensionnement_image_oeuvre(chemin_image_ouverture, dimension_appelee, nom_image_sauvegarde) :
    """ Cette fonction vérifie la résolution
    et la change si besoin et ceci afin
    que les oeuvres s'affichent correctement 
    dans le QGraphicsView (fonction avec return).
    Cela permet de gagner pas mal de temps pour
    les images qui ont une résolution importante """

    # ---------------------------------------------------------------
    # Appel de la classe pour récupérer les commandes de 
    # MurExpo en vue de les insérer dans le terminal
    # interne (dans un QTextEdit dans l'onglet Retour
    # commandes)
    capter_commandes_murexpo = CapterCommandesMurExpo()
    # ---------------------------------------------------------------

    # Appel de la classe pour récupérer le chemin du 
    # répertoire temporaire selon le système d'exploitation
    chem_rep_tempo = Chemin_Rep_Tempo()
    # Le chemin est retourné (return ds la fonction de la classe)
    chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
    
    # Ouverture de l'image avec Pillow
    img_capture = Image.open(chemin_image_ouverture)
    
    # Vérification de la résolution (en 
    # pixels) de l'image (du mur) chargée
    w, h = img_capture.size

    # ---------------------------------------------------------------
    # Retour commandes ....
    # ---------------------------------------------------------------
    # Récupération des commandes pour le terminal interne de MurExpo
    capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Résolution initiale de l'image de l'oeuvre : {}x{}\n").format(w, h))
    # Ecriture des commandes dans le fichier
    # tempo data_retour_commandes_murexpo
    capter_commandes_murexpo.liste_ecrire_commandes()
    # ---------------------------------------------------------------

    # ----------------------------------
    # Si l'image chargée a une largeur 
    # plus grande que la hauteur ...
    # ----------------------------------
    if w > h :
        # Si la largeur de l'image est plus 
        # importante que que la dimension
        # appelée (par ex 1200 pixels) ...
        if w >= dimension_appelee :
            # Calcul du ratio de l'image
            ratio = float(w)/float(h)
        
            # Calcul de future hauteur
            h = int(float(dimension_appelee)/ratio)
            # Nouvelle dimension : 
            # tuple : (largeur, hauteur)
            nouv_dim_img = (dimension_appelee, h)
            
            # Redimensionnement et sauvegarde de l'image
            img_capture.thumbnail(nouv_dim_img, Image.ANTIALIAS)
            img_capture.save(os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde))
    
            # Sauvegarde de l'image dans le répertoire temporaire
            chemin_image_sauv = os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde)

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Résolution après redimensionnement de l'image de l'oeuvre : {}x{}\n").format(nouv_dim_img[0], nouv_dim_img[1]))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        # Autrement ...
        else : 
            # Conversion en RGB et sauvegarde dans le répertoire temporaire
            img_capture.convert("RGB").save(os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde))
            # Le chemin de l'image est celui 
            # sélectionné par l'utilisateur
            chemin_image_sauv = chemin_image_ouverture
            
    # ----------------------------------
    # Si l'image chargée a une hauteur 
    # plus grande que la largeur ...
    # ----------------------------------
    elif h > w :
        # Si la hauteur de l'image est plus 
        # importante que que la dimension
        # appelée (par ex 1200 pixels) ...
        if h >= dimension_appelee :
            # Calcul du ratio de l'image
            ratio = float(h)/float(w)
        
            # Calcul de future largeur
            w = int(float(dimension_appelee)/ratio)
            # Nouvelle dimension : 
            # tuple : (largeur, hauteur)
            nouv_dim_img = (w, dimension_appelee)
            
            # Redimensionnement et sauvegarde de l'image
            img_capture.thumbnail(nouv_dim_img, Image.ANTIALIAS)
            img_capture.save(os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde))
    
            # Sauvegarde de l'image dans le répertoire temporaire
            chemin_image_sauv = os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde)

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Résolution après redimensionnement de l'image de l'oeuvre : {}x{}\n").format(nouv_dim_img[0], nouv_dim_img[1]))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        # Autrement ...
        else : 
            # Conversion en RGB et sauvegarde dans le répertoire temporaire
            img_capture.convert("RGB").save(os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde))
            # Le chemin de l'image est celui 
            # sélectionné par l'utilisateur
            chemin_image_sauv = chemin_image_ouverture
            
    # ----------------------------------
    # Si l'image chargée a une largeur 
    # et une hauteur égales ...
    # ----------------------------------
    elif w == h :
        # Si la largeur de l'image est
        # plus importante que la dimension
        # appelée et aussi que la hauteur 
        # de l'image est plus importante 
        # que la dimension appelée (par 
        # ex 1200 pixels) ...
        if w >= dimension_appelee and h >= dimension_appelee :
            # Nouvelle dimension : 
            # tuple : (largeur, hauteur)
            nouv_dim_img = (dimension_appelee, dimension_appelee)
            
            # Redimensionnement et sauvegarde de l'image
            img_capture.thumbnail(nouv_dim_img, Image.ANTIALIAS)
            img_capture.save(os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde))
    
            # Sauvegarde de l'image dans le répertoire temporaire
            chemin_image_sauv = os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde)

            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            capter_commandes_murexpo.liste_capter_commandes(_("(* Traitement) Résolution après redimensionnement de l'image de l'oeuvre : {}x{}\n").format(nouv_dim_img[0], nouv_dim_img[1]))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------

        # Autrement ...
        else : 
            # Conversion en RGB et sauvegarde dans le répertoire temporaire
            img_capture.convert("RGB").save(os.path.join(chemin_tempo_system_exploit, nom_image_sauvegarde))
            # Le chemin de l'image est celui 
            # sélectionné par l'utilisateur
            chemin_image_sauv = chemin_image_ouverture
            
    return chemin_image_sauv
    

