#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Merci à eyllanesc pour son aide est pour cette classe ... import
# --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
# --> pour les icônes à gauche dans le bouton
from push_button_icone_gauche import PushButtonIconeGauche

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture
# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

# Imports PyQt5 ----------------------------------------------------------------
from PyQt5.QtWidgets import (QWidget, QApplication, QVBoxLayout, QTextEdit, \
                             QLineEdit, QColorDialog, QLayout, QPushButton, \
                             QGroupBox, QGridLayout, QLabel, QSlider, QCheckBox)
from PyQt5.QtCore import QSize, Qt, pyqtSignal, QLocale
from PyQt5.QtGui import QFont, QFontDatabase, QIcon
# ------------------------------------------------------------------------------

import sys, os

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------


class SelectionCouleur(QWidget) :
    """ Sélection/collecte des couleurs, affichage 
    et écriture de la couleur concernée dans un 
    fichier du répertoire temporaire """
    
    # =========================================================================
    # Création du signal en vue de lier le bouton 'Retour aux réglages' 
    # (onglet Choix couleur) à l'onglet 'Conception et exportation'. 
    # 1ère partie. Un grand merci à eyllanesc !!!
    # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
    # qtabwidget-defined-in-the-main-class-in-another-class#63065558
    customSignal_onglet_retour_reglages_choix_couleur = pyqtSignal(int)
    # =========================================================================
    
    def __init__(self, parent=None):
        super(SelectionCouleur, self).__init__(parent)
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # ----------------------------------------------------------------
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        # ----------------------------------------------------------------

        # -----------------------------------------------------------------
        #
        self.texte_edit_vue_couleur = QTextEdit()
        # Placé en lecture seule
        self.texte_edit_vue_couleur.setReadOnly(True)
        #
        self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba(0, 0, 0, 255);")
        
        #self.texte_edit_vue_couleur.setStyleSheet('''{background-image: url("./ressources_murexpo/murexpo_quadrillage_canal_alpha.png");}''')
        # -----------------------------------------------------------------
        
        # Cases à cocher pour le choix de la couleur 
        # pour le panneau/cartel ou la police de caractère
        self.case_choix_cartel = QCheckBox(_("Modifier la couleur du panneau/cartel"))
        self.case_choix_cartel.setFixedHeight(32)         
        self.case_choix_police = QCheckBox(_("Modifier la couleur de la police de caractère"))
        self.case_choix_police.setFixedHeight(32)
        # La case pour la choix du cartel est sélectionnée par défaut
        self.case_choix_cartel.setChecked(True)
        
        # -----------------------------------------------------------------
        #
        label_couleur_rouge = QLabel(_("Rouge :"))
        label_couleur_vert = QLabel(_("Vert :"))
        label_couleur_bleu = QLabel(_("Bleu :"))
        label_transparence = QLabel(_("Transparence :"))
        # -----------------------------------------------------------------
        # QSlider pour la sélection de la couleur rouge
        self.slider_couleur_rouge = QSlider()
        self.slider_couleur_rouge.setOrientation(Qt.Horizontal)
        self.slider_couleur_rouge.setMinimum(0)
        self.slider_couleur_rouge.setMaximum(255)
        self.slider_couleur_rouge.setValue(0)
        self.slider_couleur_rouge.setSingleStep(1)
        # Le tooltip (renseignements sur le widget)
        self.slider_couleur_rouge.setToolTip(_("Permet de régler la couleur rouge"))
        
        # QSlider pour la sélection de la couleur verte
        self.slider_couleur_vert = QSlider()
        self.slider_couleur_vert.setOrientation(Qt.Horizontal)
        self.slider_couleur_vert.setMinimum(0)
        self.slider_couleur_vert.setMaximum(255)
        self.slider_couleur_vert.setValue(0)
        self.slider_couleur_vert.setSingleStep(1)
        # Le tooltip (renseignements sur le widget)
        self.slider_couleur_vert .setToolTip(_("Permet de régler la couleur verte"))
        
        # QSlider pour la sélection de la couleur bleue
        self.slider_couleur_bleu = QSlider()
        self.slider_couleur_bleu.setOrientation(Qt.Horizontal)
        self.slider_couleur_bleu.setMinimum(0)
        self.slider_couleur_bleu.setMaximum(255)
        self.slider_couleur_bleu.setValue(0)
        self.slider_couleur_bleu.setSingleStep(1)
        # Le tooltip (renseignements sur le widget)
        self.slider_couleur_bleu .setToolTip(_("Permet de régler la couleur bleue"))
        
        # QSlider pour la sélection de la couleur bleue
        self.slider_transparence = QSlider()
        self.slider_transparence.setOrientation(Qt.Horizontal)
        self.slider_transparence.setMinimum(0)
        self.slider_transparence.setMaximum(255)
        self.slider_transparence.setValue(255)
        self.slider_transparence.setSingleStep(1)
        # Le tooltip (renseignements sur le widget)
        self.slider_transparence.setToolTip(_("Permet de régler la transparence de la couleur générée"))
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe 1 --> elle est ici : affichage_apsc/push_button_icone_droite.py ... 
        # en fait le texte dans le bouton est centré et l'icône positionné à droite.
        # J'ai un peu modifié la classe et adaptée. Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : config_apsc/chemin_data_ressources.py)
        icone_retour = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_2_fleches_gauche_murexpo_256x256.png'))) # A CHANGER !!!
        #
        bout_retour_reglages = PushButtonIconeGauche(_("Retour aux réglages"), icon = icone_retour, iconSize = QSize(16, 16))
        bout_retour_reglages.setFixedWidth(250)
        #
        bout_appliquer = PushButtonIconeGauche(_("Appliquer"), icon = icone_retour, iconSize = QSize(16, 16))
        bout_appliquer.setFixedWidth(250)
        # Le tooltip (renseignements sur le widget)
        bout_appliquer.setToolTip(_("Permet d'appliquer le choix des couleurs."))
        
        # *****************************************************************************************
        
        # On liste le contenu du répertoire temporaire ...
        self.liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
        
        # ========================================================
        # Exception pour contrôler la permanence du choix
        # "Modifier la couleur du panneau/cartel" ou bien
        # "Modifier la couleur de la police de caractère" 
        # par l'intermédiaire des cases à cocher.
        # ========================================================
        try :
            # -------------------------------------
            # Si les choix (2 cases à cocher) ont été
            # modifiés ... Boucle de vérification.
            # -------------------------------------
            for contenu in self.liste_contenu_tmp :
                # Si le fichier tempo concerné (c'est à dire :
                # data_choix_couleur_cartel_ou_police_murexpo
                # est bien dedans ...
                if 'data_choix_couleur_cartel_ou_police_murexpo' in contenu :
                    print(_("Choix de la couleur : Modifier la couleur de la police de caractère"))
                    # Utilisation de la classe Tempo_lecture pour la lecture
                    # des données dans le répertoire (dit) temporaire (lecture
                    # du choix de la couleur, et ce dans :
                    # data_choix_couleur_cartel_ou_police_murexpo (fichier 
                    # tempo qui a une réelle utilité !)
                    rep_tempo_lecture = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_choix_couleur_cartel_ou_police_murexpo_', 'data_choix_couleur_cartel_ou_police_murexpo')
                    # ... avec appel de la méthode affichage
                    rep_tempo_choix = rep_tempo_lecture.affichage()[0]
                    # Si le fichier tempo contient bien la donnée Modifier la
                    # couleur de la police de caractère le traitement se poursuit ...
                    if rep_tempo_choix == _("Modifier la couleur de la police de caractère") :
                        # La case "Résolution programmée" est décochée (!)
                        self.case_choix_cartel.setChecked(False)
                        # La case "Modifier la couleur de la 
                        # police de caractère" est cochée (!)
                        self.case_choix_police.setChecked(True)
                        # Ecriture "Modifier la couleur de la police 
                        # de caractère" dans le rép. tempo ... 
                        # --> data_choix_couleur_cartel_ou_police_murexpo
                        # (fichier tempo qui a une réelle utilité !)
                        ecr_tempo_choix_couleur_cartel_ou_police = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_couleur_cartel_ou_police_murexpo_", "data_choix_couleur_cartel_ou_police_murexpo", ".txt", _("Modifier la couleur de la police de caractère"))           
        # Ici le fichier peut exister ... MAIS ...
        except : 
            # Si la case "Modifier la couleur 
            # du panneau/cartel" est cochée ...
            if self.case_choix_cartel.isChecked() :
                print(_("Choix de la couleur : Modifier la couleur du panneau/cartel"))
                # Ecriture "Modifier la couleur du panneau/cartel" dans le répertoire temporaire (fichier utile !)
                ecr_tempo_choix_couleur_cartel_ou_police = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_couleur_cartel_ou_police_murexpo_", "data_choix_couleur_cartel_ou_police_murexpo", ".txt", _("Modifier la couleur du panneau/cartel"))
                # La case "Modifier la couleur du 
                # panneau/cartel" est cochée (!)
                self.case_choix_cartel.setChecked(True)
                # La case "Modifier la couleur de la
                # police de caractère" est décochée (!)
                self.case_choix_police.setChecked(False)

        # *****************************************************************************************

        # ------------------------------------------
        # Appel de la fonction de mise à jour 
        # des valeurs des sliders (rouge, vert, 
        # bleu, transparence) et de l'affichage 
        # de la couleur de fond dans le QTextEdit 
        # (ici quand la case "Modifier la couleur 
        # du panneau/cartel" est sélectionnée)
        # ------------------------------------------
        self.maj_sliders_couleur_cartel()
        # --------------------------------------------
        # Appel de la fonction de mise à jour 
        # des valeurs des sliders (rouge, vert, 
        # bleu, transparence) et de l'affichage 
        # de la couleur de fond dans le QTextEdit 
        # (ici quand la case "Modifier la couleur 
        # de la police de caractère" est sélectionnée)
        # --------------------------------------------
        self.maj_sliders_couleur_police()
        
        # *****************************************************************************************
        
        groupe_1 = QGroupBox()
        groupe_2 = QGroupBox()
        groupe_3 = QGroupBox()
        groupe_4 = QGroupBox()
        
        # ------------------------------------
        grid_1 = QGridLayout()
        grid_1.setSpacing(8)
        grid_1.setContentsMargins(4, 4, 4, 4)
        
        #
        grid_1.addWidget(self.texte_edit_vue_couleur, 0, 0)
        
        # ------------------------------------
        grid_2 = QGridLayout()
        grid_2.setSpacing(8)
        grid_2.setContentsMargins(4, 4, 4, 4)
        
        grid_2.addWidget(self.case_choix_cartel, 0, 0)
        grid_2.addWidget(self.case_choix_police, 0, 1)
        
        # ------------------------------------
        grid_3 = QGridLayout()
        grid_3.setSpacing(8)
        grid_3.setContentsMargins(4, 4, 4, 4)
        
        #
        grid_3.addWidget(label_couleur_rouge, 0, 0)
        grid_3.addWidget(self.slider_couleur_rouge, 0, 1)
        grid_3.addWidget(label_couleur_vert, 1, 0)
        grid_3.addWidget(self.slider_couleur_vert, 1, 1)
        grid_3.addWidget(label_couleur_bleu, 2, 0)
        grid_3.addWidget(self.slider_couleur_bleu, 2, 1)
        grid_3.addWidget(label_transparence, 3, 0)
        grid_3.addWidget(self.slider_transparence, 3, 1)
        
        # ------------------------------------
        grid_4 = QGridLayout()
        grid_4.setSpacing(8)
        grid_4.setContentsMargins(4, 4, 4, 4)
        
        #
        grid_4.addWidget(bout_retour_reglages, 0, 0)
        grid_4.addWidget(bout_appliquer, 0, 1)
        
        #
        groupe_1.setLayout(grid_1)
        groupe_2.setLayout(grid_2)
        groupe_3.setLayout(grid_3)
        groupe_4.setLayout(grid_4)
             
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        layout.addWidget(groupe_2, 1, 0)
        layout.addWidget(groupe_3, 2, 0)
        layout.addWidget(groupe_4, 3, 0)
        
        #
        self.setLayout(layout)
        
        # =====================================================
        # Signaux
        # =====================================================
        # Les cases à cocher QCheckBox (utilisation de lambda
        # car il y a un argument dans la fonction appelée)
        # 1 Fonction pour chaque QCheckBox
        # -----------------------------------------
        # Signaux pour les cases pour les choix ("Modifier la couleur du 
        # panneau/cartel" ou "Modifier la couleur de la police de caractère")
        self.case_choix_cartel.toggled.connect(lambda : self.fonction_choix_cartel(self.case_choix_cartel))
        self.case_choix_police.toggled.connect(lambda : self.fonction_choix_police(self.case_choix_police))
        # Signal pour le bouton appliquer
        bout_appliquer.clicked.connect(self.appliquer_choix_couleur)
        # Slider de la couleur rouge
        self.slider_couleur_rouge.valueChanged.connect(self.fonction_slider_rouge)
        # Slider de la couleur verte
        self.slider_couleur_vert.valueChanged.connect(self.fonction_slider_vert)
        # Slider de la couleur bleue
        self.slider_couleur_bleu.valueChanged.connect(self.fonction_slider_bleu)
        # Slider de la transparence (canal alpha)
        self.slider_transparence.valueChanged.connect(self.fonction_slider_transparence)
        # Signal pour permettre au bouton 'Retour aux réglages'
        # d'accéder à l'onglet 'Conception et exportation'
        bout_retour_reglages.clicked.connect(self.direction_onglet_conception_et_exportation)
        # =====================================================
        
        
    def fonction_choix_cartel(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) 'Modifier la 
        couleur du panneau/cartel' a été cochée """
        
        if c.isChecked() == True :
            # La case "Modifier la couleur de la 
            # police de caractère" est décochée (!)
            self.case_choix_police.setChecked(False)
            # La case "Modifier la couleur du 
            # panneau/cartel" est cochée (!)
            self.case_choix_cartel.setChecked(True)
            # Ecriture de "Modifier la couleur du 
            # panneau/cartel" dans le fichier tempo.
            # (fichier tempo qui a une réelle utilité !)
            ecr_tempo_choix_couleur_cartel_ou_police = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_couleur_cartel_ou_police_murexpo_", "data_choix_couleur_cartel_ou_police_murexpo", ".txt", _("Modifier la couleur du panneau/cartel"))
            
            # ------------------------------------------
            # Appel de la fonction de mise à jour 
            # des valeurs des sliders (rouge, vert, 
            # bleu, transparence) et de l'affichage 
            # de la couleur de fond dans le QTextEdit 
            # (ici quand la case "Modifier la couleur 
            # du panneau/cartel" est sélectionnée)
            # ------------------------------------------
            self.maj_sliders_couleur_cartel()
        
        
    def fonction_choix_police(self, c) :
        """ Fonction servant à détecter et à écrire dans le fichier 
        tempo correspondant si la case (QCheckBox) 'Modifier la 
        couleur de la police de caractère' a été cochée' """
        
        if c.isChecked() == True :
            # La case "Modifier la couleur du 
            # panneau/cartel" est décochée (!)
            self.case_choix_cartel.setChecked(False)
            # La case "Modifier la couleur de la 
            # police de caractère" est cochée (!)
            self.case_choix_police.setChecked(True)
            # Ecriture de "Modifier la couleur du 
            # panneau/cartel" dans le fichier tempo.
            # (fichier tempo qui a une réelle utilité !)
            ecr_tempo_choix_couleur_cartel_ou_police = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_choix_couleur_cartel_ou_police_murexpo_", "data_choix_couleur_cartel_ou_police_murexpo", ".txt", _("Modifier la couleur de la police de caractère"))
        
            # --------------------------------------------
            # Appel de la fonction de mise à jour 
            # des valeurs des sliders (rouge, vert, 
            # bleu, transparence) et de l'affichage 
            # de la couleur de fond dans le QTextEdit 
            # (ici quand la case "Modifier la couleur 
            # de la police de caractère" est sélectionnée)
            # --------------------------------------------
            self.maj_sliders_couleur_police()
        
        
    def fonction_slider_rouge(self) :
        """ Fonction pour récupérer la valeur 
        du slider pour la couleur rouge """
        
        # Récupération de la valeur définie par 
        # le slider rouge (variable)
        self.rouge = self.slider_couleur_rouge.value()
        
        try :
            # Si l'utilisateur a modifié les 4 sliders (rouge, vert, bleu, transparence)
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))
        except :
            self.vert = 0
            self.bleu = 0
            #self.transparence = 100
            self.transparence = 255
            # Si l'utilisateur n'a modifié que le slider du rouge (les autres prennent les 
            # valeurs des variables définies ici). Le QTextEdit se colore aux valeurs définies ...
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))

        
    def fonction_slider_vert(self) :
        """ Fonction pour récupérer la valeur 
        du slider pour la couleur verte """
        
        self.vert = self.slider_couleur_vert.value()
        
        try :
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))
        except :
            self.rouge = 0
            self.bleu = 0
            #self.transparence = 100
            self.transparence = 255
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))
        
        
    def fonction_slider_bleu(self) :
        """ Fonction pour récupérer la valeur 
        du slider pour la couleur bleue """
        
        self.bleu = self.slider_couleur_bleu.value()
        
        try :
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))
        except :
            self.rouge = 0
            self.vert = 0
            self.transparence = 255
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence)) 
           
            
    def fonction_slider_transparence(self) :
        """ Fonction pour récupérer la valeur 
        du slider pour la transparence (le
        canal alpha) """
        
        self.transparence = self.slider_transparence.value()
        
        try :
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))
        except :
            self.rouge = 0
            self.vert = 0
            self.bleu = 0
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))

        
    def maj_sliders_couleur_cartel(self) :
        """ Fonction pour mettre en place la 
        permanence des réglages des sliders 
        (rouge, vert, bleu, transparence) 
        pour la couleur de la police de 
        caractère """
        
        for contenu in self.liste_contenu_tmp :
            # Si la case "Modifier la couleur 
            # du panneau/cartel" est cochée ...
            if self.case_choix_cartel.isChecked() :
                # Si le fichier temporaire a été crée (cela prouve
                # que l'utilisateur a modifié les réglages de la
                # couleur du panneau/cartel) ...
                if 'data_couleur_panneau_cartel_murexpo' in contenu :
                    # Utilisation de la classe Tempo_lecture pour la lecture
                    # des données dans le répertoire (dit) temporaire
                    rep_tempo_lecture_cpc = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_couleur_panneau_cartel_murexpo_', 'data_couleur_panneau_cartel_murexpo')
                    # ... avec appel de la méthode affichage
                    rep_tempo_cpc = rep_tempo_lecture_cpc.affichage()[0]
                    # Les données sont récupérées de la sorte :
                    # 39/11/137/68 et sont ensuite splitées
                    # elles deviennent : ['39', '11', '137', '68']
                    data_couleur_cpc = rep_tempo_cpc.split('/')
                    # Attribution des valeurs (des couleurs)
                    # dans chaque slider ...
                    self.slider_couleur_rouge.setValue(int(data_couleur_cpc[0]))
                    self.slider_couleur_vert.setValue(int(data_couleur_cpc[1]))
                    self.slider_couleur_bleu.setValue(int(data_couleur_cpc[2]))
                    self.slider_transparence.setValue(int(data_couleur_cpc[3]))
                    # Affichage (maj) de la couleur de fond du QTextEdit
                    self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(int(data_couleur_cpc[0]), int(data_couleur_cpc[1]), int(data_couleur_cpc[2]), int(data_couleur_cpc[3])))
        
                    # ---------------------------------------------------------------
                    # Retour commandes ....
                    # ---------------------------------------------------------------
                    # Récupération des commandes pour le terminal interne de MurExpo
                    self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Réglage des couleurs pour le panneau/cartel : valeur de rouge {}, valeur de vert {}, valeur de bleu {}, valeur de transparence {}\n").format(str(data_couleur_cpc[0]), str(data_couleur_cpc[1]), str(data_couleur_cpc[2]), str(data_couleur_cpc[3])))
                    # Ecriture des commandes dans le fichier
                    # tempo data_retour_commandes_murexpo
                    self.capter_commandes_murexpo.liste_ecrire_commandes()
                    # ---------------------------------------------------------------
        
        
    def maj_sliders_couleur_police(self) :
        """ Fonction pour mettre en place la 
        permanence des réglages des sliders 
        (rouge, vert, bleu, transparence) 
        pour la couleur du panneau/cartel """
        
        for contenu in self.liste_contenu_tmp :
            # Si la case "Modifier la couleur de 
            # la police de caractère" est cochée ...
            if self.case_choix_police.isChecked() :
                # Si le fichier temporaire a été crée (cela prouve
                # que l'utilisateur a modifié les réglages de la
                # couleur de la police de caractère) ...
                if 'data_couleur_police_de_caractere_murexpo' in contenu :
                    # Utilisation de la classe Tempo_lecture pour la lecture
                    # des données dans le répertoire (dit) temporaire
                    rep_tempo_lecture_pdc = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_couleur_police_de_caractere_murexpo_', 'data_couleur_police_de_caractere_murexpo')
                    # ... avec appel de la méthode affichage
                    rep_tempo_pdc = rep_tempo_lecture_pdc.affichage()[0]
                    # Les données sont récupérées de la sorte :
                    # 196/204/34/48 et sont ensuite splitées
                    # elles deviennent : ['196', '204', '34', '48']
                    data_couleur_pdc = rep_tempo_pdc.split('/')
                    # Attribution des valeurs (des couleurs)
                    # dans chaque slider ...
                    self.slider_couleur_rouge.setValue(int(data_couleur_pdc[0]))
                    self.slider_couleur_vert.setValue(int(data_couleur_pdc[1]))
                    self.slider_couleur_bleu.setValue(int(data_couleur_pdc[2]))
                    self.slider_transparence.setValue(int(data_couleur_pdc[3]))
                    # Affichage (maj) de la couleur de fond du QTextEdit
                    self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(int(data_couleur_pdc[0]), int(data_couleur_pdc[1]), int(data_couleur_pdc[2]), int(data_couleur_pdc[3])))
        
                    # ---------------------------------------------------------------
                    # Retour commandes ....
                    # ---------------------------------------------------------------
                    # Récupération des commandes pour le terminal interne de MurExpo
                    self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Réglage des couleurs pour la police de caractère : valeur de rouge {}, valeur de vert {}, valeur de bleu {}, valeur de transparence {}\n").format(int(data_couleur_pdc[0]), int(data_couleur_pdc[1]), int(data_couleur_pdc[2]), int(data_couleur_pdc[3])))
                    # Ecriture des commandes dans le fichier
                    # tempo data_retour_commandes_murexpo
                    self.capter_commandes_murexpo.liste_ecrire_commandes()
                    # ---------------------------------------------------------------
        
        
    def appliquer_choix_couleur(self) :
        """ Appliquer les réglages des sliders 
        Ecriture de la couleur (par exemple :
        111/139/238/74 dans le fichier concerné
        dans le répertoire temporaire """
        
        # --------------------------------------------------------------------
        # Application du choix de la couleur dans le QTextEdit
        # --------------------------------------------------------------------
        try :
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))
        except :
            self.rouge = 0
            self.vert = 0
            self.bleu = 0
            self.transparence = 100
            self.texte_edit_vue_couleur.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self.rouge, self.vert, self.bleu, self.transparence))
        
        # --------------------------------------------------------------------
        # Sauvegarde du choix de la couleur ... soit dans le fichier
        # temporaire : "data_couleur_panneau_cartel_murexpo", soit dans :
        # "data_couleur_police_de_caractere_murexpo". Ces fichiers tempo
        # vont, au final, être utilisés pour mettre en place le panneu/cartel
        # et la police de caractère sur le mur d'exposition.
        # --------------------------------------------------------------------
        # Si l'utilisateur a sélectionné la case à cocher : 
        # "Modifier la couleur du panneau/cartel"
        if self.case_choix_cartel.isChecked() :
            # Ecriture des données dans le fichier tempo : data_couleur_panneau_cartel_murexpo
            ecr_tempo_couleur_cartel = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_couleur_panneau_cartel_murexpo_", "data_couleur_panneau_cartel_murexpo", ".txt", str(self.rouge)+"/"+str(self.vert)+"/"+str(self.bleu)+"/"+str(self.transparence))
        
        # Si l'utilisateur a sélectionné la case à cocher : 
        # "Modifier la couleur de la police de caractère"
        if self.case_choix_police.isChecked() :
            # Ecriture des données dans le fichier tempo : data_couleur_police_de_caractere_murexpo
            ecr_tempo_couleur_police = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_couleur_police_de_caractere_murexpo_", "data_couleur_cartel_et_ou_police_murexpo", ".txt", str(self.rouge)+"/"+str(self.vert)+"/"+str(self.bleu)+"/"+str(self.transparence))
        
            # ---------------------------------------------------------------
            # Retour commandes ....
            # ---------------------------------------------------------------
            # Récupération des commandes pour le terminal interne de MurExpo
            self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Réglage des couleurs pour la police de caractère : valeur de rouge {}, valeur de vert {}, valeur de bleu {}, valeur de transparence {}\n").format(str(self.rouge), str(self.vert), str(self.bleu), str(self.transparence)))
            # Ecriture des commandes dans le fichier
            # tempo data_retour_commandes_murexpo
            self.capter_commandes_murexpo.liste_ecrire_commandes()
            # ---------------------------------------------------------------
        
    
    def direction_onglet_conception_et_exportation(self) :
        """ Fonction utilisable quand l'utilisateur 
        clique sur le bouton Retour aux réglages à 
        l'onglet Choix couleur ... pour qu'il puisse
        immédiatement accéder à l'onglet Conception
        et exportation """

        # =========================================================================
        # Utilisation du signal pour afficher automatiquement l'onglet 
        # Conception et exportation. 
        # 2ème partie ==> le signal est émis pour l'id 0 (du QTabWidget défini 
        # [les onglets] défini dans le script main (murexpo_gui.py)). A voir aussi
        # Une ligne de code par rapport à cela se trouve dans murexpo_gui.py. 
        # Un grand merci à eyllanesc !!!
        # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
        # qtabwidget-defined-in-the-main-class-in-another-class#63065558
        self.customSignal_onglet_retour_reglages_choix_couleur.emit(0)
        # =========================================================================
        
