#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Merci à eyllanesc pour son aide est pour cette classe ... import
# --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
# --> pour les icônes à gauche dans le bouton
from push_button_icone_gauche import PushButtonIconeGauche

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture
# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Appel du module fonte (fonction 
# fonte avec arguments) contenu 
# dans le script fonte.py ... pour
# attribuer une fonte spécifique
# à un widget (le type de police)
from fonte import fonte

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

# Imports PyQt5 ----------------------------------------------------------------
from PyQt5.QtWidgets import (QWidget, QApplication, QTableWidget, 
                             QAbstractItemView, QTableWidgetItem,
                             QHeaderView, QVBoxLayout, QTextEdit, QLineEdit)
from PyQt5.QtCore import QSize, Qt, pyqtSignal, QLocale
from PyQt5.QtGui import QFont, QFontDatabase, QIcon
# ------------------------------------------------------------------------------

import sys, os

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------


class SelectionPolice(QWidget) :
    """ Collecte des différentes polices 
    présentes sur le système. Affichage 
    dans un tableau. L'utilisateur peut
    sélectionner la police de son choix 
    et l'afficher """
    
    # =========================================================================
    # Création du signal en vue de lier le bouton 'Retour aux réglages' 
    # (onglet Choix Police) à l'onglet 'Conception et exportation'. 
    # 1ère partie. Un grand merci à eyllanesc !!!
    # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
    # qtabwidget-defined-in-the-main-class-in-another-class#63065558
    customSignal_onglet_retour_reglages = pyqtSignal(int)
    # =========================================================================
    
    def __init__(self, parent=None):
        super(SelectionPolice, self).__init__(parent)
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------
        
        # ----------------------------------------------------------------
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        # ----------------------------------------------------------------
        
        # Appel de QFontDatabase pour
        # la collecte des polices
        # présentes sur le système
        self.database = QFontDatabase()
        
        # Création du tableau
        self.tableau = QTableWidget()

        # On peut sélectionner une ligne entière du tableau
        self.tableau.setSelectionBehavior(QAbstractItemView.SelectRows)
        # Simple sélection d'une ligne entière du tableau
        self.tableau.setSelectionMode(QAbstractItemView.SingleSelection)
        
        # Appel de la fonction de la collecte des
        # polices de caractère présente sur le 
        # système ... et insertion dans le tableau
        self.collecte_et_affichage_tableau_police()
        
        # Ligne de texte pour afficher une 
        # simulation pour la police choisie
        # (la ligne de texte s'adaptera à
        # la taille de la police)
        self.ligneedit = QLineEdit()
        
        # ---------------------------------------------------------------------------
        # Pour la disposition des icônes, eyllanesc m'a bien aidé ; il a proposé une 
        # classe 1 --> elle est ici : affichage_apsc/push_button_icone_droite.py ... 
        # en fait le texte dans le bouton est centré et l'icône positionné à droite.
        # J'ai un peu modifié la classe et adaptée. Voir ici :
        # --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
        # ----------------------------------------------------------------------------
        # (--> Affichage icône avec la fonction chemin_ressources 
        # contenue dans : chemin_data_ressources.py)
        icone_retour_aux_reglages = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_3_fleches_gauche_murexpo_256x256.png')))
        #
        self.bout_retour_aux_reglages = PushButtonIconeGauche(_("Retour aux réglages"), icon = icone_retour_aux_reglages, iconSize = QSize(16, 16))
        self.bout_retour_aux_reglages.setFixedWidth(350)

        # Disposition des widgets
        self.layout = QVBoxLayout()
        self.layout.addWidget(self.tableau)
        self.layout.addWidget(self.ligneedit)
        self.layout.addWidget(self.bout_retour_aux_reglages)
        # Aligne le bouton au centre (bien)
        self.layout.setAlignment(self.bout_retour_aux_reglages, Qt.AlignCenter)
        self.setLayout(self.layout)
        
        # ------------------------------------
        # Signaux
        # ------------------------------------
        # Sélection des lignes du tableau
        self.tableau.clicked.connect(self.selection_ligne_de_tableau)
        # L'utilisateur sélectionne le bouton et retourne dans l'onglet Conception et exportation
        self.bout_retour_aux_reglages.clicked.connect(self.direction_onglet_conception_exportation)
        
        
    def collecte_et_affichage_tableau_police(self) :
        """ Insertion des différents éléments dans un tableau 
        en utilisant QFontDatabase. Les éléments en questions
        sont le type de la police, le style et la taille. Les
        éléments sont collectés dans une liste """
        
        liste_elements_police = []
        
        # Sélection du type de police, par exemple :
        # 'Noto Sans Telugu UI', 'Noto Serif',
        # 'Noto Serif Display', 'Oxygen Mono', ...
        for famille in self.database.families() :
            # Sélection du style de la police, par exemple :
            # 'Italic', 'Bold', 'Bold Italic', 'Demi Oblique'
            for style in self.database.styles(famille) :
                # Sélection de la taille de la police
                for taille in self.database.smoothSizes(famille, style) :
                    # Les éléments constitutifs des différentes polices de
                    # caractère sont insérés dans la liste (liste de listes).
                    # Exemple (extrait) :
                    # [ ..., ['Noto Sans Tai Tham', 'Regular', 9], ['Noto Sans 
                    # Tai Tham', 'Regular', 10], ['Noto Sans Tai Tham', 'Regular', 
                    # 11], ... ['wasy10', 'LyX', 22], ['wasy10', 'LyX', 24], 
                    # ['wasy10', 'LyX', 26], ['wasy10', 'LyX', 28], ['wasy10', 
                    # 'LyX', 36], ['wasy10', 'LyX', 48], ['wasy10', 'LyX', 72]]
                    liste_elements_police.append([famille, style, taille])

        # La colonne du tableau comportera 3 cellules
        self.tableau.setColumnCount(3)
        # Le tableau comportera autant des lignes qu'il
        # y a d'éléments dans la liste liste_elements_police
        self.tableau.setRowCount(len(liste_elements_police))
        # Création des entêtes du tableau
        self.tableau.setHorizontalHeaderLabels([_("Police"), _("Style"), _("Taille")])
    
        # Rend invisible l'entête (les bords) vertical
        # ... c'est à dire 1, 2, 3, 4, ... 1500, ...
        self.tableau.verticalHeader().setVisible(False)

        # ... voir exactement à quoi ça sert ...
        self.tableau.horizontalHeader().setStretchLastSection(True) 
        self.tableau.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch) 

        # Boucle principale pour parcourir les 
        # éléments de la liste (liste de listes)
        for n_1, listes in enumerate(liste_elements_police) :
            # Boucle secondaire pour parcourir les
            # chaînes de caractères représentant
            # les éléments constitutifs de la police
            for n_2, elements in enumerate(listes) :
                # Remplissage du tableau
                self.tableau.setItem(n_1, n_2, QTableWidgetItem(str(elements)))

        # Le tableau est montré
        self.tableau.show()


    def selection_ligne_de_tableau(self):
        """ Affichage dans le tableau 
        et visionnement de la police
        sélectionnée dans le 
        QTextWidget de démonstration """
        
        liste_selection_police = []
        
        # Sélection de l'item choisi dans le tableau
        for item in self.tableau.selectedItems():
            # Les caractéristiques de l'item 
            # choisi sont entrées dans la liste
            liste_selection_police.append(item.text())
            
        print("Police de caractere choisie :", liste_selection_police)
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Police de caractere choisie (police : {}, style : {}, taille : {})\n").format(liste_selection_police[0], liste_selection_police[1], liste_selection_police[2]))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
            
        # ---------------------------------------------------------
        # La police choisie est écrite dans le rep. temporaire ...
        # ---------------------------------------------------------
        ecr_police_choisie = Tempo_ecriture(self.chemin_tempo_system_exploit, os.getcwd(), "data_police_de_caractere_choisie_murexpo_", "data_police_de_caractere_choisie_murexpo", ".txt", liste_selection_police[0]+','+liste_selection_police[1]+','+str(liste_selection_police[2]))

        # Appel de la fonction (module fonte) contenue dans le script fonte.py. 
        # Fonction avec arguments. Attribution de la police (police, style et taille)
        # à la ligne d'édition de visualisation de la police choisie par l'utilisateur
        fonte(self.ligneedit, liste_selection_police[0], liste_selection_police[1], int(liste_selection_police[2]))
        # Affichage de la police choisie dans la ligne d'édition
        self.ligneedit.setText(_("aAbBcCdDeEfFgGhHiIjJkKlLmMnN"))
        
        
    def direction_onglet_conception_exportation(self) :
        """ Fonction utilisable quand l'utilisateur clique sur 
        le bouton Retour aux réglages ici dans l'onglet Choisir 
        police. En fait cela permet à l'utilisateur d'accéder à 
        l'onglet Conception et exportation """
        
        # =========================================================================
        # Utilisation du signal pour afficher automatiquement l'onglet 
        # ... . 
        # 2ème partie ==> le signal est émis pour l'id 0 (du QTabWidget défini 
        # [les onglets] défini dans le script main (murexpo_gui.py)). A voir aussi
        # Une ligne de code par rapport à cela se trouve dans murexpo_gui.py. 
        # Un grand merci à eyllanesc !!!
        # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
        # qtabwidget-defined-in-the-main-class-in-another-class#63065558
        self.customSignal_onglet_retour_reglages.emit(0)
        # =========================================================================
