#!/usr/bin/python3
# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2018/2019 (2021 pour Murexpo)
#-------------------------------------------------------------------------------

import platform, tempfile, os


class Chemin_Rep_Tempo(object):
    """ Classe qui retourne le chemin du répertoire temp 
    suivant le système d'exploitation sur lequel APSC est 
    exécuté ... sous GNU/Linux ce sera : 
    /home/utilisateur/.apsc
    ... sous windows ce sera :
    \\Users\\utilisateur\\appData\\Local\\temp
    ... et sous MacOSX ce sera :
    dans le répertoire temporaire ... par contre sous 
    MacOSX, ce sera sous : 
    /Users/nom_utilisateur/.apsc
    Ce chemin sous MacOSX est du au fait qu'il peut y avoir 
    des problèmes (de droits) pour la lecture/écriture dans 
    les chemins des répertoires temporaires du système """
    
    def __init__(self):
        
        # Si on est sous windows ...
        if platform.system() == 'Windows' :
            self.chemin_temp = tempfile.gettempdir()
        # Si on est sous GNU/Linux ou MacOS ...
        elif platform.system() == 'Darwin' or platform.system() == 'Linux' :
            self.chemin_temp = os.path.join(os.path.expanduser('~'), '.murexpo')
        
    def chemin_tempo(self) :
        """ Fonction qui retourne le chemin quand elle est appelée """
        
        return self.chemin_temp
