#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2018
#-------------------------------------------------------------------------------

import os, tempfile, platform

class Tempo_ecriture():
    '''
    Classe utilisée pour l'écriture des données (par exemple les 
    options pour l'écriture du pdf de la séquence prof avec pdfkit) 
    dans un choix de répertoires temporaires
    '''
    
    def __init__(self, chemin_recherche_1, chemin_recherche_2, nom_recherche_1, nom_recherche_2, suffixe, ecrire):
        super().__init__()
        
        # Cette classe n'utilise pas (pour l'instant) la classe
        # Chemin_Rep_Tempo se trouvant dans le fichier
        # tempo_chemin_plateforme.py (qui se trouve dans le
        # répertoire config_apsc) ... elle passe donc par
        # des conditions d'écriture des fichiers tempo 
        # selon le système d'exploitation (--> sous GNU/Linux
        # le répertoire tempo se trouve dans /home/utilisateur/.apsc, 
        # sous windows il se trouvera dans 
        # \Users\utilisateur\appData\Local\temp, sous MacOSX le 
        # répertoire tempo sera dans /Users/nom_utilisateur/.apsc
        
        # --------------------------------------------------------
        # Ecriture des fichiers dans le répertoire temporaire ...
        # Par exemple écriture dans tempfile.gettempdir() pour 
        # windows
        try :
            # Elimination des fichiers résiduels
            fichiers = os.listdir(chemin_recherche_1)
            for fich in fichiers :
                if nom_recherche_1 in fich :
                    os.remove(chemin_recherche_1+os.sep+fich)
            # Si on est sous MacOSX ... ou GNU/Linux ...
            if platform.system() == 'Darwin' or platform.system() == 'Linux' :
                # Ecriture dans le répertoire (caché) .apsc ... par exemple
                # sous MacOSX ce sera dans le chemin /Users/utilisateur/.apsc
                # (à cause de problèmes de droits dans les répertoires temporaires)
                with open(chemin_recherche_1+os.sep+nom_recherche_1+suffixe, "wt") as oa :
                    oa.write(ecrire)
                    oa.flush()
            # ... autrement si on est sous windows
            else :
                # Préparation ...
                tmp = tempfile.NamedTemporaryFile(prefix=nom_recherche_1, suffix=suffixe, delete=False, mode='w+t')
                # Ecriture options dans le fichier temporaire
                tmp.write(ecrire)
                # Permet d'écrire les données dans le fichier
                tmp.flush()
                tmp.close()
        # ... (par ex suite) ... si problème écriture dans os.getcwd()
        except :
            # Elimination des fichiers résiduels
            fichiers = os.listdir(chemin_recherche_2)
            for fich in fichiers :
                if nom_recherche_2 in fich :
                    os.remove(chemin_recherche_2+os.sep+fich)   
            # Ecriture dans le répertoire courant d'APSC
            with open(chemin_recherche_2+os.sep+nom_recherche_2+suffixe, "w") as oe :
                oe.write(ecrire)
        # --------------------------------------------------------
