#!/usr/bin/python3
#-*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2020
#-------------------------------------------------------------------------------

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture
# Ecrire les données dans le répertoire temporaire
from tempo_ecriture import Tempo_ecriture

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Import de la fonction chemin_ressources pour
# la découpe/morcellement du fond ... reflet
from chemin_data_ressources import chemin_ressources

# Appel du module (fonction avec return) de redimensionnement
from redimensionnement_oeuvre import redimensionnement_image_oeuvre

# ---------------------------------------------------------------------------
# Import de la classe (de tyrtamos) nbre_appel_fonction du fichier libdeco.py
# ---------------------------------------------------------------------------
# Voir ici :  https://www.developpez.net/forums/d1117521-2/autres-langages/
# python-zope/general-python/tester-fonction-return-ete-executee
# ---------------------------------------------------------------------------
# Merci à tyrtamos !!!
# ---------------------------------------------------------------------------
from libdeco import nbre_appel_fonction

import glob, os, platform, string, random

# Imports PyQt5 ------------------------------------------------------------
from PyQt5.QtCore import QLocale
# --------------------------------------------------------------------------

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------

# Pillow
try : import Image, ImageFilter, ImageOps, ImageChops, ImageEnhance
except ImportError : from PIL import Image, ImageFilter, ImageOps, ImageChops, ImageEnhance


class Traitement_Image_Oeuvre() :
    ''' Classe pour le taitement des images avec Pillow.
    Ajouter les cadres, les matières du support de la 
    toile, ... pour chaque oeuvre (en fait il s'agit
    de la mise en valeur de chaque oeuvre '''
    
    def __init__(self) :
        super().__init__()
        
        # ------------------------------------------------ A garder pour l'instant ---------------
        """
        ### --> https://note.nkmk.me/en/python-pillow-flip-mirror/
        from PIL import Image, ImageOps
        im = Image.open('data/src/lena.jpg')
        # ---> IMAGE EFFET MIROIR SUR LE SOL (HAUT - BAS)
        im_flip = ImageOps.flip(im)
        im_flip.save('data/dst/lena_flip.jpg', quality=95)
        # ---> IMAGE EFFET MIROIR GAUCHE - DROITE
        im_mirror = ImageOps.mirror(im)
        im_mirror.save('data/dst/lena_mirror.jpg', quality=95)
        """
        
        """
        im1 = im.filter(ImageFilter.BLUR)
        im2 = im.filter(ImageFilter.MinFilter(3))
        im3 = im.filter(ImageFilter.MinFilter)  # same as MinFilter(3)
        """
        # ------------------------------------------------ A garder pour l'instant ---------------
        
        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # Chemin (dans le répertoire temporaire) et nom des images (PNG)
        self.img_traitement = os.path.join(self.chemin_tempo_system_exploit, 'image_oeuvre_selectionnee.png')

        try :
            # Ouverture de l'image avec Pillow
            # --> notamment pour le traitement 
            # du Sous verre, bordure
            self.img = Image.open(self.img_traitement)
        except : pass
        
        try :
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire.
            # Récupération de la largeur de la baguette du cadre et
            # de l'épaisseur du cadre (profondeur)
            # ------------------------------------------------------
            # Lecture de la largeur de la baguette du cadre
            pre_lecture_largeur_baguette_cadre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_largeur_baguette_cadre_oeuvre_murexpo_', 'data_largeur_baguette_cadre_oeuvre_murexpo')
            # ... avec appel de la méthode affichage
            self.largeur_baguette_cadre = "".join(pre_lecture_largeur_baguette_cadre.affichage())
            # Lecture de l'épaisseur du cadre
            pre_lecture_epaisseur_cadre_ou_toile = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_epaisseur_cadre_ou_toile_oeuvre_murexpo_', 'data_epaisseur_cadre_ou_toile_oeuvre_murexpo')
            # ... avec appel de la méthode affichage
            self.epaisseur_cadre_ou_toile = "".join(pre_lecture_epaisseur_cadre_ou_toile.affichage())
            #
            self.largeur_baguette_cadre = int(self.largeur_baguette_cadre)
            self.epaisseur_cadre_ou_toile = int(self.epaisseur_cadre_ou_toile)
        except :
            self.largeur_baguette_cadre = 24
            self.epaisseur_cadre_ou_toile = 8
            
        # Nombre d'oeuvres à afficher sur le mur (prévisions)
        # ---> Prévoir un réglage concernant cela dans longlet Réglages
        self.nbre_ouvres = 4
        
             
    def cadre_noir_classique(self) :
        """ Fonction pour fabriquer un cadre autour 
        de l'oeuvre. Cadre simpliste (Cadre noir
        classique dans le combo Choix du média) """
        
        # ----------------------------------------------
        # Appel de la fonction generale pour 
        # la gestion du reflet
        # ----------------------------------------------
        # La fonction est appelée ici (au tout début de
        # la fonction) car le reflet sur l'oeuvre doit 
        # être fait avant la pose du cadre (!)
        # ----------------------------------------------
        self.traitement_final_reflet_sur_oeuvre()
        # ----------------------------------------------
        
        # Si l'image existe bien dans le 
        # répertoire temporaire ...
        if os.path.exists(self.img_traitement) :

            # -------------------------------------
            # Ouverture se l'image interne 
            # à la fonction
            # -------------------------------------
            # Ouverture de l'image avec Pillow
            img = Image.open(self.img_traitement)
            # Vérification de la résolution
            # (en pixels) de l'image
            w, h = img.size
            
            # --------------------------------------------------------------------------------
            # Intégration d'un cadre noir autour de l'oeuvre avec Pillow ...
            # Procédure trouvée ici : 
            # https://moonbooks.org/Articles/Ajouter-une-bordure-autour-dune-image-avec-PIL/
            # --------------------------------------------------------------------------------
            # Création d'une nouvelle image en ajoutant l'épaisseur du cadre (fois 2)
            image_cadre_001 = Image.new('RGB', (w + 2 * self.epaisseur_cadre_ou_toile, h + 2 * self.epaisseur_cadre_ou_toile)) # cadre noir
            # On colle la cadre à l'image
            image_cadre_001.paste(img, (self.epaisseur_cadre_ou_toile, self.epaisseur_cadre_ou_toile))
            
            # -------------------------------------------
            # 2ème passage
            # -------------------------------------------
            # Couleur grise foncée --> (45, 45, 45)
            image_cadre_002 = Image.new('RGB', ((w + 2 * self.epaisseur_cadre_ou_toile) + 2 * self.largeur_baguette_cadre, (h + 2 * self.epaisseur_cadre_ou_toile) + 2 * self.largeur_baguette_cadre), (45, 45, 45))
            # On colle la cadre obtenu au passage 1 à l'image
            image_cadre_002.paste(image_cadre_001, (self.largeur_baguette_cadre, self.largeur_baguette_cadre))
            
            # Sauvegarde finale
            image_cadre_002.save(self.img_traitement, "PNG")

            
    def sous_verre_avec_bordure(self) :
        """ Fonction pour mettre l'oeuvre sous verre """
        
        # A VOIR ICI --> https://www.geeksforgeeks.org/python-pil-imageops-expand-method/?ref=rp
        try :
            
            # ------------------------------------------------------
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire.
            # Récupération de la largeur de la bordure et de la 
            # couleur du bord du sous verre
            # ------------------------------------------------------
            # Lecture de la couleur (écrite sous cette forme :
            # 127, 127, 127 dans le fichier tempo)
            pre_lecture_couleur_bord_sous_verre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_largeur_couleur_bordure_oeuvre_murexpo_', 'data_largeur_couleur_bordure_oeuvre_murexpo')
            # ... avec appel de la méthode affichage
            couleur_bord_sous_verre = "".join(pre_lecture_couleur_bord_sous_verre.affichage())
            # Récupération du tuple de la valeur de la couleur 
            # à partir des données contenues dans le fichier 
            # du répertoire temporaire. 
            # =======================================================
            # Il s'agit d'un comprehension de tuple (tout d'abord
            # les données récupérées sont splitées), cela donne par 
            # ex pour le gris --> ['127', ' 127', ' 127'] ... et 
            # ensuite chaque bande (r, g, b) est convertie en
            # integer et placée dans un tuple ; résultat pour le 
            # gris : (127, 127, 127)
            # =======================================================
            couleur = tuple(int(i) for i in couleur_bord_sous_verre.split(','))
            
            # Lecture de la largeur du bord
            pre_lecture_largeur_bord_sous_verre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_largeur_bordure_oeuvre_sous_verre_murexpo_', 'data_largeur_bordure_oeuvre_sous_verre_murexpo')
            # ... avec appel de la méthode affichage
            largeur_bord_sous_verre = int("".join(pre_lecture_largeur_bord_sous_verre.affichage()))
            
            # Lecture de la hauteur du bord
            pre_lecture_hauteur_bord_sous_verre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_hauteur_bordure_oeuvre_sous_verre_murexpo_', 'data_hauteur_bordure_oeuvre_sous_verre_murexpo')
            # ... avec appel de la méthode affichage
            hauteur_bord_sous_verre = int("".join(pre_lecture_hauteur_bord_sous_verre.affichage()))
        
        except :
            couleur = (0, 0, 0)
            largeur_bord_sous_verre = 20
            hauteur_bord_sous_verre = 20
        
        try : 
            # Traitement final --> Sous verre, bordure
            im = ImageOps.expand(self.img, border = (largeur_bord_sous_verre, hauteur_bord_sous_verre), fill = couleur)
            
            # Sauvegarde finale
            im.save(self.img_traitement, "PNG")
    
            # ----------------------------------------------
            # Appel de la fonction ...
            self.traitement_final_reflet_sur_oeuvre()
            # ----------------------------------------------
        except : pass
        
        
    @nbre_appel_fonction  
    def enhance_pour_reflet(self, facteur) :
        """ Fonction le traitement de la luminosité 
        (pour la construction du reflet). Fonction
        décorée avec le module de tyrtamos (voir 
        plus haut) """
        
        # On détecte si des images issues d'un précédent morcellement
        # existent, ... si c'est le cas elles sont supprimées
        liste_recup_img_existantes = [i for i in os.listdir(self.chemin_tempo_system_exploit) if 'image_mur_decoupee_pour_reflexion_' in i]
        for parc in liste_recup_img_existantes :
            if len(liste_recup_img_existantes) > 0 :
                os.remove(os.path.join(self.chemin_tempo_system_exploit, parc))

        # Récupération de l'image du mur traitée (on s'assure que 
        # l'image du mur est bien présente dans le rep. temporaire)
        image_mur_selection = [image_mur for image_mur in os.listdir(self.chemin_tempo_system_exploit) if image_mur == 'image_mur.jpg']
        
        # Si le fichier image_mur.jpg a bien
        # été généré dans le rép. tempo ...
        if len(image_mur_selection) > 0 :
            # Ouverture de l'image du mur ...
            im = Image.open(os.path.join(self.chemin_tempo_system_exploit, "".join(image_mur_selection)))
            
            # Intensification de la luminosité et sauvegarde
            enhancer = ImageEnhance.Brightness(im)
            im_sortie = enhancer.enhance(facteur).save(os.path.join(self.chemin_tempo_system_exploit, 'image_mur.jpg'))

    
    def decoupe_morcellement_fond_pour_reflet(self) :
        """ Fonction pour le traitement de la découpe 
        / morcellement (pour la construction du reflet). 
        --> gestion de 2 situations : quand l'utilisateur 
        a choisi Mur d'exposition par défaut ou bien quand 
        l'utilisateur a choisi Mur d'exposition personnel 
        dans Configuration > Type de mur d'exposition """

        # -------------------------------------------------------
        # Récupération dur mur sélectionné ...
        # -------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture 
        # des données dans le répertoire (dit) temporaire
        pre_lecture_selection_mur = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_choix_du_mur_murexpo_', 'data_choix_du_mur_murexpo')
        # ... avec appel de la méthode affichage
        lecture_choix_mur = "".join(pre_lecture_selection_mur.affichage())
        
        # Condition pour le travail préparatoire de 
        # l'apposition du reflet sur chaque oeuvre
        if lecture_choix_mur == 'mur_001' :
            image_reflet = chemin_ressources(os.path.join('ressources_murexpo', 'mur_001_reflet.png'))
        elif lecture_choix_mur == 'mur_002' :
            image_reflet = chemin_ressources(os.path.join('ressources_murexpo', 'mur_002_reflet.png'))
        elif lecture_choix_mur == 'mur_003' :
            image_reflet = chemin_ressources(os.path.join('ressources_murexpo', 'mur_003_reflet.png'))
        elif lecture_choix_mur == 'mur_004' :
            image_reflet = chemin_ressources(os.path.join('ressources_murexpo', 'mur_004_reflet.png'))
        elif lecture_choix_mur == 'mur_005' :
            image_reflet = chemin_ressources(os.path.join('ressources_murexpo', 'mur_005_reflet.png'))
        elif lecture_choix_mur == 'mur_006' :
            image_reflet = chemin_ressources(os.path.join('ressources_murexpo', 'mur_006_reflet.png'))
        elif lecture_choix_mur == 'mur_007' :
            image_reflet = chemin_ressources(os.path.join('ressources_murexpo', 'mur_007_reflet.png'))
        elif lecture_choix_mur == 'mur_008' :
            image_reflet = chemin_ressources(os.path.join('ressources_murexpo', 'mur_008_reflet.png'))

        # Ouverture de l'image du reflet correspondante
        im = Image.open(os.path.join(self.chemin_tempo_system_exploit, "".join(image_reflet)))
        
        # Résolution de l'image
        w, h = im.size

        # ------------------------------------------------------------------------------------
        # Ré-utilisation de l'algorithme que j'avais mis en place dans EKD
        # ------------------------------------------------------------------------------------
        # Comprehension-list pour calculer les coordonnées de découpe dans le sens horizontal
        l_coord_horizontal = [(coord_x * (w / self.nbre_ouvres), coord_y * (h / 1)) for coord_y in range(1) for coord_x in range(self.nbre_ouvres)]
        # Comprehension-list pour calculer les coordonnées de découpe dans le sens vertical
        l_coord_vertical = [(coord_x * (w / self.nbre_ouvres) + (w / self.nbre_ouvres), coord_y * (h / 1) + (h / 1)) for coord_y in range(1) for coord_x in range(self.nbre_ouvres)]
        # Regroupement sens horizontal et vertical
        inter = zip(l_coord_horizontal, l_coord_vertical)
        # Regroupement final des coordonnées horizontales et verticales pour la découpe.
        # Affiche par exemple: [(0, 0, 192, 1080), (192, 0, 384, 1080), ...]
        l_coord_finales = [(int(x1[0]), int(x1[1]), int(y1[0]), int(y1[1])) for x1, y1 in inter]
        # ------------------------------------------------------------------------------------

        # Création des images après découpe et sauvegarde
        for n, decoup in enumerate(l_coord_finales) :
            im.crop(decoup).save(os.path.join(self.chemin_tempo_system_exploit, 'image_mur_decoupee_pour_reflexion_'+str(n+1).zfill(3)+'.png'))
                 
    
    def mise_en_place_reflet(self, image_selectionnee) :
        """ Fonction de mise en place du reflet. 
        Récupération de l'index de l'oeuvre 
        sélectionnée, redimensionnement de 
        l'image du mur après découpe, et 
        composition entre les deux parties """
        
        # ---------------------------------------------
        # Lecture de l'index de l'oeuvre sélectionnée
        # par l'utilisateur dans la vue
        pre_lecture_index_oeuvre_selectionnee = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_selection_index_oeuvre_ds_scene_murexpo_', 'data_selection_index_oeuvre_ds_scene_murexpo')
        # ... avec appel de la méthode affichage
        # On récupère la valeur sous forme d'un
        # integer
        index_oeuvre_selectionnee = int("".join(pre_lecture_index_oeuvre_selectionnee.affichage()))
        # ---------------------------------------------
        # Lecture de l'intensité du reflet de l'oeuvre
        pre_lecture_val_intensite_reflet = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_intensite_reflet_oeuvre_murexpo_', 'data_intensite_reflet_oeuvre_murexpo')
        # ... avec appel de la méthode affichage
        # On récupère la valeur sous forme d'un
        # integer
        val_intensite_reflet = int("".join(pre_lecture_val_intensite_reflet.affichage()))
        # ---------------------------------------------

        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)
        
        # --------------------------------------
        # Récupération des images du mur
        # après découpe/morcellement
        # --------------------------------------
        # Scan du contenu du rep. tempo.
        for contenu in liste_contenu_tmp :
            # Si les images du mur préalablement
            # découpées/morcelées sont bien présentes ...
            if 'image_mur_decoupee_pour_reflexion_' in contenu :
                # ... une liste contenant tous les chemins est crée
                liste_contenu_img_mur_decoupe = glob.glob(os.path.join(self.chemin_tempo_system_exploit, 'image_mur_decoupee_pour_reflexion_*'))
                # Mise en ordre de la liste
                liste_contenu_img_mur_decoupe.sort()

        # Ouverture de l'image sélectionnée avec
        # Pillow pour capturer largeur et hauteur
        ouverture_img_selection = Image.open(image_selectionnee)
        
        # -------------------------------------------------------------
        try :
            # On récupère l'image morcelée/découpée (dans la liste 
            # --> liste_contenu_img_mur_decoupe) à l'indice de 
            # l'oeuvre sélectionnée par l'utilisateur ==> récupéré 
            # par la variable index_oeuvre_selectionnee
            capture_img_decoup_mur_id = liste_contenu_img_mur_decoupe[index_oeuvre_selectionnee]
        except :
            # En cas de dépassement d'index (IndexError: list index 
            # out of range), un index aléatoire entre 0 et le nombre
            # d'oeuvres à afficher (en prévision)
            ### capture_img_decoup_mur_id = liste_contenu_img_mur_decoupe[random.randrange(9)] ###
            capture_img_decoup_mur_id = liste_contenu_img_mur_decoupe[random.randrange(self.nbre_ouvres - 1)]
        # -------------------------------------------------------------
        
        # On va redimensionner l'image ainsi capturée aux 
        # dimensions de l'image sélectionnée par l'utilisateur
        # ----------------------------------------------------
        # Tout d'abord l'image est ouverte dans Pillow
        # ----------------------------------------------------
        ouverture_img_mur_morcele = Image.open(capture_img_decoup_mur_id)
        # Redimensionnement et sauvegarde ...
        img_mor_morcele_redim = ouverture_img_mur_morcele.resize(ouverture_img_selection.size).save(os.path.join(self.chemin_tempo_system_exploit, 'image_mur_redimensionnee_pour_reflet.png'))
        
        # Ouverture de l'image qui vient d'être sauvegardée
        # --> image_mur_redimensionnee_pour_reflet.png
        ouv_img_reflet_vitre = Image.open(os.path.join(self.chemin_tempo_system_exploit, 'image_mur_redimensionnee_pour_reflet.png'))
        
        # Création d'un masque ... voir ici :
        # https://note.nkmk.me/en/python-pillow-composite/
        # ... en prenant en compte la valeur de l'intensité 
        # du reflet (val_intensite_reflet) définie par 
        # l'utilisateur
        masque = Image.new("L", ouv_img_reflet_vitre.size, val_intensite_reflet) # Valeurs entre 10 et 256 --> par défaut : 76
        # Composition entre l'image du mur capturée et l'image sélectionnée
        im = Image.composite(ouv_img_reflet_vitre, ouverture_img_selection, masque).save(self.img_traitement, "PNG")
        
        
    def traitement_final_reflet_sur_oeuvre(self) :
        """ Fonction finale du traitement
        du reflet (vitre) """
        
        # Lecture de du fichier tempo concerné pour savoir quel choix
        # l'utilisateur a fait (à savoir un reflet sur l'oeuvre ou non)
        pre_lecture_relief_oeuvre = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_reflet_sur_oeuvre_oeuvre_murexpo_', 'data_reflet_sur_oeuvre_oeuvre_murexpo')
        # ... avec appel de la méthode affichage
        relief_oeuvre = "".join(pre_lecture_relief_oeuvre.affichage())
        
        # --------------------------------------------------------------------
        # Si l'utilisateur a choisi d'utiliser le relief sur l'oeuvre
        # ... sinon cette condition est ignorée (donc pas de relief
        # sur l'oeuvre)
        if relief_oeuvre in [_("Reflet sur l'oeuvre"), _("Reflet &sur l'oeuvre")] :
            # ----------------------------------------------------------------
            # Compte le nombre d'appels de la fonction enhance_pour_reflet 
            # (avec la classe avec decorateur de tyrtamos). Voir plus haut.
            # ----------------------------------------------------------------
            # Teste le nombre d'exécutions de la fonction (ici la fonction
            # n'a pas encore été exécutée, d'où le == 0). Avec cette 
            # condition la fonction ne sera exécutée qu'une fois !
            # ----------------------------------------------------------------
            if self.enhance_pour_reflet.nbappels == 0 :
                # Exécution (une seule fois) de la fonction
                # enhance_pour_reflet pour le reflet ...
                # La fonction ici contient un self et 3.2,
                # c'est bien comme cela ...
                self.enhance_pour_reflet(self, 0.8) # 0.8
            # Exécution de la fonction de découpe/morcellement
            self.decoupe_morcellement_fond_pour_reflet()
            # Mise en place finale du reflet
            self.mise_en_place_reflet(self.img_traitement)
            # ----------------------------------------------------------------
            
    
    def decalage_points_gif_anim(self) :
        """ Fonction pour calculer les coordonnées 
        de décalage des points (de gauche à droite) 
        pour créer le déplacement en vue du découpage 
        des images dans le traitement des gif animé """
        
        # --------------------------------------------------------------------------------
        # Lecture du fichier tempo. data_choix_decalage_gif_anime_murexpo
        # --------------------------------------------------------------------------------
        # Utilisation de la classe Tempo_lecture pour la lecture des données dans le 
        # répertoire (dit) temporaire
        pre_lecture_choix_decalage_gif_anim = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), 'data_choix_decalage_gif_anime_murexpo_', 'data_choix_decalage_gif_anime_murexpo')
        # ... avec appel de la méthode affichage
        # variable ==> nbre_img
        decalage = "".join(pre_lecture_choix_decalage_gif_anim.affichage())
        decalage = int(decalage)
        
        # Si l'utilisateur sélectionne
        # 5 pour le décalage ...
        if decalage == 5 :
            # L'image fera 
            # 500 x 720 pix
            res_x = 500
            # Le nombre d'image
            # aura pour base 401
            nbre_img = 401
        # Si l'utilisateur sélectionne
        # 10 pour le décalage ...
        elif decalage == 10 :
            # L'image fera 
            # 500 x 720 pix
            res_x = 500
            # Le nombre d'image
            # aura pour base 201
            nbre_img = 201
        # Si l'utilisateur sélectionne
        # 10 pour le décalage ...
        elif decalage == 20 :
            # L'image fera 
            # 500 x 720 pix
            res_x = 500
            # Le nombre d'image
            # aura pour base 101
            nbre_img = 101
        # Si l'utilisateur sélectionne
        # 25 pour le décalage ...
        elif decalage == 25 :
            # L'image fera 
            # 500 x 720 pix
            res_x = 500
            # Le nombre d'image
            # aura pour base 81
            nbre_img = 81
            
        # Liste des coordonnées sous
        # la forme de tuples ds une liste
        liste_coordonnees = []
        
        # Boucle calculant chaque coordonnée
        # (x_1, y_1, x_2, y_2)
        for n in range(nbre_img) :
            x_2 = (n * decalage) + res_x
            # Pour que la coordonnée
            # n'aille pas au delà
            # des 2500 pixels
            if x_2 <= 2500 :
                x_1 = (n * decalage)
                y_1 = 0
                y_2 = 720
                # On remplit la liste des coordonnées, par
                # rexemple le résultat (si nbre_img = 18) :
                # [(0, 0, 375, 720), (125, 0, 500, 720), (250, 0, 625, 720), 
                # (375, 0, 750, 720), (500, 0, 875, 720), (625, 0, 1000, 720), 
                # (750, 0, 1125, 720), (875, 0, 1250, 720), (1000, 0, 1375, 720), 
                # (1125, 0, 1500, 720), (1250, 0, 1625, 720), (1375, 0, 1750, 720), 
                # (1500, 0, 1875, 720), (1625, 0, 2000, 720), (1750, 0, 2125, 720), 
                # (1875, 0, 2250, 720), (2000, 0, 2375, 720), (2125, 0, 2500, 720)]
                # Juste pour rexemple, 18 a été enlevé ...
                liste_coordonnees.append((x_1, y_1, x_2, y_2))
                
        return liste_coordonnees
        
        # --------------------------------------------------------------
        
