#!/usr/bin/python3
# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Author:      Lama Angelo
#
# Copyright:   (c) Lama Angelo 2021
#-------------------------------------------------------------------------------

# Je me suis inspiré de ce post sur Stackoverflow : 
# https://stackoverflow.com/questions/49670506/reading-frames-from-a-folder-and-display-in-qgraphicsview-in-pyqt4
# ... en l'adaptant complètement à mon besoin ...

# Merci à eyllanesc pour son aide est pour cette classe ... import
# --> pour les icônes à gauche dans le bouton
# --> https://stackoverflow.com/questions/53415379/align-icon-on-the-right-and-center-the-text-in-a-qpushbutton
from push_button_icone_gauche import PushButtonIconeGauche

# Lire les données dans le répertoire temporaire
from tempo_lecture import Tempo_lecture

# Le chemin du répertoire temporaire est 
# conditionné par le système d'exploitation
from tempo_chemin_plateforme import Chemin_Rep_Tempo

# Affichage des images (icônes) par la fonction chemin_ressources 
from chemin_data_ressources import chemin_ressources

# Capture des commandes (actions utilisateur et traitements)
# effectuées dans MurExpo. C'est en qque sorte un terminal.
from capter_commandes_murexpo import CapterCommandesMurExpo

import os, glob, platform

# ---------------------------------------------------------------------------------------------
# Gestion de la langue du système (et des fichiers de traduction)
# ---------------------------------------------------------------------------------------------
import gettext
try :
    # On récupère la locale
    locale = QLocale.system().name() # exemple (anglais) : en_EN
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=[locale[0:2]])
    loc_langue.install()
except :
    # Pour les français (langue par défaut)
    _ = gettext.gettext
# *********************************************************************************************
# Pour tester en anglais par exemple (il faut que les fichiers /locale/LC_MESSAGES/murexpo.po
# et /locale/LC_MESSAGES/murexpo.mo existent et qu'au moins une ligne ait été traduite dans
# PoEdit) .... et ensuite ce code ( à la palce de l'autre au dessus) :
"""
import gettext
try :
    loc_langue = gettext.translation('murexpo', localedir='locale', languages=['en'])
    loc_langue.install()
except :
    _ = gettext.gettext
"""
# *********************************************************************************************
# ---------------------------------------------------------------------------------------------

# Import de Pillow
try : import Image
except ImportError : from PIL import Image

# Import du timer du module threading
from threading import Timer

# Imports PyQt5 -----------------------------------------------------------------------
from PyQt5.QtWidgets import QWidget, QGroupBox, QPushButton, QGridLayout, \
                            QGraphicsScene, QGraphicsView, QGraphicsPixmapItem, \
                            QApplication, QLineEdit, QLabel
from PyQt5.QtCore import QSize, Qt, QTimer, pyqtSignal, QLocale
from PyQt5.QtGui import QPixmap, QIcon, QColor, QTransform
# --------------------------------------------------------------------------------------


class Visionnage(QWidget):
    ''' Classe pour visionner l'image 
    de l'exposition (mur + oeuvres) '''
    
    # =========================================================================
    # Création du signal en vue de lier le bouton 'Revenir à la page des 
    # réglages' (page 4, index 3 du QStackedWidget) à l'onglet 'Conception 
    # et exportation'. 1ère partie. Un grand merci à eyllanesc !!!
    # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
    # qtabwidget-defined-in-the-main-class-in-another-class#63065558
    customSignal_onglet_retour_conception_exportation = pyqtSignal(int)
    # =========================================================================
    
    def __init__(self, parent=None):
        super(Visionnage, self).__init__(parent)
        
        # ---------------------------------------------------------------
        # Appel de la classe pour récupérer les commandes de 
        # MurExpo en vue de les insérer dans le terminal
        # interne (dans un QTextEdit dans l'onglet Retour
        # commandes)
        self.capter_commandes_murexpo = CapterCommandesMurExpo()
        # ---------------------------------------------------------------

        # Appel de la classe pour récupérer le chemin du 
        # répertoire temporaire selon le système d'exploitation
        chem_rep_tempo = Chemin_Rep_Tempo()
        # Le chemin est retourné (return ds la fonction de la classe)
        self.chemin_tempo_system_exploit = chem_rep_tempo.chemin_tempo()
        
        # On liste le contenu du répertoire temporaire ...
        liste_contenu_tmp = os.listdir(self.chemin_tempo_system_exploit)

        # Utilisation QGraphicsScene et QGraphicsView
        self.scene  = QGraphicsScene()
        # Le fond de la scène est gris
        self.scene.setBackgroundBrush(QColor(100, 100, 100))
        # Ajout de la scène au QGraphicsView
        self.graphique_vue = QGraphicsView(self.scene)
        
        # Le label de la ligne d'édition
        label_ligne_aff_chem = QLabel(_("Chemin d'enregistrement de l'image de l'exposition :"))
        
        # La ligne pour afficher le chemin ...
        self.ligne_aff_chem_enregistr_expo = QLineEdit()
        self.ligne_aff_chem_enregistr_expo.setText(_("Pas de chemin spécifié pour l'instant"))
        self.ligne_aff_chem_enregistr_expo.setReadOnly(True)

        # (--> Affichage icône avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
        # ------------------------------------------------------------------------------------------
        # Icône pour le bouton principal d'affichage
        icone_affichage = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_visualisation_murexpo_256x256.png')))
        # Icône Zoom avant (loupe +)            
        icone_zoom_avant = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_loupe_plus_murexpo_256x256.png')))
        # Icône Zoom arrière (loupe -)            
        icone_zoom_arriere = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_loupe_moins_murexpo_256x256.png')))
        # Icône vue par défaut           
        icone_vue_par_defaut = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_vue_par_defaut_murexpo_256x256.png')))
        
        # Icône pour revenir au réglages de l'exposition           
        icone_revenir = QIcon(chemin_ressources(os.path.join('ressources_murexpo', 'icone_3_fleches_gauche_murexpo_256x256.png')))

        # Bouton pour zoomer vers l'avant (loupe +)
        self.bout_zoom_avant = PushButtonIconeGauche(_("Zoom +"), icon = icone_zoom_avant, iconSize = QSize(16, 16))
        self.bout_zoom_avant.setMaximumWidth(160)
        self.bout_zoom_avant.setMinimumWidth(160)
        # ToolTip
        self.bout_zoom_avant.setToolTip(_("Zoom avant (agrandir la vue)"))
        
        # Bouton pour zoomer vers l'arrière (loupe -)
        self.bout_zoom_arriere = PushButtonIconeGauche(_("Zoom -"), icon = icone_zoom_arriere, iconSize = QSize(16, 16))
        self.bout_zoom_arriere.setMaximumWidth(160)
        self.bout_zoom_arriere.setMinimumWidth(160)
        # ToolTip
        self.bout_zoom_arriere.setToolTip(_("Zoom arrière (rapetisser la vue)"))
        
        # Bouton pour revenir à la taille d'affichage par défaut (100%)
        self.bout_zoom_taille_defaut = PushButtonIconeGauche(_("Taille 100%"), icon = icone_vue_par_defaut, iconSize = QSize(16, 16))
        # ToolTip
        self.bout_zoom_taille_defaut.setToolTip(_("Affiche l'image à la taille d'origine, taille 100%"))
        
        # Bouton pour afficher à une taille adaptée à la fenêtre (toute l'image de l'expo est visible)
        self.bout_taille_adaptee = PushButtonIconeGauche(_("Taille adaptée à la fenêtre"), icon = icone_vue_par_defaut, iconSize = QSize(16, 16))
        # ToolTip
        self.bout_taille_adaptee.setToolTip(_("L'image de l'exposition devient entièrement visible"))
        
        # Bouton pour revenir aux réglages du mur d'exposition et du dépôt des oeuvres
        self.bout_revenir = PushButtonIconeGauche(_("Revenir à la page des réglages"), icon = icone_revenir, iconSize = QSize(16, 16))
        # ToolTip
        self.bout_revenir.setToolTip(_("Retourner aux réglages présents dans l'onglet Conception et exportation"))

        # =====================================================
        groupe_1 = QGroupBox()
        groupe_2 = QGroupBox()
        groupe_3 = QGroupBox()

        grid_1 = QGridLayout()
        grid_1.setSpacing(8)
        grid_1.setContentsMargins(4, 4, 4, 4)

        self.grid_2 = QGridLayout()
        self.grid_2.setSpacing(8)
        self.grid_2.setContentsMargins(4, 4, 4, 4)
        
        self.grid_3 = QGridLayout()
        self.grid_3.setSpacing(8)
        self.grid_3.setContentsMargins(4, 4, 4, 4)
        
        # Ajout du QGraphicsView ds la grille 1
        grid_1.addWidget(self.graphique_vue, 0, 0)#, 1, 1)

        # Ajout du label dans la grille 2
        self.grid_2.addWidget(label_ligne_aff_chem, 0, 0)
        # Ajout De la ligne d'édition affichant le chemin
        # de l'expo une fois enregistrée dans la grille 2
        self.grid_2.addWidget(self.ligne_aff_chem_enregistr_expo, 0, 1)
        
        # Ajout des boutons ds la grille 3
        self.grid_3.addWidget(self.bout_zoom_avant, 1, 0)
        self.grid_3.addWidget(self.bout_zoom_arriere, 1, 1)
        self.grid_3.addWidget(self.bout_zoom_taille_defaut, 1, 2)
        self.grid_3.addWidget(self.bout_taille_adaptee, 1, 3)
        self.grid_3.addWidget(self.bout_revenir, 1, 4)

        #
        groupe_1.setLayout(grid_1)
        groupe_2.setLayout(self.grid_2)
        groupe_3.setLayout(self.grid_3)
        
        layout = QGridLayout()
        layout.addWidget(groupe_1, 0, 0)
        layout.addWidget(groupe_2, 1, 0)
        layout.addWidget(groupe_3, 2, 0)
        
        #
        self.setLayout(layout)
        
        # =====================================================
        # Signaux
        # =====================================================
        # Signaux pour les boutons zoom avant et zoom arrière
        self.bout_zoom_avant.clicked.connect(self.afficher_zoom_avant)
        self.bout_zoom_arriere.clicked.connect(self.afficher_zoom_arriere)
        # Signal pour afficher la taille de l'image d'exposition à 100%
        self.bout_zoom_taille_defaut.clicked.connect(self.afficher_zoom_taille_defaut)
        # Signal pour que l'image de l'exposition s'adapte à la taille de la scène
        self.bout_taille_adaptee.clicked.connect(self.afficher_taille_adaptee)
        # Signal pour accéder/revenir à l'onglet Conception et exportation
        self.bout_revenir.clicked.connect(self.direction_onglet_conception_exportation)
        # =====================================================
        
        # Temps de latence pour le visionnage
        # (1000 --> 1 seconde)
        self.temps_de_latence_vision = 1000
        # Affichage direct de l'expo 
        # avec un QTimer (important !)
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.afficher)
        self.timer.start(self.temps_de_latence_vision)
         
    
    # ---------------------------------------------------------------------------------
    # Ces fonctions (événements à la souris, en l'occurrence pointeur gauche de la
    # souris + touche Shift [touche Maj avec un clavier de PC ... et Shift avec un 
    # clavier Mac]) permettent de faire défiler le panneau (la vue) de gauche à 
    # droite ou de haut en bas (en fait un pan)
    # ---------------------------------------------------------------------------------
    # Voir ce post sur stackedoverflow (merci à watersnake !) :
    # https://stackoverflow.com/questions/51566884/pyqt5-in-a-qgraphicsview-rebind-
    # scrollhanddrag-to-work-with-middle-mouse-butto
    # ---------------------------------------------------------------------------------
    
    def toggleDragMode(self) :
        if self.graphique_vue.dragMode() == QGraphicsView.ScrollHandDrag :
            self.graphique_vue.setDragMode(QGraphicsView.NoDrag)
        else :
            self.graphique_vue.setDragMode(QGraphicsView.ScrollHandDrag)

    
    def mousePressEvent(self, event):
        # Si l'utilisateur appuye en même temps sur la touche Shift
        # (Maj avec un clavier de PC ... ou Shft clavier Mac) + 
        # pointeur souris gauche, on peut translater le panneau
        if (event.buttons() == Qt.LeftButton and event.modifiers() == Qt.ShiftModifier) :
            self.toggleDragMode()
        super().mousePressEvent(event)
    
    
    def mouseReleaseEvent(self, event):
        if self.graphique_vue.dragMode() == QGraphicsView.ScrollHandDrag:
            self.setDragMode(QGraphicsView.NoDrag)
        super().mouseReleaseEvent(event)

    # ---------------------------------------------------------------------------------
    
    def wheelEvent(self, event):
        """
        # --------------------------------
        Merci à rengel pour cette solution
        pour zoomer et dézoomer la vue (et 
        donc l'image qu'elle contient) ...
        Solution à voir ici :
        https://stackoverflow.com/questions/19113532/
        qgraphicsview-zooming-in-and-out-under-mouse-position-using-mouse-wheel
        # --------------------------------
        Zoom in or out of the view.
        --> Zoom avant : roulette milieu de la souris vers le haut
        --> Zoom arrière : roulette milieu de la souris vers la bas
        """

        zoomInFactor = 1.1
        zoomOutFactor = 1 / zoomInFactor
        
        # Set Anchors
        self.graphique_vue.setTransformationAnchor(QGraphicsView.NoAnchor)
        self.graphique_vue.setResizeAnchor(QGraphicsView.NoAnchor)

        # Save the scene pos
        oldPos = self.graphique_vue.mapToScene(event.pos())
    
        # Zoom
        if event.angleDelta().y() > 0:
            zoomFactor = zoomInFactor
        else:
            zoomFactor = zoomOutFactor
        self.graphique_vue.scale(zoomFactor, zoomFactor)
    
        # Get the new position
        newPos = self.graphique_vue.mapToScene(event.pos())
    
        # Move scene to old position
        delta = newPos - oldPos
        self.graphique_vue.translate(delta.x(), delta.y())
        
    # ---------------------------------------------------------------------------------
    

    def afficher(self) :
        ''' Fonction pour afficher une image PNG de l'exposition '''
        
        # Arrêt du QTimer
        # (défini ds l'init)
        self.timer.stop()

        # ====================================================== #
        # Affichage du chemin et nom de l'mage de l'exposition 
        # dans une ligne d'édition (entre le QGraphisScene, 
        # QGraphicsView et les boutons tout en bas)
        # ====================================================== #
        try :
            # Recherche à partir du répertoire temporaire utilisé
            liste_contenu_temp_glob = glob.glob(self.chemin_tempo_system_exploit+os.sep+'data_chemin_enregistrement_image_exposition_murexpo*')
            # On ne garde que le nom de fichier (pas le chemin)
            fichier_temp = os.path.basename(liste_contenu_temp_glob[0])
            # Utilisation de la classe Tempo_lecture pour la lecture 
            # des données dans le répertoire (dit) temporaire
            lect = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), fichier_temp, fichier_temp)
            # ... avec appel de la méthode affichage
            chemin = lect.affichage()[0]
            # Affichage du chemin et nom de fichier dans la ligne d'édition
            self.ligne_aff_chem_enregistr_expo.setText(chemin)
        except :
            try :
                # Recherche à partir du répertoire temporaire utilisé
                liste_contenu_temp_glob = glob.glob(os.getcwd()+os.sep+'data_chemin_enregistrement_image_exposition_murexpo*')
                # On ne garde que le nom de fichier (pas le chemin)
                fichier_temp = os.path.basename(liste_contenu_temp_glob[0])
                # Utilisation de la classe Tempo_lecture pour la lecture 
                # des données dans le répertoire (dit) temporaire
                lect = Tempo_lecture(self.chemin_tempo_system_exploit, os.getcwd(), fichier_temp, fichier_temp)
                # ... avec appel de la méthode affichage
                chemin = lect.affichage()[0]
                # Affichage du chemin et nom de fichier dans la ligne d'édition
                self.ligne_aff_chem_enregistr_expo.setText(chemin)
            except : pass
        # ====================================================== #

        # ====================================================== #
        # On va chercher l'image finale de prévisualisation 
        # générée directement dans le répertoire dit temporaire
        # ====================================================== #
        try :
            liste_image_finale = glob.glob(os.path.join(self.chemin_tempo_system_exploit, 'sauvegarde_image_scene_finale.png'))
        except :
            liste_image_finale = glob.glob(os.path.join(os.getcwd(), 'sauvegarde_image_scene_finale.png'))
        # Affichage d'une page d'information lors du 1er démarrage de MurExpo
        # (et par exemple si l'image /tmp/sauvegarde_image_scene_finale.png est 
        # inéxistante).
        if len(liste_image_finale) == 0 :
            # (--> Affichage fond gris avec la fonction chemin_ressources contenue dans : chemin_data_ressources.py)
            liste_image_finale.append(chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png')))
        # ====================================================== #
        
        # Si l'image du rendu a bien été sauvegardée
        # dans le répertoire temporaire ...
        elif len(liste_image_finale) > 0 :
            # --------------------------------------------------------------
            # Post-traitement pour un affichage correct (si images
            # oeuvres en hors-champ, c'est à dire qui ressortent 
            # des limites du cadre de l'image)
            # --------------------------------------------------------------
            # Réouverture du rendu avec Pillow --> Dans certaines 
            # situations l'image est mal sauvegardée (rajout 
            # d'artefacts disgracieux). En la redécoupant (crop 
            # de Pillow) aux bonnes dimensions (2520 x 720), 
            # l'image est bien enregistrée correctement
            im = Image.open(liste_image_finale[0])
            im.crop((0, 0, 2520, 720)).save(liste_image_finale[0])
            # --------------------------------------------------------------
        
        # Ré-ouverture de l'image et ...
        # Détection de la taille de l'image 
        # (pour harmoniser la vue)
        im = Image.open(liste_image_finale[0])
        largeur, hauteur = im.size
        
        # Maj scène et vue QGraphicsView
        self.scene.clear()
        self.graphique_vue.items().clear()

        # Procédure d'affichage scène et vue
        images = liste_image_finale[0]
        pixmap = QPixmap(images)
        item   = QGraphicsPixmapItem(pixmap)

        # Amélioration de la qualité
        item.setTransformationMode(Qt.SmoothTransformation)
        
        #
        self.scene.addItem(item)
        self.graphique_vue.setScene(self.scene)
        
        # La scène (en passant par le QGraphicsView) s'harmonise 
        # à la taille de l'image. La taille du visionnage (la 
        # taille de la scène) s'adapte automatiquement.
        self.graphique_vue.setSceneRect(0, 0, largeur, hauteur)

        # Affichage de la vue
        self.graphique_vue.show()
        
        # Remise en service du QTimer (défini ds l'init)
        self.timer.start(self.temps_de_latence_vision)
        
        
    def afficher_zoom_avant(self) :
        ''' Fonction de zoom avant '''

        # Appel de la fonction pour le travail 
        # de la 1ère partie de l'affichage
        item = self.travail_affichage()
        
        # Amélioration de la qualité quand on zoome
        item.setTransformationMode(Qt.SmoothTransformation)
        #
        self.scene.addItem(item)
        self.graphique_vue.setScene(self.scene)
        
        # On zoome vers l'avant
        self.graphique_vue.scale(1.00 / 0.950, 1.00 / 0.950)
        
        # Affichage de la vue
        self.graphique_vue.show()
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Visionnage de l'exposition ... clic sur le bouton du zoom avant\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
            
    def afficher_zoom_arriere(self) :
        ''' Fonction de zoom arrière '''
        
        # Appel de la fonction pour le travail 
        # de la 1ère partie de l'affichage
        item = self.travail_affichage()
        
        # Amélioration de la qualité quand on zoome
        item.setTransformationMode(Qt.SmoothTransformation)
        
        #
        self.scene.addItem(item)
        self.graphique_vue.setScene(self.scene)
        
        # On zoome vers l'arrière
        self.graphique_vue.scale(0.950, 0.950)
        
        # Affichage de la vue
        self.graphique_vue.show()
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Visionnage de l'exposition ... clic sur le bouton du zoom arrière\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        
    def afficher_zoom_taille_defaut(self) :
        ''' Fonction de zoom pour revenir à la taille 
        affichée par défaut (présente par défaut et à 
        la base dans la vue). En fait taille 100% '''
        
        # Appel de la fonction pour le travail 
        # de la 1ère partie de l'affichage
        item = self.travail_affichage()
        
        # Amélioration de la qualité quand on zoome
        item.setTransformationMode(Qt.SmoothTransformation)
        #
        self.scene.addItem(item)
        self.graphique_vue.setScene(self.scene)
        
        # On revient à la taille d'affichage par défaut
        self.graphique_vue.setTransform(QTransform())
        
        # Affichage de la vue
        self.graphique_vue.show()
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Visionnage de l'exposition ... clic sur le bouton de l'affichage à 100%\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        
    def afficher_taille_adaptee(self) :
        ''' Fonction d'affichage de l'image 
        en vue image entière (en fait on 
        voit toute l'exposition) '''

        # Appel de la fonction pour le travail 
        # de la 1ère partie de l'affichage
        item = self.travail_affichage()
        
        # Amélioration de la qualité quand on zoome
        item.setTransformationMode(Qt.SmoothTransformation)
        #
        self.scene.addItem(item)
        self.graphique_vue.setScene(self.scene)
        
        # Adapter la vue (le QGraphicsView) pour qu'il soit totalement 
        # visible --> L'image déployée s'adapte à la taille de la scène
        self.graphique_vue.fitInView(item, Qt.KeepAspectRatio)

        # Affichage de la vue
        self.graphique_vue.show()
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Visionnage de l'exposition ... clic sur le bouton de l'affichage en vue entière\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
    
    
    def travail_affichage(self) :
        ''' Pour le travail du zomm avant 
        (zoom +), zoom arrière (zoom -) et
        l'affichage de la taille par défaut '''
        
        # ====================================================== #
        # On va chercher l'image finale de prévisualisation 
        # générée directement dans le répertoire dit temporaire
        # ====================================================== #
        try :
            liste_image_finale = glob.glob(os.path.join(self.chemin_tempo_system_exploit, 'sauvegarde_image_scene_finale.png'))
        except :
            liste_image_finale = glob.glob(os.path.join(os.getcwd(), 'sauvegarde_image_scene_finale.png'))
        # Si l'image du montage (exposition) 
        # final(e) n'existe pas ...
        if len(liste_image_finale) == 0 :
            # (--> Affichage avec la fonction chemin_ressources  contenue dans : chemin_data_ressources.py)
            liste_image_finale.append(chemin_ressources(os.path.join('ressources_murexpo', 'fond_gris.png')))
        # ====================================================== #
        
        # ---------------------------------------------------------------
        # Retour commandes ....
        # ---------------------------------------------------------------
        # Récupération des commandes pour le terminal interne de MurExpo
        self.capter_commandes_murexpo.liste_capter_commandes(_("(* Action) Visionnage de l'exposition\n"))
        # Ecriture des commandes dans le fichier
        # tempo data_retour_commandes_murexpo
        self.capter_commandes_murexpo.liste_ecrire_commandes()
        # ---------------------------------------------------------------
        
        # Procédure d'affichage scène et vue
        images = liste_image_finale[0]
        pixmap = QPixmap(images)
        item   = QGraphicsPixmapItem(pixmap)
        
        return item
    
    
    def direction_onglet_conception_exportation(self) :
        """ Fonction utilisable quand l'utilisateur 
        clique sur le bouton Revenir à la page des 
        réglages à la page 4 du QStackedWidget. En 
        fait cela permet à l'utilisateur d'accéder/ 
        revenir à l'onglet Conception et exportation """

        # =========================================================================
        # Utilisation du signal pour afficher automatiquement l'onglet 
        # ... . 
        # 2ème partie ==> le signal est émis pour l'id 0 (du QTabWidget défini 
        # [les onglets] défini dans le script main (murexpo_gui.py)). A voir aussi
        # Une ligne de code par rapport à cela se trouve dans murexpo_gui.py. 
        # Un grand merci à eyllanesc !!!
        # A voir ici : https://stackoverflow.com/questions/63065472/call-and-use-a-
        # qtabwidget-defined-in-the-main-class-in-another-class#63065558
        self.customSignal_onglet_retour_conception_exportation.emit(0)
        # =========================================================================
